/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* PISTONE: animazione con semigrafici in pagina testo */


#include "Jconio.c"                                 /*libreria di console*/
#include <stdio.h>                                  /*per sprintf, sscanf*/
#include <stdlib.h>
#include <time.h>


int  AnimaPistone(int iniz);                        /*prototipi funzioni*/
void Sincro(double sec);
int  InputInt(char *prompt, int mi, int ma);
void ClrEol(void);


#define POSX  35        /*posizione del disegno*/
#define POSY  6


/********************************************************************
 * main
 ********************************************************************/

main()
{
    time_t t;           /*tempo limite, riferito al clock <1>*/
    double tp;          /*tempo minimo tra i passi, in secondi*/
    int    r;

    Jclrscr();                                      /*cancella il video*/
    Jcursor(0);                                     /*spegne cursore <2>*/
    Jgotoxy(15,2);                                  /*mostra titolo*/
    Jputs("Animazione con semigrafici e libreria di console:");

    Jgotoxy(POSX,POSY);                             /*prepara disegno <3>*/
    AnimaPistone(0);

    Jgotoxy(1,21);                                  /*input tempo passo <4>*/
    tp = 1.0 / InputInt("Passi al secondo",1,30);
    Jgotoxy(1,22);                                  /*e tempo limite <5>*/
    t = CLK_TCK * InputInt("Tempo limite, secondi",3,30);
    t += clock();                                   /*calcola limite <6>*/

    Sincro(0.0);                                    /*avvia timer <7>*/
    while (clock() < t) {                           /*se nel limite <8>*/
        do {                                        /*fa un ciclo*/
            r = AnimaPistone(1);                    /*fa un passo <9>*/
            Sincro(tp);                             /*sincronizza <10>*/
        } while (r);
    }

    Jcursor(1);                                     /*riaccende cursore*/
    Jgotoxy(1,23);                                  /*cursore in fondo*/
}

/* Note su main:
    <1>  Sarebbe pi leggibile usare la funzione time invece di clock (vedi
         lezione B8), ma clock  generalmente pi precisa e richiede meno
         tempo in esecuzione: non serve usare difftime per calcolare il tempo
         trascorso. clock ritorna infatti un valore di tipo clock_t, che
         indica il tempo trascorso dall'avviamento del programma.
    <2>  Il cursore ha senso solo quando l'utente pu interagire, cio in
         genere durante un input. Adottiamo quindi la convenzione di tenerlo
         sempre spento, tranne appunto in occasione degli input. In questo
         modo l'animazione non viene disturbata. Occorre per ricordarsi di
         riaccenderlo a fine programma.
    <3>  Se chiamata con 0 come argomento, la AnimaPistone si inizializza,
         disegnando il pistone in posizione di riposo.
    <4>  tp contiene il tempo minimo, in secondi, che deve intercorrere tra
         l'inizio di un passo del pistone e l'inizio del passo successivo.
         E' quindi l'inverso del numero di passi al secondo. E' essenziale
         scrivere 1.0 e non 1, altrimenti si ha una divisione intera con
         perdita del resto, e il risultato  quasi sempre zero.
    <5>  Non avendo ancora introdotto la possibilit di 'sbirciare' la
         tastiera per sapere se  stato premuto un tasto, l'unico modo per
         fermare il programma  stabilire un limite. Usiamo un tempo limite
         anzich un numero massimo di cicli, perch la velocit  variabile.
         Moltiplicando il tempo desiderato (in secondi) per il numero di tick
         del clock per secondo (CLK_TCK, definita in <time.h>), si ottiene il
         numero di tick da attendere.
    <6>  Sommando al clock corrente il numero di tick da attendere, si ha
         il momento (valore di clock) in cui scadr il ritardo desiderato.
    <7>  La Sincro, che sincronizza l'animazione alla velocit desiderata,
         va chiamata una prima volta con 0.0 per inizializzarla.
    <8>  L'animazione continua finch il valore restituito dalla funzione
         clock non raggiunge (o supera) il tempo limite fissato. Per estetica,
         a questo punto lasciamo che il pistone termini comunque il ciclo
         (il che pu comportare ancora alcuni secondi).
    <9>  Se chiamata con 1 come argomento, la AnimaPistone calcola la
         posizione successiva del pistone e la disegna, ritornando 0 quando
         il pistone  in posizione di riposo. Non c' bisogno di conservare
         nel main informazioni riguardanti la posizione corrente del pistone,
         quindi le lasciamo al loro posto: dentro la AnimaPistone.
    <10> La Sincro aspetta che sia trascorso il tempo indicato, in secondi
         (tp), dalla chiamata precedente alla Sincro stessa. Questo  diverso
         da una semplice attesa di un certo tempo: se il computer  veloce,
         una parte del tempo viene impiegato nel disegno ed il resto viene
         fatto passare dalla Sincro; se il computer  lento ed il disegno
         impiega pi del tempo desiderato tra un passo e l'altro, la Sincro
         ritorna immediatamente senza introdurre alcun ritardo. Data la
         variet di computer in circolazione, un controllo automatico di
         velocit come questo  essenziale perch l'animazione non risulti
         troppo lenta su un computer e troppo veloce su un altro.
*/


/********************************************************************
 * AnimaPistone: Se iniz==0 si inizializza disegnando il pistone alle
 *               coordinate video correnti, in posizione di riposo,
 *               Se iniz==1 muove il pistone alla posizione successiva,
 *               ritorna 0 quando si trova alla posizione originaria.
 ********************************************************************/

int AnimaPistone(int iniz)     /*<1>*/
{
    const  int  pmax = 8;               /*posizione massima pistone*/
    const  int  pmed = 5;               /*posizione di cambio pressione*/

    static int  posx;                   /*coordinate disegno sul video <2>*/
    static int  posy;
    static int  pv;                     /*posizione corrente del pistone*/
    static int  direz;                  /*direzione di movimento (1=gi)*/

    static char gp[] = "          ";   /*gambo del pistone <3>*/
    static char cc[] = "        ";   /*cima cilindro, con gambo pistone*/
    static char gc[] = "        ";   /*gambo del pistone, con cilindro*/
    static char pc[] = "͵";   /*pistone, con cilindro*/
    static char g1[] = "     ";   /*diverse pressioni del gas*/
    static char g2[] = "    ";
    static char g3[] = "";
    static char g4[] = "";
    static char bc[] = "";   /*base cilindro*/

    char *pg1, *pg2;                    /*pointer a pressioni correnti <4>*/
    int  y;

    if (iniz == 0) {                                /*inizializza*/
        posx = Jposx();                             /*guarda dove si trova*/
        posy = Jposy();
        pv = 0;                                     /*posizione pistone*/
        direz = 1;                                  /*e direzione movimento*/
    } else {
        pv += direz;                                /*avanza il pistone*/
        if (pv == pmax || pv == 0) {                /*inverte in fondo*/
           direz = -direz;
        }
    }

    Jgotoxy(posx,posy);
    Jputs(gp);                                      /*gambo pistone*/
    Jgotoxy(posx,posy+1);
    Jputs(cc);                                      /*cima cilindro*/

    if (pv < pmed) {                                /*bassa pressione*/
        pg1 = g1;                                   /*<5>*/
        pg2 = g2;
    } else if (pv < pmax) {                         /*media pressione*/
        pg1 = pg2 = g3;
    } else {                                        /*alta pressione*/
        pg1 = pg2 = g4;
    }

    for (y = 0; y <= pmax; y++) {
        Jgotoxy(posx,posy+y+2);                     /*si posiziona*/
        if (y < pv) {                               /*sopra il pistone*/
            Jputs(gc);
        } else if (y == pv) {                       /*pistone*/
            Jputs(pc);
        } else if (y%2) {                           /*gas dispari*/
            Jputs(pg1);
        } else {                                    /*gas pari*/
            Jputs(pg2);
        }
    }
    Jgotoxy(posx,posy+pmax+3);                      /*ultimo gas*/
    Jputs(pg2);
    Jgotoxy(posx,posy+pmax+4);                      /*base cilindro*/
    Jputs(bc);

    return (pv != 0);                               /*zero se a riposo*/
}

/*  Note su AnimaPistone:
    <1> Questa funzione va chiamata una volta all'inizio del programma
        passando zero come argomento, per inizializzarla. La tecnica usata
        per l'animazione  una delle peggiori: ridisegnare tutto ogni volta.
        Si potrebbe facilmente ottimizzare disegnando solo i caratteri
        modificati; il guadagno in velocit sarebbe notevole.
    <2> Le variabili statiche conservano il loro valore tra una chiamata e
        l'altra della funzione. In questo modo, coordinate del disegno,
        posizione e direzione del pistone restano locali alla funzione e
        non c' bisogno di variabili globali: tutto quanto riguarda il
        disegno  'privato'. L'isolamento tra le varie parti del lavoro 
        importante nella programmazione strutturata: migliora l'affidabilit
        e la leggibilit del programma.
    <3> E' pi chiaro definire all'inizio della funzione una serie di
        costanti stringa, piuttosto che spargerle tra le istruzioni della
        funzione stessa: un'eventuale modifica risulta molto pi semplice.
        Le stringhe sono definite static, in modo che non vengano assegnate
        ad ogni chiamata della funzione ma lo siano una sola volta, alla
        partenza del programma.
    <4> Il nome di una stringa (es. gp)  un pointer a char. Copiandolo in
        pg1, pg1 punta alla stessa stringa e pu essere passata a tutte le
        funzioni che ricevono una stringa (es. Jputs).
    <5> Vedi nota precedente.
*/


/********************************************************************
 * Sincro: Aspetta che siano passati sec secondi dalla chiamata
 *         precedente (se prima chiamata, aspetta sec secondi).
 ********************************************************************/

void Sincro(double sec)     /*<1>*/
{
    static int     prima = -1;  /*flag di prima chiamata <2>*/
    static clock_t tprec = 0;   /*tempo alla chiamata precedente*/

    clock_t t;                  /*istante da aspettare*/

    if (prima) {                                    /*se prima chiamata*/
        tprec = clock();                            /*prende riferimento*/
        prima = 0;                                  /*non lo rif pi*/
    }

    t = tprec+((clock_t)(sec*CLK_TCK));             /*calcola riferim. <3>*/
    while (clock() < t)                             /*e lo aspetta <4>*/
        ;

    tprec = clock();                                /*per prossima volta <5>*/
}

/*  Note su Sincro:
    <1> La Sincro aspetta il tempo indicato, non riferito all'istante della
        chiamata corrente ma a quello della chiamata precedente. Serve per
        costruire temporizzazioni indipendenti dalla velocit della macchina,
        ad esempio per sincronizzare i 'fotogrammi' di un'animazione, come
        in questo caso (vedi nota 10 del main).
    <2> Usare una variabile statica inizializzata con un certo valore  la
        tecnica classica per riconoscere la prima chiamata di una funzione,
        in modo che questa possa fare le necessarie inizializzazioni.
    <3> 'Mette la sveglia' all'istante calcolato sommando il tempo richiesto
        con l'istante del ritorno dalla chiamata precedente. Per il calcolo,
        vedi nota 5 del main.
    <4> Questo ciclo non pu essere arrestato in alcun modo. Occorre quindi
        fare attenzione a non richiedere ritardi troppo lunghi. Una versione
        pi evoluta di Sincro potrebbe uscire anche con la pressione di un
        tasto (la relativa funzione  introdotta nella lezione E2).
    <5> L'istante del ritorno viene registrato nella variabile statica tprec,
        in modo da servire come riferimento alla successiva chiamata.
*/


/********************************************************************
 * InputInt: mostra prompt, input intero compreso tra mi e ma,
 *           ritorna il valore introdotto, ripete se non valido.
 ********************************************************************/

int InputInt(char *prompt, int mi, int ma)      /*<1>*/
{
    char buf[128];      /*buffer di linea*/
    char pr[80];        /*per costruzione stringa di prompt*/
    int  n;             /*numero introdotto*/
    int  x,y;           /*posizione di input*/

    sprintf(pr,"%s (%d..%d): ",prompt,mi,ma);       /*prompt, limiti <2>*/
    Jputs(pr);
    x = Jposx();                                    /*registra posizione <3>*/
    y = Jposy();

    do {
        Jcursor(1);                                 /*accende cursore <4>*/
        if (gets(buf) == NULL) {                    /*legge stringa*/
            *buf = 0;                               /*nulla se EOF*/
        }
        Jcursor(0);                                 /*spegne cursore*/

        n = atoi(buf);                              /*converte a numero*/
        if (*buf == 0 || n < mi || n > ma) {        /*controlla se valido*/
            Jputch('\a');                           /*no, segnala errore*/
            Jgotoxy(x,y);                           /*riposiziona <5>*/
            ClrEol();                               /*pulisce riga <6>*/
            *buf = 0;                               /*e annulla*/
        }
    } while (*buf == 0);                            /*ripete se non ok*/

    return n;                                       /*ritorna valore <7>*/
}

/*  Note su InputInt:
    <1> La InputInt deriva dal programma TABULA. E' stata modificata per
        mantenere l'input sulla stessa linea in caso di errore, e per usare
        la libreria di console Jackson. Mostra il cursore solo durante
        l'input.
    <2> Non essendoci una Jprintf, occorre stampare in una stringa con la
        sprintf e poi stampare la stringa con la Jputs.
    <3> Ricorda la posizione corrente, in modo da potervi poi tornare in
        caso di errore. Dato che si resta sulla stessa linea, il prompt
         stato stampato una volta per tutte, all'inizio.
    <4> Il cursore viene acceso solo quando 'tocca all'utente': durante
        l'input con la gets. Verr poi spento non appena terminato l'input.
    <5> In caso di errore, ritorna alla posizione registrata in precedenza
        (nota 3), in modo da rimanere sulla stessa riga. Non siamo ancora
        all'input perfetto: bisognerebbe anche limitare il numero massimo
        di caratteri che l'utente pu introdurre. Per farlo servono per le
        funzioni di input di console, esaminate nella lezione E2.
    <6> La ClrEol cancella il resto della linea, lasciando il cursore nella
        posizione corrente.
    <7> Il cursore (non visibile) viene lasciato dove si trova. Prima di
        un'altra operazione occorrer dunque posizionarlo con Jgotoxy.
*/


/********************************************************************
 * ClrEol: cancella dalla posizione corrente a fine riga, meno un
 *         carattere per evitare scrolling sull'ultima linea.
 ********************************************************************/

void ClrEol(void)   /*<1>*/
{
    int x;          /*posizione corrente*/
    int x1;

    x = x1 = Jposx();                               /*registra posizione <2>*/
    while (x1++ < 79) {                             /*cancella con spazi <3>*/
        Jputch(' ');
    }

    Jgotoxy(x,Jposy());                             /*torna dov'era <4>*/
}

/*  Note su ClrEol:
    <1> Il nome deriva da 'clear to end of line', funzione che in altri
        computer  normalmente disponibile.
    <2> E' necessario conoscere la posizione corrente per potervi poi
        riportare il cursore.
    <3> La cancellazione con una serie di spazi non  la pi veloce, ma in
        questo caso  pi che sufficiente. Non arriva al carattere 80 per
        evitare uno scrolling (scorrimento verso l'alto) del video quando
        la cancellazione avviene sull'ultima riga (la 25).
    <4> La posizione verticale non  cambiata, quindi non c'era bisogno di
        registrarla: la si richiede con Jposy.
*/
