/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* TABULA: converte tabulazioni in spazi, leggendo da un file */


#include <stdio.h>
#include <stdlib.h>


void TabulaFile(FILE *ft, int ptab);                /*prototipi funzioni*/
int InputInt(char *prompt, int mi, int ma);


/********************************************************************
 * main
 ********************************************************************/

main()
{
    char nomeFile[] = "TESTO.TAB";      /*nome del file da leggere <1>*/
    FILE *f;                            /*<2>*/
    int  t;

    printf("\n Conversione di Tab in spazi dal file %s:\n\n",nomeFile);

    if ( (f = fopen(nomeFile,"r")) == NULL) {       /*apre file <3>*/
        printf("- Non trovo il file"
               " \"%s\".\a\n",nomeFile);            /*segnala errore <4>*/
    } else {
        t = InputInt("Passo di tabulazione",1,16);  /*dimensione tab <5>*/
        putchar('\n');                              /*a capo*/
        TabulaFile(f,t);                            /*stampa tabulato <6>*/
    }
    fclose(f);                                      /*chiude file <7>*/
}

/* Note su main:
    <1> Invece di usare una costante stringa, sarebbe il caso di chiedere
        all'utente il nome del file da tabulare, usando ad esempio una gets.
        Bisognerebbe per dare la possibilit di listare i file disponibili
        o, meglio ancora, scegliere direttamente tra i file presenti nella
        directory (vedremo pi avanti nel corso esperti come si pu fare).
        Alternativamente, si potrebbe trasformare TABULA in un filtro, come
        illustrato nel programma ACCENTI di questa stessa lezione.
    <2> f  un pointer ad una struttura di tipo FILE, o pi familiamente un
        pointer a file (o 'stream'). La libreria di standard I/O (stdio) usa
        i pointer a file per tutte le operazioni sui file.
    <3> Indicando "r", il file viene aperto in sola lettura:  impossibile
        modificarne il contenuto, anche per sbaglio. E' fondamentale tener
        conto della possibilit che l'apertura del file possa fallire, ad
        esempio perch il file non c'. In questo caso la fopen ritorna NULL.
    <4> Se il file non si pu aprire, non si deve proseguire col programma:
        il suo pointer (f)  infatti NULL ed ogni chiamata alle funzioni di
        stdio pu creare problemi. Notare l'uso di \" per introdurre le
        virgolette in mezzo ad una stringa.
    <5> La InputInt chiede all'utente (e ritorna) un numero compreso tra i
        due limiti dati. E' definita pi avanti nel programma.
    <6> La TabulaFile stampa il contenuto del file, convertendo ogni Tab
        in un adeguato numero di spazi.
    <7> Anche se i file aperti vengono automaticamente chiusi quando il
        programma termina (ma non lo sono se 'si pianta'),  sempre meglio
        richiederne esplicitamente la chiusura quando si ha finito di usarli,
        anche e specialmente nel caso di file aperti in scrittura.
*/


/********************************************************************
 * TabulaFile: stampa il contenuto del file ft usando pt come passo
 *             di tabulazione (quando incontra Tab, aggiunge spazi
 *             per arrivare alla successiva colonna di tabulazione.
 ********************************************************************/

void TabulaFile(FILE *ft, int ptab)     /*<1>*/
{
    int ph;     /*pos. orizz., in caratteri mancanti al prossimo tab <2>*/
    int c;

    ph = ptab;                                      /*azzera a ptab <3>*/
    while ( (c = fgetc(ft)) != EOF) {               /*scandisce file <4>*/
        if (c != '\t') {                            /*se non  un Tab <5>*/
            putchar(c);                             /*lo stampa*/
            if (c == '\n') {                        /*se newline <6>*/
                ph = ptab;                          /*riazzera*/
            } else if (--ph <= 0) {                 /*si avvicina <7>*/
                ph = ptab;                          /*se arrivato, riparte*/
            }
        } else {                                    /*se  un Tab*/
            while (ph--)  putchar(' ');             /*tabula con spazi <8>*/
            ph = ptab;                              /*e riazzera*/
        }
    }
}

/*  Note su TabulaFile:
    <1> La TabulaFile riceve due argomenti nei parametri indicati: il primo 
        un pointer al file da leggere, il secondo  un numero che indica la
        spaziatura delle colonne di tabulazione, cio la larghezza di un tab
        (per la chiamata, vedi nota 6 del main).
    <2> La tecnica usata per far s che un Tab mandi alla successiva colonna
        di tabulazione (le colonne di tabulazione sono spaziate ogni pt
        caratteri)  quella del conteggio alla rovescia: la variabile ph
        contiene il numero di caratteri mancanti alla prossima colonna di
        tabulazione. In questo modo si evitano calcoli (es. divisioni).
    <3> Il conteggio alla rovescia viene avviato a inizio file, che  anche
        un inizio riga.
    <4> la fgetc legge un carattere dal file (puntato da) ft. Come sempre in
        C, per contenere un singolo carattere si usa un int. Se il file 
        finito la fgetc ritorna EOF (end of file), che tra l'altro  un
        valore non confondibile con nessun carattere stampabile. Non pu
        infatti essere contenuto in un char, ma ha proprio bisogno di un int.
    <5> Il carattere speciale '\t'  il Tab. Il suo codice ASCII  9, ma 
        molto meglio usare il suo nome simbolico '\t'.
    <6> Se la linea  finita (newline), il conteggio alla rovescia va fatto
        ripartire daccapo.
    <7> Se la linea non  finita, il conteggio alla rovescia avanza. Se
        arriva a zero, siamo su una colonna di Tab e lo riavviamo con ptab
        per contare quanti caratteri mancano alla prossima tabulazione.
    <8> Quando si incontra un Tab, basta stampare il numero di spazi che
        mancano alla successiva colonna di tabulazione, cio il valore che
         tenuto sempre aggiornato in ph.
*/


/********************************************************************
 * InputInt: mostra prompt, input intero compreso tra mi e ma,
 *           ritorna il valore introdotto, ripete se non valido.
 ********************************************************************/

int InputInt(char *prompt, int mi, int ma)      /*<1>*/
{
    char buf[128];      /*buffer di linea <2>*/
    int  n;             /*numero introdotto*/

    do {
        printf("%s (%d..%d): ",prompt,mi,ma);       /*prompt, limiti*/
        if (gets(buf) == NULL) {                    /*legge stringa*/
            *buf = 0;                               /*nulla se EOF <3>*/
        }
        n = atoi(buf);                              /*converte a numero <4>*/
        if (*buf == 0 || n < mi || n > ma) {        /*controlla se valido*/
            putchar('\a');                          /*segnala errore <5>*/
            *buf = 0;                               /*e annulla*/
        }
    } while (*buf == 0);                            /*ripete se non ok*/

    return n;                                       /*ritorna valore*/
}

/*  Note su InputInt:
    <1> La inputInt riceve una stringa da stampare come prompt (sotto forma
        di pointer al primo carattere dalla stringa), e due int che indicano
        rispettivamente il valore minimo e massimo accettabili (per la
        chiamata, vedi nota 5 del main).
    <2> Nei computer MS-DOS, la gets legge al massimo 127 caratteri pi lo
        zero finale. Usando come buffer (posto dove accumulare i caratteri
        introdotti) una stringa di 128 caratteri, non si corre quindi alcun
        rischio che la gets vada a scrivere oltre la fine del buffer stesso.
    <3> L'utente pu premere F6 o Ctrl-Z, causando il ritorno immediato della
        gets con il valore EOF. In questo caso la stringa buf[] non viene
        alterata, quindi per evitare problemi mettiamo uno zero nel primo
        carattere (*buf equivale a buf[0]), ottenendo una stringa nulla.
    <4> Per leggere un int da una stringa,  inutile usare la sscanf. La atoi
        (che richiede lo header <stdlib.h>) equivale a sscanf(buf,"%d").
    <5> Il carattere '\a' non viene stampato, ma fa suonare un beep al
        computer (alert).
*/
