/*
   Questa libreria fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* JCONIO: 'libreria' di funzioni per Input/Ouput su console   */
/*         compilabili sia con Turbo C 2.0 che con Quick C 2.0 */
/*         Versione E2: output su video e input da tastiera.   */


#if     !defined(__JCONIO_DEF_)             /*compila solo se non compilata*/
#define __JCONIO_DEF_


/* Nota:
    Queste funzioni usano informazioni e tecniche di programmazione
    introdotte nel Corso di C in lezioni successive alla E2. Per
    usarle non  comunque necessario conoscere queste tecniche.
*/


/****************** Prototipi delle funzioni di output *******************/

void Jtexttcolor(int col);          /*Sceglie il colore di foreground*/
void Jtextback(int col);            /*Sceglie il colore di background*/
int  Jgettextcolor(void);           /*Ritorna il colore di foreground*/
int  Jgettextback(void);            /*Ritorna il colore di background*/
void Jclrscr(void);                 /*Cancella al colore di background*/
void Jgotoxy(int x,int y);          /*Sposta il cursore alla posizione x,y*/
void Jputch(int c);                 /*Stampa il carattere c*/
void Jputs(char *s);                /*Stampa la stringa s, non va a capo*/
void Jcursor(int vis);              /*Spegne cursore (0) o riaccende (1)*/
int  Jposx(void);                   /*Ritorna la colonna corrente*/
int  Jposy(void);                   /*Ritorna la linea corrente*/

/* Note:
    - La Jputs si comporta in modo diverso tra Turbo C e Quick C quando
      incontra un newline (\n). Per evitare problemi, usare sempre la
      combinazione \r\n (CRLF) per andare a capo. Inoltre la Jputs non
      aggiunge alcun \n dopo la stampa della stringa.
    - Non  fornita un'equivalente della printf, per la difficolt di
      realizzarla con il Quick C. Bisogna stampare in una stringa con la
      sprintf, e poi stampare la stringa con la Jputs.
    - Rispetto alla versione E1, sono state aggiunte le funzioni per
      conoscere i colori correnti (gettextcolor e gettextback).


/******************* Prototipi delle funzioni di input *******************/

int  Jgetkey(int cur);              /*Legge ogni tasto, mostra cursore se 1*/
void Jclrkey(void);                 /*Cancella il buffer di tastiera*/


/*************************************************************************/


#include <dos.h>                            /*richiesto per chiamate a BIOS*/
#include <conio.h>                          /*usa librerie di console*/


/********************************************************************
 * Per il Turbo C 2.0 la maggior parte delle funzioni desiderate sono
 * gi disponibili nella libreria di console: basta cambiare i nomi.
 ********************************************************************/

#ifdef __TURBOC__                           /*compila se Turbo C*/

  #define Jtextcolor    textcolor           /*definisce equivalenze nomi*/
  #define Jtextback     textbackground
  #define Jclrscr       clrscr
  #define Jgotoxy       gotoxy
  #define Jputch        putch
  #define Jputs         cputs
  #define Jposx         wherex
  #define Jposy         wherey

  /* Jgettextcolor: ritorna colore corrente di foreground */

  int Jgettextcolor(void)
  {
      struct text_info ti;

      gettextinfo(&ti);                     /*legge dati correnti*/
      return (ti.attribute & 0x8f);         /*estrae foreground+blink*/
  }

  /* Jgettextback: ritorna colore corrente di background */

  int Jgettextback(void)
  {
      struct text_info ti;

      gettextinfo(&ti);                     /*legge dati correnti*/
      return ((ti.attribute & 0x70) >> 4);  /*estrae background*/
  }


/********************************************************************
 * Per il Quick C 2.0 serve molto pi lavoro, per costruire quello
 * che manca e semplificare l'uso di quello che c'. L'uso della
 * libreria <graph> imposto dalla Microsoft comporta una notevole
 * inefficienza in termini di spazio quando si vuole usare solamente
 * il video alfanumerico.
 ********************************************************************/

#else                                       /*compila se Quick C*/

  #include <graph.h>                        /*usa la libreria grafica*/

  #define BLACK             0               /*nomi dei colori in modo testo*/
  #define BLUE              1
  #define GREEN             2
  #define CYAN              3
  #define RED               4
  #define MAGENTA           5
  #define BROWN             6
  #define LIGHTGRAY         7
  #define DARKGRAY          8
  #define LIGHTBLUE         9
  #define LIGHTGREEN        10
  #define LIGHTCYAN         11
  #define LIGHTRED          12
  #define LIGHTMAGENTA      13
  #define YELLOW            14
  #define WHITE             15

  #define BLINK             16              /*aggiungere per il lampeggio*/

  /* macro per sfruttare funzioni fornite nella libreria del Quick C 2.0: */

  #define Jtextcolor(col)  ((void)_settextcolor(col))
  #define Jtextback(col)   ((void)_setbkcolor((long)(col)))
  #define Jgettextcolor()  ((int)(_gettextcolor()))
  #define Jgettextback()   ((int)(_getbkcolor()))
  #define Jclrscr()        _clearscreen(_GCLEARSCREEN)
  #define Jgotoxy(x,y)     ((void) _settextposition(y,x));
  #define Jputs            _outtext

  /* Jputch: stampa il carattere c */

  void Jputch(int c)
  {
      char b[2];                            /*stringa di 2 char*/

      if (c == '\a') {                      /*se beep, lo suona*/
          putch(c);
      } else {
          b[0] = (char)c; b[1] = 0;         /*costruisce stringa*/
          _outtext(b);                      /*e la stampa*/
      }
  }

  /* Jposx: ritorna posizione orizzontale (colonna) del cursore */

  int Jposx(void)
  {
      struct rccoord rc;                    /*contiene posizione x,y*/

      rc = _gettextposition();              /*legge posizione*/
      return(rc.col);                       /*ritorna colonna*/
  }

  /* Jposx: ritorna posizione verticale (riga) del cursore */

  int Jposy(void)
  {
      struct rccoord rc;                    /*contiene posizione x,y*/

      rc = _gettextposition();              /*legge posizione*/
      return(rc.row);                       /*ritorna riga*/
  }


#endif                                      /*fine compilazione condizionale*/


/********************************************************************
 * Jcursor: spegne il cursore se vis  falsa (0), lo accende se vera.
 * Il controllo dell'accensione del cursore non  disponibile in
 * nessuna delle librerie fornite, per cui lo costruiamo usando
 * chiamate dirette al BIOS, compilabili da entrambi i compilatori.
 ********************************************************************/

void Jcursor(int vis)
{
    static int           v = 1;             /*di solito, cursore visibile*/
    static unsigned char start;             /*linea iniziale del cursore*/
    static unsigned char end;               /*linea finale del cursore*/

    union REGS ir;                          /*per interfaccia BIOS*/
    union REGS or;

    if (vis != v) {                         /*agisce solo se cambia*/
        if (vis) {                          /*se deve mostrarlo*/
            ir.h.ah = 0x01;                 /*Set Cursor Size*/
            ir.h.ch = start;                /*valori salvati prima*/
            ir.h.cl = end;
            int86(0x10,&ir,&or);            /*chiama BIOS Video*/
        } else {                            /*se deve nasconderlo*/
            ir.h.ah = 0x03;                 /*Read Cursor Pos, Size*/
            ir.h.bh = 0;                    /*pagina video 0*/
            int86(0x10,&ir,&or);            /*chiama BIOS Video*/
            start = or.h.ch;                /*registra dati cursore*/
            end = or.h.cl;
            ir.h.ah = 0x01;                 /*Set Cursor Size*/
            ir.h.ch = 0x20;                 /*nessun cursore*/
            int86(0x10,&ir,&or);            /*chiama BIOS Video*/
        }
        v = vis;                            /*registra nuovo stato*/
    }
}


/********************************************************************
 * Jgetkey: legge qualunque tasto, inclusi i tasti speciali con
 * doppio codice, per i quali ritorna 1000 pi il codice di scansione.
 * Se cur==vero, accende il cursore prima dell'input e lo spegne
 * dopo l'input, se cur==falso non tocca il cursore.
 ********************************************************************/

int Jgetkey(int cur)
{
    int c;

    if (cur)  Jcursor(1);                   /*mostra cursore se richiesto*/

    c = getch();                            /*legge un tasto*/
    if (c == 0) {                           /*se speciale*/
        c = getch()+1000;                   /*ne legge un altro, corregge*/
    }

    if (cur)  Jcursor(0);                   /*nasconde cursore se richiesto*/
    return c;                               /*ritorna tasto*/
}


/********************************************************************
 * Jclrkey: cancella il buffer di tastiera leggendo ed ignorando
 * tutti i caratteri contenuti.
 ********************************************************************/

void Jclrkey(void)
{
    while (kbhit()) {                       /*finch ci sono caratteri*/
        getch();                            /*li legge e li ignora*/
    }
}


#endif                                      /*(se non compilata)*/
