/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* LIFE: il celebre gioco matematico - by Enrico Colombini 1990 */


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include "Jconio.c"


#define DH    40        /*dimensioni della matrice di cellule*/
#define DV    24

#define VIVA  1         /*simboli di cellula viva e morta <1>*/
#define MORTA ' '


typedef char Gen[DV+2][DH+2];     /*matrice di generazione, <2> <3>*/


/* prototipi funzioni: */

long Controllo(Gen *gp, long ng);
void GenNulla(Gen *gp);
void GenCasuale(Gen *gp);
void StampaGen(Gen *gp);
void NuovaGen(Gen *vgp, Gen *ngp);
void CollegaBordi(Gen *gp);
void StampaNumGen(long ng);
void Messaggio(char *s, int h);


/* Note sulle dichiarazioni:
    <1> Nei PC, il carattere con codice 1 disegna una faccina. Non lo
        scriviamo come carattere perch si tratta di un carattere di
        controllo che in genere gli editor non possono riprodurre.
    <2> Il tipo Gen  un array bidimensionale (matrice) di char, usato per
        contenere una generazione di Life. Gli elementi da 1 a DV (o DH)
        contengono le cellule, mentre gli elementi 0 e DV+1 (o DH+1) formano
        una 'cornice' attorno alla matrice di cellule, usata nell'algoritmo
        per il calcolo della generazione successiva.
    <3> In un array multidimensionale (come il tipo Gen), il secondo indice
         quello che varia pi velocemente. In altri termini, avendo indicato
        la dimensione verticale prima di quella orizzontale, gli elementi di
        una stessa riga sono adiacenti (mentre non lo sono quelli di una
        stessa colonna). Un array di tipo Gen pu essere considerato un array
        di DV+2 stringhe, ciascuna di DH+2 caratteri. Ciascuna riga pu
        dunque essere trattata come una stringa.
*/


/********************************************************************
 * main
 ********************************************************************/

main()
{
    Gen   g[2];       /*due generazioni successive di cellule <1>*/
    int   gcorr;      /*generazione corrente (0/1)*/
    int   gnew;       /*e nuova generazione*/
    long  ngen;       /*contatore generazioni*/

    Jcursor(0);                                     /*spegne cursore*/
    srand((unsigned int)time(NULL));                /*avvia random <2>*/
    Jclrscr();                                      /*cancella schermo*/

    gcorr = 0;                                      /*inizia con gen 0*/
    GenCasuale(g+gcorr);                            /*estrae cellule <3>*/
    StampaGen(g+gcorr);                             /*e le mostra*/
    ngen = 0;                                       /*azzera contatore*/

    Jclrkey();                                      /*pulisce buffer*/
    do {
        ngen = Controllo(g+gcorr,ngen);             /*comando/edit <4>*/
        if (ngen != -1L) {
            gnew = 1-gcorr;                         /*indice nuova gen <5>*/
            NuovaGen(g+gcorr,g+gnew);               /*calcola nuova gen*/
            StampaGen(g+gnew);
            gcorr = gnew;                           /*cambia gen*/
            ngen++;                                 /*e la conta*/
        }
    } while (ngen != -1L);

    Jclrscr();                                      /*cancella schermo*/
    Jcursor(1);                                     /*riaccende cursore*/
}

/* Note su main:
    <1> g[]  un array di due elementi, ciascuno dei quali  una matrice
        rettangolare (vedi nota 1 delle dichiarazioni). Questo  un esempio
        di come l'uso di typedef possa migliorare molto la leggibilit.
    <2> Il solito avviamento del generatore di numeri pseudocasuali.
    <3> g+gcorr equivale a: &g[gcorr], cio all'indirizzo dell'elemento gcorr
        dell'array g. Questo elemento  a sua volta un array bidimensionale,
        che viene quindi passato per riferimento (indirizzo del suo primo
        elemento), ed  modificabile dalla funzione chiamata.
    <4> La Controllo consente di avanzare passo-passo o di continuo, e di
        editare la generazione corrente. Torna -1L se  stato richiesto di
        terminare l'esecuzione del programma.
    <5> gcorr (generazione corrente) e gnew (nuova generazione) vengono
        scambiate ad ogni ciclo, e si alternano tra 0 e 1. La nuova
        generazione viene quindi calcolata alternativamente in g[0] e g[1].
*/


/********************************************************************
 * Controllo: aspetta comando, edita generazione *gp. Alla prima
 *            chiamata aspetta un comando in ogni caso, alle chiamate
 *            successive si ferma solo se richiesto (modo passo-passo)
 *            o se  stato premuto un tasto. Ritorna il numero della
 *            generazione, -1L se  richiesta l'uscita dal programma.
 ********************************************************************/

long Controllo(Gen *gp, long ng)
{
    enum { SP = 32, ESC = 27,                       /*codici tasti <1>*/
           SU = 1072, GIU = 1080,
           SIN = 1075, DES = 1077,
           F1 = 1059, F5 = 1063, F6 = 1064 };
    enum { POSINFO = 15 };                          /*pos. informazioni*/
    enum { CH = 1+DH/2, CV = 1+DV/2 };              /*centro, per edit*/

    static char istr1[] = "F1=passo, CR=vai, F5=clear,"
                          " F6=random, Esc=fine, ";
    static char istr2[] = { 27, 26, 24, 25, 0 };    /*simboli frecce <2>*/
    static char istr3[] = ", SP=edit";
    static char tasto[] = "             <premi un tasto per fermare>";
    static int  passo = 1;        /*flag di passo-passo, si ferma se vero*/
    static int  h = CH;           /*posizione di edit nella matrice*/
    static int  v = CV;

    int  pos;       /*posizione corrispondente sul video*/
    char *pc;       /*pointer a cellula corrente <3>*/
    int  c,esce;
    char buf[81];

    StampaNumGen(ng);                               /*mostra n.generaz.*/
    if (kbhit()) {                                  /*se premuto tasto <4>*/
        Jclrkey();                                  /*toglie da buffer*/
        passo = 1;                                  /*e si ferma*/
    }
    if (passo) {                                    /*se deve fermarsi*/
        sprintf(buf,"%s%s%s",istr1,istr2,istr3);    /*mostra istruzioni*/
        esce = 0;                                   /*esce a richiesta*/

        do {                                        /*aspetta comando*/
            Messaggio(buf,POSINFO);
            pc = (*gp)[v]+h;                        /*punta a cellula <5>*/
            pos = 1+2*(h-1);                        /*calcola pos. video <6>*/
            Jgotoxy(pos,v);                         /*ridisegna cellula <7>*/
            Jputch( (*pc) ? VIVA : MORTA);
            Jgotoxy(pos,v);
            c = Jgetkey(1);                         /*aspetta tasto*/

            switch(c) {                             /*esegue tasto*/
            case F1:                                /*un passo:*/
                esce = 1;
                break;
            case '\r':                              /*vai di continuo:*/
                passo = 0;
                esce = 1;
                break;
            case F5:                                /*clear:*/
            case F6:                                /*random:*/
                if (c == F5) {
                    GenNulla(gp);
                } else {
                    GenCasuale(gp);
                }
                StampaGen(gp);
                h = CH; v = CV;                     /*rimette al centro*/
                ng = 0;                             /*azzera generazione*/
                StampaNumGen(ng);                   /*anche su video*/
                break;
            case ESC:                               /*fine:*/
                esce = 1;
                ng = -1L;
                break;
            case SU:                                /*sposta in alto*/
                if (--v < 1)  v = DV;               /*<8>*/
                break;
            case GIU:                               /*sposta in basso*/
                if (++v > DV)  v = 1;
                break;
            case SIN:                               /*sposta a sinistra*/
                if (--h < 1)  h = DH;
                break;
            case DES:                               /*sposta a destra*/
                if (++h > DH)  h = 1;
                break;
            case SP:                                /*inverte cellula*/
                *pc = 1-*pc;
                break;
            default:                                /*tasto non valido:*/
                Jputch('\a');
                break;
            }
        } while (! esce);

        if (passo == 0 && ng != -1L) {              /*se Vai e non Esci*/
            Messaggio(tasto,POSINFO);               /*"tasto per fermare"*/
        } else {
            Messaggio("",POSINFO);                  /*toglie istruzioni*/
        }
    }
    return(ng);                                     /*numero generazione*/
}

/* Note su Controllo:
    <1> La enum consente di definire costanti locali ad una funzione, purch
        siano di tipo int.
    <2> Si potrebbe definire una stringa unica per le istruzioni da stampare
        sulla linea di stato, se fosse possibile introdurre in una stringa
        caratteri con codice arbitrario (es. 25) non rappresentabile dai
        normali editor. Questo  in effetti possibile in C, anche se non
         ancora stato introdotto nel corso.
    <3> pc sar usata per puntare ad una singola cellula nella matrice.
    <4> La kbhit ritorna vero (diverso da zero) se c' almeno un tasto nel
        buffer di tastiera, in attesa di essere letto.
    <5> Se g fosse una variabile di tipo Gen (matrice di cellule), g[v][h]
        sarebbe la cellula (char) alla posizione v,h (gli elementi partono
        da 1 perch l'elemento 0  usato nei calcoli e non contiene cellule).
        Poich gp  un pointer ad una Gen, *gp  una Gen e (*gp)[v][h]  una
        singola cellula (le parentesi sono obbligatorie perch altrimenti []
        avrebbero precedenza su *). L'indirizzo di questa cellula  quindi
        &(*gp)[v][h]. Ricordando che una Gen  organizzata per righe (vedi
        nota 3 delle dichiarazioni) e quindi (*gp)[v]  una stringa, possiamo
        avere lo stesso indirizzo scrivendo semplicemente (*gp)[v]+h.
        Ci vuole un po' per abituarsi a queste notazioni, che usiamo perch
        in C sono la norma. Con un minimo sacrificio di efficienza, si
        potrebbe comunque fare a meno del pointer pc e riferirsi alla cellula
        usando la notazione pi comprensibile ai non iniziati: (*gp)[v][h].
    <6> Per migliorare la regolarit della disposizione su video, le cellule
        sono spaziate orizzontalmente di due in due caratteri.
    <7> La posizione verticale su video di una cellula corrisponde al suo
        indice nella matrice, quella orizzontale  stata calcolata alla
        linea precedente.
    <8> Abbreviazione per:  v = v-1; if (v < 1)  v = DV;  In pratica, sposta
        il cursore a sinistra; se era gi sul bordo sinistro, lo riporta sul
        bordo destro (wrap-around).
*/


/********************************************************************
 * GenNulla: azzera la generazione *gp.
 ********************************************************************/

void GenNulla(Gen *gp)      /*<1>*/
{
    int  v;

    for (v = 1; v <= DV; v++) {                     /*per ogni riga*/
        memset((*gp)[v]+1,0,DH);                    /*la azzera <2>*/
    }
}

/* Note su GenNulla:
    <1> gp  un pointer ad una variabile di tipo Gen, cio una matrice di
        cellule (char).
    <2> Riempie i caratteri da 1 a DH della stringa (*gp)[v] con zeri, cio
        azzera una riga della matrice di cellule (vedi nota 5 di Controllo).
*/


/********************************************************************
 * GenCasuale: estrae generazione casuale in *gp.
 ********************************************************************/

void GenCasuale(Gen *gp)
{
    int  estraz;    /*numero di estrazioni (cellule vive)*/
    int  v,h;

    GenNulla(gp);                                   /*azzera tutto*/
    estraz = (DH*DV)/5;                             /*20% di cellule vive*/
    while (estraz--) {                              /*estrae coordinate <1>*/
        v = 1+rand()%DV;                            /*vert. tra 1 e DV*/
        h = 1+rand()%DH;                            /*orizz. tra 1 e DH*/
        (*gp)[v][h] = 1;                            /*mette cellula viva*/
    }
}

/* Note su GenCasuale:
    <1> Piuttosto che stabilire per ogni cellula se  viva o morta,  molto
        pi rapido azzerarle (ucciderle) tutte, poi estrarre un certo numero
        di cellule vive in posizioni casuali (conviene se la percentuale di
        cellule vive  bassa rispetto al totale).
*/


/********************************************************************
 * NuovaGen: calcola nuova generazione *ngp a partire dalla vecchia
 *           generazione *vgp.
 ********************************************************************/

void NuovaGen(Gen *vgp, Gen *ngp)     /*<1>*/
{
    char *rprec;     /*pointer alla riga precedente <2>*/
    char *riga;      /*pointer alla riga corrente*/
    char *rsucc;     /*pointer alla riga successiva*/
    int  tprec;      /*somma valori terna precedente (a sinistra) <3>*/
    int  tcorr;      /*somma valori terna corrente*/
    int  cc;         /*cellula corrente*/
    int  nc;         /*nuova cellula*/
    int  nv;         /*numero vicini*/
    int  v,h;

    GenNulla(ngp);                                  /*azzera la nuova*/
    CollegaBordi(vgp);                              /*riporta bordi <4>*/

    for (v = 1; v <= DV; v++) {                     /*per ogni riga*/
        rprec = (*vgp)[v-1];                        /*punta a riga prec.<5>*/
        riga = (*vgp)[v];                           /*a riga corrente*/
        rsucc = (*vgp)[v+1];                        /*e a riga successiva*/
        tprec = (*rprec++)+(*riga++)+(*rsucc++);    /*tot.terna sinistra <6>*/
        tcorr = (*rprec++)+(*riga++)+(*rsucc++);    /*tot.terna corrente*/

        for (h = 1; h <= DH; h++) {                 /*per ogni colonna*/
            cc = *(riga-1);                         /*cellula corrente <7>*/
            nv = tprec+tcorr-cc;                    /*subtot. prime 2 terne*/
            tprec = tcorr;                          /*fa scorrere terne*/
            tcorr = (*rprec++)+(*riga++)+(*rsucc++);/*calcola successiva*/
            nv += tcorr;                            /*n. totale vicini*/

            if (nv == 3) {                          /*nasce o resta <8>*/
                nc = 1;
            } else if (nv == 2) {                   /*resta*/
                nc = cc;
            } else {                                /*muore*/
                nc = 0;
            }
            (*ngp)[v][h] = nc;                      /*cambia cellula <9>*/
        }
    }
}

/* Note su NuovaGen:
    <1> La nuova matrice di cellule *ngp viene calcolata sulla base del
        contenuto della vecchia matrive *vgp.
    <2> L'algoritmo di Life comporta il calcolo del numero di cellule vive
        nelle 8 caselle attorno a ciascuna cellula (i vicini). Questa
        funzione lo fa in modo efficiente, usando una tecnica per somma di
        terne (che non dettagliamo in queste note). In ogni caso, rprec deve
        puntare inizialmente alla riga precedente quella corrente, riga deve
        puntare alla riga corrente, e rsucc alla riga successiva.
    <3> tprec conterr la somma dei valori delle tre cellule a sinistra di
        quella corrente. Poich ogni cellula pu contenere solo 0 o 1, questo
         il numero dei vicini di sinistra.
    <4> La CollegaBordi riporta nella cornice attorno alla matrice di
        cellule (indici 0, DV+1, DH+1) le cellule presenti nel bordo opposto
        della matrice. Questo consente di ottenere un effetto 'toroide',
        cio di un piano continuo: i bordi destro e sinistro si toccano, e
        cos pure il bordo superiore ed inferiore.
    <5> (*vgp)[v-1]  l'indirizzo del primo char della riga v-1, cio
        &(*vgp)[v-1][0] (vedi nota 3 delle dichiarazioni).
    <6> Calcola la somma delle tre cellule di sinistra, ciascuna puntata dal
        suo pointer, ed avanza tutti e tre i pointer.
    <7> In questo momento, riga punta alla cellula a destra di quella
        corrente, che  quindi puntata da (riga-1).
    <8> Questo  il cuore dell'algoritmo: la decisione se far nascere o
        morire la cellula sulla base del numero di vicini.
    <9> Il valore calcolato viene scritto nella nuova generazione. Il
        procedimento  ripetuto per tutte le cellule.
*/


/********************************************************************
 * CollegaBordi: riporta i bordi della generazione *gp accanto ai
 *               bordi opposti, per produrre un effetto toroidale
 *               (piano finito ma illimitato).
 ********************************************************************/

void CollegaBordi(Gen *gp)    /*<1>*/
{
    int  h,v;

    for (h = 1; h <= DH; h++) {
        (*gp)[0][h] = (*gp)[DV][h];                 /*riporta sopra <2>*/
        (*gp)[DV+1][h] = (*gp)[1][h];               /*e sotto*/
    }
    for (v = 1; v <= DV; v++) {
        (*gp)[v][0] = (*gp)[v][DH];                 /*a sinistra <3>*/
        (*gp)[v][DH+1] = (*gp)[v][1];               /*e a destra*/
    }
    (*gp)[0][0] = (*gp)[DV][DH];                    /*angolo sup.sin. <4>*/
    (*gp)[DV+1][0] = (*gp)[1][DH];                  /*inf.sinistro*/
    (*gp)[0][DH+1] = (*gp)[DV][1];                  /*sup.destro*/
    (*gp)[DV+1][DH+1] = (*gp)[1][1];                /*inf.destro*/
}

/* Note su CollegaBordi:
    <1> Vedi nota 4 di NuovaGen.
    <2> Si poteva risparmiare tempo usando una memcpy, dato che le cellule
        di una stessa riga sono adiacenti (vedi nota 3 delle dichiarazioni)
        e la riga pu quindi trattata come una stringa.
    <3> Qui invece non si poteve usare memcpy perch le cellule di una
        stessa colonna non sono adiacenti all'interno della matrice.
    <4> Anche i quattro angoli vanno riportati accanto all'angolo opposto.
*/


/********************************************************************
 * StampaGen: mostra generazione *gp su video.
 ********************************************************************/

void StampaGen(Gen *gp)
{
    enum { LV = DH*2 };   /*lunghezza linea da stampare sul video*/
    char lin[LV];         /*linea da stampare <1>*/
    int  pos;             /*posizione nella linea da stampare*/
    char *riga;           /*pointer a una riga dell'array di cellule*/
    int  v,h;

    memset(lin,' ',DH*2);                           /*vuota car.dispari <2>*/
    lin[LV-1] = 0;                                  /*termina stringa <3>*/
    for (v = 1; v <= DV; v++) {                     /*per ogni riga*/
        riga = (*gp)[v];                            /*punta alla riga <4>*/
        pos = 0;                                    /*azzera pos. stampa*/
        for (h = 1; h <= DH; h++) {                 /*prepara linea*/
            lin[pos] = riga[h] ? VIVA : MORTA;      /*simboli cellule <5>*/
            pos += 2;
        }
        Jgotoxy(1,v);                               /*stampa linea*/
        Jputs(lin);
    }
}

/* Note su StampaGen:
    <1> E' molto pi veloce stampare un'intera linea in un sol colpo con la
        Jputs che non stampare un carattere alla volta con la Jputch.
        Preparaiamo quindi la linea da stampare nel buffer lin[].
    <2> Le cellule sono stampate su video solo nei caratteri posti in
        posizioni pari nella stringa (0,2,4,...), mentre nelle altre
        posizioni sono stampati spazi. Riempiamo quindi velocemente la linea
        di spazi, per poi cambiare solo le posizioni dispari.
    <3> Per poter stampare la linea come stringa con la Jputs, occorre lo
        zero finale.
    <4> Come al solito, (*gp)[v]  l'indirizzo della riga v della matrice
        puntata da gp (vedi nota 3 delle dichiarazioni).
    <5> Notare che riga (un pointer a char) viene usato con le parentesi
        quadre, come fosse un array. Le due notazioni sono intercambiabili,
        senza nessuna penalizzazione in termini di velocit. Sarebbe pi
        efficiente mettere in lin[] solo i simboli delle cellule vive, dato
        che il simbolo di una cellula morta  comunque uno spazio (che c'
        gi in lin[]), ma questo causerebbe problemi se si decidesse in
        seguito di cambiare il simbolo di cellula morta (definito all'inizio
        del programma).
*/


/********************************************************************
 * StampaNumGen: stampa numero generazione su linea di stato.
 ********************************************************************/

void StampaNumGen(long ng)
{
    char buf[81];   /*<1>*/

    sprintf(buf,"gen: %-7ld",ng);
    Jgotoxy(1,25);
    Jputs(buf);
}

/* Note su StampaNumGen:
    <1> buf[]  evidentemente sovradimensionato, ma  sempre meglio sbagliare
        per eccesso (si spreca memoria) che per difetto (col rischio di
        misteriosi malfunzionamenti), tanto pi che si tratta di una
        variabile temporanea che sparisce al termine della funzione.
*/


/********************************************************************
 * Messaggio: stampa stringa s[] su linea di stato (linea 25)
 *            alla posizione h, cancella il resto della linea.
 ********************************************************************/

void Messaggio(char *s, int h)
{
    int n;

    Jgotoxy(h,25);
    Jputs(s);
    n = 78-Jposx();
    while (n--)  Jputch(' ');
}
