/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* LISSAJ: figure di Lissajous su video e stampante */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "Jconio.c"


/* prototipi funzioni: */

void InputDati(void);
void CalcolaFigura(void);
void DisegnaFigura(void);
int  InputStringa(char *prompt, char *s, int lmax, int col, int back);
int  InputInt(char *prompt, int n, int mi, int ma, int col, int back);
int  Scelta(char *prompt, char *scelte, char *def);
int  SiNo(char *domanda, char *def);
int  CalcolaMcd(int n1, int n2);


#define INCOL  BLACK      /*colori per input*/
#define INBACK LIGHTGRAY

#define HMAX    80        /*massimo numero colonne*/
#define VMAX    100       /*massimo numero linee*/
#define SIMBVID 4         /*simbolo per video*/

typedef struct {          /*dati per un disp.uscita <1>*/
    char nome;            /*'v'=video, 's'=stampante <2>*/
    int  v;               /*n. righe*/
    int  h;               /*n.colonne*/
    char simb;            /*simbolo da stampare*/
} DatiDispositivo;


DatiDispositivo video = { 'v', 25, 79, SIMBVID };  /*dati del video <3>*/
DatiDispositivo stamp = { 's', 60, 79, '#' };      /*dati della stampante*/

const double pi = 3.1415926536;   /*pigreco*/


/********************* variabili globali ***************************/

DatiDispositivo disp;             /*dispositivo corrente <4>*/
char mat[VMAX][HMAX+1];           /*matrice per disegno <5>*/
int  freqh = 2;                   /*frequenza orizzontale, con default <6>*/
int  freqv = 3;                   /*frequenza verticale, con default*/


/* Note sulle dichiarazioni:
    <1> Il tipo DatiDispositivo  una struttura che raccoglie i dati
        del dispositivo usato per l'output della figura.
    <2> Poich non si possono confrontare direttamente due strutture,
        la componente 'nome' permette di identificare immediatamente a
        cosa si riferisca il contenuto della struttura (video o stampante).
    <3> video  una variabile di tipo DatiDispositivo (quindi una struttura)
        che  inizializzata con i dati dello schermo video.
    <4> In questa struttura disp verr copiata la struttura 'video' o la
        struttura 'stampante', a seconda del dispositivo scelto. Sarebbe pi
        efficiente limitarsi a preparare un pointer alla struttura voluta
        per evitare di doverla copiare, ma si tratta di strutture molto
        piccole che sono copiate una sola volta, quindi la soluzione 
        accettabile.
    <5> mat[][]  usabile come un array di stringhe, ciascuna delle quali
        rappresenta una riga della figura (vedi nota 3 delle dichiarazioni
        del programma LIFE).
    <6> Trattandosi di variabili globali, il valore indicato viene assegnato
        una sola volta, all'avviamento del programma.
*/


/********************************************************************
 * main
 ********************************************************************/

main()
{

    Jcursor(0);                                     /*spegne cursore*/

    do {
        Jclrscr();                                  /*cancella video*/
        Jgotoxy(18,3);
        Jputs("- Figure di Lissajous, dal Corso di C -");

        InputDati();                                /*freq. e dispositivo*/
        CalcolaFigura();                            /*calcola in matrice*/
        DisegnaFigura();                            /*stampa matrice*/

        Jclrscr();
        Jgotoxy(1,3);
    } while (SiNo("Un'altra figura (s/n) ? ",""));  /*ancora? <1>*/

    Jputs("\r\n");                                  /*a capo*/
    Jcursor(1);                                     /*riaccende cursore*/
}

/* Note su main:
    <1> La SiNo ritorna vero se l'utente risponde s (s). Il secondo
        argomento (la stringa nulla) serve per indicare un'eventuale
        risposta di default.
*/


/********************************************************************
 * InputDati: input frequenze e dispositivo nelle variabili globali
 *            freqh, freqv e disp.
 ********************************************************************/

void InputDati(void)
{
    int sc;

    Jgotoxy(1,6);
    freqh = InputInt("Frequenza orizzontale"          /*sceglie frequenze*/
            " (1..5) : ",freqh,1,5,INCOL,INBACK);
    Jgotoxy(1,8);
    freqv = InputInt("Frequenza verticale"
            " (1..5)   : ",freqv,1,5,INCOL,INBACK);
    Jgotoxy(1,10);
    sc = Scelta(
         "Su video o stampante (v/s) ?   ","vs","v"); /*e disp. di output*/
    disp = (sc == 0) ? video : stamp;                 /*dati dispositivo <1>*/
}

/* Note su InputDati:
    <1> La variabile sc poteva essere evitata usando direttamente la funzione
        Scelta nell'espressione condizionale.
*/


/********************************************************************
 * CalcolaFigura: calcola figura di Lissajous nella matrice globale
 *                mat[][], con dimensioni adatte al dispositivo scelto.
 ********************************************************************/

void CalcolaFigura(void)
{
    int    ch,cv;         /*coordinate del centro della matrice*/
    double amph,ampv;     /*ampiezza orizzontale e verticale*/
    double passo;         /*passo base*/
    double passoh,passov; /*passo orizzontale, verticale*/
    int    npassi;        /*numero passi*/
    double angh,angv;     /*angoli iniziali*/
    int    conta;         /*contapassi, per indicare che lavora*/
    int    h,v;

    for (v = 0; v < disp.v; v++) {                  /*riempie di spazi*/
        memset(mat[v],' ',disp.h);                  /*<1>*/
        mat[v][disp.h] = 0;                         /*zero per stampa <2>*/
    }
    ch = disp.h/2;                                  /*coord. del centro*/
    cv = disp.v/2;
    amph = ch-1;                                    /*ampiezze <3>*/
    ampv = cv-1;
    passo = (disp.v > disp.h) ?
            pi/disp.v : pi/disp.h;                  /*passo base*/
    passoh = passo/freqv;                           /*corregge orizz. <4>*/
    passov = passo/freqh;                           /*corregge passo vert.*/
    npassi = (int)(floor(2.0*pi/passoh+0.5));       /*passi/ciclo orizz.<5>*/
    if (freqh != freqv) {                           /*per numero cicli*/
        npassi *= freqh;
    }
    angh = 0.0;                                     /*angoli iniziali*/
    angv = pi/2.0;

    Jputs("\r\n\r\nCalcolo figura...");
    conta = 0;                                      /*azzera contapassi*/
    while (npassi-- > 0) {
        h = ch+(int)(floor(amph*sin(angh)+0.5));    /*calcola punto <6>*/
        v = cv+(int)(floor(ampv*sin(angv)+0.5));
        mat[v][h] = disp.simb;                      /*mette simbolo*/
        angh += passoh;                             /*aggiorna angoli*/
        angv += passov;
        if (++conta >= 25) {                        /*mostra che lavora <7>*/
            Jputch('.');
            conta = 0;
        }
    }
}

/* Note su CalcolaFigura:
    <1> Poich i caratteri di ciascuna riga della matrice sono adiacenti
        nella matrice, possono essere trattati come una stringa. In questo
        caso, la memset riempie velocemente di spazi un'intera riga.
    <2> Il carattere in pi indicato nella dimensione orizzontale di mat[][]
        (vedi dichiarazioni) serve per contenere lo zero finale, che consente
        di stampare un'intera riga con una sola Jputs (invece che dover
        stampare un carattere alla volta con la Jputch).
    <3> amph  un double e ch  un int. Assegnando un int ad un double non
        si ha comunque perdita di informazioni.
    <4> Non entriamo nei dettagli dell'algoritmo, che consiste comunque nel
        far 'ruotare' i due angoli angh e angv aggiungendo a ciascuno di
        questi il rispettivo passo (passoh, passov) ad ogni ciclo. I passi
        vengono calcolati in modo da dare l'illusione di una maggiore o
        minore velocit (passo pi lungo: meno cicli per fare un giro; e
        viceversa), in base alle frequenze indicate dall'utente. Il sistema
        usato per determinare il numero di passi potrebbe essere molto
        migliorato: se i due numeri hanno divisori in comune, la figura
        viene tracciata pi volte senza necessit.
    <5> Per approssimare un double (in questo caso: due pigreco / passoh)
        all'intero pi vicino, si aggiunge 0.5 e lo si approssima all'intero
        sottostante con la funzione floor. Il risultato  ancora un double,
        che per non ha parte frazionaria e potrebbe quindi essere assegnato
        ad un int (in questo caso: npassi) senza perdita di precisione. Per
        scrupolo, segnaliamo chiaramente il cambio di tipo 'in peggio' con
        un esplicito cast: (int).
    <6> sin(angh) varia tra -1 e +1. Moltiplicandolo per amph (l'ampiezza
        orizzontale calcolata in precedenza) si ottiene un valore che
        varia tra -amph e +amph. Questo valore viene poi arrotondato
        all'intero pi vicino (vedi nota precedente) in modo da determinare
        la posizione nella matrice in cui scrivere un simbolo per 'accendere'
        un punto sul video (un punto grande come un carattere, visto che
        stiamo ancora usando il video alfanumerico).
    <7> Stampa un punto ogni 25 passi, per far vedere all'utente che il
        programma lavora, altrimenti (specie sui computer meno veloci) questi
        potrebbe credere che il programma sia 'morto', mentre sta solo
        eseguendo numerosi calcoli.
*/


/********************************************************************
 * DisegnaFigura: disegna contenuto dell'array nel dispositivo scelto.
 ********************************************************************/
void DisegnaFigura(void)
{
    int v;

    if (disp.nome == 'v') {                         /*se video <1>*/
        Jclrscr();                                  /*ripulisce*/
        for (v = 0; v < disp.v; v++) {              /*per ogni riga*/
            Jgotoxy(1,v);                           /*si posiziona*/
            Jputs(mat[v]);                          /*e la stampa <2>*/
        }
        Jclrkey();                                  /*pulisce buffer <3>*/
        while (! kbhit())                           /*aspetta tasto*/
            ;
        Jclrkey();                                  /*e ripulisce*/
            ;
    } else if (disp.nome == 's') {                  /*se stampante*/
        Jputs("\r\n\r\nStampa su carta...");
        for (v = 0; v < disp.v; v++) {              /*per ogni riga*/
            fprintf(stdprn,"%s\r\n",mat[v]);        /*la stampa <4>*/
        }
        fputc('\f',stdprn);                         /*fine, Form Feed <5>*/
    }
}

/* Note su DisegnaFigura:
    <1> Identifica il dispositivo usato sulla base della componente 'nome'
        della struttura disp.
    <2> Lo zero aggiunto dalla CalcolaFigura dopo la fine di ogni riga
        consente di stamparla con una semplice Jputs.
    <3> Prima di aspettare un tasto  bene ripulire il buffer di tastiera
        da eventuali tasti in attesa, specialmente se in precedenza sono
        state fatte operazini di una certa durata (durante le quali l'utente
        potrebbe aver premuto dei tasti).
    <4> Questo programma non considera i possibili errori dovuti alla
        mancata connessione della stampante, alla stampante non attiva
        (off line) o semplicemente alla mancanza di carta. Vedremo pi
        avanti nel corso come tenerli nella debita considerazione.
    <5> Il carattere di Form Feed fa avanzare la carta fino all'inizio
        della pagina successiva.
*/


/********************************************************************
 * InputStringa: mostra campo di input con default usando colori col
 *               e back, input controllato di stringa nell'array s[]
 *               non oltre lmax caratteri, accetta correzioni con
 *               backspace, mette zero finale, ritorna lunghezza della
 *               stringa introdotta.
 ********************************************************************/

int InputStringa(char *prompt, char *s, int lmax, int col, int back)
{
    enum { BS = 8, BACK = 1075 };      /*codici tasti*/

    char buf[81];       /*buffer di linea*/
    int  ls;            /*lunghezza stringa*/
    int  px,py,pxi;     /*posizione originaria e di input*/
    int  cf,cb;         /*colori originari*/
    int  i,c,ok;

    px = Jposx();                                   /*registra posizione*/
    py = Jposy();
    cf = Jgettextcolor();                           /*e colori*/
    cb = Jgettextback();

    Jputs(prompt);                                  /*stampa prompt*/
    pxi = Jposx();                                  /*posizione di input*/

    Jtextcolor(col);                                /*mette colori di input*/
    Jtextback(back);
    Jputs(s);                                       /*mostra default*/
    ls = strlen(s);                                 /*lunghezza corrente*/
    for (i = ls; i < lmax; i++) {                   /*evidenzia campo*/
        Jputch(' ');
    }
    Jgotoxy(pxi+ls,py);                             /*torna a fine default*/

    s += ls;                                        /*punta alla fine*/
    Jclrkey();                                      /*cancella buffer*/
    do {
        c = Jgetkey(1);
        ok = (c == '\r');
        if ( (c == BS || c == BACK) && ls > 0) {
            ls--;
            s--;
            Jgotoxy(pxi+ls,py);
            Jputch(' ');
            Jgotoxy(pxi+ls,py);
        } else if (isprint(c) && ls < lmax) {
            *s++ = c;
            ls++;
            Jputch(c);
        } else if (! ok) {
            Jputch('\a');
        }
    } while (! ok);

    *s = 0;                                         /*termina stringa*/
    Jgotoxy(px,py);                                 /*torna dov'era*/
    Jtextcolor(cf); Jtextback(cb);                  /*risistema colori*/

    return ls;                                      /*ritorna lunghezza*/
}

/* Note su InputStringa:
    <1>  InputStringa  commentata nel programma INPCON.
*/


/********************************************************************
 * InputInt: Input controllato di un intero. Come InputStringa, ma
 *           ritorna un valore compreso tra mi e ma, con default n.
 ********************************************************************/

int InputInt(char *prompt, int n, int mi, int ma, int col, int back)
{
    enum { LBUF = 6 };

    char buf[LBUF+1];

    sprintf(buf,"%d",n);                                  /*prepara default*/
    while (! (InputStringa(prompt,buf,LBUF,col,back) > 0
              && (isdigit(*buf) || *buf == '-')
              && (n = atoi(buf)) >= mi
              && n <= ma)) {
        Jputch('\a');
    }

    return n;                                             /*valore numero*/
}

/* Note su InputInt:
    <1> InputInt  commentata nel programma INPCON.
*/


/********************************************************************
 * Scelta: sceglie tra due o pi opzioni introducendo un carattere,
 *         con default def[]. Ritorna la posizione del carattere
 *         nella stringa scelte[], che dev'essere composta di soli
 *         caratteri minuscoli. Usa colori INCOL, INBACK.
 ********************************************************************/
int  Scelta(char *prompt, char *scelte, char *def)
{
    char buf[2] = { 0, 0 };     /*spazio per risposta, con zero finale <1>*/
    char *pos;                  /*posizione del carattere in scelte[]*/
    int  ok;

    *buf = *def;                                    /*mette default <2>*/
    do {
        InputStringa(prompt,buf,1,INCOL,INBACK);    /*legge carattere*/
        buf[0] = tolower(buf[0]);                   /*minuscolo <3>*/
        pos = strstr(scelte,buf);                   /*cerca in scelte <4>*/
        ok = (buf[0] != 0 && pos != NULL);          /*guarda se valido <5>*/
        if (! ok) {
            Jputch('\a');                           /*segnala errore*/
            buf[0] = 0;                             /*annulla scelta*/
        }
    } while (! ok);

    return pos-scelte;                              /*indice in scelte <6>*/
}

/* Note su Scelta:
    <1> Definisce una stringa composta da due caratteri con codice 0, in
        modo che basti cambiare il primo carattere per avere una stringa
        di un carattere, pi lo zero finale che la termina.
    <2> Per evitare problemi, copia solamente un carattere dalla stringa
        def[] alla stringa buf[]. Lo zero finale  comunque assicurato:
        c'era gi in precedenza (vedi nota 1 qui sopra).
    <3> Sono ammesse sia le maiuscole che le minuscole.
    <4> La strstr cerca una stringa (buf) in un'altra stringa (scelte) e
        ne ritorna la posizione sotto forma di pointer, o NULL se non c'.
    <5> La scelta  valida se la stringa introdotta buf[] non  nulla
        (ma  di un carattere), e se  elencata nella stringa scelte[].
    <6> La differenza tra due pointer che puntano a caratteri della
        stessa stringa  la distanza tra i due caratteri. Dato che scelte[]
         il nome della stringa (e quindi un pointer al primo carattere,
        la differenza  la posizione nella stringa del carattere puntato da
        pos.
*/


/********************************************************************
 * SiNo: scelta si/no con default def[], ritorna vero se s.
 ********************************************************************/
int  SiNo(char *domanda, char *def)
{
    return Scelta(domanda,"ns",def);                /*<1>*/
}

/* Note su SiNo:
    <1> Il C offre anche un modo per evitare di definire funzioni cos
        semplici, e risparmiare anche tempo di esecuzione. Sono le macro,
        che esamineremo pi avanti nel corso.
*/
