/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* BINARIO: equivalente decimale di un numero binario */

/*
   Per compilare con il Turbo C 2.0, usare BINARIO.PRJ.   *<1>*
   Per compilare con il Quick C 2.0, usare BINARIO.MAK.
*/


#include <stdio.h>

#include "Jconio.h"     /*<2>*/


/* prototipi funzioni: */

void PreparaSchermo(void);
void StampaDec(char *bin);
void Inverse(int inv);


/* costanti: <3> */

enum {
    POSX  = 19,         /*posizione del primo digit binario*/
    POSY  = 9,
    STEPX = 4,          /*distanza x tra i digit*/
    POSXD = POSX+38,    /*posizione risultato decimale*/
    ISTRX = POSX-2,     /*posizione istruzioni*/
    ISTRY = POSY+5,

    SU    = 1072,       /*tasti usati*/
    GIU   = 1080,
    SINI  = 1075,
    DEST  = 1077,
    ESC   = 27
};

/* Note sulle dichiarazioni:
    <1> Il programma  composto di due moduli: BINARIO.C e JCONIO.C:
        bisogna comunicare questo al compilatore. Per il Turbo C 2.0
        si sceglie Project Name dal menu Project e si legge BINARIO.PRJ;
        per il Quick C 2.0 si sceglie Set Program List dal menu Make e
        si legge BINARIO.MAK.
    <2> Per usare correttamente le funzioni della libreria di console
        Jackson, si deve includere il corrispondente header. Le virgolette
        indicano di cercare il file nella directory corrente.
    <3> Le costanti intere possono essere definite con una enum al posto
        di varie #define.
*/


/********************************************************************
 * main
 ********************************************************************/

main()
{
    static char bin[] = "00000000";   /*numero binario a 8 digit*/

    int dig = 0;                                    /*digit relativo (0..7)*/
    int i,c;

    Jcursor(0);                                     /*spegne cursore*/
    Jclrscr();                                      /*cancella schermo*/

    PreparaSchermo();                               /*titoli e riquadro*/
    Jclrkey();                                      /*pulisce buffer <1>*/
    Inverse(1);                                     /*colori scambiati*/

    do {
        for (i = 0; i < 8; i++) {                   /*aggiorna i digit <2>*/
            Jgotoxy(POSX+STEPX*i,POSY);
            Jputch(bin[i]);
        }
        Jgotoxy(POSXD,POSY);                        /*e il risultato <3>*/
        StampaDec(bin);

        Jgotoxy(POSX+STEPX*dig,POSY);               /*va su digit corrente*/
        c = Jgetkey(1);                             /*aspetta comando <4>*/

        if (c == SU) {                              /*freccia in alto = 1*/
            bin[dig] = '1';
        } else if (c == GIU) {                      /*freccia in basso = 0*/
            bin[dig] = '0';
        } else if (c == SINI) {                     /*cambia digit corrente*/
            dig = (dig-1+8)%8;                      /*mantiene in 7..0 <5>*/
        } else if (c == DEST) {
            dig = (dig+1)%8;
        } else if (c != ESC) {
            Jputch('\a');                           /*segnala errore*/
        }

    } while (c != ESC);                             /*esce con Esc*/

    Inverse(0);                                     /*colori normali*/
    Jclrscr();                                      /*cancella schermo*/
    Jcursor(1);                                     /*riaccende cursore*/
}

/* Note su main:
    <1> Cancellando il buffer di tastiera, si evita che eventuali tasti
        battuti prima dell'inizio del programma siano accettati come comandi.
    <2> Aggiorna i digit scritti su video in modo che coincidano con quelli
        contenuti nella stringa bin[].
    <3> La stampaDec stampa l'equivalente decimale del numero binario
        contenuto in bin[].
    <4> Passando 1 come parametro, la Jgetkey mostra il cursore solo mentre
         in attesa di un tasto.
    <5> dig-1  il nuovo valore che si desidera. Per evitare una if che
        controlli se dig  sceso sotto zero (e nel caso lo riporti a 7),
        basta prendere il resto della divisione per 8 (modulo: %). Dato che
        per questa operazione funziona solo per numeri positivi, si deve
        aggiungere 8 a dig in modo che sia sicuramente positivo (aggiungendo
        8 il resto della divisione per 8 evidentemente non cambia). Da notare
        comunque che la if  pi veloce: il modulo  lento come la divisione.
*/


/********************************************************************
 * PreparaSchermo: disegna titoli e riquadro.
 ********************************************************************/

void PreparaSchermo (void)
{
    static char *riq[] = {
        " Ŀ     Ŀ ",
        "                           =        ",
        "       "
    };
    static char *istr[] = {                             /*<1>*/
        "\x1b\x1a   Sceglie cifra da modificare\r\n",
        "\x18    Mette cifra 1\r\n",
        "\x19    Mette cifra 0\r\n",
        "Esc  Termina il programma"
    };

    int i;

    Inverse(1);                                     /*colori scambiati*/
    Jgotoxy(30,4);                                  /*titolo*/
    Jputs(" - Numeri binari - ");

    Inverse(0);                                     /*colori normali*/
    for (i = 0; i < 4; i++) {                       /*scrive istruzioni*/
        Jgotoxy(ISTRX,ISTRY+i);
        Jputs(istr[i]);
    };

    Inverse(1);
    for (i = 0; i < 3; i++) {                       /*disegna riquadro*/
        Jgotoxy(POSX-3,POSY-1+i);
        Jputs(riq[i]);
    };
    Inverse(0);
}

/* Note su PreparaSchermo:
    <1> Un carattere speciale (come il simbolo della freccia a sinistra) pu
        essere introdotto in una stringa con il suo codice esadecimale
        preceduto da \x.
*/


/********************************************************************
 * StampaDec: mostra l'equivalente decimale a tre cifre del numero
 *            rappresentato in binario a 8 bit nella stringa bin[].
 ********************************************************************/

void StampaDec(char *bin)
{
    int  d;       /*equivalente decimale*/
    char buf[4];  /*buffer di stampa <1>*/
    int  p2;      /*potenza di 2 (peso) corrente <2>*/
    int  n;

    d = 0;
    p2 = 1;                                         /*peso digit a destra*/
    for (n = 7; n >= 0; n--) {                      /*legge da destra <3>*/
        if (bin[n] == '1') {                        /*aggiorna se 1*/
            d += p2;
        }
        p2 += p2;                                   /*peso seguente*/
    }

    sprintf(buf,"%03d",d);                          /*stampa in buffer <4>*/
    Jputs(buf);                                     /*e su video*/
}

/* Note su StampaDec:
    <1> 4 caratteri bastano perch ne verranno stampati esplicitamente 3
        nella stringa, pi lo zero finale (vedi nota 4).
    <2> p2  il peso della colonna corrente: quella pi a destra ha peso 1.
    <3> Volendo rendere la funzione indipendente dal numero di bit nella
        stringa, basta inizializzare n con strlen(bin) invece che con 7, e
        naturalmente modificare la dichiarazione di buf (nota 1) e la sprintf
        che segue per avere spazio sufficiente per il numero convertito.
    <4> Il numero  sempre di 3 cifre, con gli eventuali zeri non
        significativi (cio precedenti) stampati in ogni caso.
*/


/********************************************************************
 * Inverse: Inverte colori se inv  vero, torna normale se falso.
 ********************************************************************/

void Inverse(int inv)
{
    static int inverse = 0;   /*ricorda lo stato corrente <1>*/
    static int nc,nb;         /*colori normali del testo*/

    if (inv && ! inverse) {                         /*se non gi inverse <2>*/
        nc = Jgettextcolor();                       /*ricorda colori <3>*/
        nb = Jgettextback();
        Jtextcolor(nb);                             /*li scambia*/
        Jtextback( (nc > 7) ? nc-8 : nc);           /*max 7 per backgr. <4>*/
    } else if (! inv && inverse) {                  /*se non gi normale <5>*/
        Jtextcolor(nc);                             /*rimette a posto*/
        Jtextback(nb);
    }
    inverse = inv;                                  /*ricorda stato <6>*/
}

/* Note su Inverse:
    <1> Una variabile statica non perde il valore tra una chiamata e l'altra
        della funzione. Questa variabile ricorda se siamo attualmente in
        modo normale o in inverse (colori di carattere e fondo scambiati).
    <2> Scambia i colori solo se non  in inverse, per evitare doppi scambi.
    <3> Le variabili nc e nb conservano i colori normali, cio quelli che
        c'erano alla prima richiesta di inverse.
    <4> Dato che il colore massimo per il background  7, se il colore del
        carattere (foreground) era maggiore di 7 si sottrae 8, in modo da
        mantenere la stessa tinta, anche se pi scura.
    <5> Ritorna normale solo se non lo era gi.
    <6> Ricordando lo stato corrente si evitano errori in caso di due
        chiamate uguali di seguito (non andrebbero fatte, ma non si sa mai).
*/
