/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* CONVERTE: rappresentazione di un numero in base 2, 8, 10 e 16 */


#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>


/* prototipi funzioni: */

int  InputNumero(long *np);
void StampaInBase(int base, long num, char *s);
void InputStringa(char *prompt, char *s);


/* costanti e variabili globali: */

enum { PREF2 = '%', PREF8 = '@', PREF16 = '$' };    /*prefissi*/
enum { SUFF2 = 'b', SUFF8 = 'q', SUFF16 = 'h' };    /*e suffissi*/

const long maxnum = LONG_MAX/16-16;                 /*limite massimo <1>*/

char digit[] = "0123456789abcdef";        /*validi per tutte le basi <2>*/

/* Note sulle dichiarazioni:
    <1> LONG_MAX  una costante definita in <limits.h> che indica il
        massimo valore ammissibile per un long. Considerando la tecnica usata
        nella InputInt per la conversione in un valore numerico della stringa
        introdotta, maxnum  il massimo valore che, nel peggiore dei casi,
        non potr dare overflow durante i calcoli.
    <2> La stringa digit[] contiene i caratteri usati per i digit in tutte
        le basi da 2 a 16, nell'ordine corrispondente al rispettivo valore.
        E' globale perch verr usata da due funzioni, ma nessuna di queste
        la modifica, per cui  in pratica una costante. Digit vuol dire
        'cifra' in inglese.
*/


/********************************************************************
 * main
 ********************************************************************/

main()    /*<1>*/
{
    long n;           /*numero decimale*/

    puts("\n - Conversione di base numerica -\n\n"
       "I numeri possono essere dati in uno dei seguenti formati:\n\n"
       " Decimale    : nessun prefisso n suffisso (es. 198).\n"
       " Binario     : prefisso % o suffisso b (es. %11000110, 11000110b).\n"
       " Ottale      : prefisso @ o suffisso q (es. @306, 306).\n"
       " Esadecimale : prefisso $ o suffisso h (es. $c6, c6h).");

    while (InputNumero(&n)) {                       /*chiede numero <2>*/
        printf("\n ");
        StampaInBase(2,n,"(bin) = ");               /*stampa in binario <3>*/
        StampaInBase(8,n,"(oct) = ");               /*stampa in ottale*/
        StampaInBase(10,n,"(dec) = ");              /*stampa in decimale*/
        StampaInBase(16,n,"(hex)\n");               /*stampa in esadecimale*/
    }
}

/* Note su main:
    <1> Il programma usa le funzioni di stdio e non la libreria di console
        Jackson, in quanto questa  stata modificata rispetto alle versioni
        riportate nelle lezioni 1 e 2. La differenza  spiegata pi avanti in
        questa stessa lezione. Avendo usato solo funzioni standard ANSI, il
        programma risulta portatile anche su altri computer.
    <2> La InputNumero accetta un numero scritto in base 2, 8, 10 o 16, e
        mette il valore del numero nella variabile puntata dal suo argomento
        (n in questo caso). Ritorna falso (0) se viene premuto Return senza
        introdurre alcun numero.
    <3> La StampaInBase stampa il secondo argomento nella base numerica
        specificata dal primo argomento, poi stampa la stringa indicata.
*/


/********************************************************************
 * InputNumero: accetta un numero in binario, ottale, decimale o
 *              esadecimale, mette numero introdotto in *np, ritorna
 *              falso (0) se solo Return.
 ********************************************************************/

int InputNumero(long *np)
{
    char s[128];       /*stringa introdotta*/
    int  ok;           /*flag di numero valido*/
    int  base;         /*base numerica indicata*/
    char *pi,*pf;      /*pointer a inizio e fine numero*/
    int  pref,suff;    /*possibili prefisso e suffisso*/
    int  d;            /*valore di un digit*/
    long pnum;         /*numero precedente, per test validit*/
    char *p;

    do {

        InputStringa("\nNumero positivo da "        /*input senza spazi <1>*/
          "convertire (Return per smettere): ",s);
        if (*s == 0) {                              /*esce se nulla <2>*/
            return(0);
        }

        base = 10;                                  /*si suppone decimale*/
        pi = s;                                     /*punta a inizio numero*/
        pf = s+strlen(s)-1;                         /*e a fine numero*/
        pref = tolower(*pi);                        /*possibile prefisso <3>*/
        if (pref == PREF2)  base = 2;
        else if (pref == PREF8)  base = 8;
        else if (pref == PREF16)  base = 16;
        if (base != 10) {                           /*se c'era un prefisso*/
            pi++;                                   /*lo toglie*/
        } else {                                    /*se no, cerca suffisso*/
            suff = tolower(*pf);                    /*possibile suffisso*/
            if (suff == SUFF2)  base = 2;
            else if (suff == SUFF8)  base = 8;
            else if (suff == SUFF16)  base = 16;
            if (base != 10) {                       /*se c'era un suffisso*/
                pf--;                               /*lo toglie*/
            }
        }

        *np = 0;                                    /*converte il numero <4>*/
        ok = 1;
        while (ok && pi <= pf) {                    /*scandendo i caratteri*/
            p = strchr(digit,tolower(*pi));         /*cerca in 0..f <5>*/
            if (p == NULL || (d = p-digit) >= base) {     /*valido? <6>*/
                printf("\n- '%c' non  un digit valido"
                       " in base %d.\n\n\a",*pi,base);
                ok = 0;
            } else {                                /*ok, conta cifra <7>*/
                if (*np >= maxnum) {                /*test overflow*/
                    puts("\n- Il numero  troppo "
                         "grande per essere convertito.\a");
                    ok = 0;
                } else {
                    *np = (*np)*base+d;             /*aggiorna totale*/
                }
            }
            pi++;                                   /*digit successivo*/
        }

    } while (! ok);                                 /*ripete se non ok*/

    return (1);                                     /*se arriva qui, ok <8>*/
}

/* Note su InputNumero:
    <1> La InputStringa accetta anche una stringa nulla, e toglie gli spazi
        extra iniziali e finali (se ci sono).
    <2> Sarebbe sempre preferibile non mettere un'istruzione return in mezzo
        ad una funzione. Abbiamo fatto questa eccezione per evitare di
        indentare tutte le linee seguenti fino al termine della funzione.
    <3> Dato che pi punta al primo carattere della stringa, esaminiamo se
        questo  un prefisso che indica una certa base. Bisogna controllare
        prima i prefissi dei suffissi, altrimenti un numero come $101b
        potrebbe essere considerato 101 binario (suffisso b) invece che 101b
        esadecimale (prefisso $). Solo se non ci sono prefissi, si pu
        vedere se c' un eventuale suffisso. In ogni caso, tolower converte
        in lettere minuscole (i numeri non cambiano) per cercare poi nella
        stringa globale digit[].
    <4> Il risultato finale andr in *np, quindi operiamo direttamente su
        *np. Dovendo fare molte operazioni, sarebbe pi efficiente definire
        una variabile provvisoria (es. n) ed assegnarne poi il risultato a
        *np alla fine dei calcoli. L'operazione di dereferencing, cio
        accedere al valore puntato da un pointer (come *np),  infatti di
        solito molto pi lenta di un accesso diretto al contenuto di una
        normale variabile (come n).
    <5> p punta alla posizione nella stringa globale digit[] del carattere
        puntato da pi. Ad esempio, se *pi  '2', p punta al '2' in digit[].
    <6> Se la ricerca in digit[] ha avuto successo, assegna a d il valore
        del carattere (ricavato dalla sua posizione in digit[]), e lo
        confronta con la base corrente per vedere se  accettabile (ad
        esempio, '8' non  accettabile in base 8).
    <7> Per ogni nuova cifra, il numero viene moltiplicato per la base e
        viene aggiunto il valore della nuova cifra. Ad esempio, se *np
        (il totale corrente) vale 3 ed il nuovo carattere  un '2' (che vale
        2) e siamo in base 10, *np viene moltiplicato per 10 e viene aggiunto
        2, ottenendo appunto 32, cio il numero rappresentato in base 10
        dalla sequenza di caratteri '3', '2'. Tutto questo potrebbe essere
        semplificato usando le possibilit della sscanf per l'input di
        numeri in base 8 e 16, ma la InputInt mostra come funziona il
        meccanismo della conversione da una stringa ad un valore numerico.
    <8> C'era una possibilit di uscita con una return precedente (nota 2),
        meglio indicarlo chiaramente con questo commento.
*/


/********************************************************************
 * StampaInBase: stampa il valore di num nella base indicata,
 *               seguito dalla stringa s[].
 ********************************************************************/

void StampaInBase(int base, long num, char *s)
{
    char buf[80];     /*buffer per costruire la stringa <1>*/
    int  i;

    if (num == 0) {                                 /*0=caso speciale <2>*/
        putchar(0);
    } else {                                        /*non zero: converte*/
        i = 0;
        while (num > 0) {
            buf[i++] = digit[num%base];             /*aggiunge digit <3>*/
            num /= base;                            /*aggiorna numero <4>*/
        }

        while(i) {                                  /*stampa a rovescio <5>*/
            putchar(buf[--i]);
        }
    }

    printf("%s",s);                                 /*stampa stringa <6>*/
}

/* Note su StampaInBase:
    <1> Le cifre non possono essere stampate direttamente perch vengono
        prodotte in ordine inverso. Le accumuliamo quindi in questa stringa,
        che verr poi stampata a rovescio.
    <2> Dato che non vengono stampati zeri non significativi (cio prima del
        numero), un valore 0 non verrebbe stampato affatto senza questa if.
    <3> num%base  il valore dell'ultimo digit a destra (ad esempio, nel caso
        di 235 in base 10, 235%10 vale 5). Questo valore viene usato come
        indice nella stringa digit[] per trovare il carattere corrispondente,
        che viene accumulato nella stringa buf[].
    <4> num viene aggiornato in modo da togliere il digit appena considerato
        (ad esempio, 235/10 vale 23).
    <5> (i) equivale a (i != 0).
    <6> s potrebbe essere stampata anche con printf(s), ma sorgerebbero
        problemi nel caso s contenesse caratteri di %, che la printf
        interpreterebbe come inizio di specifiche di stampa.
*/


/********************************************************************
 * InputStringa: mostra prompt, input stringa (anche nulla) in s[],
 *               senza spazi iniziali e finali (quello che segue il
 *               primo spazio, viene ignorato).
 ********************************************************************/

void InputStringa(char *prompt, char *s)
{
    char buf[128];     /*buffer di linea*/
    int  ok;           /*flag di stringa valida*/
    char *p;

    do {                                            /*legge stringa*/
        printf(prompt);
        ok = (gets(buf) != NULL);                   /*non accetta EOF*/
        if (! ok)  putchar('\a');
    } while (! ok);                                 /*esce con ok vero*/

    p = buf;                                        /*cerca inizio <1>*/
    while (isspace(*p))  p++;
    while (*p && (! isspace(*p))) {                 /*copia in s*/
        *s++ = *p++;
    }
    *s = 0;                                         /*termina stringa*/
}

/* Note su InputStringa:
    <1> Si potrebbe leggere la stringa usando semplicemente la sscanf con
        "%s", ma abbiamo preferito mantenere lo spirito 'a basso livello'
        del programma, trattando anche le stringhe come i numeri. Se non
        altro, questo mostra la complessit interna di scanf (sscanf), e
        come sia conveniente farne uso (quando possibile) per risparmiare
        lavoro.
*/
