/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* ESAMINA: mostra il contenuto di un file in esadecimale ed ASCII */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>


/* prototipi funzioni: */

int  Esamina(FILE *f, long inizio, long fine);
void MostraLinea(long nl, int val[], int nv);
long Valore(char *s);
void Errore(char *m);


/* costanti: */

#define PREF16 '$'      /*prefisso e suffisso per esadecimale*/
#define SUFF16 'h'

#define LUN    16       /*lunghezza di una linea da mostrare*/


/********************************************************************
 * main
 ********************************************************************/

main(int argc, char *argv[])
{
    FILE *f;
    long inizio,fine;    /*posizione iniziale e finale da esaminare*/
    int  c;

    if (argc < 2 || argc > 4) {                     /*controlla n.arg <1>*/
        Errore("Uso: esamina <file> [ <da> [ <a> ] ]");
    }
    if ( (f = fopen(argv[1],"rb")) == NULL) {       /*legge in binario <2>*/
        Errore("Non trovo il file indicato.\a");
    }

    inizio = 0;                                     /*pos. di default <3>*/
    fine = LONG_MAX;

    if (argc > 2) {                                 /*legge pos. iniziale*/
        inizio = Valore(argv[2]);
        if (inizio < 0) {
             Errore("La posizione iniziale dev'essere un intero positivo\n"
                 "  decimale (es. 435) o esadecimale (es. 1b3h, $1b3).\a");
         }
    }
    if (argc > 3) {                                 /*e posizione finale*/
        fine = Valore(argv[3]);
        if (fine < 0) {
             Errore("La posizione finale dev'essere un intero positivo\n"
                 "  decimale (es. 435) o esadecimale (es. 1b3h, $1b3).\a");
         }
    }
    if (inizio > fine) {
        Errore("La posizione finale dev'essere maggiore\n"
               "  od uguale alla posizione iniziale.\a");
    }

    if (Esamina(f,inizio,fine) == 0) {              /*mostra il contenuto*/
        Errore("Si  verificato un errore durante la lettura del file.\a");
    }

    fclose(f);                                      /*chiude il file*/
    return 0;                                       /*tutto ok <4>*/
}

/* Note su main:
    <1> ESAMINA va chiamato con almeno un argomento (il nome del file da
        esaminare) e altri due opzionali (posizione iniziale e finale nel
        file). Inoltre c' il primo argomento che  il nome stesso del
        programma; quindi devono esserci da 2 a 4 argomenti. In caso
        contrario viene stampata la sintassi richiesta per l'uso di ESAMINA.
        La Errore stampa il messaggio ed esce dal programma.
    <2> Per poter leggere tutti i caratteri contenuti nel file, questo deve
        essere aperto in lettura in modo binario. Se l'apertura non riesce,
        quasi certamente il nome dato non corrisponde ad un file accessibile.
    <3> Poich non abbiamo ancora visto come conoscere la lunghezza di un
        file, assumiamo come posizione finale di default il massimo valore
        possibile come long. In questo modo, siamo sicuri che la posizione
        finale (fine) si trova oltre il termine del file.
    <4> Il valore indicato viene ritornato al DOS, o comunque al programma
        chiamante, come nel caso di exit (vedi nota 1 di Errore). Ritornare
        zero indica che tutto va bene.
*/


/********************************************************************
 * Esamina: mostra il contenuto del file f, da inizio a fine,
 *          ritorna 0 in caso di errore.
 ********************************************************************/

int Esamina(FILE *f, long inizio, long fine)
{
    int  linea[LUN];      /*linea di caratteri (EOF=no car.)*/
    long i,n;
    int  c,ok;

    putchar('\n');
    n = 0;                                          /*posizione nel file*/
    while (n <= fine && ! feof(f)) {                /*legge dal file <1>*/
        ok = 0;
        for (i = 0; i < LUN; i++) {                 /*per linee lunghe LUN*/
            c = fgetc(f);                           /*legge comunque <2>*/
            if (n+i < inizio || n+i > fine) {       /*EOF se non valido*/
                c = EOF;
            }
            linea[i] = c;
            if (c != EOF)  ok = 1;                  /*va mostrata <3>*/
        }
        if (ok) {                                   /*se contiene qualcosa*/
            MostraLinea(n,linea,LUN);               /*la mostra*/
        }
        n += LUN;                                   /*linea seguente <4>*/
    }
}

/* Note su Esamina:
    <1> Poich il ciclo termina solamente al raggiungimento del valore
        finale o alla fine del file (feof), occorre un sistema per uscire
        quando si vuole smettere di esaminare il contenuto di un file lungo.
        Questo si pu ottenere premendo Ctrl-C, in quanto le funzioni di
        stdio (come printf) terminano il programma se viene premuto Ctrl-C
        (mentre Ctrl-S permette di effettuare una pausa). Sarebbe pi
        elegante (e pi pratico) usare la kbhit della libreria di console
        (conio) per fare una pausa quando viene premuto un tasto, con la
        possibilit di continuare o terminare l'esecuzione del programma.
    <2> Vengono letti anche i caratteri che non saranno mostrati su video
        (perch precedono la posizione iniziale indicata come argomento),
        per poter arrivare al punto desiderato del file. Il C consente anche
        di posizionarsi direttamente sul punto voluto, ma non abbiamo ancora
        introdotto le relative istruzioni. In ogni caso, chiamando la fgetc
        oltre la fine del file si ottiene EOF. Abbiamo usato int e non char
        per l'array linea[] in modo da poter distinguere EOF da un normale
        carattere (i normali caratteri occupano tutte le 256 possibili
        combinazioni di un char).
    <3> Il flag ok ricorda se almeno un elemento di linea contiene un
        carattere da mostrare. In caso contrario, l'intera linea non viene
        mostrata.
    <4> La posizione si sposta a passo LUN (che  16) per migliorare la
        leggibilit ed identificare velocemente i byte cercati nel file.
*/


/********************************************************************
 * MostraLinea: mostra array val[] di nv elementi in esadecimale e
 *              in ASCII su una stessa linea, con numero nl. Non
 *              mostra gli elementi EOF.
 ********************************************************************/

void MostraLinea(long nl, int val[], int nv)
{
    const int simbCtrl = 30;    /*simbolo per caratteri di controllo*/

    int i,c;

    printf("%06lx:  ",nl);                          /*mostra posizione <1>*/
    for (i = 0; i < nv; i++) {                      /*mostra in hex*/
        if ( (c = val[i]) != EOF) {                 /*se valido*/
            printf("%02x ",c);                      /*<2>*/
        } else {
            printf("   ");                          /*<3>*/
        }
        if (i == 7)  putchar(' ');                  /*spazio dopo 8*/
    }
    printf("  ");

    for (i = 0; i < nv; i++) {                      /*mostra in ASCII <4>*/
        if ( (c = val[i]) != EOF) {                 /*se valido*/
            if (c < 32)  c = simbCtrl;              /*ctrl: mostra simbolo*/
            putchar(c);
        } else {
            putchar(' ');
        }
    }
    putchar('\n');
}

/* Note su MostraLinea:
    <1> La posizione viene stampata come numero esadecimale di 6 cifre. Il
        programma  quindi 'limitato' a file di 16 Mbyte.
    <2> Nella parte sinistra del video sono mostrati i codici esadecimali
        dei caratteri contenuti nel file, 2 digit per carattere.
    <3> Per mantenere l'allineamento, se il carattere non  da stampare
        sono stampati al suo posto degli spazi.
    <4> Nella parte destra del video sono indicati i caratteri corrispondenti
        ai codici stampati a sinistra. Per evitare interferenze con la
        stampa (es. codici 10, 13), al posto dei caratteri di controllo
        (codici 0..31) viene stampato il simbolo simbCtrl.
*/


/********************************************************************
 * Valore: ritorna il valore della stringa s[] considerata come numero
 *         positivo decimale o esadecimale (prefisso $ o suffisso h),
 *         o <0 se non riesce ad effettuare una conversione valida.
 ********************************************************************/

long Valore(char *s)
{
    long v = -1L;     /*valore da calcolare, con default <1>*/

    if (*s == PREF16) {                             /*prefisso hex <2>*/
        sscanf(s+1,"%lx",&v);
    } else if (s[strlen(s)-1] == SUFF16) {          /*suffisso hex <3>*/
        sscanf(s,"%lx",&v);
    } else {
        sscanf(s,"%ld",&v);                         /*decimale*/
    }

    return v;                                       /*valore, o errore <4>*/
}

/* Note su Valore:
    <1> Nel caso in cui la stringa non rappresenti un numero valido, la
        sscanf non altera la variabile in cui dovr mettere il valore, e
        lascia quindi questo -1L (L indica long), che significa errore.
    <2> Se la stringa inizia con il prefisso di esadecimale ($), converte
        a partire dal secondo carattere (s+1) usando "%lx", che significa
        'esadecimale long'.
    <3> Se l'ultimo carattere della stringa  il prefisso di esadecimale (h),
        converte usando ancora "%lx". La h finale non  un digit valido, e
        quindi non disturba la conversione.
    <4> Dato che la funzione ritorna un numero negativo per segnalare errore,
        non c' bisogno di controllare che il valore ritornato sia positivo:
        un numero negativo scritto dall'utente  automaticamente non valido.
*/


/********************************************************************
 * Errore: stampa messaggio di errore ed esce dal programma.
 ********************************************************************/

void Errore(char *m)
{
    printf("\n- %s\n",m);
    exit(1);                                        /*codice errore <1>*/
}

/* Note su Errore:
    <1> Questo valore viene ritornato al DOS e pu essere usato (ad esempio)
        da un file di comando (.BAT) che chiama ESAMINA, per sapere se questo
         stato correttamente eseguito.
*/
