/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* LIBRARCH: funzioni di archivio per LIBRI */


#include <stdio.h>
#include <string.h>

#include "libri.h"
#include "Jconio.h"


/********************************************************************
 * NuovoArchivio: crea nuovo archivio con una sola scheda vuota.
 ********************************************************************/

void NuovoArchivio(void)
{
    numLibri = 1;                                   /*c' una scheda*/
    corrente = 0;                                   /* questa*/
    libri[0] = libroNullo;                          /*la vuota*/
}


/********************************************************************
 * Inserisce scheda: inserisce una nuova scheda dopo la scheda
 *                   corrente, ritorna falso se non c' pi spazio.
 ********************************************************************/

int InserisceScheda(void)
{
    int ok,n;

    ok = (numLibri < MAX_LIBRI);                    /*solo se ci sta*/
    if (! ok) {
        Errore("L'archivio e' pieno.");
    } else {                                        /*ci sta, inserisce*/
        n = numLibri-1;                             /*dall'ultima*/
        while (n > corrente) {                      /*all'indietro*/
            libri[n+1] = libri[n];                  /*sposta avanti*/
            n--;
        }
        numLibri++;                                 /*aggiorna totale*/
        corrente++;                                 /*va sulla nuova*/
        libri[corrente] = libroNullo;               /*la azzera <1>*/
        MostraCorrente();                           /*la mostra*/
        ModificaScheda();                           /*e la edita*/
    }

    return ok;
}

/* Note su InserisceScheda:
    <1> La nuova scheda inserita viene azzerata, e subito si chiama la
        ModificaScheda per consentire all'utente di introdurrere i dati
        nella nuova scheda.
*/


/********************************************************************
 * CancellaScheda: cancella scheda corrente, vero se cancellata.
 ********************************************************************/

int CancellaScheda(void)
{
    int ok,n;

    ok = (numLibri > 1);                            /*guarda se pu*/
    if (! ok) {
        Errore("Non puoi cancellare l'unica scheda.");
    } else {
        n = corrente;                               /*da quella corrente*/
        while (n < numLibri-1) {                    /*all'ultima*/
            libri[n] = libri[n+1];                  /*sposta indietro*/
            n++;
        }
        numLibri--;                                 /*aggiorna totale*/
        if (corrente >= numLibri) {                 /*corregge se ultima <1>*/
            corrente--;
        }
        MostraCorrente();                           /*aggiorna video*/
    }
}

/* Note su CancellaScheda:
    <1> Quando si cancella l'ultima scheda, bisogna tornare alla precedente
        per evitare che la scheda corrente rimanga quella cancellata, che si
        trova ora oltre la fine dell'archivio.
*/


/********************************************************************
 * ModificaScheda: modifica la scheda corrente, vero se cambiata.
 ********************************************************************/

int ModificaScheda(void)
{
    Libro edit;     /*scheda di lavoro <1>*/
    int   mod;      /*vero se modificata*/

    edit = libri[corrente];                         /*copia scheda corrente*/
    InputStringa(edit.titolo,LTITOLO,XDATI,YTITOLO);        /*input titolo*/
    InputStringa(edit.autore,LAUTORE,XDATI,YAUTORE);        /*autore*/
    InputStringa(edit.editore,LEDITORE,XDATI,YEDITORE);     /*editore*/

    mod = (strcmp(edit.titolo,libri[corrente].titolo) != 0  /*modif.? <2>*/
        || strcmp(edit.autore,libri[corrente].autore) != 0
        || strcmp(edit.editore,libri[corrente].editore) != 0);

    libri[corrente] = edit;                         /*registra modifiche*/

    return mod;                                     /*vero se modificata*/
}

/* Note su ModificaScheda:
    <1> E' comodo creare una scheda provvisoria su cui lavorare, invece di
        dovere continuamente riferirsi a libri[corrente].
    <2> Se l'utente non altera il contenuto degli elementi (le tre stringhe)
        della scheda, non  il caso di segnare che  stata modificata. Non
         possibile in C confrontare direttamente due strutture con !=, cosa
        comunque inutile perch in realt ci interessano solo i caratteri di
        ciascuna stringa fino allo zero finale, e non quelli non usati.
*/


/********************************************************************
 * MostraCorrente: mostra dati scheda corrente.
 ********************************************************************/

void MostraCorrente(void)
{
    char buf[81];

    sprintf(buf,"  Scheda n.%d di %d (max %d)  ",   /*prepara statistiche*/
            corrente+1,numLibri,MAX_LIBRI);
    PutsAt(40-(strlen(buf)/2),YTITOLO-3,buf);       /*le mostra*/
    PutsAt(XDATI,YTITOLO,libri[corrente].titolo);   /*mostra dati scheda*/
    PutNchar(XDATI+LTITOLO-Jposx(),' ');            /*cancella resto campo*/
    PutsAt(XDATI,YAUTORE,libri[corrente].autore);
    PutNchar(XDATI+LAUTORE-Jposx(),' ');
    PutsAt(XDATI,YEDITORE,libri[corrente].editore);
    PutNchar(XDATI+LEDITORE-Jposx(),' ');
}


/********************************************************************
 * ImpostaSelezione: input criteri di selezione.
 ********************************************************************/

void ImpostaSelezione(void)
{
    InputStringa(selezione.titolo,XSEL_LEN,XSEL_DAT,YSELEZ);      /*titolo*/
    InputStringa(selezione.autore,XSEL_LEN,XSEL_DAT,YSELEZ+1);    /*autore*/
    InputStringa(selezione.editore,XSEL_LEN,XSEL_DAT,YSELEZ+2);   /*editore*/
}


/********************************************************************
 * CercaSelezione: va alla pi vicina scheda che risponde ai criteri
 *                 di selezione nella direzione indicata (+1/-1),
 *                 ritorna vero se trovata.
 ********************************************************************/

int CercaSelezione(int dir)
{
    Libro *ps;          /*pointer alla scheda corrente <1>*/
    int   n,ok,fuori;

    n = corrente;                                   /*dalla scheda corrente*/
    ok = fuori = 0;

    while (! ok && ! fuori) {                       /*cerca fino al limite*/
        n += dir;                                   /*nella direzione scelta*/
        fuori = (n < 0 || n >= numLibri);           /*controlla limite*/
        if (! fuori) {                              /*se nei limiti*/
            ps = libri+n;                           /*punta alla scheda <2>*/
            ok =                                    /*verifica criteri <3>*/
                (strstr(ps->titolo,selezione.titolo) != NULL
                 && strstr(ps->autore,selezione.autore) != NULL
                 && strstr(ps->editore,selezione.editore) != NULL);
        }
    }

    if (ok) {                                       /*trovata, ci va*/
        corrente = n;
    }

    return (ok);                                    /*e avvisa*/
}

/* Note su CercaSelezione:
    <1> ps  un pointer ad una variabile di tipo Libro, che  una struttura
        definita con typedef in LIBRI.H.
    <2> libri+n , come al solito, l'indirizzo dell'elemento n dell'array
        libri, cio &libri[n].
    <3> La scheda risponde ai criteri di selezione se tutte e tre le stringhe
        indicate nella selezione sono contenute nelle stringhe della scheda
        stessa. Se non avessimo puntato ps alla scheda da esaminare (vedi
        nota 1), invece di ps->titolo avremmo scritto: libri[n].titolo, e
        lo stesso per gli altri due confronti. Sarebbero dunque stati
        necessari tre accessi all'elemento n dell'array libri[]. L'uso dei
        pointer a strutture pu aumentare molto l'efficienza di un programma.
*/
