/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* LIBRFILE: funzioni di registrazione/rilettura archivio per LIBRI */


#include <stdio.h>
#include <errno.h>

#include "libri.h"


/********************************************************************
 * RegistraArchivio: scrive archivio nel file, crea file di backup,
 *                   ritorna 0 se errore.
 ********************************************************************/

int RegistraArchivio(void)
{
    FILE *arc;
    int  ok;

    Messaggio("Registrazione archivio...");         /*<1>*/

    remove(nomeCopia);                              /*cancella ev.copia <2>*/
    rename(nomeArchivio,nomeCopia);                 /*tiene copia <3>*/
    arc = fopen(nomeArchivio,"w");                  /*apre in scrittura*/

    ok = (arc != NULL                                   /*<4>*/
      && fwrite(&numLibri,sizeof(numLibri),1,arc) == 1  /*scrive num.schede*/
      && fwrite(&corrente,sizeof(corrente),1,arc) == 1  /*scheda corrente*/
      && fwrite(libri,sizeof(Libro),numLibri,arc) == numLibri); /*contenuto*/

    Messaggio("");                                  /*toglie messaggio*/

    if (! ok) {                                     /*informa su ev. errore*/
        if (_doserrno == EACCES) {                  /*<5>*/
            Errore("Il disco  protetto dalla scrittura.");
        } else {                                    /*poca info*/
            Errore("Non  stato possibile registrare l'archivio.");
        }
    }

    fclose(arc);                                    /*chiude file*/
    return ok;                                      /*informa se ok*/
}

/* Note su RegistraArchivio:
    <1> Quando un'operazione pu impiegare un certo tempo,  bene avvisare
        l'utente che il programma non  'morto' ma sta lavorando e non si
        aspetta alcun intervento da parte sua.
    <2> Cancella l'eventuale file di copia preesistente. Dato che questo pu
        non esistere, non controlla se l'operazione  riuscita.
    <3> Il file di archivio corrente viene cambiato di nome, in modo che
        non venga perso se accade qualche problema durante la registrazione
        sullo stesso file. Dato che il file di archivio pu non esistere,
        non controlla se l'operazione  riuscita.
    <4> Dato che gli operandi di && sono esaminati in sequenza (fermandosi
        non appena uno risulta falso), questa assegnazione alla variabile
        logica ok  in realt l'intera operazione di scrittura, che viene
        interrotta se una delle fwrite non ritorna il valore corretto. Se
        ok  falsa fin dall'inizio (perch la fopen non  riuscita), nessuna
        delle fwrite viene eseguita.
    <5> In caso di errore  utile dare all'utente quante pi informazioni
        possibili sulla relativa causa. Purtroppo, sia con il Turbo C 2.0
        che con il Quick C 2.0, le informazioni accessibili sono piuttosto
        limitate, specialmente usando _doserrno (che si comporta allo stesso
        modo nei due compilatori) invece di errno. Inoltre, l'errore viene
        prima intercettato da MS-DOS che presenta all'utente un intimidatorio
        "Protetto da scrittura: Abbandonare, Riprovare, Fallire ?". Solo se
        l'utente preme F appare il messaggio di errore "Il disco  protetto
        dalla scrittura". Vedremo pi avanti nel corso esperti come evitare
        il messaggio del DOS, o renderlo pi 'umano'.
*/


/********************************************************************
 * LeggeArchivio: legge archivio da file, ritorna 0 se errore.
 ********************************************************************/

int LeggeArchivio(void)
{
    FILE *arc;
    int  ok;

    Messaggio("Lettura archivio...");
    arc = fopen(nomeArchivio,"r");                  /*apre in lettura*/

    ok = (arc != NULL                                   /*<1>*/
      && fread(&numLibri,sizeof(numLibri),1,arc) == 1   /*legge num.schede*/
      && fread(&corrente,sizeof(corrente),1,arc) == 1   /*scheda corrente*/
      && fread(libri,sizeof(Libro),numLibri,arc) == numLibri);  /*contenuto*/

    Messaggio("");                                  /*toglie messaggio*/
    fclose(arc);                                    /*chiude file*/
    return ok;                                      /*informa se ok*/
}

/* Note su LeggeArchivio:
    <1> Vedi nota 4 di RegistraArchivio.
*/
