/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* LIBRI: archivio per biblioteca (main file) <1> */


#include <stdio.h>
#include <string.h>

#include "libri.h"                            /*<2>*/
#include "Jconio.h"


/* variabili globali (definizioni): <3>*/

Libro libri[MAX_LIBRI];                       /*archivio biblioteca <4>*/
int   numLibri;                               /*numero libri contenuti*/
int   corrente;                               /*scheda (libro) corrente*/
int   modificato;                             /*flag di archivio modificato*/
Libro selezione;                              /*selezione corrente*/

Libro libroNullo = { "", "", "" };            /*scheda vuota <5>*/

char nomeArchivio[] = "LIBRI.DAT";            /*file di archivio <6>*/
char nomeCopia[]  = "LIBRI.COP";              /*e file di backup <7>*/


/* Note su variabili globali:
    <1> In un programma suddiviso in pi file sorgente, il file principale
        (main file)  quello che contiene la funzione main e che ha di solito
        il nome stesso del programma (LIBRI.C).
    <2> Lo header LIBRI.H contiene le definizioni di costanti, le
        dichiarazioni di tutte le variabili globali ed i prototipi di
        funzione. E' il punto di riferimento comune dell'intero programma.
    <3> Le variabili globali dichiarate in LIBRI.H risultano dichiarate in
        tutti i file sorgente (che includono lo header LIBRI.H). Sono invece
        definite una sola volta, qui nel main.
    <4> La dimensione di un array si indica solo nella sua definizione. La
        costante MAX_LIBRI  definita nello header LIBRI.H.
    <5> La variabile libroNullo  di tipo Libro, che  una struttura definita
        nel solito header LIBRI.H. Viene qui inizializzata con tre stringhe
        nulle, in modo da disporre di una scheda (record) vuota, copiabile
        in un record dell'archivio con una semplice assegnazione.
    <6> Anche la stringa nomeArchivio, dichiarata nel main senza indicarne
        la lunghezza, viene definita qui nel main.
    <7> nomeCopia  il nome del file di backup, cio della copia di sicurezza
        che viene fatta prima della registrazione su disco per conservare la
        versione precedente in caso di problemi. Convenzionalmente si usa il
        suffisso .BAK (backup), ma abbiamo preferito italianizzarlo perch
        lo stesso suffisso viene usato dall'editor del Turbo C quando si
        lavora sul file LIBRI.C.
*/


/********************************************************************
 * main
 ********************************************************************/

main()
{
    char buf[81];
    int  si;

    Jcursor(0);                                     /*spegne cursore*/
    PreparaSchermo();                               /*disegna schermo*/
    selezione = libroNullo;                         /*azzera selezione <1>*/

    if (LeggeArchivio() == 0) {                     /*legge dal file*/
        Jputch('\a');                               /*errore, segnala*/
        Inverse(1);
        sprintf(buf,
            "Non riesco a leggere il file \"%s\"."
            " Creo un nuovo archivio (s/n)? ",
            nomeArchivio);
        Inverse(1);
        si = SiNo(buf);                             /*chiede se creare <2>*/
        Inverse(0);
        if (si) {
            NuovoArchivio();                        /*si, lo crea*/
        } else {
            numLibri = 0;                           /*no archivio*/
        }
    }
    modificato = 0;                                 /*non modificato <3>*/

    if (numLibri > 0) {                             /*se c' archivio*/
        MostraCorrente();                           /*mostra scheda*/
        CicloComandi();                             /*lavora <4>*/
    }

    Jclrscr();                                      /*fine, cancella*/
    Jputs("\r\n\r\n\r\n- Buona lettura!\r\n");
    Jcursor(1);                                     /*riaccende cursore*/
}

/* Note su main:
    <1> Questa assegnazione copia un'intera struttura (vedi nota 5 delle
        variabili globali).
    <2> La SiNo (in LIBRUTIL.C) stampa la stringa come domanda e accetta
        come risposta 's' o 'n', torna vero se 's'.
    <3> Questa variabile logica (flag) ricorda se l'archivio  stato
        modificato dopo l'ultima registrazione, in modo da evitare che
        si esca accidentalmente perdendo tutte le modifiche fatte.
    <4> La CicloComandi attende ed esegue i comandi dell'utente. E'
        separata dal main solo per evitare funzioni troppo lunghe.
*/


/********************************************************************
 * CicloComandi: attende ed esegue comandi, esce solo se fine lavoro.
 ********************************************************************/

void CicloComandi(void)
{
    int finito = 0;
    int c;

    do {
        Jclrkey();                                  /*pulisce buffer*/
        c = Domanda("Scegli un comando: ");         /*Aspetta un comando <1>*/
        switch(c) {                                 /*<2>*/

        case PREC:                                  /*scheda precedente:*/
            if (corrente > 0) {                     /*se non  la prima*/
                corrente--;                         /*ci va*/
                MostraCorrente();                   /*e la mostra*/
            }
            break;
        case SUCC:                                  /*scheda successiva:*/
            if (corrente < numLibri-1) {            /*se non  l'ultima*/
                corrente++;                         /*ci va*/
                MostraCorrente();                   /*e la mostra*/
            }
            break;
        case PREC_SEL:
            if (CercaSelezione(-1)) {               /*precedente selezionata*/
                MostraCorrente();                   /*mostra se trovata*/
            }
            break;
        case SUCC_SEL:
            if (CercaSelezione(1)) {                /*successiva selezionata*/
                MostraCorrente();                   /*mostra se trovata*/
            }
            break;
        case INS_SCHEDA:                            /*inserisce scheda:*/
            if (InserisceScheda()) {                /*esegue se pu*/
                modificato = -1;                    /*segna se modificato*/
            }
            break;
        case DEL_SCHEDA:                            /*cancella scheda:*/
            if (CancellaScheda()) {                 /*esegue se pu*/
                modificato = -1;                    /*segna se modificato*/
            }
            break;
        case MOD_SCHEDA:                            /*modifica scheda:*/
            if (ModificaScheda()) {                 /*esegue*/
                modificato = -1;                    /*segna se modificato*/
            }
            break;
        case SELEZIONE:
            ImpostaSelezione();                     /*criteri di selezione*/
            break;
        case REGISTRA:
            if (RegistraArchivio()) {               /*registra archivio:*/
                modificato = 0;                     /*segna se ok*/
            }
            break;
        case FINE:                                  /*fine programma:*/
            if ( (! modificato)                     /*se modif., chiede <3>*/
              || SiNo("Non hai registrato le "
                      "modifiche. Esci lo stesso? ")) {
                finito = -1;
            }
            break;
        default:                                    /*comando non valido*/
            Jputch('\a');                           /*segnala errore*/
            break;
        }
    } while (! finito);
}

/* Note su CicloComandi:
    <1> La Domanda (in LIBRUTIL.C) stampa la stringa come domanda ed attende
        che sia premuto un tasto, del quale ritorna il codice.
    <2> LIBRI  un tipico esempio di programma event-driven, cio pilotato
        dagli eventi, che in questo caso sono i tasti premuti dall'utente.
        Il programma  sempre in attesa di un tasto in questo ciclo: quando
        lo riceve esegue il comando corrispondente e poi torna qui. La
        voluminosa switch  tipica dei programmi event-driven.
    <3> Vedi nota 3 del main.
*/


/********************************************************************
 * PreparaSchermo: disegna finestra, diciture e descrizioni comandi.
 ********************************************************************/

void PreparaSchermo(void)
{
    char *istruz[] = {                              /*<1>*/
        "\x1b   Scheda precedente",                 /*<2>*/
        "\x18   Scheda successiva",
        "F1  Inserisce nuova scheda",
        "F2  Modifica scheda corrente",
        "F5  Registra su disco",
        "     \x18   Scheda precedente selezionata",
        "     \x19   Scheda successiva selezionata",
        "Ctrl F1  Cancella scheda corrente",
        "     F3  Cambia criteri di selezione",
        "     F6  Fine lavoro"
    };

    int i;

    Jclrscr();
    PutsAt(18,1,"- Archivio per biblioteca - dal Corso di C -");
    DisegnaFinestra(3,3,78,13);                     /*finestra di scheda*/

    PutsAt(XDESCRIZ,YTITOLO,"Titolo :");            /*descrizioni campi*/
    PutsAt(XDESCRIZ,YAUTORE,"Autore :");
    PutsAt(XDESCRIZ,YEDITORE,"Editore:");
    PutsAt(X1ISTRUZ,YSELEZ,"Attuali criteri di selezione:");
    PutsAt(XSEL_TIT,YSELEZ,"Titolo :");
    PutsAt(XSEL_TIT,YSELEZ+1,"Autore :");
    PutsAt(XSEL_TIT,YSELEZ+2,"Editore:");

    for (i = 0; i < 5; i++) {
        PutsAt(X1ISTRUZ,YISTRUZ+i,istruz[i]);       /*colonna di sinistra*/
        PutsAt(X2ISTRUZ,YISTRUZ+i,istruz[i+5]);     /*colonna di destra*/
    }
}

/* Note su PreparaSchermo
    <1> Per comodit le istruzioni sono contenute in un array di stringhe,
        o pi esattamente in un array di pointer a char, ciascun elemento
        del quale  automaticamente inizializzato in modo che punti ad una
        delle stringhe elencate.
    <2> \x1b  il carattere avente codice esadecimale 1b, che se stampato
        sul video del PC appare come una freccia a sinistra.
*/
