/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* LIBRI.H: header per LIBRI <1>*/


/* costanti: */

#define PREC       1075    /*freccia a sinistra: scheda precedente*/
#define SUCC       1077    /*freccia a destra: scheda successiva*/
#define PREC_SEL   1072    /*freccia in alto: precedente selezionata*/
#define SUCC_SEL   1080    /*freccia in basso: successiva selezionata*/
#define INS_SCHEDA 1059    /*F1: inserisce nuova scheda*/
#define DEL_SCHEDA 1094    /*Ctrl-F1: cancella scheda corrente*/
#define MOD_SCHEDA 1060    /*F2: modifica scheda corrente*/
#define SELEZIONE  1061    /*F3: cambia criteri di selezione*/
#define REGISTRA   1063    /*F5: registra archivio*/
#define FINE       1064    /*F6: fine lavoro*/

#define YTITOLO    7          /*posizioni verticali campi su video: titolo*/
#define YAUTORE    YTITOLO+2  /*autore*/
#define YEDITORE   YTITOLO+4  /*editore*/
#define XDESCRIZ   7          /*posizione orizzontale descrizioni e dati*/
#define XDATI      20
#define YSELEZ     15         /*posizione verticale input selezione*/
#define XSEL_TIT   39         /*posizione orizz. titoli per input selezione*/
#define XSEL_DAT   48         /*posizione orizzontale input selezione*/
#define XSEL_LEN   30         /*lunghezza input selezione*/
#define YISTRUZ    19         /*posizione verticale istruzioni*/
#define X1ISTRUZ   3          /*posizioni orizzontali istruzioni*/
#define X2ISTRUZ   39

#define LTITOLO    50         /*lunghezze campi, in caratteri: titolo*/
#define LAUTORE    30         /*autore*/
#define LEDITORE   30         /*editore*/


/*definisce il tipo Libro (un record): <2>*/

typedef struct {
    char titolo[LTITOLO+1];                      /*<3>*/
    char autore[LAUTORE+1];
    char editore[LEDITORE+1];
} Libro;

#define MAX_MEM    50000U                        /*memoria per archivio <4>*/
#define MAX_LIBRI  (MAX_MEM / sizeof(Libro))     /*numero massimo record*/


/* variabili globali (dichiarazioni): */

extern Libro libri[];                            /*archivio biblioteca <5>*/
extern int   numLibri;                           /*numero libri contenuti*/
extern int   corrente;                           /*scheda (libro) corrente*/
extern int   modificato;                         /*flag di archivio modificato*/
extern Libro selezione;                          /*selezione corrente*/
extern Libro libroNullo;                         /*scheda vuota*/

extern char nomeArchivio[];                      /*file di archivio <6>*/
extern char nomeCopia[];                         /*e file di backup*/

extern int _doserrno;                            /*numero errore dos <7>*/


/* prototipi funzioni: <8>*/

void CicloComandi(void);                         /*file LIBRI.C (main)*/
void PreparaSchermo(void);

void NuovoArchivio(void);                        /*file LIBRARCH.C*/
int  InserisceScheda(void);
int  CancellaScheda(void);
int  ModificaScheda(void);
void MostraCorrente(void);
void ImpostaSelezione(void);
int  CercaSelezione(int dir);

int  LeggeArchivio(void);                        /*file LIBRFILE.C*/
int  RegistraArchivio(void);

int  Domanda(char *d);                           /*file LIBRUTIL.C*/
int  SiNo(char *d);
void Errore(char *e);
void Messaggio(char *s);
void PutsAt(int x, int y, char *s);
void PutNchar(int n, int c);
int  InputStringa(char *s, int lmax, int px, int py);
void Inverse(int inv);
void DisegnaFinestra(int x1, int y1, int x2, int y2);


/* Note su dichiarazioni e costanti:
    <1> Questo header riunisce tutte le dichiarazioni di costanti, variabili
        globali e prototipi di funzione del programma LIBRI, in modo che
        siano accessibili da tutti i file sorgente C del programma stesso.
    <2> I dati di ciascun libro della biblioteca sono contenuti in una scheda
        dell'archivio, tecnicamente un 'record', che in C si rappresenta
        molto bene con una struttura.
    <3> Il tipo Libro  una struttura contenente tre array di char di
        lunghezza fissa, adatti a contenere stringhe (il +1 serve appunto per
        lo zero finale). Lo spazio occupato da una variabile di tipo Libro
         costante, indipendentemente dalla lunghezza delle stringhe scritte
        nei suoi tre elementi. Questo semplifica il programma, anche se non
        consente di sfruttare a fondo la memoria disponibile.
    <4> Nonostante i PC IBM e compatibili abbiano in genere una memoria di
        640 Kbyte, per sfruttare pi di 64 Kbyte occorre usare particolari
        accorgimenti, come vedremo nel seguito del corso esperti. Dato che
        occorre lasciare spazio nei 64K per le normali variabili, teniamo
        un po' di spazio di riserva e destiniamo 50000 byte a contenere
        l'archivio dei libri (in questo programma l'archivio risiede tutto
        in memoria). Dividendo la memoria disponibile per la dimensione di
        un record (scheda per un libro), otteniamo il numero massimo di
        record che possono stare in memoria. Questo verr usato nel file
        principale LIBRI.C per dimensionare l'array libri[], cio l'archivio.
    <5> Non si deve indicare nello header la dimensione degli array,
        altrimenti la dichiarazione diventa una definizione (il compilatore
        riserva spazio per l'array), e una definizione non pu essere ripetuta
        (nemmeno in file diversi) mentre questo header viene incluso da ogni
        file sorgente. Ci limitiamo quindi a dichiarare che l'array  extern,
        cio definito altrove, senza indicarne qui la dimensione.
    <6> nomeArchivio e copiaArchivio sono array di char, cio stringhe,
        definiti anch'essi nel file principale LIBRI.C.
    <7> _doserrno  una variabile globale definita nelle librerie del
        Turbo C 2.0 e del Quick C 2.0, che fornisce alcune informazioni
        sulla causa dell'ultimo errore relativo alle operazioni sui file.
    <8> I prototipi di funzione sono, per comodit di lettura, suddivisi
        a seconda del file in cui  definita la funzione corrispondente.
*/
