/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* LIBRUTIL: funzioni di uso generale (utility) per LIBRI */


#include <string.h>
#include <ctype.h>

#include "libri.h"
#include "Jconio.h"


/********************************************************************
 * Domanda: stampa stringa d[] su linea di stato, torna col primo
 *          carattere premuto.
 ********************************************************************/

int Domanda(char *d)
{
    int c;

    Messaggio(d);                                   /*mostra domanda*/
    Jclrkey();                                      /*pulisce buffer*/
    c = Jgetkey(1);                                 /*aspetta tasto*/
    Messaggio("");                                  /*ripulisce*/
    return c;                                       /*ritorna tasto*/
}


/********************************************************************
 * SiNo: mostra domanda d[] su linea di stato, aspetta 's' o 'n'
 *       (anche maiuscoli), ritorna vero se 's'.
 ********************************************************************/

int SiNo(char *d)
{
    int c,ok;

    do {
        c = tolower(Domanda(d));                    /*aspetta risposta*/
        ok = (c == 's' || c == 'n');                /*controlla*/
        if (! ok) {
            Jputch('\a');                           /*segnala errore*/
        }
    } while (!ok);
    return (c == 's');
}


/********************************************************************
 * Errore: mostra messaggio di errore e[] in inverse, aspetta Return.
 ********************************************************************/

void Errore(char *e)
{
    Inverse(1);                                     /*va in inverse*/
    Messaggio(e);                                   /*mostra messaggio*/
    Jputch('\a');
    Jputs("  Premi Return per continuare: ");
    while (Jgetkey(1) != '\r') {                    /*aspetta Return*/
        Jputch('\a');
    }

    Inverse(0);                                     /*rimette a posto*/
    Messaggio("");
}


/********************************************************************
 * Messaggio: stampa stringa s[] su linea di stato (linea 25), o
 *            cancella linea di stato se s[]  una stringa nulla.
 ********************************************************************/

void Messaggio(char *s)
{
    Jgotoxy(1,25);
    if (*s) {                                       /*non nulla, stampa*/
        Jputs("  ");
        Jputs(s);
    } else {                                        /*nulla, cancella*/
        Inverse(0);                                 /*assicura normale*/
        PutNchar(78,' ');
    }
}


/********************************************************************
 * PutsAt: stampa stringa alla posizione x,y.
 ********************************************************************/

void PutsAt(int x, int y, char *s)    /*<1>*/
{
    Jgotoxy(x,y);                                   /*si posiziona*/
    Jputs(s);                                       /*e stampa*/
}

/* Note su PutsAt
    <1> La funzione non conserva la posizione corrente del cursore.
*/


/********************************************************************
 * PutNchar: stampa n caratteri c di seguito.
 ********************************************************************/

void PutNchar(int n, int c)
{
    while (n-- > 0) {
        Jputch(c);
    }
}


/********************************************************************
 * InputStringa: mostra campo di input con default alla posizione
 *               indicata, input controllato di stringa nell'array
 *               s[] non oltre lmax caratteri, accetta correzioni
 *               con backspace, mette zero finale, ritorna lunghezza
 *               della stringa introdotta.
 ********************************************************************/

int InputStringa(char *s, int lmax, int px, int py)  /*<1>*/
{
    enum { CR = 13, BS = 8, BACK = 1075 };      /*codici tasti*/

    char buf[81];       /*buffer di linea*/
    int  ls;            /*lunghezza stringa*/
    char *s1;           /*pointer a stringa originaria, per ristampa*/
    int  i,c,ok;

    s1 = s;                                         /*inizio stringa*/

    Jgotoxy(px,py);                                 /*si posiziona*/
    Inverse(1);                                     /*campo in inverse*/
    Jputs(s);                                       /*mostra default*/
    ls = strlen(s);                                 /*lunghezza corrente*/
    for (i = ls; i < lmax; i++) {                   /*evidenzia campo*/
        Jputch(' ');
    }
    Jgotoxy(px+ls,py);                              /*torna a fine default*/

    s += ls;                                        /*punta alla fine*/
    Jclrkey();                                      /*cancella buffer*/
    do {
        c = Jgetkey(1);
        ok = (c == CR);
        if ( (c == BS || c == BACK) && ls > 0) {
            ls--;
            s--;
            Jgotoxy(px+ls,py);
            Jputch(' ');
            Jgotoxy(px+ls,py);
        } else if (isprint(c) && ls < lmax) {
            *s++ = c;
            ls++;
            Jputch(c);
        } else if (! ok) {
            Jputch('\a');
        }
    } while (! ok);

    *s = 0;                                         /*termina stringa*/
    Jgotoxy(px,py);                                 /*torna dov'era*/
    Inverse(0);                                     /*colori normali*/
    Jputs(s1);                                      /*ristampa normale*/
    for (i = ls; i < lmax; i++) {                   /*de-evidenzia campo*/
        Jputch(' ');
    }

    return ls;                                      /*ritorna lunghezza*/
}

/* Note su InputStringa:
    <1>  InputStringa  simile a quella presentata nel programma INPCON, con
         due differenze: non ha prompt, e rimette in colore normale il
         campo di input al termine dell'operazione.
*/


/********************************************************************
 * Inverse: Inverte colori se inv  vero, torna normale se falso.
 ********************************************************************/

void Inverse(int inv)         /*<1>*/
{
    static int inverse = 0;   /*ricorda lo stato corrente*/
    static int nc,nb;         /*colori normali del testo*/

    if (inv && ! inverse) {                         /*se non gi inverse*/
        nc = Jgettextcolor();                       /*ricorda colori*/
        nb = Jgettextback();
        Jtextcolor(nb);                             /*li scambia*/
        Jtextback( (nc > 7) ? nc-8 : nc);           /*max 7 per backgr.*/
    } else if (! inv && inverse) {                  /*se non gi normale*/
        Jtextcolor(nc);                             /*rimette a posto*/
        Jtextback(nb);
    }
    inverse = inv;                                  /*ricorda stato*/
}

/* Note su Inverse:
    <1> Inverse  commentata nel programma BINARIO.
*/


/********************************************************************
 * DisegnaFinestra: disegna finestra generica con riquadro per titolo.
 ********************************************************************/

void DisegnaFinestra(int x1, int y1, int x2, int y2)
{
    int la,y;
    int c1,c2,c3;

    la = x2-x1-1;                                   /*calcola larghezza*/
    Jgotoxy(x1,y1);                                 /*bordo superiore*/
    for (y = y1; y <= y2; y++) {
        Jgotoxy(x1,y);
        if (y == y1) {                              /*bordo superiore*/
            c1 = ''; c2 = ''; c3 = '';
        } else if (y == y1+2) {                     /*bordo sotto titolo*/
            c1 = ''; c2 = ''; c3 ='';
        } else if (y == y2) {                       /*bordo inferiore*/
            c1 = ''; c2 = ''; c3 = '';
        } else {                                    /*bordi laterali*/
            c1 = c3 = ''; c2 = ' ';
        }
        Jputch(c1);
        if (c2 != ' ') {
            PutNchar(la,c2);
        } else {
            Jgotoxy(x2,y);
        } Jputch(c3);
    }
}
