/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* ALFAPAR: stampa le parole di una frase in ordine alfabetico */


/*** AVVERTENZA: questo programma  molto semplice all'apparenza, ma fa uso
                 di concetti abbastanza complessi come la doppia indirezione.
                 Non  obbligatorio capirli a fondo per programmare in C, ma
                 possono semplificare parecchio il lavoro in certe occasioni.
***/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>


#define MAXPAROLE 64     /*numero massimo parole nella frase <1>*/


/* prototipi funzioni: */

int ConfrontaParole(char **p1, char **p2);


/********************************************************************
 * main
 ********************************************************************/

main()
{
    char frase[128];            /*frase introdotta <2>*/
    char sep[] = " \t.,;:'`";   /*separatori tra le parole <3>*/
    char *parole[MAXPAROLE];    /*pointer alle parole <4>*/
    char *p;
    int  ok,n,i;

    puts("\n- Ordinamento alfabetico delle parole di una frase -\n");

    do {                                            /*input frase*/
        printf("Frase: ");
    } while (gets(frase) == NULL);                  /*non accetta EOF*/

    n = 0;
    if ( (p = strtok(frase,sep)) != NULL) {         /*cerca prima parola <5>*/
        parole[n++] = p;                            /*ok, la registra*/
        while ( (p = strtok(NULL,sep)) != NULL) {   /*e cerca successive <6>*/
            parole[n++] = p;
        }
    }

    if (n > 0) {                                    /*se c'erano parole <7>*/
        qsort(parole,n,sizeof(parole[0]),
              ConfrontaParole);                     /*ordine alfabetico <8>*/

        putchar('\n');                              /*stampa parole ordinate*/
        for (i = 0; i < n; i++) {
            puts(parole[i]);                        /*<9>*/
        }
    }

}

/* Note su main:
    <1> Dato che l'input  effettuato con la gets, che non accetta pi di
        127 caratteri, e che tra una parola e l'altra ci dev'essere almeno
        un carattere separatore, non ci possono essere pi di 64 parole, nel
        caso improbabile che siano tutte composte da una sola lettera (per
        quanto improbabili, i casi estremi non vanno mai trascurati).
    <2> Spazio per 127 caratteri (limite della gets in MS-DOS), pi lo zero
        finale. ALFAPAR usa solo le funzioni di stdio.
    <3> La stringa sep[] contiene tutti i caratteri che sono considerati
        separatori validi tra le parole della frase. Servir alla funzione di
        libreria strtok per estrarre le parole dalla frase. \t  il Tab.
    <4> parole[]  un array di pointer a char, ciascun elemento del quale
        punter all'inizio di una delle parole in frase[]. Viene usato come
        array indice per riordinare le parole della frase senza muoverle
        fisicamente in memoria.
    <5> La strtok cerca in frase[] il primo carattere della prima parola (che
        non sia uno dei separatori elencati in sep[]), e ritorna un pointer
        a questo carattere (p), o NULL se la frase non contiene parole
        valide. Se  stata trovata una parola, il primo elemento di parole[]
        (un pointer a char) viene messo uguale a p, perch punti al primo
        carattere della prima parola trovata in frase[]. La strtok termina
        poi la parola trovata con uno zero finale nella frase stessa, al
        posto del separatore che terminava la parola. La frase viene quindi
        alterata: se la si vuole conservare, occorre farne prima una copia.
    <6> Passando NULL come primo argomento, la strtok continua a cercare
        nella stessa frase dal punto dov'era rimasta all'ultima chiamata.
        Ritorna quindi un pointer al primo carattere della parola successiva,
        che termina mettendo uno zero nella frase stessa. Queste poche linee
        di programma svolgono, grazie alla strtok, il lavoro dell'intero
        programma SEPARA della lezione B7.
    <7> Non  il caso di riordinare delle parole inesistenti.
    <8> La qsort viene chiamata per riordinare l'array indice parole[], che
        contiene i pointer agli elementi che si vogliono in ordine (le parole
        della frase). Le parole in s non vengono spostate, ma la qsort
        riordina l'array indice in modo che i suoi elementi puntino alle
        varie parole in frase[] nell'ordine voluto. parole[0] punter alla
        prima parola in ordine alfabetico, parole[1] alla seconda, eccetera.
        qsort confronta le parole attraverso i pointer usando la funzione
        ConfrontaParole.
    <9> Dato che gli elementi di parole[] puntano alle parole della frase in
        ordine alfabetico, parole[] appare come un array di stringhe
        correttamente ordinate.
*/


/********************************************************************
 * ConfrontaParole: confronto alfabetico di due parole, indiretto
 *                  attraverso pointer. Ritorna < 0 se la prima
 *                  precede la seconda. Funziona solo con i normali
 *                  caratteri ASCII, non con le lettere accentate.
 ********************************************************************/

int ConfrontaParole(char **p1, char **p2)           /*<1>*/
{
    return strcmp(*p1,*p2);                         /*<2>*/
}

/* Note su ConfrontaParole
    <1> La qsort passa a ConfrontaParole due pointer ai dati da confrontare.
        Questi dati puntati da **p1 e **p2, cio *p1 e *p2, sono due elementi
        dell'array indice parole[], cio a loro volta dei pointer a char.
        Ecco il motivo della doppia indirezione **p1, che vuol dire appunto
        'pointer a pointer a char'.
    <2> Poich vogliamo confrontare gli elementi puntati da *p1 e *p2 (cio
        le parole), e non *p1 e *p2 direttamente (come sarebbe nel caso di un
        normale ordinamento con qsort), passiamo alla strcmp gli indirizzi
        delle due stringhe, cio appunto *p1 e *p2. Il confronto della strcmp
        non tiene conto di maiuscole e minuscole, il che pu creare qualche
        problema. Di solito i compilatori offrono anche la funzione non
        standard stricmp, che tratta allo stesso modo lettere maiuscole e
        lettere minuscole.
*/
