/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* ORDINA: confronto tra varie tecniche di ordinamento */


/* Questo programma va linkato con la libreria di console Jackson, usando
   il file ORDINA.PRJ per il Turbo C 2.0, o ORDINA.MAK per il Quick C 2.0.
*/


#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "Jconio.h"


/* prototipi funzioni: */

void EstraeDati(double d[], int n);
void MostraDati(double d[], int n);
void Selezione(double d[], int n);
void Inserzione(double d[], int n);
void BubbleSort(double d[], int n);
void ShellSort(double d[], int n);
void Quicksort(double d[], int n);
int  Confronta(double *d1, double *d2);
void PremiReturn(void);


/********************************************************************
 * main
 ********************************************************************/

main()
{
    enum { NDATI = 200 };                       /*numero dati da riordinare*/
    enum { F1 = 1059, F2, F3, F4, F5, F6 };     /*codici tasti <1>*/

    double  dati[NDATI];                        /*dati da riordinare <2>*/
    clock_t inizio,fine;                        /*tempo iniziale, finale <3>*/
    char    buf[80];
    int     c,ok,finito;

    srand( (unsigned int)time(NULL));               /*avvia random*/
    Jcursor(0);                                     /*spegne cursore*/

    do {
        Jclrscr();                                  /*cancella schermo*/
        Jclrkey();                                  /*e buffer tastiera*/
        Jputs("\r\n\n\n\n- Confronto tra tecniche di ordinamento -\r\n\n"
            "  F1) Sort per selezione\r\n"
            "  F2) Sort per inserzione\r\n"
            "  F3) Bubble sort\r\n"
            "  F4) Shell sort\r\n"
            "  F5) Quicksort\r\n"
            "  F6) Esci\r\n\n"
            "Scegli premendo F1..F6: ");
        do {                                        /*legge tasto valido*/
            c = Jgetkey(1);
            ok = (c >= F1 && c <= F6);              /*(sono in ordine) <4>*/
            if (! ok)  Jputch('\a');
        } while (! ok);
        finito = (c == F6);

        if (! finito) {
            EstraeDati(dati,NDATI);                 /*dati da riordinare*/
            Jclrscr();                              /*li stampa*/
            Jputs("Estrazione casuale dati da ordinare:\r\n\n");
            MostraDati(dati,NDATI);

            Jclrscr();
            inizio = clock();                       /*tempo iniziale*/
            if (c == F1) {                          /*esegue sort scelto*/
                Selezione(dati,NDATI);
            } else if (c == F2) {
                Inserzione(dati,NDATI);
            } else if (c == F3) {
                BubbleSort(dati,NDATI);
            } else if (c == F4) {
                ShellSort(dati,NDATI);
            } else if (c == F5) {
                Quicksort(dati,NDATI);
            }
            fine = clock();                         /*tempo finale <5>*/
            sprintf(buf,"eseguito in %3.2f secondi.\r\n\n",
                    ((double)(fine-inizio))/CLK_TCK);
            Jputs(buf);
            MostraDati(dati,NDATI);                 /*stampa dati ordinati*/
        }

    } while (! finito);

    Jclrscr();                                      /*cancella schermo*/
    Jcursor(1);                                     /*riaccende cursore*/
}

/* Note su main:
    <1> Poich i codici dei tasti indicati sono numeri progressivi, basta
        assegnare il primo. La enum provvede poi automaticamente ad
        assegnare valori crescenti (1060,1061,...) alle successive costanti.
    <2> Usiamo un array di double perch il riordino di un array int 
        troppo veloce. Confronti ed assegnazioni di double rendono meglio
        l'idea del comportamento tipico in condizioni reali.
    <3> Usiamo clock invece di time per avere una maggiore risoluzione (circa
        1/18.2 secondi sui PC). La funzione time ha invece , sempre sui
        PC IBM e compatibili, una risoluzione tipica di un secondo.
    <4> Funziona perch i codici dei tasti F1..F6 sono numeri interi
        progressivi (vedi nota 1).
    <5> Dividendo il tempo impiegato per la costante CLK_TCK (definita in
        <time.h>), si ottiene il tempo in secondi. Notare l'uso del cast
        (double) per assicurare che la divisione venga eseguita in double,
        evitando il rischio di una divisione intera che perderebbe la
        parte frazionaria (dipende da come  definito CLK_TCK, meglio non
        rischiare ed usare invece il cast).
*/


/********************************************************************
 * EstraeDati: estrae n interi casuali nell'array d[].
 ********************************************************************/

void EstraeDati(double d[], int n)
{
    int i;

    for (i = 0; i < n; i++) {                       /*in ogni elemento*/
        d[i] = rand()/100.0;                        /*un double a caso <1>*/
    }
}

/* Note su EstraeDati:
    <1> Mette due cifre decimali ai soli fini estetici: i valori dei dati
        non interessano, basta che vengano ordinati correttamente.
*/


/********************************************************************
 * MostraDati: mostra gli n interi contenuti nell'array d[].
 ********************************************************************/

void MostraDati(double d[], int n)
{
    int  i;
    char buf[80];

    for (i = 0; i < n; i++) {                       /*per ogni elemento*/
        sprintf(buf,"%7.2f",d[i]);                  /*stampa in 7 spazi*/
        if(Jposx() > 72) {                          /*a capo se non ci sta*/
            Jputs("\r\n");
        }
        Jputs(buf);                                 /*mostra dato*/
    }
    Jputs("\r\n");
    PremiReturn();                                  /*aspetta Return*/
}


/********************************************************************
 * Selezione: riordina per selezione l'array d[] di n elementi.
 ********************************************************************/

void Selezione(double d[], int n)     /*<1>*/
{
    int    i,j,imin;
    double temp;

    Jputs("Ordinamento per selezione in corso...");
    for (i = 0; i < n; i++) {                       /*per ogni numero*/
        imin = i;                                   /*indice minimo da i <2>*/
        for (j = i+1; j < n; j++) {
            if (d[j] < d[imin]) {                   /*ricorda indice minimo*/
                imin = j;
            }
        }
        temp = d[i];                                /*scambia col minimo <3>*/
        d[i] = d[imin];
        d[imin] = temp;
    }
}

/* Note su Selezione:
    <1> A partire dal primo elemento dell'array, viene selezionato l'elemento
        di valore pi basso e viene messo al primo posto, scambiandolo con
        quello che stava al primo posto; poi si avanza i al secondo elemento,
        si cerca il nuovo minimo e si scambia col secondo, eccetera. In
        pratica, si selezionano gli elementi uno alla volta in ordine
        crescente, e si mettono a partire dall'inizio dell'array.
    <2> imin  l'indice dell'elemento di valore pi basso tra quelli ancora
        da ordinare. Per cercare l'elemento di valore pi basso, si suppone
        che sia il primo (imin = i). Se poi se ne trova uno pi basso, si
        cambia imin in modo che contenga l'indice dell'elemento pi basso.
    <3> per lo scambio occorre una variabile intermedia: il C non ha
        istruzioni per scambiare due variabili, ed una funzione sarebbe
        troppo inefficiente.
*/


/********************************************************************
 * Inserzione: riordina per inserzione l'array d[] di n elementi.
 ********************************************************************/

void Inserzione(double d[], int n)    /*<1>*/
{
    int    i,j;
    double z;

    Jputs("Ordinamento per inserzione in corso...");
    for (i = 1; i < n; i++) {
        z = d[i];                                   /*num. da inserire <2>*/
        j = i;                                      /*cerca all'indietro*/
        while (j > 0 && d[j-1] > z) {               /*<3>*/
            d[j] = d[j-1];                          /*fa posto <4>*/
            j--;
        }
        d[j] = z;                                   /*inserisce*/
    }
}

/* Note su Inserzione:
    <1> Ogni elemento viene via via inserito al giusto posto tra quelli gi
        ordinati. All'inizio si considera ordinato solo il primo elemento,
        quindi si parte dal secondo. Ciascun elemento viene inserito come
        quando si ordina una mano di carte da gioco.
    <2> z  il valore da inserire al posto giusto tra i precedenti.
    <3> Torna indietro tra gli elementi gi ordinati finch non ne trova
        uno minore o uguale a z. z pu quindi essere inserito subito dopo
        l'elemento trovato. Notare che se la prima condizione (j > 0) non 
        vera, la seconda non viene nemmeno provata, evitando cos di accedere
        ad un elemento inesistente dell'array d[].
    <4> Mentre va indietro, sposta avanti gli elementi sfruttando il posto
        in cui c'era z. In questo modo, quando trova il posto giusto deve
        solo scriverci z (il contenuto originale  gi stato spostato avanti).
*/


/********************************************************************
 * BubbleSort: riordina per scambi l'array d[] di n elementi.
 ********************************************************************/

void BubbleSort(double d[], int n)    /*<1>*/
{
    int    i,scambi;
    double temp;

    Jputs("Bubble sort in corso...");
    do {
        scambi = 0;
        for (i = 1; i < n; i++) {                   /*scandisce coppie*/
            if (d[i] < d[i-1]) {
                temp = d[i];
                d[i] = d[i-1];
                d[i-1] = temp;
                scambi = 1;                         /*e segna <2>*/
            }
        }
    } while (scambi);                               /*esce se no scambi*/
}

/* Note su BubbleSort:
    <1> A partire dalla prima coppia di elementi, l'array viene scandito
        confrontando coppie di elementi adiacenti. Se sono nell'ordine
        sbagliato, vengono scambiati tra loro. Si ripete il procedimento
        finch in una passata non viene eseguito alcuno scambio perch gli
        elementi sono tutti in ordine.
    <2> Il flag (variabile logica) scambi serve per segnare che  stato fatto
        almeno uno scambio, e quindi si deve fare un'altra passata.
*/


/********************************************************************
 * ShellSort: riordina per Shell sort l'array d[] di n elementi.
 ********************************************************************/

void ShellSort(double d[], int n)    /*<1>*/
{
    int    dist;   /*distanza tra gli elementi da ordinare in una passata*/
    int    i,j;
    double z;

    Jputs("Shell sort in corso...");
    for (dist = 1; dist < n; dist = dist*3+1)       /*distanza iniziale <2>*/
        ;

    while ( (dist = dist/3) > 0) {                  /*fino a distanza 1*/
        for (i = dist; i < n; i++) {                /*<3>*/
            z = d[i];                               /*num. da inserire*/
            j = i;                                  /*cerca all'indietro*/
            while (j-dist >= 0 && d[j-dist] > z) {
                d[j] = d[j-dist];                   /*fa posto*/
                j -= dist;
            }
            d[j] = z;                               /*inserisce*/
        }
    }
}

/* Note su ShellSort:
    <1> Viene eseguito un sort per inserzione considerando solo un elemento
        ogni dist elementi, partendo con dist abbastanza grande. Questo
        sposta velocemente verso gli estremi gli elementi che hanno molta
        strada da fare per raggiungere il proprio posto. dist viene poi
        ridotta gradualmente fino ad 1, cio a un sort per inserzione puro,
        che per  velocissimo perch gli elementi sono gi quasi a posto.
    <2> Una sequenza particolarmente efficace per dist  (a rovescio) la
        serie 1, 4, 13, 40, 121, ... (moltiplica per tre e aggiunge uno).
        Notare che tutto il lavoro  fatto nella for stessa: l'istruzione
        controllata  un'istruzione nulla.
    <3> Il ciclo interno  un normale sort per inserzione, solo che viene
        considerato solo un elemento ogni dist.
*/


/********************************************************************
 * Quicksort: riordina con Quicksort l'array d[] di n elementi.
 ********************************************************************/

void Quicksort(double d[], int n)
{
    Jputs("Quicksort in corso...");
    qsort(d,n,sizeof(d[0]),Confronta);
}


/***** Confronta: confronta due elementi, per Quicksort. *****/

int Confronta(double *d1, double *d2)
{
    if (*d1 > *d2)  return 1;                       /*<1>*/
    else if (*d1 < *d2)  return -1;
    else return 0;
}

/* Note su Quicksort:
    <1> Non si pu fare semplicemente *d1-*d2, perch il valore ritornato
        dalla funzione dev'essere un int: si perderebbe la parte frazionaria
        e numeri come 12.47 e 12.34 (diversi solo nella parte frazionaria)
        sarebbero considerati uguali. In questo modo, tra l'altro,
        l'esecuzione  pi veloce. I tre return sono accettabili visto che le
        esigue dimensioni della funzione non creano problemi di leggibilit.
*/


/********************************************************************
 * PremiReturn: aspetta tasto Return.
 ********************************************************************/

void PremiReturn(void)
{
    Jclrkey();
    Jputs("\r\nPremi Return per continuare: ");
    while (Jgetkey(1) != '\r') {
        Jputch('\a');
    }
}
