/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* RIMBALZA: controllo di un pixel in pagina grafica */


/* NOTA: date le numerose differenze nell'uso della grafica tra il Turbo C
         2.0 ed il Quick C 2.0, abbiamo preferito scrivere due programmi
         separati. La #ifdef che segue decide quale dei due sar compilato,
         in base al compilatore impiegato.
*/


/*========================= Versione per Turbo C 2.0 ======================*/

#ifdef __TURBOC__


#include <stdlib.h>
#include <conio.h>        /*<1>*/
#include <graphics.h>     /*<2>*/


/* prototipi funzioni: */

void Rimbalza(void);


/********************************************************************
 * main
 ********************************************************************/

main()
{
    int driver = DETECT;                            /*sceglie da s <3>*/
    int modo;

    initgraph(&driver,&modo,NULL);                  /*avvia grafica <4>*/
    if (driver < 0) {                               /*controlla se ok*/
        cputs("\r\n- Non sono in grado di usare"
              " la grafica di questo computer.");   /*esce se non pu <5>*/
        exit(1);
    }

    Rimbalza();                                     /*disegna*/

    closegraph();                                   /*torna in modo testo*/
}

/* Note su main:
    <1> Usiamo la libreria di console del compilatore (conio), dato che
        cputs, getch e kbhit (le uniche funzioni usate) non pongono problemi
        di portatilit, almeno tra Turbo C 2.0 e Quick C 2.0. Tra l'altro,
        questa versione del programma funziona con il solo Turbo C 2.0, a
        causa delle chiamate alle librerie grafiche.
    <2> Non basta includere lo header, bisogna anche avvisare il linker del
        Turbo C 2.0, con l'apposita scelta da menu, di cercare le funzioni
        chiamate non solo nella libreria standard ma anche in quella grafica.
    <3> Indicando DETECT, la initgraph sceglier il modo grafico con la
        maggiore definizione tra quelli disponibili.
    <4> NULL indica di cercare il driver grafico necessario per usare la
        scheda disponibile (file .BGI) nella directory corrente.
    <5> In questa uscita anticipata dal programma, non c' bisogno di tornare
        in modo testo, visto che se arriva qui non  riuscito ad avviare la
        grafica. Il valore di driver indica la causa del fallimento (es.
        mancanza del file .BGI adeguato), e pu essere usato per fornire
        informazioni pi dettagliate all'utente.
*/


/********************************************************************
 * Rimbalza: fa rimbalzare un pixel lasciando una traccia, cambia
 * colore ogni 50 pixel, esce quando si preme un tasto.
 ********************************************************************/

void Rimbalza(void)
{
    int maxx,maxy;     /*limiti pixel orizzontale, verticale*/
    int maxcol;        /*massimo numero di colore (bianco)*/
    int x,y;           /*posizione corrente*/
    int col;           /*colore corrente*/
    int dx,dy;         /*direzione corrente (+1/-1)*/
    int n;

    maxx = getmaxx();                               /*legge limiti video <1>*/
    maxy = getmaxy();
    maxcol = getmaxcolor();                         /*e limite colori <2>*/

    x = y = 0;                                      /*angolo sup. sinistro*/
    dx = dy = 1;                                    /*verso destra, basso*/
    col = maxcol;                                   /*inizia in bianco*/
    n = 0;                                          /*per cambio colore*/
    while (kbhit())  getch();                       /*pulisce buffer <3>*/

    while (! kbhit()) {                             /*esce con un tasto*/
        putpixel(x,y,col);                          /*colora pixel <4>*/
        if (x == maxx && dx == 1)  dx = -1;         /*bordo destro <5>*/
        else if (x == 0 && dx == -1)  dx = 1;       /*bordo sinistro*/
        else if (y == maxy && dy == 1)  dy = -1;    /*bordo inferiore*/
        else if (y == 0 && dy == -1)  dy = 1;       /*bordo superiore*/
        x += dx;                                    /*nuova posizione*/
        y += dy;
        if (++n >= 50) {                            /*cambia colore <6>*/
            if (++col > maxcol)  col = 0;
            n = 0;
        }
    }
    getch();                                        /*ripulisce <7>*/
}

/* Note su Rimbalza:
    <1> Le coordinate valide per i pixel vanno da 0 a maxx e da 0 a maxy.
    <2> I numeri validi per i colori vanno da 0 a maxcol.
    <3> Dato che il programma si fermer premendo un tasto, ci assicuriamo
        che non ci sia gi un carattere in attesa nel buffer.
    <4> Accende il pixel alle coordinate x,y al colore col.
    <5> Il rimbalzo contro il bordo viene effettuato invertendo la direzione
        quando 'va a sbattere' contro un lato del video.
    <6> Cambiando questo limite, ad esempio con maxy+1, si ottengono altri
        effetti interessanti.
    <7> Il carattere che ha causato la fine del ciclo  ancora in attesa
        nel buffer. Meglio toglierlo, altrimenti  come se venisse battuto
        quando il programma termina.
*/



/*========================= Versione per Quick C 2.0 ======================*/

#else


#include <stdlib.h>
#include <conio.h>        /*<1>*/
#include <graph.h>        /*<2>*/


/* prototipi funzioni: */

void Rimbalza(void);


/********************************************************************
 * main
 ********************************************************************/

main()
{
    int modi[] = {          /*lista dei modi in ordine di preferenza <3>*/
        _VRES16COLOR,       /*VGA 640 x 480*/
        _ERESCOLOR,         /*EGA colore 640 x 350*/
        _ERESNOCOLOR,       /*EGA monocromatica 640 x 350*/
        _ORESCOLOR,         /*Olivetti 640 x 400 <4>*/
        _HERCMONO,          /*Hercules 720 x 348 <5>*/
        _HRESBW,            /*CGA 640 x 400*/
        _DEFAULTMODE        /*segna fine lista <6>*/
    };
    int i;

    i = 0;                                          /*avvia grafica <7>*/
    while (modi[i] != _DEFAULTMODE
      && _setvideomode(modi[i]) == 0) {
        i++;
    }

    if (modi[i] == _DEFAULTMODE) {                  /*controlla se ok*/
        cputs("\r\n- Non sono in grado di usare"
              " la grafica di questo computer.");   /*esce se non pu <8>*/
        exit(1);
    }

    Rimbalza();                                     /*disegna*/

    _setvideomode(_DEFAULTMODE);                    /*torna in modo testo*/
}

/* Note su main:
    <1> Usiamo la libreria di console del compilatore (conio), dato che
        cputs, getch e kbhit (le uniche funzioni usate) non pongono problemi
        di portatilit, almeno tra Turbo C 2.0 e Quick C 2.0. Tra l'altro,
        questa versione del programma funziona con il solo Quick C 2.0, a
        causa delle chiamate alle librerie grafiche.
    <2> Non basta includere lo header, bisogna anche installare la libreria
        grafica nella libreria standard, usando il programma SETUP fornito
        con il Quick C 2.0, in modo che il linker possa trovarle.
    <3> Questa tecnica, che funziona col Quick C 2.0 ma non col Turbo C 2.0,
        consente di scegliere il modo preferito in base ad una lista che
        elenca i modi in ordine di preferenza.
    <4> Per quanto sia definito come COLOR, questo modo avvia in realt
        correttamente anche il modo monocromatico Olivetti 640 x 400, es. M24.
    <5> Il modo Hercules funziona solo se si  prima eseguito l'apposito
        programma MSHERC.COM.
    <6> Usiamo _DEFAULTMODE per terminare la lista, dato che  un modo che
        non chiamiamo (serve per tornare al modo testo originale).
    <7> Prova tutti i modi nell'ordine indicato dall'array modi[],
        verificando ogni volta se il passaggio in grafica  riuscito. Questa
        tecnica non funziona col Turbo C 2.0, perch la relativa funzione
        (initgraph) non ritorna alcuna informazione utile per sapere se il
        modo richiesto era possibile o meno in base alla scheda installata.
    <8> In questa uscita anticipata dal programma, non c' bisogno di tornare
        in modo testo, visto che se arriva qui non  riuscito ad avviare la
        grafica.
*/


/********************************************************************
 * Rimbalza: fa rimbalzare un pixel lasciando una traccia, cambia
 * colore ogni 50 pixel, esce quando si preme un tasto.
 ********************************************************************/

void Rimbalza(void)
{
    struct videoconfig v;   /*dati del modo video corrente <1>*/
    int maxx,maxy;          /*limiti pixel orizzontale, verticale*/
    int maxcol;             /*massimo numero di colore (bianco)*/
    int x,y;                /*posizione corrente*/
    int col;                /*colore corrente*/
    int dx,dy;              /*direzione corrente (+1/-1)*/
    int n;

    _getvideoconfig(&v);                   /*legge dati modo video: <2>*/
    maxx = v.numxpixels-1;                 /*limiti video <3>*/
    maxy = v.numypixels-1;
    maxcol = v.numcolors-1;                /*e limite colori <4>*/

    x = y = 0;                                      /*angolo sup. sinistro*/
    dx = dy = 1;                                    /*verso destra, basso*/
    col = maxcol;                                   /*inizia in bianco*/
    _setcolor(col);
    n = 0;                                          /*per cambio colore*/
    while (kbhit())  getch();                       /*pulisce buffer <5>*/

    while (! kbhit()) {                             /*esce con un tasto*/
        _setpixel(x,y);                             /*colora pixel <6>*/
        if (x == maxx && dx == 1)  dx = -1;         /*bordo destro <7>*/
        else if (x == 0 && dx == -1)  dx = 1;       /*bordo sinistro*/
        else if (y == maxy && dy == 1)  dy = -1;    /*bordo inferiore*/
        else if (y == 0 && dy == -1)  dy = 1;       /*bordo superiore*/
        x += dx;                                    /*nuova posizione*/
        y += dy;
        if (++n >= 50) {                            /*cambia colore <8>*/
            if (++col > maxcol)  col = 0;
            _setcolor(col);
            n = 0;
        }
    }
    getch();                                        /*ripulisce <9>*/
}

/* Note su Rimbalza:
    <1> Una variabile di tipo struct videoconfig viene usata dalla
        _getvideoconfig per ritornare informazioni sul modo corrente.
    <2> La _getvideoconfig scrive nella struttura indicata tutte le
        informazioni utili sul modo video corrente.
    <3> Le coordinate valide per i pixel vanno da 0 a maxx e da 0 a maxy.
    <4> I numeri validi per i colori vanno da 0 a maxcol.
    <5> Dato che il programma si fermer premendo un tasto, ci assicuriamo
        che non ci sia gi un carattere in attesa nel buffer.
    <6> Accende il pixel alle coordinate x,y al colore corrente, scelto in
        precedenza con la _setcolor.
    <7> Il rimbalzo contro il bordo viene effettuato invertendo la direzione
        quando 'va a sbattere' contro un lato del video.
    <8> Cambiando questo limite, ad esempio con maxy+1, si ottengono altri
        effetti interessanti.
    <9> Il carattere che ha causato la fine del ciclo  ancora in attesa
        nel buffer. Meglio toglierlo, altrimenti  come se venisse battuto
        quando il programma termina.
*/


#endif                                      /*fine scelta Turbo C/Quick C*/
