/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* SPIRALI: disegna spirali poligonali casuali */


/* NOTA: anche questo file, come RIMBALZA.C, contiene due programmi distinti,
         il primo scritto con le funzioni grafiche del Turbo C 2.0, il secondo
         con quelle del Quick C 2.0. La scelta  automatica con una #ifdef.

         Questo sistema comincia a diventare troppo scomodo: nella lezione
         E5  mostrato come costruire una 'libreria' di funzioni grafiche
         portatili, simile alla Jconio usata per le funzioni di console.
*/


/*========================= Versione per Turbo C 2.0 ======================*/

#ifdef __TURBOC__


#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <conio.h>
#include <graphics.h>


/* costanti e variabili globali */

#define PAUSA  3   /*3 secondi di pausa tra le spirali*/

int    maxcol;     /*limite colori*/
int    centroX;    /*coordinate del centro schermo*/
int    centroY;
double quadra;     /*fattore di quadratura*/
double pi;         /*pigreco*/
double grad;       /*costante di conversione gradi/radianti;


/* prototipi funzioni: */

void Spirale(void);


/********************************************************************
 * main
 ********************************************************************/

main()
{
    int    driver = DETECT;     /*sceglie da s*/
    int    modo;
    int    dimx,dimy;           /*dimensioni video (numero pixel)*/
    time_t t;

    initgraph(&driver,&modo,NULL);                  /*avvia grafica <1>*/
    if (driver < 0) {                               /*controlla se ok*/
        cputs("\r\n- Non sono in grado di usare"
              " la grafica di questo computer.");   /*esce se non pu*/
        exit(1);
    }

    maxcol = getmaxcolor();                         /*legge limite colori*/
    dimx = getmaxx()+1;                             /*e dimensioni video*/
    dimy = getmaxy()+1;
    centroX = dimx/2;                               /*calcola centro <2>*/
    centroY = dimy/2;
    quadra = (dimx/(double)dimy)/(4.0/3.0);         /*quadratura <3>*/
    pi = 4.0*atan(1.0);                             /*pigreco <4>*/
    grad = pi/180.0;                                /*costante convers. <5>*/

    srand((unsigned int)time(NULL));                /*avvia random*/
    while (kbhit())  getch();                       /*pulisce buffer*/

    while (! kbhit()) {                             /*esce con un tasto*/
        Spirale();                                  /*disegna una spirale*/
        t = time(NULL)+PAUSA;                       /*ritardo o tasto <6>*/
        while (! kbhit() && time(NULL) < t)
            ;
    }
    if (kbhit())  getch();                          /*ripulisce ev. tasto*/

    closegraph();                                   /*torna in modo testo*/
}

/* Note su main:
    <1> Il passaggio in modo grafico  identico a quello di RIMBALZA.
    <2> Dato che il numero di pixel  pari, non esiste un centro esatto.
    <3> Il fattore di quadratura sar usato per correggere la dimensione
        orizzontale in modo da far apparire le figure con le medesime
        proporzioni sia in verticale che in orizzontale, indipendentemente
        dalla forma dei pixel su un dato computer. Il cast con (double) 
        necessario per evitare che dimx/dimy diventi una divisione intera
        con perdita del resto.
    <4> Sfruttando il fatto che 45 a /4 e la tangente di 45  1.0, si
        pu ricavare il valore di pigreco senza doverlo scrivere.
    <5> E' comodo disporre di una costante per convertire tra gradi
        sessagesimali (un giro = 360) e radianti (un giro = 2).
    <6> L'attesa del tempo stabilito (PAUSA secondi) viene interrotta se
        l'utente preme un tasto per terminare il programma. Volendo una
        maggiore precisione, si dovrebbe usare clock invece di time.
*/


/********************************************************************
 * Spirale: disegna una spirale poligonale casuale.
 ********************************************************************/

void Spirale(void)   /*<1>*/
{
    double r;          /*raggio*/
    double ia;         /*incremento angolo*/
    double ir;         /*incremento raggio*/
    double rlim;       /*limite raggio*/
    double s,c;        /*seno e coseno correnti <2>*/
    double cp;         /*coseno precedente*/
    double isin,icos;  /*incremento seno e coseno*/
    int    x,y;

    rlim = centroY*2.5;                             /*fissa limite*/
    ia = (double)((rand()%3600)/10.0);              /*estrae incrementi <3>*/
    ir = 1.0+(double)((rand()%200)/100.0);
    isin = sin(ia*grad);
    icos = cos(ia*grad);
    r = 0.0;                                        /*parte dal centro*/
    s = sin(0.0);                                   /*con angolo 0 gradi*/
    c = cos(0.0);

    setcolor(1+rand()%maxcol);                      /*e colore, non nero <4>*/
    cleardevice();                                  /*cancella schermo*/
    moveto(centroX,centroY);                        /*va al centro*/
    while (r < rlim) {
        cp = c;                                     /*copia coseno*/
        c = c*icos-s*isin;                          /*nuovo coseno <5>*/
        s = s*icos+cp*isin;                         /*nuovo seno*/
        x = centroX+(int)(floor(r*c*quadra+0.5));   /*nuove coordinate <6>*/
        y = centroY+(int)(floor(r*s+0.5));
        lineto(x,y);                                /*traccia linea*/
        r += ir;                                    /*incrementa raggio*/
    }
}

/* Note su Spirale:
    <1> Una spirale poligonale si ottiene ruotando a velocit costante
        attorno al centro ed aumentando progressivamente il raggio. Ad
        intervalli regolari (in termini di angolo) si fissa un punto e lo
        si collega con un segmento rettilineo al punto precedente.
    <2> La funzione Spirale usa un trucco per non dover chiamare due funzioni
        trigonometriche relativamente lente (seno e coseno) ad ogni giro.
        E' possibile calcolare una sola volta, all'inizio, le due variabili
        isin ed icos, ed usarle con semplici moltiplicazioni per calcolare
        la posizione successiva lungo una circonferenza.
    <3> La rotazione si ottiene aumentando ad ogni giro l'angolo corrente
        di un certo valore (in realt usiamo il trucco descritto in nota 2).
        Eseguendo rand()%3600 si ottiene un valore tra 0 e 3599, e
        dividendolo per 10.0 si suddivide l'angolo giro in decimi di grado:
        il valore di ia risulta quindi compreso tra 0.0 e 359.9 gradi. Il
        valore di ir (incremento raggio) sar invece compreso tra 1.0 e 2.99.
    <4> Estrae un colore tra quelli disponibili, escluso lo zero. Nel caso
        di un monitor monocromatico, estrae quindi sempre il bianco.
    <5> Questa  la scorciatoia per evitare di calcolare ad ogni giro seno e
        coseno dell'angolo corrente. Non approfondiamo la tecnica
        trigonometrica usata.
    <6> La coordinata orizzontale viene moltiplicata per quadra (il fattore
        di quadratura) per assicurare proporzioni 'quadrate'. Usiamo il
        solito sistema per approssimare un double all'intero pi vicino:
        aggiungere 0.5 ed approssimare con floor all'intero inferiore. Il
        cast con (int) non  strettamente necessario, ma mostra senza
        possibilit di dubbio che viene eseguito un troncamento della parte
        decimale passando da un double ad un int (la parte decimale era
        comunque gi zero per effetto della floor).
*/



/*========================= Versione per Quick C 2.0 ======================*/

#else


#include <stdlib.h>
#include <stddef.h>     /*per definire NULL*/
#include <time.h>
#include <math.h>
#include <conio.h>
#include <graph.h>


/* costanti e variabili globali */

#define PAUSA  3   /*3 secondi di pausa tra le spirali*/

int    maxcol;     /*limite colori*/
int    centroX;    /*coordinate del centro schermo*/
int    centroY;
double quadra;     /*fattore di quadratura*/
double pi;         /*pigreco*/
double grad;       /*costante di conversione gradi/radianti;


/* prototipi funzioni: */

void Spirale(void);


/********************************************************************
 * main
 ********************************************************************/

main()
{
    int modi[] = {          /*lista dei modi in ordine di preferenza*/
        _VRES16COLOR,       /*VGA 640 x 480*/
        _ERESCOLOR,         /*EGA colore 640 x 350*/
        _ERESNOCOLOR,       /*EGA monocromatica 640 x 350*/
        _ORESCOLOR,         /*Olivetti 640 x 400*/
        _HERCMONO,          /*Hercules 720 x 348*/
        _HRESBW,            /*CGA 640 x 400*/
        _DEFAULTMODE        /*segna fine lista*/
    };
    struct videoconfig v;   /*dati del modo video corrente*/
    int    dimx,dimy;       /*dimensioni video (numero pixel)*/
    time_t t;
    int    i;

    i = 0;                                          /*avvia grafica <1>*/
    while (modi[i] != _DEFAULTMODE
      && _setvideomode(modi[i]) == 0) {
        i++;
    }

    if (modi[i] == _DEFAULTMODE) {                  /*controlla se ok*/
        cputs("\r\n- Non sono in grado di usare"
              " la grafica di questo computer.");   /*esce se non pu*/
        exit(1);
    }

    _getvideoconfig(&v);                            /*legge dati modo video:*/
    maxcol = v.numcolors;                           /*legge limite colori*/
    dimx = v.numxpixels;                            /*e dimensioni video*/
    dimy = v.numypixels;
    centroX = dimx/2;                               /*calcola centro <2>*/
    centroY = dimy/2;
    quadra = (dimx/(double)dimy)/(4.0/3.0);         /*quadratura <3>*/
    pi = 4.0*atan(1.0);                             /*pigreco <4>*/
    grad = pi/180.0;                                /*costante convers. <5>*/

    srand((unsigned int)time(NULL));                /*avvia random*/
    while (kbhit())  getch();                       /*pulisce buffer*/

    while (! kbhit()) {                             /*esce con un tasto*/
        Spirale();                                  /*disegna una spirale*/
        t = time(NULL)+PAUSA;                       /*ritardo o tasto <6>*/
        while (! kbhit() && time(NULL) < t)
            ;
    }
    if (kbhit())  getch();                          /*ripulisce ev. tasto*/

    _setvideomode(_DEFAULTMODE);                    /*torna in modo testo*/
}

/* Note su main:
    <1> Il passaggio in modo grafico  identico a quello di RIMBALZA.
    <2> Dato che il numero di pixel  pari, non esiste un centro esatto.
    <3> Il fattore di quadratura sar usato per correggere la dimensione
        orizzontale in modo da far apparire le figure con le medesime
        proporzioni sia in verticale che in orizzontale, indipendentemente
        dalla forma dei pixel su un dato computer. Il cast con (double) 
        necessario per evitare che dimx/dimy diventi una divisione intera
        con perdita del resto.
    <4> Sfruttando il fatto che 45 a /4 e la tangente di 45  1.0, si
        pu ricavare il valore di pigreco senza doverlo scrivere.
    <5> E' comodo disporre di una costante per convertire tra gradi
        sessagesimali (un giro = 360) e radianti (un giro = 2).
    <6> L'attesa del tempo stabilito (PAUSA secondi) viene interrotta se
        l'utente preme un tasto per terminare il programma. Volendo una
        maggiore precisione, si dovrebbe usare clock invece di time.
*/


/********************************************************************
 * Spirale: disegna una spirale poligonale casuale.
 ********************************************************************/

void Spirale(void)   /*<1>*/
{
    double r;          /*raggio*/
    double ia;         /*incremento angolo*/
    double ir;         /*incremento raggio*/
    double rlim;       /*limite raggio*/
    double s,c;        /*seno e coseno correnti <2>*/
    double cp;         /*coseno precedente*/
    double isin,icos;  /*incremento seno e coseno*/
    int    x,y;

    rlim = centroY*2.5;                             /*fissa limite*/
    ia = (double)((rand()%3600)/10.0);              /*estrae incrementi <3>*/
    ir = 1.0+(double)((rand()%200)/100.0);
    isin = sin(ia*grad);
    icos = cos(ia*grad);
    r = 0.0;                                        /*parte dal centro*/
    s = sin(0.0);                                   /*con angolo 0 gradi*/
    c = cos(0.0);

    _setcolor(1+rand()%maxcol);                     /*e colore, non nero <4>*/
    _clearscreen(_GCLEARSCREEN);                    /*cancella schermo*/
    _moveto(centroX,centroY);                       /*va al centro*/
    while (r < rlim) {
        cp = c;                                     /*copia coseno*/
        c = c*icos-s*isin;                          /*nuovo coseno <5>*/
        s = s*icos+cp*isin;                         /*nuovo seno*/
        x = centroX+(int)(floor(r*c*quadra+0.5));   /*nuove coordinate <6>*/
        y = centroY+(int)(floor(r*s+0.5));
        _lineto(x,y);                               /*traccia linea*/
        r += ir;                                    /*incrementa raggio*/
    }
}

/* Note su Spirale:
    <1> Una spirale poligonale si ottiene ruotando a velocit costante
        attorno al centro ed aumentando progressivamente il raggio. Ad
        intervalli regolari (in termini di angolo) si fissa un punto e lo
        si collega con un segmento rettilineo al punto precedente.
    <2> La funzione Spirale usa un trucco per non dover chiamare due funzioni
        trigonometriche relativamente lente (seno e coseno) ad ogni giro.
        E' possibile calcolare una sola volta, all'inizio, le due variabili
        isin ed icos, ed usarle con semplici moltiplicazioni per calcolare
        la posizione successiva lungo la circonferenza.
    <3> La rotazione si ottiene aumentando ad ogni giro l'angolo corrente
        di un certo valore (in realt usiamo il trucco descritto in nota 2).
        Eseguendo rand()%3600 si ottiene un valore tra 0 e 3599, e
        dividendolo per 10.0 si suddivide l'angolo giro in decimi di grado:
        il valore di ia risulta quindi compreso tra 0.0 e 359.9 gradi. Il
        valore di ir (incremento raggio) sar invece compreso tra 1.0 e 2.99.
    <4> Estrae un colore tra quelli disponibili, escluso lo zero. Nel caso
        di un monitor monocromatico, estrae quindi sempre il bianco.
    <5> Questa  la scorciatoia per evitare di calcolare ad ogni giro seno e
        coseno dell'angolo corrente. Non approfondiamo la tecnica
        trigonometrica usata.
    <6> La coordinata orizzontale viene moltiplicata per quadra (il fattore
        di quadratura) per assicurare proporzioni 'quadrate'. Usiamo il
        solito sistema per approssimare un double all'intero pi vicino:
        aggiungere 0.5 ed approssimare con floor all'intero inferiore. Il
        cast con (int) non  strettamente necessario, ma mostra senza
        possibilit di dubbio che viene eseguito un troncamento della parte
        decimale passando da un double ad un int (la parte decimale era
        comunque gi zero per effetto della floor).
*/


#endif                                      /*fine scelta Turbo C/Quick C*/
