/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* AGEDATA: funzioni di data/tempo per AGENDA */


#include <stdio.h>
#include <time.h>

#include "agenda.h"


/********************************************************************
 * LeggeData: legge data odierna in *pd, calcola se bisestile.
 ********************************************************************/

void LeggeData(Data *pd)
{
    time_t    t;     /*tempo, in formato interno*/
    struct tm *tp;   /*e come pointer a struttura*/

    t = time(NULL);                                 /*legge data*/
    tp = localtime(&t);                             /*converte*/
    pd->anno = tp->tm_year+1900;                    /*ricava dati <1>*/
    pd->giorno = tp->tm_yday;
    pd->mese = tp->tm_mon;
    pd->gmese = tp->tm_mday;
    pd->gsett = (tp->tm_wday+6)%7;                  /*0 = luned*/
    pd->bisest = (pd->anno%4 == 0);                 /*calcola se bisestile*/
    pd->ultimo = (pd->bisest) ? 365 : 364;          /*ultimo giorno valido*/
}

/* Note su LeggeData:
    <1> Avremmo potuto usare direttamente delle struct tm nel programma, ma
        abbiamo preferito definire la struttura Data, progettata su misura
        per le necessit di AGENDA.
*/


/********************************************************************
 * RicavaData: calcola mese, gmese, gsett da giorno in data *dp,
 *             ritorna zero se giorno non valido.
 ********************************************************************/

int RicavaData(Data *pd)   /*<1>*/
{
    int mese;        /*temporanee per lavoro pi veloce*/
    int gsett;
    int g;
    int ok,b,n;

    mese = 0;
    g = pd->giorno;

    ok = (g >= 0 && g <= pd->ultimo);               /*guarda se valido <2>*/
    if (ok) {
        while (g+1 > mesi[mese].giorni) {           /*determina mese <3>*/
            g -= mesi[mese].giorni;
            mese++;
        }

        n = pd->anno-1988;                          /*anni da bisestile <4>*/
        b = (n+3)/4;                                /*anni bisest trascorsi*/
        gsett = 4;                                  /*1.o Gen 1988: Venerd*/
        gsett += (n*365+b);                         /*1.o Gen corrente*/
        gsett = (gsett+pd->giorno)%7;               /*giorno selezionato*/

        pd->mese = mese;                            /*risultati <5>*/
        pd->gmese = g+1;
        pd->gsett = gsett;
    }

    return ok;                                      /*zero se non valido*/
}

/* Note su RicavaData:
    <1> AGENDA lavora basandosi sempre sul giorno progressivo dall'inizio
        dell'anno: questa funzione provvede a ricavare gli altri dati.
        Basarsi su un solo dato riduce le possibili cause di errori.
    <2> Se il giorno corrente non  valido, RicavaData ritorna zero.
    <3> Determina il mese corrente avanzando dall'inizio dell'anno. In g
        rimane il giorno del mese (riferito a zero, bisogner aggiungere 1).
    <4> Per calcolare il giorno della settimana, occorre conoscere cos'era
        il primo giorno dell'anno. Questo viene determinato partendo da una
        data nota (il 1.o Gennaio 1988, anno bisestile), e calcolando i
        giorni trascorsi (modulo 7). Se fosse necessaria una maggiore
        efficienza, si potrebbe aggiungere ad ogni mese della tabella mesi[]
        il giorno della settimana corrispondente al primo del mese.
    <5> Per comodit ed efficienza, i calcoli sono stati eseguiti usando
        variabili semplici invece che le componenti della struttura passata
        per riferimento. Alla fine bisogna ovviamente ricopiare i risultati
        nella struttura.
*/


/********************************************************************
 * SpostaData: sposta data *dp di n giorni, corregge se non valido.
 ********************************************************************/

int SpostaData(Data *pd, int n)
{
    int g;

    g = pd->giorno;                                 /*giorno prog. corrente*/
    g += n;                                         /*sposta avanti/indietro*/
    if (g < 0)  g = 0;                              /*corregge se fuori*/
    else if (g > pd->ultimo)  g = pd->ultimo;
    pd->giorno = g;                                 /*assegna giorno progr.*/
    RicavaData(pd);                                 /*sistema dati restanti*/
}


/********************************************************************
 * CambiaMese: sposta data *dp avanti (1) o indietro (-1) di un mese.
 ********************************************************************/

void CambiaMese(Data *pd, int n)
{
    int mese;       /*nuovo mese*/
    int g;          /*nuovo giorno progressivo*/
    int i;

    mese = pd->mese+n;                              /*nuovo mese <1>*/
    if (mese < 0) {                                 /*corregge se fuori*/
        g = 0;                                      /*1 Gennaio*/
    } else if (mese > 11) {
        g = pd->ultimo;                             /*31 Dicembre*/
    } else {
        if (pd->gmese > mesi[mese].giorni) {        /*se oltre fine mese*/
            pd->gmese = mesi[mese].giorni;          /*rientra*/
        }

        g = 0;                                      /*calcola giorno progr.*/
        for (i = 0; i < mese; i++) {
            g += mesi[i].giorni;                    /*di mese in mese*/
        }
        g += (pd->gmese-1);                         /*pi il giorno del mese*/
    }

    pd->giorno = g;                                 /*giorno calcolato*/
    RicavaData(pd);                                 /*sistema il resto*/
}

/* Note su CambiaMese :
    <1> Per ridurre al minimo le possibilit di errore, anche qui i calcoli
        sono eseguiti sul giorno progressivo, ed alla fine le altre
        componenti della struttura sono sistemate con la RicavaData.
*/
