/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* AGEFILE: funzioni di accesso al file AGENDA */



#include <stdio.h>
#include <string.h>

#include "agenda.h"


/* variabili usate solo in questo modulo (AGEFILE): <1> */

FILE *fage;                 /*pointer al file di agenda*/
int  erfile;                /*vero se errore di accesso al file <2>*/


/* Note sulle dichiarazioni:
    <1> E' inutile dichiarare globali queste variabili, visto che sono
        usate solamente in questo modulo (vedi anche nota 4 di AGENDA.C).
    <2> Per evitare di rendere inutilmente globale questa variabile, abbiamo
        creato un'apposita funzione per consultarla dall'esterno: ErFile.
        In ogni caso, erfile contiene semplicemente un valore logico (zero
        se tutto bene, diverso da zero se errore). Si potrebbe usarla per
        contenere un numero indicante il tipo di errore, ma in AGENDA non
        ce n' particolare bisogno (al massimo, si potrebbe distinguere tra
        errori di lettura e di scrittura).
*/


/********************************************************************
 * ApreFile: apre file di agenda, lo crea se non c', ritorna 1 se ok.
 ********************************************************************/

int ApreFile(anno)
{
    char nomeFage[13];     /*nome del file di agenda <1>*/
    char buf[81];
    int  ok;

    sprintf(nomeFage,"%d.AGE",anno);                /*crea nome file <2>*/
    fage = fopen(nomeFage,"r+b");                   /*apre esistente <3>*/
    ok = (fage != NULL);                            /*verifica se c'era <4>*/
    if (! ok) {
        sprintf(buf,"- Non trovo l'agenda per il"   /*no, chiede se creare*/
                    " %d. Ne creo una nuova? ",anno);
        ok = SiNo(buf);
        if (ok) {                                   /*se s, lo crea <5>*/
            ok = CreaFile(nomeFage);
            if (! ok) {
                Errore("Non ho potuto creare il file di agenda.");
            } else {
                fage = fopen(nomeFage,"r+b");       /*creazione ok, lo apre*/
                ok = (fage != NULL);                /*verifica se tutto ok*/
                if (! ok) {
                    Errore("Non ho potuto aprire il file di agenda.");
                }
            }
        }
    }
    return(ok);                                     /*1=creato e aperto*/
}

/* Note su ApreFile:
    <1> Un nome di file di MS-DOS  composto al massimo da 12 caratteri:
        8 per il nome, 3 per il suffisso, 1 per il punto separatore. Ne
        aggiungiamo 1 per lo zero finale. Anche se non usiamo tutti questi
        caratteri, meglio non correre rischi: in caso di modifiche, 
        facile sbagliarsi. In ogni caso, il nome del file  necessario solo
        in questa funzione: tutte le altre si riferiranno al file con il
        suo pointer (fage).
    <2> Il nome del file  il numero dell'anno pi .AGE (es. 1992.AGE).
    <3> Apre il file in lettura e scrittura binaria.
    <4> Se il file non c'era, chiede se deve crearlo. La variabile ok 
        tenuta costantemente aggiornata, in modo che contenga un valore
        significativo al ritorno dalla funzione (vera se il file c' ed
         stato aperto correttamente).
    <5> Per non suddividere un'operazione tra pi funzioni, la CreaFile
        richiude il file dopo averlo creato. L'apertura per usarlo avviene
        comunque in questa funzione. Ovviamente la CreaFile ha bisogno del
        nome da dare al file che crea.
*/


/********************************************************************
 * CreaFile: crea nuovo file di agenda col nome dato, ritorna 1 se ok.
 ********************************************************************/

int CreaFile(char *nome)
{
    FILE *f;
    char buf[81];
    Nota nn;
    int  ok,i;

    sprintf(buf,"Sto creando il file di agenda %s... ",nome);
    Inverse(1); Messaggio(buf); Inverse(0);

    f = fopen(nome,"wb");                           /*apre in scrittura <1>*/
    ok = (f != NULL);                               /*verifica se ok*/

    if (ok) {
        for (i = 0; i < NRIGHE; i++) {              /*crea nota nulla*/
            memset(nn[i],0,LRIGA+1);                /*piena di zeri <2>*/
        }
        for (i = 0; i < 366 && ! ferror(f); i++) {  /*scrive 366 record <3>*/
            fwrite(nn,sizeof(nn),1,f);              /*di note nulle <4>*/
        }

        ok = ! ferror(f);                           /*controlla se errore*/
        if (fclose(f) == EOF) {                     /*e chiude <5>*/
            ok = 0;
        }
    }

    Messaggio("");                                  /*leva messaggio*/
    return(ok);
}

/* Note su CreaFile:
    <1> Il file viene aperto in scrittura binaria, dato che non serve
        leggerlo. Verr infatti richiuso prima di ritornare.
    <2> Riempie il file di note nulle. Basterebbe uno zero all'inizio di
        ogni riga, ma riempiendo tutta la Nota di zeri si semplifica
        l'eventuale esame del file con un programma di utility (es. ESAMINA).
    <3> Scrive un record per ogni giorno dell'anno. Si potevano anche passare
        i dati dell'anno in corso per decidere se creare o no il record 365
        (che serve solo per gli anni bisestili), ma costa meno crearlo in
        ogni caso: lo spreco  trascurabile. In caso di errore di scrittura,
        termina subito il ciclo.
    <4> Registra le note una di seguito all'altra, con dimensione costante.
        Ciascuna nota occuper quindi uno spazio ben definito, di lunghezza
        pari a sizeof(Nota) byte. Dato che nn  gi un array, non serve
        scrivere &nn:  comunque un indirizzo.
    <5> Il file viene chiuso solo se l'apertura  riuscita, altrimenti f
        vale NULL e non ha senso chiamare la fclose. Dato che la chiusura
        causa la scrittura dei dati non ancora fisicamente trasferiti sul
        disco, si deve controllare che non avvengano errori.
*/


/********************************************************************
 * ChiudeFile: chiude file di agenda, segna ev. errore in erfile.
 ********************************************************************/

void ChiudeFile(void)    /*<1>*/
{
    if (fclose(fage) == EOF) {                      /*chiude*/
        erfile = 1;                                 /*segna ev. errore*/
    }
}

/* Note su ChiudeFile:
    <1> Quest'operazione  svolta da una funzione nonostante la sua
        semplicit, per simmetria con la CreaFile. A differenza di questa,
        un eventuale errore viene segnato in erfile e non ritornato dalla
        funzione.
*/


/********************************************************************
 * LeggeNota: legge nota giorno g in n, segna ev. errore in erfile.
 ********************************************************************/

void LeggeNota(Nota n,int g)
{
    if (fseek(fage,(long)g*sizeof(Nota),SEEK_SET)   /*posiziona <1> */
      || fread(n,sizeof(Nota),1,fage) < 1) {        /*legge <2>*/
        erfile = 1;                                 /*segna ev. errore <3>*/
    }
}

/* Note su LeggeNota:
    <1> Le note sono registrate come blocchi contigui di lunghezza costante,
        quindi per trovare la nota numero g basta posizionarsi nel punto
        del file (g*lunghezza di una nota). Si deve indicare sizeof(Nota)
        e non sizeof(n), perch il tipo Nota  in realt un array, che viene
        passato per riferimento. Quindi la variabile n non  una Nota, ma
        contiene l'indirizzo di una Nota, cio  un pointer alla Nota.
        Essendo un pointer, la sua dimensione (sizeof)  tipicamente 2 (o 4),
        quindi non ha niente a che vedere con la dimensione di una Nota
        (che  102). La fseek ritorna zero se il posizionamento  riuscito.
    <2> Una volta posizionati con fseek sul punto desiderato, si legge con
        una normale fread. Notare l'uso dell'OR (||) per eseguire la fread
        solo se la fseek ha avuto esito positivo.
    <3> Se l'operazione non  riuscita, la variabile erfile diventa vera.
*/


/********************************************************************
 * ScriveNota: registra nota n in giorno g, segna ev. errore in erfile.
 ********************************************************************/

void ScriveNota(Nota n,int g)     /*<1>*/
{
    if (fseek(fage,(long)g*sizeof(Nota),SEEK_SET)   /*posiziona*/
      || fwrite(n,sizeof(Nota),1,fage) < 1) {       /*scrive*/
        erfile = 1;                                 /*segna ev. errore*/
    }
}

/* Note su ScriveNota:
    <1> ScriveNota  perfettamente simmetrica a LeggeNota
*/


/********************************************************************
 * ErFile: ritorna vero se c' stato un errore di accesso.
 ********************************************************************/

int ErFile(void)
{
    return(erfile);                                 /*<1>*/
}

/* Note su ErFile:
    <1> Questa funzione serve per consentire agli altri moduli di consultare
        erfile, senza per questo dover rendere nota la variabile anche dove
        non ce n' bisogno. Pi una variabile resta confinata nella sua zona,
        pi il programma risulta robusto e meno soggetto a problemi.
*/
