/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* AGENDA: agenda settimanale valida un anno (main file) */


#include "agenda.h"
#include "Jconio.h"


/* costanti: <1> */

#define G_PREC   1075       /*freccia a sinistra: giorno precedente*/
#define G_SUCC   1077       /*freccia a destra: giorno successivo*/
#define S_PREC   1072       /*freccia in alto: settimana precedente*/
#define S_SUCC   1080       /*freccia in basso: settimana successiva*/
#define M_PREC   1073       /*PgUp: mese precedente*/
#define M_SUCC   1081       /*PgDn: mese successivo*/
#define ED_NOTE  1059       /*F1: edita note giorno selezionato*/
#define VA_OGGI  1060       /*F2: torna al giorno corrente*/
#define FINE     1064       /*F6: fine lavoro*/


/* variabili globali (definizioni): <2> */

Mese mesi[] = {             /*dati dei mesi, per anno non bisestile <3>*/
    "Gen",31, "Feb",28, "Mar",31, "Apr",30, "Mag",31, "Giu",30,
    "Lug",31, "Ago",31, "Set",30, "Ott",31, "Nov",30, "Dic",31
};

Giorno giorni[] = {         /*dati dei giorni, inclusa posizione video*/
    "Luned",3,3, "Marted",22,3, "Mercoled",41,3, "Gioved",60,3,
    "Venerd",3,12, "Sabato",22,12, "Domenica",41,12
};


/* variabili usate solo in questo modulo (main): <4> */

Data oggi;                  /*dati giorno odierno*/
Data gsel;                  /*giorno correntemente selezionato (0..365)*/


/* Note su dichiarazioni e costanti:
    <1> I codici dei tasti usati come comandi di AGENDA sono usati solo in
        questo modulo, per cui  inutile definirli nello header: meglio
        confinarli qui, dove si usano.
    <2> Qui sono definite le variabili globali dell'intero programma AGENDA,
        dichiarate nello header AGENDA.H.
    <3> Le due tabelle sono usate come costanti, con l'eccezione della
        durata di Febbraio che viene aggiustata all'avviamento del programma,
        a seconda che l'anno sia bisestile o meno.
    <4> Queste variabili sono usate solo in questo modulo; valgono quindi le
        stesse considerazioni fatte alla nota 1 per le costanti. Nella
        lezione 6  introdotta una tecnica per 'nasconderle' anche al linker,
        realizzando cos un isolamento completo (al momento, una variabile
        con lo stesso nome dichiarata in un altro file darebbe errore).
*/


/********************************************************************
 * main
 ********************************************************************/

main()
{
    Jcursor(0);                                     /*spegne cursore*/
    Jclrscr();                                      /*pulisce video*/

    LeggeData(&oggi);                               /*legge data odierna <1>*/

    if (oggi.anno < 1990 || oggi.anno > 1999) {     /*controlla*/
        Jputs("\r\n- La data dell'orologio"
              " interno non  valida:\r\n"
              "  l'anno dev'essere compreso"
              " tra 1990 e 1999.\r\n\n");
    } else {                                        /*ok, lavora*/
        mesi[1].giorni = (oggi.bisest) ? 29 : 28;   /*aggiusta febbraio*/

        DisegnaFinestra(oggi.anno);                 /*prepara schermo*/
        Menu(1);                                    /*menu principale*/
        if (ApreFile(oggi.anno)) {                  /*apre (o crea) file*/
            CicloComandi();                         /*input e esegue comandi*/
            ChiudeFile();                           /*fine, chiude file*/
        }
    }

    if (ErFile()) {                                 /*segnala ev. errore <2>*/
        Errore("Errore di accesso al disco.");
    }
    Jclrscr();                                      /*cancella schermo*/
    Jcursor(1);                                     /*riaccende cursore*/
}

/* Note su main:
    <1> La LeggeData scrive informazioni sulla data corrente nella struttura
        che le viene passata per riferimento (cio come indirizzo).
    <2> La funzione ErFile (nel modulo AGEFILE) ritorna vero se si 
        verificato un errore di lettura o scrittura del file di agenda.
*/


/********************************************************************
 * CicloComandi: attende ed esegue comandi, esce solo se fine lavoro.
 ********************************************************************/

void CicloComandi(void)
{
    int c;

    gsel = oggi;                                    /*seleziona oggi <1>*/
    do {
        MostraSettimana(&gsel);                     /*mostra settimana <2>*/
        if (! ErFile()) {                           /*ok, accetta comando*/
            Jclrkey();                              /*pulisce buffer*/
            c = Domanda("Scegli un comando: ");     /*aspetta un comando*/
        } else {                                    /*errore file, esce*/
            c = FINE;
        }

        switch(c) {                                 /*esegue comando <3>*/

        case G_PREC:                                /*giorno precedente*/
            SpostaData(&gsel,-1);
            break;
        case G_SUCC:                                /*giorno successivo*/
            SpostaData(&gsel,1);
            break;
        case S_PREC:                                /*settimana precedente*/
            SpostaData(&gsel,-7);
            break;
        case S_SUCC:                                /*settimana successiva*/
            SpostaData(&gsel,7);
            break;
        case M_PREC:                                /*mese precedente*/
            CambiaMese(&gsel,-1);
            break;
        case M_SUCC:                                /*mese successivo*/
            CambiaMese(&gsel,1);
            break;
        case ED_NOTE:                               /*edita note giorno*/
            EditNota(&gsel);
            break;
        case VA_OGGI:                               /*torna a data odierna*/
            gsel = oggi;
            break;
        case FINE:                                  /*fine programma*/
            break;                                  /*(non fa nulla) <4>*/
        default:                                    /*comando non valido*/
            Jputch('\a');                           /*segnala errore*/
            break;
        }
    } while (c != FINE);                            /*esce con FINE*/
}

/* Note su CicloComandi:
    <1> Questa assegnazione copia un'intera struttura di tipo Data.
    <2> Per quanto la MostraSettimana vega chiamata ad ogni ciclo, contiene
        un meccanismo per non ridisegnare la settimana corrente se  gi
        visibile sullo schermo.
    <3> Come in tutti i programmi event-driven, tutte le operazioni sono
        comandate da uno switch basato sul comando introdotto dall'utente.
    <4> Il caso FINE  trattato nella condizione di fine ciclo, ma occorre
        comunque mettere questa scelta, altrimenti esegue il default e suona
        un beep per segnalare errore. Dato che le eventuali modifiche vengono
        sempre registrate immediatamente nel file, la scelta FINE pu essere
        eseguita senza rischio di perdere dati (in LIBRI chiedevamo invece
        una conferma se i dati non erano stati registrati).
*/
