/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* AGEUTIL: funzioni di uso generale (utility) per AGENDA <1> */


#include <string.h>
#include <ctype.h>

#include "agenda.h"
#include "Jconio.h"


/* Nota:
    <1> Queste funzioni sono le stesse di LIBRUTIL (lezione 3), con
        l'eccezione di PutNchar che  stata resa pi efficiente.
*/


/********************************************************************
 * Domanda: stampa stringa d[] su linea di stato, torna col primo
 *          carattere premuto.
 ********************************************************************/

int Domanda(char *d)
{
    int c;

    Messaggio(d);                                   /*mostra domanda*/
    Jclrkey();                                      /*pulisce buffer*/
    c = Jgetkey(1);                                 /*aspetta tasto*/
    Messaggio("");                                  /*ripulisce*/
    return c;                                       /*ritorna tasto*/
}


/********************************************************************
 * SiNo: mostra domanda d[] su linea di stato, aspetta 's' o 'n'
 *       (anche maiuscoli), ritorna vero se 's'.
 ********************************************************************/

int SiNo(char *d)
{
    int c,ok;

    do {
		c = Domanda(d);                             /*aspetta risposta*/
		c = tolower(c);
        ok = (c == 's' || c == 'n');                /*controlla*/
        if (! ok) {
            Jputch('\a');                           /*segnala errore*/
        }
    } while (!ok);
    return (c == 's');
}


/********************************************************************
 * Errore: mostra messaggio di errore e[] in inverse, aspetta Return.
 ********************************************************************/

void Errore(char *e)
{
    Inverse(1);                                     /*va in inverse*/
    Messaggio(e);                                   /*mostra messaggio*/
    Jputch('\a');
    Jputs("  Premi Return per continuare: ");
    while (Jgetkey(1) != '\r') {                    /*aspetta Return*/
        Jputch('\a');
    }

    Inverse(0);                                     /*rimette a posto*/
    Messaggio("");
}


/********************************************************************
 * Messaggio: stampa stringa s[] su linea di stato (linea 25), o
 *            cancella linea di stato se s[]  una stringa nulla.
 ********************************************************************/

void Messaggio(char *s)
{
    Jgotoxy(1,25);
    if (*s) {                                       /*non nulla, stampa*/
        Jputs("  ");
        Jputs(s);
    } else {                                        /*nulla, cancella*/
        Inverse(0);                                 /*assicura normale*/
        PutNchar(78,' ');
    }
}


/********************************************************************
 * PutsAt: stampa stringa alla posizione x,y.
 ********************************************************************/

void PutsAt(int x, int y, char *s)    /*<1>*/
{
    Jgotoxy(x,y);                                   /*si posiziona*/
    Jputs(s);                                       /*e stampa*/
}

/* Note su PutsAt
    <1> La funzione non conserva la posizione corrente del cursore.
*/


/********************************************************************
 * PutNchar: stampa n (max 80) caratteri c di seguito.
 ********************************************************************/

void PutNchar(int n, int c)
{
    char buf[81];                                   /*per stampa rapida <1>*/

    if (n > 80)  n = 80;                            /*per sicurezza <2>*/
    memset(buf,c,n);                                /*riempie buffer*/
    buf[n] = 0;                                     /*mette zero finale*/
    Jputs(buf);                                     /*e stampa*/
}

/* Note su PutNchar:
    <1> Dato che la stampa di una stringa con Jputs  molto pi veloce delle
        stampa carattere per carattere con Jputch, viene costruita nel
        buffer buf[] una stringa da stampare in un sol colpo.
    <2> La funzione non dovrebbe mai essere chiamata con n > 80, ma un buon
        controllo risparmia inutili problemi (sarebbe alterata memoria in
        posti imprevedibili: il tipico errore difficile da individuare).
*/


/********************************************************************
 * InputStringa: mostra campo di input con default alla posizione
 *               indicata, input controllato di stringa nell'array
 *               s[] non oltre lmax caratteri, accetta correzioni
 *               con backspace, mette zero finale, ritorna lunghezza
 *               della stringa introdotta.
 ********************************************************************/

int InputStringa(char *s, int lmax, int px, int py)
{
    enum { CR = 13, BS = 8, BACK = 1075 };      /*codici tasti*/

    char buf[81];       /*buffer di linea*/
    int  ls;            /*lunghezza stringa*/
    char *s1;           /*pointer a stringa originaria, per ristampa*/
    int  i,c,ok;

    s1 = s;                                         /*inizio stringa*/

    Jgotoxy(px,py);                                 /*si posiziona*/
    Inverse(1);                                     /*campo in inverse*/
    Jputs(s);                                       /*mostra default*/
    ls = strlen(s);                                 /*lunghezza corrente*/
    for (i = ls; i < lmax; i++) {                   /*evidenzia campo*/
        Jputch(' ');
    }
    Jgotoxy(px+ls,py);                              /*torna a fine default*/

    s += ls;                                        /*punta alla fine*/
    Jclrkey();                                      /*cancella buffer*/
    do {
        c = Jgetkey(1);
        ok = (c == CR);
        if ( (c == BS || c == BACK) && ls > 0) {
            ls--;
            s--;
            Jgotoxy(px+ls,py);
            Jputch(' ');
            Jgotoxy(px+ls,py);
        } else if (isprint(c) && ls < lmax) {
            *s++ = c;
            ls++;
            Jputch(c);
        } else if (! ok) {
            Jputch('\a');
        }
    } while (! ok);

    *s = 0;                                         /*termina stringa*/
    Jgotoxy(px,py);                                 /*torna dov'era*/
    Inverse(0);                                     /*colori normali*/
    Jputs(s1);                                      /*ristampa normale*/
    for (i = ls; i < lmax; i++) {                   /*de-evidenzia campo*/
        Jputch(' ');
    }

    return ls;                                      /*ritorna lunghezza*/
}


/********************************************************************
 * Inverse: Inverte colori se inv  vero, torna normale se falso.
 ********************************************************************/

void Inverse(int inv)
{
    static int inverse = 0;   /*ricorda lo stato corrente*/
    static int nc,nb;         /*colori normali del testo*/

    if (inv && ! inverse) {                         /*se non gi inverse*/
        nc = Jgettextcolor();                       /*ricorda colori*/
        nb = Jgettextback();
        Jtextcolor(nb);                             /*li scambia*/
        Jtextback( (nc > 7) ? nc-8 : nc);           /*max 7 per backgr.*/
    } else if (! inv && inverse) {                  /*se non gi normale*/
        Jtextcolor(nc);                             /*rimette a posto*/
        Jtextback(nb);
    }
    inverse = inv;                                  /*ricorda stato*/
}
