/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* AGEVIDEO: funzioni di disegno note su video per AGENDA */


#include <stdio.h>

#include "agenda.h"
#include "Jconio.h"


/* costanti usate nelle scritte dei menu: */

#define F_SU     "\x18"     /*simbolo freccia in alto*/
#define F_GIU    "\x19"     /*freccia in basso*/
#define F_SIN    "\x1b"     /*freccia a sinistra*/
#define F_DES    "\x1a"     /*freccia a destra*/


/********************************************************************
 * MostraSettimana: mostra settimana contenente il giorno *dp,
 *                  ritorna in caso di errore di lettura.
 ********************************************************************/

void MostraSettimana(Data *dp)
{
    static int primoGiorno = -1;    /*primo giorno sett.<1>*/

    int  primoVis;                  /*primo giorno visibile*/
    int  gs;                        /*giorno della settimana*/
    Nota n;
    Data d;
    char buf[81];

    d = *dp;                                        /*data selezionata*/
    primoVis = d.giorno-d.gsett;                    /*primo visibile <2>*/
    for (gs = 0; gs < 7 && ! ErFile(); gs++) {      /*scandisce settimana*/
        Jgotoxy(giorni[gs].hpos-1,giorni[gs].vpos); /*si posiziona*/
        d.giorno = primoVis+gs;                     /*giorno progressivo*/
        if (RicavaData(&d) == 0) {                  /*ricava altri dati <3>*/
            PutNchar(LRIGA+2,' ');                  /*cancella se non valido*/
            if (primoVis != primoGiorno) {          /*se deve mostrare <4>*/
                CancellaNota(gs);                   /*pulisce spazio nota*/
            }
        } else {                                    /*giorno valido*/
            Inverse(d.giorno == dp->giorno);        /*inverse se selez. <5>*/
            sprintf(buf," %-9s %2d %s ",            /*mostra data*/
                    giorni[d.gsett].nome,
                    d.gmese,mesi[d.mese].nome);
            Jputs(buf);
            Inverse(0);                             /*fine event. inverse*/

            if (primoVis != primoGiorno) {          /*se deve mostrare*/
                LeggeNota(n,d.giorno);              /*legge da file <6>*/
                if (! ErFile()) {
                    MostraNota(n,gs);               /*ok, mostra su video*/
                }
            }
        }
    }

    primoGiorno = primoVis;                         /*per prox volta <7>*/
}

/* Note su MostraSettimana:
    <1> Questa variabile statica, che quindi conserva il suo valore tra una
        chiamata e l'altra, serve per evitare di ridisegnare la settimana
        che  gi visibile sul video. Contiene il primo giorno (Luned)
        della settimana attualmente visibile sullo schermo, sotto forma di
        giorno progressivo dall'inizio dell'anno. Viene inizializzata con
        un valore impossibile (-1) per essere certi che alla prima chiamata
        della funzione la settimana venga comunque mostrata sul video.
    <2> primoVis  il Luned della settimana da visualizzare, cio il primo
        giorno in alto a sinistra.
    <3> AGENDA si basa sempre sul giorno progressivo dall'inizio dell'anno.
    <4> Se la settimana da mostrare  uguale a quella gi visibile, non
        la ridisegna (questo test  ripetuto pi avanti). Per i giorni non
        validi (es. dopo la fine dell'anno), cancella sia il nome del giorno
        che il riquadro delle note.
    <5> Il giorno correntemente selezionato  mostrato a colori invertiti.
    <6> Per ciascun giorno, legge dal file la nota del giorno e, se non ci
        sono stati errori, la scrive nel riquadro corrispondente.
    <7> Vedi nota 1.
*/


/********************************************************************
 * CancellaNota: cancella nota giorno gs della settimana.
 ********************************************************************/

void CancellaNota(int gs)
{
    int h,v,i;

    h = giorni[gs].hpos;                            /*posizione nota*/
    v = giorni[gs].vpos+2;
    for (i = 0; i < NRIGHE; i++) {                  /*cancella*/
        Jgotoxy(h,v++);                             /*una riga alla volta*/
        PutNchar(LRIGA,' ');                        /*<1>*/
    }
}

/* Note su CancellaNota:
    <1> La PutNchar  stata resa pi efficiente per il programma AGENDA.
*/


/********************************************************************
 * MostraNota: mostra nota n nel giorno gs della settimana.
 ********************************************************************/

void MostraNota(Nota n,int gs)
{
    int h,v,h2,i;

    h = giorni[gs].hpos;                            /*posizione nota*/
    v = giorni[gs].vpos+2;
    h2 = h+LRIGA;                                   /*limite area nota*/
    for (i = 0; i < NRIGHE; i++) {
        PutsAt(h,v++,n[i]);                         /*mostra contenuto*/
        PutNchar(h2-Jposx(),' ');                   /*pulisce resto riga*/
    }
}


/********************************************************************
 * EditNota: edit nota data *dp, ritorna se errore di accesso a disco.
 ********************************************************************/

void EditNota(Data *dp)     /*<1>*/
{
    Nota n;
    int h,v,i;

    LeggeNota(n,dp->giorno);                        /*legge nota dal file*/
    if (! ErFile()) {                               /*se lettura ok*/
        Menu(2);                                    /*menu di edit <2>*/
        h = giorni[dp->gsett].hpos;                 /*posizione nota*/
        v = giorni[dp->gsett].vpos+2;
        for (i = 0; i < NRIGHE; i++) {              /*input linee <3>*/
            InputStringa(n[i],LRIGA,h,v++);
        }
        ScriveNota(n,dp->giorno);                   /*registra nota in file*/
        Menu(1);                                    /*menu principale <4>*/
    }
}

/* Note su EditNota:
    <1> La nota viene letta dal file, modificata, e riscritta nel file.
    <2> Per evidenziare il fatto che i comandi del menu principale non sono
        disponibili durante l'editing, lo nasconde e mostra le istruzioni
        sui tasti disponibili.
    <3> Sarebbe certamente pi elegante fare un unico input 'rettangolare',
        confinando il cursore nel riquadro della nota (continuando
        automaticamente, alla fine di ogni riga, su quella successiva).
        Lasciamo volentieri questo esercizio al lettore di buona volont.
    <4> Terminata l'operazione, ripristina il normale menu dei comandi.
*/


/********************************************************************
 * DisegnaFinestra: mostra finestra settimanale, con anno indicato.
 ********************************************************************/

void DisegnaFinestra(int anno)
{
    char lin[] = "";
    char buf[81];
    int i;

    PutNchar(32,'');
    sprintf(buf," Agenda %d ",anno); Jputs(buf);
    PutNchar(32,''); Jputs("\r\n");
    sprintf(buf,"%s%s%s%s\r\n",lin,lin,lin,lin); Jputs(buf);
    sprintf(buf,"%18s%18s%18s%18s\r\n","","","",""); Jputs(buf);
    sprintf(buf,"%s%s%s%s\r\n",lin,lin,lin,lin); Jputs(buf);
    for (i = 0; i < 6; i++) {
        sprintf(buf,"%18s%18s%18s%18s\r\n","","","",""); Jputs(buf);
    }
    sprintf(buf,"%s%s%s%s\r\n",lin,lin,lin,lin); Jputs(buf);
    sprintf(buf,"%18s%18s%18s\r\n","","",""); Jputs(buf);
    sprintf(buf,"%s%s%s\r\n",lin,lin,lin); Jputs(buf);
    for (i = 0; i < 6; i++) {
        sprintf(buf,"%18s%18s%18s\r\n","","",""); Jputs(buf);
    }
    sprintf(buf,"%s%s%s\r\n",lin,lin,lin,lin); Jputs(buf);

}


/********************************************************************
 * Menu: 1=menu principale, 2=edit note.
 ********************************************************************/

void Menu(int m)
{
    char *menu1[] = {     /*<1>*/
        "  " F_SIN " " F_DES "      Giorno precedente/successivo",
        "  " F_SU  " " F_GIU "      Settimana precedente/successiva",
        " Pg Up/Dn  Mese precedente/successivo",
        "F1  Note giorno corrente",
        "F2  Torna alla data odierna",
        "F6  Fine lavoro"
    };

    char *menu2[] = {
        "",
        "<Backspace> per correggere,",
        "<Return> per accettare la riga:",
        "","",""
    };

    char **menu;                                    /*punter al menu <2>*/
    int  i;

    for (i = 0; i < 3; i++) {                       /*cancella area menu*/
        Jgotoxy(1,21+i);
        PutNchar(78,' ');
    }

    menu = (m == 1) ? menu1 : menu2;                /*sceglie menu <3>*/
    for (i = 0; i < 3; i++) {                       /*e lo mostra*/
        PutsAt(1,21+i,menu[i]);                     /*<4>*/
        PutsAt(48,21+i,menu[i+3]);
    }
}

/* Note su Menu:
    <1> Un menu  un array di 6 pointer a char, che puntano rispettivamente
        alle tre istruzioni di sinistra ed alle tre istruzioni di destra.
    <2> La variabile menu punta ad un pointer a char, esattamente come fa il
        nome di un array di pointer a char. E' quindi possibile assegnare
        a questa variabile l'indirizzo di menu1 o di menu2.
    <3> Vedi nota precedente.
    <4> La variabile menu pu essere usata come fosse un array di stringhe.
        Difatti contiene l'indirizzo di un array di pointer a char; perci
        menu[0] (ovvero *menu)  il primo di questi pointer, che  appunto
        l'indirizzo della prima stringa. Come si vede,  assai pi facile
        da usare che da spiegare.
*/
