/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* FIOCCO: disegna un fiocco di neve ricorsivo (curva di Koch) */


#include <stdio.h>
#include <math.h>
#include <conio.h>                /*per aspettare un tasto*/

#include "Jgraph.h"


#define ORDMAX  6                 /*massimo ordine del fiocco*/

/* costanti e variabili globali: */

double grad;                      /*cost. di conversione gradi/radianti <1>*/
double quadra;                    /*fattore di quadratura*/
double posx,posy;                 /*posizione corrente di disegno <2>*/


/* prototipi delle funzioni usate: */

void Fiocco(int n);
void TracciaLato(double la,double ang,int n);
void TracciaLinea(double lun, double a);


/* Note sulle dichiarazioni:
    <1> Non  possibile definire una costante calcolata con operazioni
        non intere (es. double), quindi grad non pu essere definita const:
        il suo valore sar calcolato prima di iniziare a disegnare.
    <2> Il meccanismo usato richiede di conservare la posizione corrente
        a cui si  arrivati col disegno. Non basta conservare le coordinate
        dell'ultimo pixel disegnato, perch questo  un'approssimazione del
        valore effettivo: gli errori di approssimazione si accumulerebbero,
        portando in breve a vistose irregolarit nel disegno.
*/

/********************************************************************
 * main: il primo argomento  l'ordine del fiocco da disegnare.
 ********************************************************************/

main(int argc, char *argv[])
{
    int ord;            /*ordine del fiocco*/
    int c;

    if (argc != 2) {                                /*controlla argomento*/
        printf("\n- Uso: FIOCCO <ordine>\n");
    } else if (sscanf(argv[1],"%d",&ord) < 1        /*lo legge <1>*/
               || ord < 1 || ord > ORDMAX) {
        printf("\n- L'ordine dev'essere compreso"
               " tra 1 e %d.\n",ORDMAX);
    } else if (Jinitgraph() == 0) {                 /*inizializza grafica*/
        puts("\n- Non sono in grado di usare"
             " la grafica di questo computer.");
    } else {                                        /*se ci riesce:*/
        Fiocco(ord);                                /*disegna il fiocco*/
        while (kbhit())  getch();                   /*pulisce buffer <2>*/
        while (! kbhit())  ;                        /*aspetta tasto <3>*/
        getch();                                    /*ripulisce <4>*/
        Jclosegraph();                              /*torna in modo testo*/
    }
}

/* Note su main:
    <1> Se la scanf non riesce a leggere un valore valido in ord, ritorna 0
        (zero valori letti). Il primo confronto logico risulta quindi vero,
        ed il risultato dell'operatore OR (||)  automaticamente vero. Non
        vengono quindi eseguiti i successivi due confronti, evitando di
        esaminare la variabile non valida (ord).
    <2> Finch ci sono caratteri in attesa, ne legge. Sia kbhit che getch
        sono uguali nelle librerie di console dei due compilatori che usiamo.
        Cicli semplicissimi come questo possono essere messi su una sola
        linea senza danno per la leggibilit.
    <3> Finch il buffer di tastiera  vuoto, aspetta.
    <4> Toglie dal buffer di tastiera il carattere battuto, per evitare che
        venga poi ricevuto dal DOS quando il programma termina.
*/


/********************************************************************
 * Fiocco: disegna un fiocco di neve di ordine n.
 ********************************************************************/

void Fiocco(int n)
{
    double lato;                                    /*lato livello 1 <1>*/

    grad = atan(1.0)/45.0;                          /*cost. conversione*/
    quadra = (Jgetdimx()*3.0)/(Jgetdimy()*4.0);     /*fattore quadratura*/
	posx = (double)Jgetdimx()/2;                    /*posiz. iniziale <2>*/
    posy = 0.0;
    lato = Jgetdimy()*6.0/7.0;                      /*lato iniziale <3>*/

    Jmoveto((int)posx,(int)posy);                   /*va all'inizio <4>*/
    TracciaLato(lato,240.0,n);                      /*lato ricorsivo <5>*/
    TracciaLato(lato,360.0,n);
    TracciaLato(lato,120.0,n);                      /*completa triangolo*/
}


/* Note su Fiocco:
    <1> Il lato iniziale  quello del triangolo puro e semplice.
    <2> Inizia dal vertice superiore del triangolo, che sta in mezzo al
        bordo superiore dello schermo.
    <3> La costante 6.0/7.0  calcolata empiricamente per tenere la figura
        interamente visibile sullo schermo.
    <4> Non serve approssimare, perch sia posx che posy non hanno parte
        frazionaria (per ora).
    <5> La Fiocco si limita a disegnare un triangolo. Sar la TracciaLato a
        fare il lavoro di suddivisione ricorsiva del lato stesso.
*/


/********************************************************************
 * TracciaLato: disegna lato ricorsivo verso angolo a, livello n.
 ********************************************************************/

void TracciaLato(double lato, double ang, int n)
{
    if (n <= 1) {                                   /*se ultimo livello <1>*/
        TracciaLinea(lato,ang);                     /*semplice linea*/
    } else {                                        /*se non  l'ultimo <2>*/
        lato /= 3.0;                                /*suddivide*/
        n--;                                        /*scende di livello*/
        TracciaLato(lato,ang,n);                    /*si richiama 4 volte*/
        TracciaLato(lato,ang-60,n);
        TracciaLato(lato,ang+60,n);
        TracciaLato(lato,ang,n);
    }
}


/* Note su TracciaLato:
    <1> Questa  la condizione finale della ricorsione. Se  vera, la
        TracciaLato disegna una semplice linea retta nella direzione ang.
    <2> Se la ricorsione non  finita, la TracciaLato richiama quattro volte
        se stessa per disegnare i quattro segmenti in cui viene spezzata
        la linea. Ciascuno di questi pu a sua volta essere spezzato in
        quattro, e cos di seguito. Notare che non viene fisicamente
        disegnato nulla finch non si arriva all'ultimo livello.
*/


/********************************************************************
 * TracciaLinea: disegna linea lun da pos. corrente verso l'angolo a.
 ********************************************************************/

void TracciaLinea(double lun, double a)             /*<1>*/
{
    posx += (lun*quadra*cos(a*grad));               /*calcola nuova pos.<2>*/
    posy -= (lun*sin(a*grad));
    Jlineto((int)(floor(posx+0.5)),(int)(floor(posy+0.5)));   /*disegna <3>*/
}


/* Note su TracciaLinea:
    <1> TracciaLinea lavora in coordinate polari relative (turtlegraphics),
        come il programma SPIRALI della lezione E4. Viene tracciata una
        linea che parte dal termine della precedente (posx,posy) e va nella
        direzione indicata (in gradi sessagesimali) per una lunghezza lun.
    <2> Questi calcoli sono piuttosto lenti: il programma potrebbe essere
        accelerato di parecchio evitando le operazioni ripetitive. Ad
        esempio, si potrebbe considerare che l'angolo varia di 60 in 60
        gradi, e quindi pu assumere solo 6 diversi valori. Basterebbe
        quindi conservare in due array i valori dello spostamento x e y per
        i 6 possibili angoli, e semplicemente leggerli ed usarli. Il
        guadagno in velocit sarebbe notevole. Un ulteriore aumento di
        velocit si potrebbe ottenere eliminando la ricorsione, anche se
        a spese della chiarezza e leggibilit del programma.
    <3> Il valore di posx e posy viene approssimato al pixel pi vicino.
*/
