/*
   Questo file fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* JGRAPH.H: header per la libreria grafica Jackson,    */
/*           sia per Turbo C 2.0 che per Quick C 2.0.   */
/*           Versione E5: funzioni elementari.          */


#if     !defined(__JGRAPH_H_)       /*include solo se non gi incluso*/
#define __JGRAPH_H_


/* Nota:
    A differenza della libreria di console Jackson (Jconio), questa libreria
    grafica  intesa prevalentemente per uso didattico. Le differenze tra
    i due compilatori la rendono infatti particolarmente limitata. Volendo
    sfruttare a fondo la grafica, occorre purtroppo abbandonare ogni pretesa
    di portatilit dei programmi.
*/


/******************* Prototipi delle funzioni grafiche *******************/

int  Jinitgraph(void);              /*va in modo grafico, ritorna 1 se ok*/
void Jclosegraph(void);             /*torna nel modo testo originario*/
int  Jgetdimx(void);                /*ritorna numero di pixel orizzontali*/
int  Jgetdimy(void);                /*ritorna numero di pixel verticali*/
int  Jgetcolors(void);              /*ritorna numero colori disponibili*/
void Jclrscreen(void);              /*cancella video al colore di fondo*/
void Jsetpixel(int x,int y,int c);  /*colora il pixel x,y al colore c*/
int  Jgetpixel(int x,int y);        /*ritorna il colore del pixel x,y*/
void Jsetcolor(int c);              /*sceglie il colore c per le linee*/
void Jmoveto(int x,int y);          /*va alla posizione x,y (non disegna)*/
void Jlineto(int x,int y);          /*traccia linea fino alla posizione x,y*/


/********************************************************************
 * Macro eseguite compilando con il Turbo C 2.0.
 ********************************************************************/

#ifdef __TURBOC__                                   /*compila se Turbo C*/

  #include <graphics.h>                             /*libreria grafica*/

  /* macro per sfruttare funzioni fornite nella libreria del Turbo C 2.0: */

  #define Jclosegraph       closegraph              /*<1>*/
  #define Jgetdimx()        (getmaxx()+1)           /*<2>*/
  #define Jgetdimy()        (getmaxy()+1)
  #define Jgetcolors()      (getmaxcolor()+1)
  #define Jclrscreen        cleardevice
  #define Jsetpixel         putpixel
  #define Jgetpixel         getpixel
  #define Jsetcolor         setcolor
  #define Jmoveto           moveto
  #define Jlineto           lineto

  /*la funzione Jinitgraph  definita in Jgraph.c*/

  /* Note sulle macro per il Turbo C 2.0:
      <1> Se nella libreria esiste una funzione perfettamente equivalente
          (compresi numero e tipo dei parametri), basta sostituire il nome:
          Tutti i Jclosegraph diventeranno closegraph, e quindi tutte le
          chiamate Jclosegraph() diventeranno closegraph().
      <2> Volendo fare delle varianti, non basta semplicemente sostituire
          il nome della funzione, ma occorre riportare per esteso l'intera
          sostituzione da eseguire.
  */


/********************************************************************
 * Macro eseguite compilando con il Quick C 2.0.
 ********************************************************************/

#else                                               /*compila se Quick C*/

  #include <graph.h>                                /*libreria grafica*/

  /* macro per sfruttare funzioni fornite nella libreria del Quick C 2.0: */

  #define Jclosegraph()     _setvideomode(_DEFAULTMODE)
  #define Jgetdimx()        JgetQuickGraphInfo(1)           /*<1>*/
  #define Jgetdimy()        JgetQuickGraphInfo(2)
  #define Jgetcolors()      JgetQuickGraphInfo(3)
  #define Jclrscreen        _clearscreen(_GCLEARSCREEN)
  #define Jsetpixel(x,y,c)  {                               /*<2>*/ \
                              int temp = _getcolor();       /*<3>*/ \
                              _setcolor(c); _setpixel(x,y); \
                              _setcolor(temp); \
                            }
  #define Jgetpixel         _getpixel
  #define Jsetcolor         _setcolor
  #define Jmoveto           _moveto
  #define Jlineto           _lineto

  int  JgetQuickGraphInfo(int n);               /*usata internamente <4>*/

  /*le funzioni Jinitgraph e JgetQuickGraphInfo sono definite in Jgraph.c*/

  /* Note sulle macro per il Quick C 2.0:
      <1> Per evitare un'eccessiva inefficienza di queste tre funzioni,
          abbiamo usato un meccanismo che ricorda i dati dell'ultima
          chiamata, ed abbiamo unificato le tre funzioni in una sola. Il
          nome  stato scelto in modo da ridurre al minimo le probabilit
          di un uso accidentale dello stesso nome da parte dell'utente.
      <2> Una macro pu anche effettuare una sostituzione complessa, come
          questa: ad ogni Jsetpixel viene sostituita questa istruzione
          composta. Occupa pi spazio di una semplice chiamata di funzione,
          ma  leggermente pi efficiente. La differenza tra le due librerie
          obbligava ad una scelta: ottimizzare la chiamata a Jsetpixel per
          Turbo C 2.0 o per il Quick C 2.0. Abbiamo scelto il primo.
          Il compilatore C ignora gli a capo nel sorgente, il preprocessore
          no: per continuare una macro sulla linea successiva occorre
          terminarla con un backslash (\).
      <3> Una variabile pu essere dichiarata dopo ogni graffa aperta, anche
          dopo quella che apre un'istruzione composta. Si tratta in questo
          caso di una variabile locale ad un blocco, che non disturba
          eventuali variabili con lo stesso nome, e sparisce alla fine del
          blocco, cio alla graffa chiusa.
      <4> Vedi nota 2.
  */


#endif                                      /*fine scelta Turbo C / Quick C*/


#endif                                      /*(se header non gi incluso)*/
