/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* RETTANG: disegna un rettangolo <1>*/


#include <stdio.h>
#include <ctype.h>              /*<2>*/


void InputDatiRettangolo(void);                         /*prototipi funzioni*/
void DisegnaRettangolo(int b, int a, int ch, int cv);
int  InputInt(int mi, int ma);
int  InputCar(void);


int base;       /*base del rettangolo, in numero di caratteri <3>*/
int altezza;    /*altezza del rettangolo, in numero di linee*/
int carh;       /*carattere per linee orizzontali*/
int carv;       /*carattere per linee verticali*/


/* Note sulle dichiarazioni:
    <1> Dato che il C standard ANSI non prevede istruzioni grafiche (che
        in effetti differiscono tra i diversi compilatori e sono trattate
        nella parte esperti), il rettangolo  disegnato usando normali
        caratteri sullo schermo alfanumerico.
    <2> Lo header <ctype.h> va incluso per poter usare la funzione isgraph
        impiegata nella InputCar.
    <3> Queste variabili, dichiarate fuori da ogni funzione (anche da main)
        sono variabili globali, accessibili da tutte le funzioni.
*/


/********************************************************************
 * main
 ********************************************************************/

main()      /*<1>*/
{
    printf("\n Disegno di un rettangolo con normali caratteri:\n\n");

    InputDatiRettangolo();                          /*chiede i dati <2>*/
    putchar('\n');                                  /*a capo*/
    DisegnaRettangolo(base,altezza,carh,carv);      /*e disegna <3>*/
}

/*  Note su main:
    <1> Questa  la tipica struttura di un vero programma C: il main si
        limita a chiamare una serie di funzioni, ciascuna delle quali svolge
        una parte del lavoro.
    <2> La InputDatiRettangolo non ritorna alcun valore, ma scrive invece
        nelle variabili globali i dati introdotti dall'utente (con le
        istruzioni finora introdotte, non c'era altra scelta: una funzione
        pu ritornare un solo valore, e qui ne servono quattro).
    <3> E' apparentemente inutile passare alla DisegnaRettangolo queste
        quattro variabili: sono infatti globali e quindi la DisegnaRettangolo
        potrebbe leggerle senza problemi. Bisogna per cercare di rendere
        le funzioni il pi possibile indipendenti: passando le variabili
        come argomenti, la DisegnaRettangolo non ha bisogno di variabili
        globali, e pu essere usata tale e quale anche in altri programmi.
*/


/********************************************************************
 * InputDatiRettangolo: input dati nelle variabili globali
 *                      base, altezza, carh e carv. <1>
 ********************************************************************/

void InputDatiRettangolo(void)      /*<2>*/
{
    printf("Base del rettangolo, in caratteri : ");
    base = InputInt(2,78);                          /*<3>*/
    printf("Altezza rettangolo, in caratteri  : ");
    altezza = InputInt(2,22);
    printf("Carattere per i lati orizzontali  : ");
    carh = InputCar();                              /*<4>*/
    printf("Carattere per i lati verticali    : ");
    carv = InputCar();
}                                                   /*<5>*/

/*  Note su InputDatiRettangolo:
    <1> I valori introdotti sono registrati nelle variabili globali, visto
        che non  possibile ritornare direttamente pi di un valore.
    <2> La funzione  di tipo void perch non ritorna alcun valore, e tra
        le parentesi c' void perch non ha parametri.
    <3> Una funzione pu chiamare un'altra funzione. Qui ad esempio la
        InputDatiRettangolo chiama la InputInt per far introdurre un valore
        compreso tra i due limiti indicati dagli argomenti. Non c' alcuna
        differenza tra una chiamata dal main o da un'altra funzione.
    <4> La InputCar, definita pi sotto, chiede all'utente di introdurre
        un normale carattere ASCII stampabile.
    <5> Una funzione di tipo void non ritorna un valore, quindi non ha
        alcuna istruzione return (che di fatto sarebbe un errore).
*/


/********************************************************************
 * DisegnaRettangolo: disegna rettangolo di base b ed altezza a,
 *                    usando il carattere ch per i lati orizzontali
 *                    ed il carattere cv per i lati verticali.
 ********************************************************************/

void DisegnaRettangolo(int b, int a, int ch, int cv)    /*<1>*/
{
    int h,v;

    for (h = 0; h < b; h++)  putchar(ch);           /*lato superiore*/
    putchar('\n');
    for (v = 1; v < a-1; v++) {                     /*parte centrale*/
        putchar(cv);                                /*lato sinistro*/
        for (h = 1; h < b-1; h++)  putchar(' ');    /*spazi intermedi*/
        putchar(cv);                                /*lato destro*/
        putchar('\n');
    }
    for (h = 0; h < b; h++)  putchar(ch);           /*lato inferiore*/
    putchar('\n');
}

/*  Note su DisegnaRettangolo:
    <1> Questa funzione lavora esclusivamente in base ai valori degli
        argomenti passati dal chiamante, e ricevuti nei parametri indicati.
        Poich non usa alcuna variabile globale,  del tutto indipendente
        e riutilizzabile. E' in sostanza un mattone da costruzione.
*/


/********************************************************************
 * InputInt: input di intero compreso tra mi e ma, ritorna il valore
 *           introdotto, ripete l'input se il valore non  valido.
 ********************************************************************/

int InputInt(int mi, int ma)        /*<1>*/
{
    int num;        /*numero introdotto*/
    int risulta;    /*risultato della scanf*/
    int valido;     /*flag di numero valido*/

    do {
        risulta = scanf("%d",&num);                 /*legge numero*/
        if (risulta != EOF) {                       /*ripulisce se non EOF*/
            while (getchar() != '\n')
                ;
        }
        valido = (risulta == 1 && num >= mi
                  && num <= ma);                    /*controlla se ok*/
        if (! valido) {                             /*segnala errore*/
            printf("Devi introdurre un intero"
                   " compreso tra %d e %d: \a",mi,ma);
        }
    } while (! valido);                             /*ripete se non ok*/

    return num;                                     /*ritorna valore*/
}

/*  Note su InputInt:
    <1> InputInt  equivalente alla InputLong descritta nel programma INVESTE.
*/


/********************************************************************
 * InputCar: input di normale carattere ASCII, ritorna il carattere
 *            introdotto, ripete l'input se il carattere non  valido.
 ********************************************************************/

int InputCar(void)  /*<1>*/
{
    int c;          /*carattere introdotto*/
    int valido;     /*flag di carattere valido*/

    do {
        c = getchar();                              /*legge carattere*/
        if (c != EOF && c != '\n') {                /*ripulisce se deve <2>*/
            while (getchar() != '\n')
                ;
        }
        valido = isgraph(c);                        /*controlla se ok <3>*/
        if (! valido) {                             /*segnala errore*/
            printf("Devi introdurre un carattere"
                   " stampabile del normale set ASCII: \a");
        }
    } while (! valido);                             /*ripete se non ok*/

    return c;                                       /*ritorna valore <4>*/
}

/*  Note su InputCar:
    <1> La InputCar  simile alla precedente InputInt, ma accetta un singolo
        carattere stampabile. Se viene battuta una linea di pi caratteri,
        ritorna il primo carattere della linea stessa.
    <2> Occorre ripulire il resto della linea solo se il primo carattere non
         un Return (che la getchar riceve come \n) o un Ctrl-Z o F6 (che
        viene ricevuto come EOF). In questi due casi, infatti, la linea  gi
        finita e non restano caratteri in attesa.
    <3> La isgraph, che richiede lo header <ctype.h>, ritorna vero (diverso
        da zero) solo se il carattere  un normale carattere stampabile del
        set ASCII standard. Sono quindi esclusi lo spazio ed i caratteri
        speciali del PC IBM, ad esempio le lettere accentate.
    <4> In C, per contenere un singolo carattere si usa un int. La InputCar
         infatti dichiarata di tipo int e torna l'int contenuto in c, che
         appunto il codice del carattere. In C un carattere ed il suo
        codice ASCII sono la stessa cosa.
*/
