/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* SIMBOLI: mostra i simboli dei caratteri di controllo (codice 0..31) */


#include <stdio.h>
#include <dos.h>          /*per le chiamate a BIOS*/


/* prototipi delle funzioni usate: */

void StampaCarattere(int c);
void LeggePosizione(int *h, int *v);
void PosizionaCursore(int h, int v);


/********************************************************************
 * main
 ********************************************************************/

main()
{
    int c,hpos,vpos;

    puts("\nSimboli corrispondenti ai caratteri di controllo:\n\n");

    for (c = 0; c < 32; c++) {
        printf("    %02d: ",c);                    /*mostra codice*/
        StampaCarattere(c);                        /*e simbolo <1>*/
        LeggePosizione(&hpos,&vpos);               /*avanza di 1 <2>*/
        PosizionaCursore(hpos+1,vpos);
        if (c%8 == 7) {                            /*a capo ogni 8*/
            puts("\n");                            /*(due volte) <3>*/
        }
    }
}

/* Note su main:
    <1> La StampaCarattere chiama direttamente il BIOS, e pu quindi
        stampare i simboli corrispondenti a qualunque carattere (da 0 a 255),
        senza interpretarne nessuno (es. 10, cio '\n') in modo speciale.
    <2> Dato che la StampaCarattere non avanza il cursore, ne leggiamo la
        posizione con la LeggePosizione e lo posizioniamo una colonna pi
        avanti con la PosizionaCursore.
    <3> Stampa due '\n', perch la puts ne aggiunge sempre uno alla fine
        della stringa stampata.
*/


/********************************************************************
 * StampaCarattere: stampa il carattere con codice c, non avanza.
 ********************************************************************/

void StampaCarattere(int c)
{
    union REGS r;     /*registri da passare al BIOS <1>*/

    r.h.ah = 0x0a;                          /*Write Character Only <2>*/
    r.h.al = c;                             /*codice del carattere <3>*/
    r.h.bh = 0;                             /*pagina video 0 <4>*/
    r.x.cx = 1;                             /*numero di volte <5>*/
    int86(0x10,&r,&r);                      /*chiama BIOS Video <6>*/
}

/* Note su StampaCarattere:
    <1> La union REGS  definita in <dos.h> e contiene due strutture
        sovrapposte, rappresentanti entrambe i registri del microprocessore.
        Se la variabile r  di tipo union REGS, la struttura r.h vede i
        registri a 8 bit per volta (es. r.h.al, r.h.ah), mentre la struttura
        r.x vede i registri a 16 bit (es. r.x.ax). al e ah sono in realt
        le due met (parte bassa e parte alta) del registro ax. I valori
        scritti in r saranno fisicamente trasferiti nei registri della CPU
        dalla funzione int86.
    <2> La funzione 0ah dell'interrupt 10h del BIOS scrive un carattere sul
        video, senza avanzare il cursore. Il numero della funzione va
        scritto in ah prima della chiamata, mentre il numero del software
        interrupt da chiamare va passato alla int86.
    <3> r.h.al  un numero a 8 bit, e non pu contenere tutti i valori
        possibili per un int, come ad esempio c. Ma sappiamo che c contiene
        il codice di un carattere, per cui il troncamento del byte pi
        significativo non ha importanza.
    <4> Su molti computer sono disponibili pi pagine video, che possono
        essere alternate sul video senza perderne il contenuto. Noi usiamo
        solo la pagina 0, che esiste sicuramente su tutti i modelli.
    <5> Il BIOS consente di stampare pi copie consecutive del carattere.
    <6> La int86 chiama il software interrupt numero 10h, corrispondente
        alle funzioni del BIOS che riguardano il video. Prima della chiamata
        vengono copiati nei registri della CPU i valori contenuti nella
        union REGS passata per riferimento (cio r). La funzione specifica
        da eseguire viene scelta in base al valore del registro ah. Dopo
        la chiamata, i valori dei registri vengono ricopiati nella seconda
        union REGS indicata, che in questo caso  ancora r (usando la stessa
        union si perdono i valori originari, ma non ne abbiamo pi bisogno).
*/


/********************************************************************
 * LeggePosizione: legge posizione corrente cursore in *h e *v.
 ********************************************************************/

void LeggePosizione(int *h, int *v)
{
    union REGS r;     /*registri da passare al (e ricevere dal) BIOS*/

    r.h.ah = 0x03;                          /*Read Cursor Info <1>*/
    r.h.bh = 0;                             /*pagina video 0 <2>*/
    int86(0x10,&r,&r);                      /*chiama BIOS Video <3>*/
    *v = r.h.dh;                            /*ritorna riga <4>*/
    *h = r.h.dl;                            /*e colonna*/
}

/* Note su LeggePosizione:
    <1> La funzione 03h dell'interrupt 10h del BIOS legge la posizione
        corrente del cursore nei registri dh e dl.
    <2> Vedi nota 4 di StampaCarattere.
    <3> Vedi nota 6 di StampaCarattere.
    <4> Le componenti della struttura .h di una union REGS sono definite in
        <dos.h> come unsigned char, per cui non sono possibili errori nella
        assegnazione ad un int (ricordiamo che assegnando un signed char ad
        un int non  garantito che il contenuto degli 8 bit a sinistra sia
        zero). In ogni caso, comunque, questi valori sono inferiori a 128
        e quindi il problema non si porrebbe lo stesso. Ma in C bisogna
        sempre tenere gli occhi ben aperti quando si fanno assegnazioni tra
        variabili intere di tipo diverso.
*/


/********************************************************************
 * PosizionaCursore: va alla posizione h,v.
 ********************************************************************/

void PosizionaCursore(int h, int v)
{
    union REGS r;     /*registri da passare al BIOS*/

    r.h.ah = 0x02;                          /*Set Cursor Position <1>*/
    r.h.bh = 0;                             /*pagina video 0 <2>*/
    r.h.dh = v;                             /*riga <3>*/
    r.h.dl = h;                             /*e colonna*/
    int86(0x10,&r,&r);                      /*chiama BIOS Video <4>*/
}

/* Note su PosizionaCursore:
    <1> La funzione 03h dell'interrupt 10h del BIOS posiziona il cursore
        sulla riga e colonna indicate nei registri dh e dl.
    <2> Vedi nota 4 di StampaCarattere.
    <3> Vedi nota 3 di STampaCarattere (riga e colonna ci stanno in 8 bit).
    <4> Vedi nota 6 di StampaCarattere.
*/
