/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* DADI: lanci di dadi a sei facce */


#include <stdio.h>
#include <stdlib.h>     /*<1>*/
#include <time.h>       /*<2>*/


int  ChiedeLanci(void);                     /*prototipi funzioni*/
void LanciaDadi(int n, int r[]);
void MostraUscite(int r[]);
int  Dado(int f);
int  InputInt(int mi, int ma);


#define MAXLANCI 300    /*numero massimo di lanci dei dadi*/
#define MINRIS   2      /*minimo risultato possibile*/
#define MAXRIS   12     /*massimo risultato possibile*/


main()
{
    int uscite[MAXRIS+1];   /*uscite di ogni possibile risultato (2-12) <3>*/
    int lanci;              /*numero di lanci di due dadi*/
    int ok;

    printf("\n Lanci di due dadi a sei facce:\n\n");

    srand((unsigned int)time(NULL));                /*prepara random <4>*/

    while ( (lanci = ChiedeLanci()) > 0) {          /*chiede n.lanci <5>*/
        LanciaDadi(lanci,uscite);                   /*esegue lanci*/
        MostraUscite(uscite);                       /*stampa uscite*/
    }
}

/* Note su main:
    <1> Lo header <stdlib.h>  necessario per usare le funzioni
        srand e rand per la generazione di numeri pseudocasuali.
    <2> Lo header <time.h>  necessario per usare la funzione time.
    <3> Per comodit di lettura, gli elementi dell'array uscite[]
        corrispondono ai numeri delle uscite stesse. Ad esempio, l'elemento
        9 indica quante volte  uscito il 9. Di conseguenza, gli elementi
        0 ed 1 non sono usati. E' invece usato l'elemento 12, per cui la
        dimensione dell'array dev'essere 12+1 (MAXRIS+1).
    <4> La funzione time (introdotta nella lezione B8) restituisce un valore
        di tipo dipendente dal compilatore, che qui viene convertito con
        un cast ad un unsigned int, e poi passato alla srand per avviare
        il generatore di numeri pseudocasuali con un valore imprevedibile.
        La funzione time ritorna infatti un valore del quale non ci
        interessa il significato, ma che  variabile con il passare del
        tempo. La costante NULL passta a time  definita in <stdio.h>.
    <5> Vedi nota 1 di ChiedeLanci.
*/


/********************************************************************
 * ChiedeLanci: chiede quanti lanci fare (1-MAXLANCI), ritorna numero
 *              lanci richiesti, o zero per indicare fine lanci.
 ********************************************************************/

int ChiedeLanci(void)
{
    int n;

    printf("Quante volte vuoi lanciare (0 per smettere)? ");
    return InputInt(0,MAXLANCI);                    /*<1>*/
}

/*  Note su ChiedeLanci:
    <1> La ChiedeLanci ritorna semplicemente il valore restituito dalla
        InputInt. Questo consente di semplificare e rendere pi leggibile
        il ciclo del main (riferimento 5), eseguendo l'input nel test stesso.
*/


/********************************************************************
 * LanciaDadi: lancia n coppie di dadi, mette numero di uscite per
 *             ciascun risultato nel corrispondente elemento di r[].
 ********************************************************************/

void LanciaDadi(int n, int r[])     /*<1>*/
{
    int ris;        /*risultato di un lancio*/
    int i;

    printf("\neseguo i lanci...");

    for (i = MINRIS; i <= MAXRIS; i++)  r[i] = 0;   /*azzera r[] */

    while (n-- > 0) {
        ris = Dado(6)+Dado(6);                      /*somma di due dadi <2>*/
        r[ris]++;                                   /*<3>*/
    }

    putchar('\n');
}

/*  Note su LanciaDadi:
    <1> r[]  in realt l'array uscite[] passato da main.
    <2> Tirare due dadi a sei facce e sommarne i risultati  ben diverso che
        tirare un dado a 12 facce!
    <3> Questa espressione incremente r[ris], cio aggiunge 1 all'elemento
        di r[] corripondente al numero uscito. Ad esempio, se  uscito un 5
        viene incrementato r[5], che contiene quindi il numero di 5 usciti
        nel totale dei lanci.
*/


/********************************************************************
 * MostraUscite: disegna istogramma numero uscite per ogni risultato,
 *               leggendo dal corrispondente elemento di r[].
 ********************************************************************/

void MostraUscite(int r[])
{
    int i,j;

    putchar('\n');

    for (i = MINRIS; i <=MAXRIS; i++) {             /*per ogni risultato*/
        printf("%2d)  %2d  ",i,r[i]);
        for (j = r[i]; j > 0; j--)  putchar('#');   /*come istogramma <1>*/
        putchar('\n');
    }

    putchar('\n');
}

/*  Note su MostraUscite:
    <1> Per disegnare tanti asterischi quante sono le uscite del numero i
        cio r[i] asterischi, un ciclo a scendere  pi efficiente di un
        ciclo a salire: il confronto avviene infatti con zero anzich con
        r[i], evitando quindi una lettura dell'array ad ogni ciclo.
*/


/********************************************************************
 * Dado: lancia un dado a f facce, ritorna numero uscito.
 ********************************************************************/

int Dado(int f)
{
    return (1+rand()%f);                            /*<1>*/
}

/*  Note su Dado:
    <1> La rand ritorna un numero compreso tra 0 e RAND_MAX. Il resto della
        divisione per f (ottenuto con l'operatore %)  un numero compreso tra
        0 e f-1, perci basta aggiungere 1 per ottenere un numero compreso
        tra 1 e f. Da notare che la 'casualit' del risultato ottenuto con
        questo sistema diventa statisticamente meno valida col crescere del
        valore di f (nel nostro caso, l'errore  trascurabile).
*/


/********************************************************************
 * InputInt: input di intero compreso tra mi e ma, ritorna il valore
 *           introdotto, ripete l'input se il valore non  valido.
 ********************************************************************/

int InputInt(int mi, int ma)        /*<1>*/
{
    int num;        /*numero introdotto*/
    int risulta;    /*risultato della scanf*/
    int valido;     /*flag di numero valido*/

    do {
        risulta = scanf("%d",&num);                 /*legge numero*/
        if (risulta != EOF) {                       /*ripulisce se non EOF*/
            while (getchar() != '\n')
                ;
        }
        valido = (risulta == 1 && num >= mi
                  && num <= ma);                    /*controlla se ok*/
        if (! valido) {                             /*segnala errore*/
            printf("Devi introdurre un intero"
                   " compreso tra %d e %d: \a",mi,ma);
        }
    } while (! valido);                             /*ripete se non ok*/

    return num;                                     /*ritorna valore*/
}

/*  Note su InputInt:
    <1> InputInt  commentata nel programma RETTANG della lezione B5.
*/
