/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* FINESTRA: copiatura e ripristino di un'area video */


#include <stddef.h>       /*per definire NULL*/
#include <stdlib.h>
#include <string.h>

#include "Jconio.h"       /*Versione E6 con Jgettext e Jputtext*/


#define  PH  8            /*posizione finestra*/
#define  PV  8
#define  DH  32           /*e dimensioni*/
#define  DV  5


/* prototipi funzioni: */

int  Finestra(int x1, int y1, int x2, int y2);
void CorniceVuota(int x1, int y1, int x2, int y2);
void PutNchar(int n, int c);
void AspettaReturn(void);


/********************************************************************
 * main
 ********************************************************************/

main()
{
    Jcursor(0);                                     /*spegne cursore <1>*/
    Jclrscr();                                      /*cancella schermo*/

    Jgotoxy(1,6);
    Jputs("Per 'aprire' una finestra sul video alfanumerico,\r\n"
          "si deve salvare il rettangolo sottostante copiandolo\r\n"
          "in un'area di memoria.\r\n\n"
          "Quest'area pu essere richiesta usando la funzione\r\n"
          "malloc, come viene fatto da questo programma.\r\n\n"
          "Premi Return per aprire una finestra salvando\r\n"
          "l'area sottostante: ");

    AspettaReturn();
    Finestra(PH,PV,PH+DH-1,PV+DV-1);                /*apre finestra <2>*/

    Jgotoxy(PH+2,PV+1);                             /*<3>*/
    Jputs("Premi nuovamente Return per");
    Jgotoxy(PH+2,PV+2);
    Jputs("chiudere la finestra e");
    Jgotoxy(PH+2,PV+3);
    Jputs("ripristinare l'area video: ");

    AspettaReturn();
    Finestra(0,0,0,0);                              /*chiude finestra <4>*/

    Jgotoxy(1,16);                                  /*risistema cursore*/
    Jcursor(1);                                     /*e lo riaccende*/
}

/* Note su main:
    <1> E' particolarmente importante spegnere il cursore se il programma
        viene compilato con il Quick C. Infatti in questo caso le funzioni
        Jgettext e Jputtext sono costruite con chiamate al BIOS carattere
        per carattere: si vedrebbe un fastidioso cursore correre lungo ogni
        area video che viene salvata o ripristinata.
    <2> Le coordinate indicate si riferiscono all'area interna della
        finestra, quella scrivibile. Attorno a questa c' una cornice
        di 1 carattere per lato. Non teniamo qui conto della possibilit
        di errore per mancanza di memoria in cui salvare il contenuto
        dell'area video: in un programma professionale sarebbe meglio farlo.
    <3> Scrivere in una finestra in questo modo  molto scomodo. Il programma
        FINESTRE mostra un sistema pi pratico.
    <4> Se il primo argomento  zero, gli altri sono ignorati. Le dimensioni
        della finestra da chiudere sono contenute in variabili statiche
        della funzione Finestra, e quindi non c' bisogno di indicarle.
*/


/********************************************************************
 * Finestra: apre finestra con area interna x1,y1..x2,y2, o la chiude
 *           se x1 == 0. Ritorna vero se ok.
 ********************************************************************/

int Finestra(int x1, int y1, int x2, int y2)
{
    static char *p = NULL;       /*pointer a memoria, ricorda se aperta <1>*/
    static int wx1,wx2,wy1,wy2;  /*coordinate esterne finestra aperta*/
    int ok,d;

    if (x1 > 0) {                                   /*apre finestra*/
        ok = (x1 > 1 && x2 < 80                     /*verifica <2>*/
              && y1 > 1 && y1 < 25
              && x2-x1 >= 0 && y2-y1 >= 0
              && p == NULL);
        if (ok) {
            wx1 = x1-1; wx2 = x2+1;                 /*coord. esterne <3>*/
            wy1 = y1-1; wy2 = y2+1;
            d = (wx2-wx1+1)*(wy2-wy1+1)*2;          /*calcola dimensione <4>*/
            p = malloc(d);                          /*chiede memoria <5>*/
            ok = (p != NULL);                       /*verifica se la riceve*/
            if (ok) {
                Jgettext(wx1,wy1,wx2,wy2,p);        /*ok, salva area <6>*/
                CorniceVuota(wx1,wy1,wx2,wy2);      /*disegna finestra*/
            }
        }
    } else {                                        /*chiude finestra*/
        ok = (p != NULL);                           /*solo se aperta <7>*/
        if (ok) {
            Jputtext(wx1,wy1,wx2,wy2,p);            /*recupera area <8>*/
            free(p);                                /*libera memoria <9>*/
            p = NULL;                               /*segna che  chiusa*/
        }
    }
    return(ok);                                     /*vero se aperta*/
}

/* Note su Finestra:
    <1> p punta all'area di memoria in cui viene copiato il contenuto
        dell'area video sottostante la finestra, in modo da poterla poi
        ripristinare. Ad evitare errori, se non c' una finestra aperta
        p contiene NULL. p deve essere statica perch va conservata tra
        una chiamata e l'altra, altrimenti sarebbe impossibile recuperare
        l'area video (come pure liberare la memoria occupata).
    <2> E' meglio controllare che le coordinate fornite abbiano senso,
        piuttosto che combinare guai. Inoltre, se p non  NULL vuol dire
        che c' gi una finestra aperta. Non se ne pu aprire pi d'una
        con questa funzione (ma vedi il programma FINESTRE).
    <3> Tiene conto della cornice attorno all'area usabile della finestra.
    <4> Per ogni carattere da salvare servono due byte: uno contiene il
        codice del carattere, l'altro gli attributi (es. colore).
    <5> Chiede d byte di memoria; se non sono disponibili, ritorna NULL.
    <6> Copia nell'area di memoria appena allocata l'area video sottostante,
        in modo da poterla ripristinare (vedi anche nota 4).
    <7> Se p  NULL, non c' nessuna finestra da chiudere.
    <8> Ripristina l'area video sottostante, facendo in effetti sparire la
        finestra e riportando il contenuto precedente del video.
    <9> La memoria usata per conservare l'area video non serve pi: viene
        quindi liberata. Questo  importante, altrimenti si esaurisce
        progressivamente la memoria disponibile. Mettiamo poi NULL in p
        per comodit: informa che non ci sono finestre aperte.
*/


/********************************************************************
 * CorniceVuota: disegna cornice x1,y1..x2,y2, cancella area interna.
 ********************************************************************/

void CorniceVuota(int x1, int y1, int x2, int y2)    /*<1>*/
{
    int h,v;

    h = x2-x1-1;                                    /*larghezza interna*/
    Jgotoxy(x1,y1);                                 /*bordo superiore*/
    Jputch(''); PutNchar(h,''); Jputch('');
    for (v = y1+1; v < y2; v++) {                   /*bordi laterali*/
        Jgotoxy(x1,v);
        Jputch(''); PutNchar(h,' '); Jputch('');
    }
    Jgotoxy(x1,y2);                                 /*bordo inferiore*/
    Jputch(''); PutNchar(h,''); Jputch('');
}

/* Note su CorniceVuota:
    <1> Questa funzione disegna in pratica una nuova finestra vuota,
        cancellando il contenuto dell'area video sottostante.
*/


/********************************************************************
 * PutNchar: stampa n (max 80) caratteri c di seguito.
 ********************************************************************/

void PutNchar(int n, int c)
{
    char buf[81];                                   /*per stampa rapida <1>*/

    if (n > 80)  n = 80;                            /*per sicurezza*/
    memset(buf,c,n);                                /*riempie buffer*/
    buf[n] = 0;                                     /*mette zero finale*/
    Jputs(buf);                                     /*e stampa*/
}

/* Note su PutNchar:
    <1> PutNchar  commentata nel file AGEUTIL della lezione E5.
*/


/********************************************************************
 * AspettaReturn: aspetta il tasto Return.
 ********************************************************************/

void AspettaReturn(void)
{
    Jclrkey();                                      /*cancella buffer*/
    while (Jgetkey(1) != '\r') {                    /*aspetta Return*/
        Jputch('\a');                               /*altrimenti beep*/
    }
}
