/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* FINUTIL: funzioni per aprire, chiudere, e scrivere in finestre <1> */


#include <stddef.h>       /*per definire NULL*/
#include <stdlib.h>
#include <string.h>

#include "finutil.h"
#include "Jconio.h"


/*Definizioni usate solo in questo modulo: */

#define MAXFIN  100                 /*numero massimo finestre <2>*/

typedef struct {                    /*definisce il tipo DatiFinestra <3>*/
    int  cx1,cy1,cx2,cy2;           /*rettangolo interno finestra*/
    int  scurx,scury;               /*posizione cursore salvata*/
    char *save;                     /*area sottostante salvata <4>*/
} DatiFinestra;


/* Variabili statiche di questo modulo, non visibili dall'esterno: <5>*/

static DatiFinestra stack[MAXFIN];  /*stack di finestre aperte <6>*/
static int nfin = 0;                /*numero finestre nello stack <7>*/


/* Funzioni statiche di questo modulo, non visibili dall'esterno: <8> */

static void CorniceVuota(int x1, int y1, int x2, int y2);
static void PutNchar(int n, int c);


/* Note sulle dichiarazioni:
    <1> Questo  un modulo del tutto isolato ed indipendente. Sono
        accessibili dall'esterno solo le tre funzioni ApreFinestra,
        ChiudeFinestra e Wputs, i cui prototipi sono definiti in FINUTIL.H.
    <2> MAXFIN  il numero massimo di finestre che possono essere aperte
        contemporaneamente.
    <3> Una variabile di tipo DatiFinestra racchiude tutte le informazioni
        specifiche di ogni finestra, che devono essere conservate sia per
        poter scrivere nella finestra, sia per chiuderla correttamente.
    <4> Questo  ovviamente solo un pointer: l'area di memoria viene
        allocata con malloc e risiede nello heap.
    <5> Queste variabili, essendo statiche ma dichiarate al di fuori da
        ogni funzione, sono visibili da tutte le funzioni del modulo
        FINUTIL, ma non da altri moduli.
    <6> L'array stack[] contiene i dati di tutte le finestre aperte. E'
        usato come uno stack, cio aggiungendo nuovi dati in fondo all'array
        e rileggendoli in ordine inverso a quello di scrittura.
    <7> nfin  il numero di finestre correntemente aperte, ed  anche usato
        come indice in stack[]. I dati dell'ultima finestra aperta sono in
        stack[nfin-1]. Si dice anche che nfin lavora da stack pointer (pi
        correttamente, in questo caso, stack index): indica il primo posto
        libero nello stack.
    <8> Queste funzioni sono usate solamente all'interno del modulo: si
        risparmia cos lavoro al linker, e si evitano conflitti con eventuali
        nomi identici nel programma che usa FINUTIL.
*/


/*********************************************************************
 * ApreFinestra: crea una nuova finestra con area interna x1,y1,x2,y2,
 *               ritorna il numero della finestra creata, zero se non
 *               la pu creare (non c' memoria,  finito lo stack
 *               delle finestre aperte, o gli argomenti sono errati).
 ********************************************************************/

int ApreFinestra(int x1, int y1, int x2, int y2)
{
    int wx1,wx2,wy1,wy2;  /*coordinate esterne finestra aperta*/
    DatiFinestra *fp;     /*punta a finestra corrente nello stack <1>*/
    int ok,d;

    if (nfin >= MAXFIN) {                           /*guarda se ci sta <2>*/
        ok = 0;
    } else {
        ok = (x1 > 1 && x2 < 80                     /*verifica <3>*/
              && y1 > 1 && y1 < 25
              && x2-x1 >= 0 && y2-y1 >= 0);

        if (ok) {                                   /*ok, crea finestra*/
            wx1 = x1-1; wx2 = x2+1;                 /*coord. esterne*/
            wy1 = y1-1; wy2 = y2+1;                 /*(aggiunge cornice)*/
            d = (wx2-wx1+1)*(wy2-wy1+1)*2;          /*calcola dimensione*/

            fp = stack+nfin;                        /*fin. corrente <4>*/
            ok = ( (fp->save = malloc(d)) != NULL); /*chiede memoria <5>*/

            if (ok) {
                Jgettext(wx1,wy1,wx2,wy2,fp->save); /*ok, salva area <6>*/
                fp->scurx = Jposx();                /*salva pos.cursore <7>*/
                fp->scury = Jposy();
                fp->cx1 = x1; fp->cy1 = y1;         /*coordinate interno*/
                fp->cx2 = x2; fp->cy2 = y2;
                CorniceVuota(wx1,wy1,wx2,wy2);      /*disegna finestra*/
                Jgotoxy(x1,y1);                     /*cursore nell'angolo*/
                nfin++;                             /*conta finestra <8>*/
            }
        }
    }
    return (ok) ? nfin : 0;                         /*numero finestra se ok*/
}

/* Note su ApreFinestra:
    <1> fp  un pointer ad una variabile di tipo DatiFinestra. Viene usato
        per puntare ai dati della finestra corrente, in modo da potervi
        accedere pi velocemente.
    <2> Se lo stack  finito, cio ci sono gi MAXFIN finestre aperte, non
        ne apre un'altra.
    <3> Controlla la validit delle coordinate ricevute.
    <4> stack+nfin equivale a &stack[nfin], cio all'indirizzo della prima
        posizione libera nell'array stack[].
    <5> Registra il pointer all'area di memoria allocata da malloc nella
        componente save della finestra corrente. Se  NULL, l'allocazione
        della memoria richiesta non  stata possibile (non ce n'era).
    <6> Se tutto  andato bene, copia nella memoria appena allocata l'area
        video che sar coperta dalla nuova finestra.
    <7> Registra anche la posizione corrente del cursore, in modo da poterla
        ripristinare quando la finestra verr chiusa.
    <8> nfin contiene il numero della nuova finestra. Se lo stack  pieno,
        va notato che stack[nfin] non punta ad un elemento valido. Questo
        non sar comunque mai usato, grazie al test della nota 2. Nella
        ChiudeFinestra, nfin sar decrementata prima di essere usata.
*/


/*********************************************************************
 * ChiudeFinestra: chiude l'ultima finestra aperta, ritorna numero
 *                 finestra corrente, zero se non ci sono fin. aperte.
 ********************************************************************/

int ChiudeFinestra(void)
{
    DatiFinestra *fp;     /*punta a finestra corrente nello stack <1>*/
    int ok;

    ok = (nfin > 0);                                /*se ce ne sono <2>*/
    if (ok) {                                       /*chiude l'ultima*/
        nfin--;                                     /*toglie da stack <3>*/
        fp = stack+nfin;                            /*fin. da chiudere <4>*/
        Jputtext(fp->cx1-1,fp->cy1-1,
                 fp->cx2+1,fp->cy2+1,fp->save);     /*recupera area <5>*/
        free(fp->save);                             /*libera memoria <6>*/
        Jgotoxy(fp->scurx,fp->scury);               /*risistema cursore <7>*/
    }
    return (ok) ? nfin : 0;                         /*numero finestra se ok*/
}

/* Note su ChiudeFinestra:
    <1> Vedi nota 1 di ApreFinestra.
    <2> Se non ci sono finestre aperte, non tenta di chiuderne e ritorna 0.
    <3> Dopo questo decremento, stack[nfin] contiene i dati della finestra
        corrente (vedi anche nota 8 di ApreFinestra).
    <4> Vedi nota 4 di ApreFinestra.
    <5> Rimette sul video l'area sottostante la finestra chiusa, che era
        stata salvata durante la ApreFinestra (vedi nota 6 relativa).
    <6> La memoria allocata con malloc (vedi nota 5 di ApreFinestra) deve
        essere liberata con free per evitare di esaurire, prima o poi,
        la memoria disponibile.
    <7> Rimette il cursore nella posizione in cui si trovava prima che la
        finestra venisse aperta (vedi nota 7 di ApreFinestra).
*/


/*********************************************************************
 * Wputs: scrive la stringa s nella finestra corrente, a partire dalla
 *        posizione attuale (l'ultima stampata). Ogni \n nella stringa
 *        manda a capo nella finestra. I caratteri che escono dai
 *        bordi della finestra non sono stampati.
 ********************************************************************/

void Wputs(char *s)
{
    DatiFinestra *fp;     /*punta a finestra corrente nello stack <1>*/
    int c,x,x2,ok;

    if (nfin > 0) {                                 /*se ce ne sono <2>*/
        fp = stack+nfin-1;                          /*punta a corrente <3>*/
        x2 = fp->cx2;                               /*legge limite x <4>*/
        x = Jposx();                                /*e posizione corrente*/

        ok = 1;                                     /*per ora, va bene*/
        while ( (c = *s++) != 0 && ok) {            /*scandisce stringa <5>*/
            if (c == '\n') {                        /*newline: a capo*/
                ok = (Jposy() < fp->cy2);           /*se non in fondo <6>*/
                if (ok) {
                    x = fp->cx1;                    /*esegue a capo <7>*/
                    Jgotoxy(x,Jposy()+1);
                }
            } else if (x++ <= x2) {                 /*se ci sta, stampa <8>*/
                Jputch(c);
            }
        }
    }
}

/* Note su Wputs:
    <1> Vedi nota 1 di ApreFinestra.
    <2> E' il caso di non stampare in una finestra inesistente.
    <3> Dato che stack[nfin]  il primo posto libero per registrare i dati
        di una nuova finestra, stack[nfin-1] contiene i dati della finestra
        corrente (vedi anche nota 4 di ApreFinestra).
    <4> Questo valore verr consultato spesso, per cui  pi efficiente
        copiarlo in una variabile semplice.
    <5> Se ha superato l'ultima linea stampabile della finestra, esce dal
        ciclo anche se non ha finito di scandire la stringa.
    <6> Un \n sull'ultima linea stampabile non manda a capo, ma fa diventare
        falsa la variabile ok.
    <7> Per andare a capo, porta la posizione orizzontale al bordo sinistro
        ed avanza quella verticale alla linea seguente.
    <8> Non stampa caratteri oltre il bordo destro dell'area interna.
*/



/************* Funzioni statiche, non visibili dall'esterno **************/


/********************************************************************
 * CorniceVuota: disegna cornice x1,y1..x2,y2, cancella area interna.
 ********************************************************************/

static void CorniceVuota(int x1, int y1, int x2, int y2)
{
    int h,v;

    h = x2-x1-1;                                    /*larghezza interna*/
    Jgotoxy(x1,y1);                                 /*bordo superiore*/
    Jputch(''); PutNchar(h,''); Jputch('');
    for (v = y1+1; v < y2; v++) {                   /*bordi laterali*/
        Jgotoxy(x1,v);
        Jputch(''); PutNchar(h,' '); Jputch('');
    }
    Jgotoxy(x1,y2);                                 /*bordo inferiore*/
    Jputch(''); PutNchar(h,''); Jputch('');
}


/********************************************************************
 * PutNchar: stampa n (max 80) caratteri c di seguito.
 ********************************************************************/

static void PutNchar(int n, int c)
{
    char buf[81];                                   /*per stampa rapida <1>*/

    if (n > 80)  n = 80;                            /*per sicurezza*/
    memset(buf,c,n);                                /*riempie buffer*/
    buf[n] = 0;                                     /*mette zero finale*/
    Jputs(buf);                                     /*e stampa*/
}

/* Note su PutNchar:
    <1> PutNchar  commentata nel file AGEUTIL della lezione E5.
*/
