/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* XDIR: lista tutti i file (anche nascosti) della directory corrente */


#include <stdio.h>
#include <string.h>

#include "Jdirect.h"          /*per funzioni di accesso a directory*/


#define  MAXLDIR  64          /*lunghezza massima nome directory*/


/* prototipi delle funzioni usate: */

void MostraDatiFile(FileInfo *fi);


/********************************************************************
 * main
 ********************************************************************/

main()
{
    FileInfo fi;          /*informazioni su un file <1>*/
    int  att;             /*attributi desiderati*/
    char dir[MAXLDIR+1];  /*nome della directory corrente*/

    att = FA_HIDDEN | FA_SYSTEM | FA_DIREC;         /*tutti i file <2>*/
    getcwd(dir,MAXLDIR);                            /*legge nome dir <3>*/
    printf("\nDirectory di %s:\n\n",dir);           /*lo mostra*/

    if (findfirst("*.*",&fi,att) != 0) {            /*legge primo file <4>*/
        printf("La directory  vuota.");            /*controlla*/
    } else {
        MostraDatiFile(&fi);                        /*mostra primo file <5>*/
        while (findnext(&fi) == 0) {                /*finch ne trova <6>*/
            MostraDatiFile(&fi);                    /*mostra i successivi*/
        }
    }
}

/* Note su main:
    <1> Il tipo FileInfo  definito in JDIRECT.H.
    <2> Ciascuna di queste costanti  un int con un solo bit a 1. Usando un
        OR bitwise (|) si mettono a 1 tutti e tre i bit, richiedendo di
        mostrare, oltre ai file normali, anche quelli nascosti, di sistema,
        e le (sotto)directory.
    <3> Mette nella stringa dir il nome della directory corrente. Una
        interessante miglioria a XDIR consisterebbe nella possibilit di
        indicare la directory da listare, come per il comando DIR del DOS.
        In questo caso bisognerebbe andarci con chdir e poi, finito il
        lavoro, ritornare alla directory originale con chdir(dir).
    <4> La findfirst cerca il primo file che risponde ai requisiti dati.
        Poich questi sono "*.*", vanno bene per qualunque file. Si potrebbe
        migliorare XDIR accettando come argomento la stringa contenente
        i file desiderati (es. XDIR *.C per listare solo i sorgenti C). La
        variabile att contiene gli attributi del file trovato (vedi nota 2).
    <5> La MostraDatiFile stampa una linea contenente informazioni sul file
        contenuto nella struttura fi.
    <6> La findnext cerca il file successivo rispondente agli stessi criteri
        indicati nella precedente findfirst. Ritorna zero se lo trova. L'uso
        di findfirst e findnext pu interferire con alcune operazioni del
        DOS effettuate con le chiamate 'vecchio stile' ereditate dal CP/M,
        che comunque sono in disuso (il problema riguarda un'area speciale
        chiamata DTA, Disk Transfer Area, usata da entrambe).
*/


/********************************************************************
 * MostraDatiFile: informazioni sul file *fi.
 ********************************************************************/

void MostraDatiFile(FileInfo *fp)   /*<1>*/
{
    static char *mesi[] = { "Gen", "Feb", "Mar", "Apr", "Mag", "Giu",
                            "Lug", "Ago", "Set", "Ott", "Nov", "Dic" };
    char n[13];  /*nome*/
    char s[4];   /*e suffisso*/
    char *p;

    strcpy(n,fp->name);                             /*copia nome <2>*/
    s[0] = 0;                                       /*nessun suffisso*/
    p = strchr(n,'.');                              /*cerca separatore*/
    if (p != NULL && p > n) {                       /*se lo trova <3>*/
        *p = 0;                                     /*termina nome in n[]*/
        strcpy(s,p+1);                              /*e copia suffisso*/
    }
    printf("%-9s%-3s",n,s);                         /*mostra nome espanso*/

    if (fp->attrib & FA_DIREC) {                    /*se directory <4>*/
        printf("%-8s","<DIR>");
    } else {                                        /*se file normale*/
        printf("%8ld",fp->size);                    /*mostra lunghezza <5>*/
    }

    printf("   %2d %3s %2d",fp->date.day,           /*mostra data <6>*/
           mesi[fp->date.month-1],fp->date.year+80);

    printf("  %2d:%02d:%02d   ",fp->time.hours,     /*mostra ora*/
           fp->time.min,fp->time.sec*2);

    if (fp->attrib & FA_SYSTEM) {                   /*mostra attributi <7>*/
        printf("System  ");
    }
    if (fp->attrib & FA_HIDDEN) {
        printf("Nascosto  ");
    }
    if (fp->attrib & FA_RDONLY) {
        printf("Protetto");
    }

    putchar('\n');                                  /*a capo*/
}

/* Note su MostraDatiFile:
    <1> fp  un pointer ad una struttura di tipo FileInfo, definita
        in JDIRECT.H.
    <2> Il campo name nella struttura FileInfo contiene il nome del file ed
        il relativo suffisso, separati da un punto (es. "XDIR.C"). Queste
        linee separano nome e suffisso, in modo da poterli poi stampare
        allineati e senza punto, come fa il comando DIR.
    <3> Se il punto separatore viene trovato nel primo carattere del nome
        (p == n) si tratta di uno dei nomi speciali di directory "." e "..",
        che vanno stampati tali e quali.
    <4> Se il file descritto nella struttura *fp  una directory, il
        corrispondente bit di fp->attrib  a 1. Per controllare un singolo
        bit si fa un AND bitwise (&) con l'adatta maschera, cio FA_DIREC.
    <5> fp->size  la lunghezza del file, in byte.
    <6> fp->date  una struttura contenente tre bit field, descritta in
        JDIRECT.H. In pratica, si tratta di un int in cui alcuni bit hanno
        un significato particolare. Il C permette di trattarlo come una
        struttura, e scrivendo fp->date.day si ottiene il valore contenuto
        nei bit da 0 a 4, come se si fosse scritto: (fp->date & 0x1f).
        fp->date.month legge invece il valore contenuto nei bit 5..8, come
        se si fosse scritto ((fp->date & 0x1e0) >> 5). Questo  descritto
        solo per spiegare cosa fa il C: non occorre saperlo per usare i
        bit field.
    <7> Per ogni speciale attributo presente (vedi nota 2), stampa la
        corrispondente descrizione.
*/
