/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* FIGURA: sceglie una figura in base alla stringa data */


#include <stdio.h>
#include <string.h>
#include <ctype.h>


int  InputStringa(char *prompt, char *s, int lmax); /*prototipi funzioni*/
void StrMinuscola(char *s);
void MostraFigura(char *f);


#define MAXLEN 20       /*lunghezza massima di una stringa introdotta*/


main()
{
    char figura[MAXLEN+1];  /*spazio per nome figura, pi zero finale*/
    int  finito;

    printf("\n Scelta di una figura:\n\n");

    do {
        InputStringa("Che figura geometrica desideri (\"Quadrato\",\n"
                     "\"Triangolo\", o \"Nessuna\" per smettere)? ",
                     figura,MAXLEN);                /*<1>*/
        StrMinuscola(figura);                       /*tutte minuscole <2>*/
        finito = (strcmp(figura,"nessuna") == 0);   /*confronta <3>*/
        if (! finito) {
            MostraFigura(figura);                   /*e disegna*/
        }
    } while (! finito);
}

/*  Note su main:
    <1> La combinazione \" non chiude la stringa, ma mette nella stringa il
        carattere di virgolette (").
    <2> Dato che la stringa figura[]  passata per riferimento (come tutti
        gli array), la StrMinuscola pu modificarne il contenuto (in questo
        caso, convertire tutte le lettere maiuscole a minuscole).
    <3> La strcmp ritorna zero se le due stringhe coincidono. Alcuni
        compilatori offrono anche funzioni per fare un confronto tra
        stringhe senza tener conto di maiuscole e minuscole (es. stricmp),
        ma queste funzioni non sono definite nello standard ANSI del C.
*/


/********************************************************************
 * MostraFigura: disegna figura richiesta in f, se disponibile.
 ********************************************************************/

void MostraFigura(char *f)      /*<1>*/
{
    putchar('\n');
    if (strcmp(f,"quadrato") == 0) {
       puts("+--------+\n"
            "|        |\n"
            "|        |\n"
            "|        |\n"
            "+--------+");
    } else if (strcmp(f,"triangolo") == 0) {
        puts("|\\\n"
             "|  \\\n"
             "|    \\\n"
             "|      \\\n"
             "+--------");
    } else {
        puts("- Mi spiace, al momento dispongo"
             " solamente di quadrati e triangoli.");
    }
    putchar('\n');
}

/*  Note su MostraFigura:
    <1> Una stringa pu essere ricevuta indifferentemente come array di
        char (f[]) o come indirizzo in un pointer a char (*f). La seconda
        forma  di uso pi comune.
*/


/********************************************************************
 * StrMinuscola: converte la stringa s a sole lettere minuscole.
 ********************************************************************/

void StrMinuscola(char *s)      /*<1>*/
{
    while ( (*s = tolower(*s)) != 0)                /*<2>*/
        s++;
}

/*  Note su StrMinuscola:
    <1> s riceve l'indirizzo del primo carattere della stringa, cio punta
        al primo carattere. La stringa , come sempre, passata per
        riferimento. Il pointer s  per locale, e quindi modificabile a
        piacimento senza per questo alterare il contenuto della stringa.
    <2> Modificando *s si altera invece il contenuto della stringa. Questa
        espressione  semplificabile in: while (*s = tolower(*s)), dato che
        in C un valore diverso da zero  considerato vero. E' per probabile
        che il compilatore segnali un warning (avvertimento), indicando
        la possibilit che si sia inavvertitamente usato = invece di == in
        un test (in questo caso, invece, era voluto). Il warning pu essere
        semplicemente ignorato, ma  bene evitarlo ed indicare esplicitamente
        il confronto con zero, come abbiamo fatto (non costa nulla: il codice
        prodotto non cambia se il compilatore non  stupido). Un programma
        deve sempre compilare senza warning, se possibile.
*/


/********************************************************************
 * InputStringa: mostra prompt, input stringa nell'array stringa s[]
 *               non oltre lmax caratteri, mette zero finale, ritorna
 *               lunghezza della stringa introdotta.
 ********************************************************************/

int InputStringa(char *prompt, char *s, int lmax)       /*<1>*/
{
    char buf[128];      /*buffer di linea*/
    int  ls;            /*lunghezza stringa*/

    do {
        printf("%s ",prompt);                       /*mostra prompt*/
        if (gets(buf) == NULL) {                    /*legge stringa*/
            ls = 0;                                 /*nulla se EOF*/
        } else {
            ls = strlen(buf);                       /*calcola lunghezza*/
        }
        if (ls == 0)  putchar('\a');                /*segnala errore*/
    } while (ls == 0);                              /*ripete se non ok*/

    if (lmax > 127)  lmax = 127;                    /*limite della gets <2>*/
    buf[lmax] = 0;                                  /*tronca a lmax <3>*/
    strcpy(s,buf);                                  /*copia in s <4>*/

    return ls;                                      /*ritorna lunghezza*/
}

/*  Note su InputStringa:
    <1> A differenza della InputStringa del programma SEPARA, questa fa uso
        della gets per leggere un'intera linea, della quale tiene poi solo
        il numero di caratteri richiesto.
    <2> E' bene controllare il valore dei parametri, quando una chiamata con
        un argomento errato potrebbe causare problemi. In questo caso, una
        chiamata con lmax maggiore di 127 causerebbe una scrittura oltre la
        fine della stringa buf[], in un punto indeterminato della memoria.
    <3> Scrivendo uno zero in una stringa, i caratteri successivi vengono
        ignorati quando la si stampa o la si copia: lo zero  infatti il
        terminatore. Se la stringa fosse pi corta di lmax, questo zero
        finirebbe pi avanti dello zero posto da gets (ma sempre all'interno
        di buf[]) e sarebbe quindi ignorato: la lettura di una stringa
        da parte delle funzioni standard termina comunque al primo zero.
    <4> Alternativamente, si poteva usare strncpy per copiare solo i primi
        lmax caratteri, aggiungendo poi lo zero finale in s[]. Dato che il
        C non dispone delle stringhe dinamiche del BASIC, occorre fare
        l'abitudine a queste manipolazioni dirette del contenuto degli
        array di char.
*/
