/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* GRAFILE: stampa su carta un'immagine grafica del contenuto di un file */


#include <stdio.h>
#include <string.h>
#include <ctype.h>


#define  MAXL     480             /*lunghezza massima di una linea <1>*/
#define  BUFL     MAXL+2          /*lunghezza di una stringa nel buffer <2>*/

#define  ESC      "\x1b"          /*Escape, come stringa <3>*/
#define  SP_STR   ESC "3\x18"     /*spaziatura verticale stretta <4>*/
#define  SP_NORM  ESC "2"         /*spaziatura normale*/
#define  GR_SING  ESC "K"         /*grafica in singola densit*/


/* prototipi funzioni: */

void StampaFile(FILE *f, FILE *pr);
void StampaStriscia(char (*s)[BUFL], int h, FILE *pr);


/* Note sulle dichiarazioni:
    <1> La lunghezza massima della linea nel file da stampare  anche il
        massimo numero di pixel in orizzontale, dato che per ogni carattere
        viene stampato un pixel. 480  in molte stampanti la massima
        definizione possibile nel modo grafico in singola densit.
    <2> Aggiungiamo un carattere per il \n che termina la linea (che la fgets
        legge quando incontra un CRLF, a differenza della gets da tastiera),
        ed uno per lo zero finale.
    <3> Come stringa, in modo che si combini con le stringhe adiacenti.
    <4> Due strisce grafiche sono pi vicine tra loro di due normali righe
        di caratteri: questo comando restringe la spaziatura verticale (tra
        le righe) al valore adeguato per i grafici, mentre il successivo la
        riporta al valore normale per la stampa di testi.
*/


/********************************************************************
 * main
 ********************************************************************/

main(int argc, char *argv[])
{
    FILE *f;        /*file da stampare*/
    FILE *pr;       /*stampante come file binario*/

    if (argc != 2) {
        puts("\n- Uso: grafile <file>");
    } else {

        f = fopen(argv[1],"r");                     /*apre in lettura <1>*/
        if (f == NULL) {                            /*avvisa se non trova*/
            printf("\nNon trovo il file \"%s\".\n",
                   argv[1]);
        } else {
            printf("\nSto stampando un'immagine grafica"
                   " del file \"%s\"...",argv[1]);

            pr = fopen("lpt1","wb");                /*apre stampante <2>*/
            if (pr == NULL) {
                puts("\n\nLa stampante non risponde.");   /*<3>*/
            } else {
                StampaFile(f,pr);                   /*ok, esegue stampa*/
                fclose(pr);                         /*chiude stampante*/
            }

            fclose(f);                              /*chiude file*/
            putchar('\n');
        }
    }
}

/* Note su main:
    <1> Il nome del file da stampare  il primo (ed unico) argomento
        indicato dall'utente dopo il nome del programma stesso.
    <2> Non usiamo il file predefinito stdprn, perch dobbiamo usare la
        stampante in modo binario: tutti i byte devono passare inalterati.
    <3> Con MS-DOS questa linea non sar probabilmente mai eseguita: se la
        stampante non risponde, o il DOS non se ne accorge oppure d un
        errore hardware. Questo programma non considera la possibilit di
        errori hardware: si potrebbe aggiungerla facilmente, applicando il
        meccanismo esaminato nel programma ERRORI della lezione E6.
*/


/********************************************************************
 * StampaFile: stampa immagine grafica del contenuto del file f
 *             sulla stampante pr, un pixel per ogni carattere
 *             (bianco se spaziatura, altrimenti nero).
 ********************************************************************/

void StampaFile(FILE *f, FILE *pr)
{
    char buf[8][BUFL];        /*buffer per 8 linee da stampare <1>*/
    int  i,ls,lm;
    char *p;

    fputs("\r\n",pr);                               /*deve essere a capo*/
    fputs(SP_STR,pr);                               /*spaziatura grafica <2>*/

    do {
        lm = 0;
        for (i = 0; i < 8; i++) {                   /*legge 8 linee*/
            if (fgets(buf[i],BUFL,f) != NULL) {     /*se lettura ok <3>*/
                p = strchr(buf[i],'\n');            /*toglie \n finale <4>*/
                if (p != NULL)  *p = 0;
                ls = strlen(buf[i]);                /*lunghezza stringa*/
                if (ls > lm)  lm = ls;              /*aggiorna lun. massima*/
            } else {                                /*altrimenti azzera <5>*/
                buf[i][0] = 0;
            }
        }

        if (lm > 0) {                               /*stampa se ce n' <6>*/
            StampaStriscia(buf,lm,pr);
        }

    } while (lm > 0);                               /*ancora se ce n' <7>*/

    fputs(SP_NORM,pr);                              /*spaziatura normale <8>*/
}

/* Note su StampaFile:
    <1> Questo  un array bidimensionale di caratteri, ed  considerato
        come un array di 8 array unidimensionali (stringhe), ciascuno di
        MAXL caratteri. Non si dovrebbero usare numeri nei programmi, ma
        questo 8  parte fondamentale dell'algoritmo stesso:  il numero di
        bit in un byte (oltre che di pixel stampati contemporaneamente),
        ed  ben difficile che possa essere cambiato senza dover modificare
        profondamente il programma.
    <2> Vedi nota 4 delle dichiarazioni.
    <3> buf[i]  un array unidimensionale di MAXL caratteri: vedi nota 1.
    <4> Il newline \n, che  presente alla fine di ogni linea tranne
        eventualmente l'ultima, viene trasformato in uno 0. La stringa viene
        qui in pratica accorciata di un carattere.
    <5> Se il file non contiene un numero di linee multiplo di 8, le
        ultime sono considerate come stringhe nulle.
    <6> Se  stata letta almeno una linea, stampa la striscia.
    <7> Smette quando non ci sono pi linee: il file  finito.
    <8> Rimette a posto la stampante (vedi nota 4 delle dichiarazioni).
*/


/********************************************************************
 * StampaStriscia: stampa le 8 stringhe dell'array s[][] in forma
 *                 grafica sulla stampante pr. La lunghezza della
 *                 stringa pi lunga  di nc caratteri.
 ********************************************************************/

void StampaStriscia(char (*s)[BUFL], int nc, FILE *pr)    /*<1>*/
{
    static p2[8] = { 128, 64, 32, 16, 8, 4, 2, 1 }; /*pesi degli aghi <2>*/
    int fine[8];
    int h,v;
    int b,c;

    fputs(GR_SING,pr);                              /*singola densit <3>*/
    fputc(nc%256,pr);                               /*numero di colonne <4>*/
    fputc(nc/256,pr);

    for (v = 0; v < 8; v++) {                       /*flag di fine <5>*/
        fine[v] = 0;
    }

    for (h = 0; h < nc; h++) {                      /*scandisce colonne*/
        b = 0;                                      /*byte grafico <6>*/
        for (v = 0; v < 8; v++) {                   /*esamina colonna <7>*/
            c = s[v][h];                            /*legge carattere*/
            if (c == 0)  fine[v] = 1;               /*segna se finita <8>*/
            if (! fine[v] && ! isspace(c)) {        /*decide se stampare <9>*/
                b += p2[v];                         /*s, mette pixel <10>*/
            }
        }
        fputc(b,pr);                                /*stampa colonna*/
    }

    fputs("\r\n",pr);                               /*fine striscia, a capo*/
}

/* Note su StampaStriscia:
    <1> L'array bidimensionale di char passato dalla StampaFile viene
        ricevuto come un pointer ad un array di BUFL char. In effetti quello
        che  passato  un pointer alla prima delle 8 stringhe lunghe MAXL
        char: le due dichiarazioni sono equivalenti. Incrementando s[],
        questo punter al successivo array di MAXL char, cio alla stringa
        contenente la successiva linea da stampare.
    <2> Sono i pesi dei bit contenuti in un byte, e quindi i valori da
        aggiungere al byte da mandare alla stampante per stampare i pixel
        corrisponenti: 128 stampa il pixel pi in alto, 1 quello pi in basso.
    <3> Predispone la stampante alla stampa di una striscia grafica in
        singola densit.
    <4> Invia di seguito la parte bassa e la parte alta di nc. Dopo questi
        due byte, la stampante attende nc byte corrispondenti alle colonne
        di 8 pixel da stampare.
    <5> Questi 8 flag servono per ricordare se ciascuna delle 8 linee da
        esaminare  arrivata alla fine. Le linee possono infatti avere
        lunghezza differente (la pi lunga ha nc caratteri), e non si
        devono esaminare oltre lo 0 finale.
    <6> In b viene costruito il byte grafico da inviare alla stampante, che
        rappresenta una colonna di 8 pixel e conterr 1 per ciascun pixel da
        stampare.
    <7> Per costruire una colonna, esamina il corrispondente carattere delle
        8 linee che la formano.
    <8> Quando incontra lo 0 finale di una linea, lo segna in modo da non
        stampare pi pixel per quella linea.
    <9> Per le linee che non sono ancora state esaminate fino in fondo,
        stampa un pixel se il carattere corrente non  un carattere di
        spaziatura (es. spazio,Tab). Se fine[v] segna che la linea  gi
        terminata, il carattere c non viene nemmeno esaminato.
   <10> Aggiunge al byte in costruzione il peso corrispondente al pixel da
        stampare.
*/
