/*
   Questo file fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* GRAFUTIL: funzioni per input e varie in modo grafico */


#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include "Jgraph.h"
#include "Jconio.h"

#include "grafutil.h"


#define  CURS  0x16    /*simbolo per simulare il cursore*/

static col;            /*colore usato da tutte le funzioni di questo modulo*/


/********************************************************************
 * GsetUtilColor: sceglie il colore usato da tutte le funzioni di
 *                questo modulo.
 ********************************************************************/

void GsetUtilColor(int c)
{
    col = c;                                        /*<1>*/
}

/* Note su GsetUtilColor:
    <1> Dato che col  una variabile statica del modulo GRAFUTIL, si pu
        usare come una globale ma non  visibile dall'esterno. Il colore
        viene quindi cambiato attraverso un'apposita funzione.
*/


/********************************************************************
 * GinputInt: input valore intero compreso tra mi e ma in colore col,
 *            senza default. Ritorna valore, o zero se solo Return.
 ********************************************************************/

int GinputInt(int mi, int ma)
{
    int  val;
    char buf[6];
    int  ok;

    do {
        Jgputs("     ",col);                        /*pulisce campo <1>*/
        Jgotoxy(Jgposx()-5,Jgposy());               /*torna dov'era*/
        *buf = 0;                                   /*niente default*/
        GinputStringa(buf,5);                       /*max 5 cifre <2>*/
        if (*buf == 0) {                            /*solo Return: 0*/
            val = 0;
            ok = 1;
        } else {
            sscanf(buf,"%d",&val);                  /*legge valore*/
            ok = (val >= mi && val <= ma);          /*lo controlla*/
        }
        if (! ok)  Jputch('\a');                    /*segnala ev. errore*/
    } while (! ok);                                 /*ripete se errore*/

    return val;                                     /*ritorna valore*/
}

/* Note su Ginputint:
    <1> La funzione accetta al massimo un input di 5 cifre.
    <2> La GinputStringa simula un cursore lampeggiante.
*/


/********************************************************************
 * GinputStringa: mostra campo di input con default in colore col,
 *                input controllato di stringa nell'array s[] non
 *                oltre lmax caratteri, accetta correzioni con
 *                backspace, mette zero finale, ritorna lunghezza
 *                della stringa introdotta, lascia cursore dov'era.
 ********************************************************************/

int GinputStringa(char *s, int lmax)                /*<1>*/
{
    enum { CR = 13, BS = 8, BACK = 1075 };          /*codici tasti*/

    char buf[81];       /*buffer di linea*/
    int  ls;            /*lunghezza stringa*/
    int  px,py;
    int  i,c,ok;

    px = Jgposx(); py = Jgposy();                   /*legge posizione*/
    Jgputs(s,col);                                  /*mostra default*/
    ls = strlen(s);                                 /*lunghezza corrente*/

    s += ls;                                        /*punta alla fine*/
    Jclrkey();                                      /*cancella buffer*/
    do {
        c = Ggetkey(CURS);                          /*legge carattere <2>*/
        ok = (c == CR);
        if ( (c == BS || c == BACK) && ls > 0) {
            ls--;
            s--;
            Jgotoxy(px+ls,py);
            Jgputch(' ',col);
            Jgotoxy(px+ls,py);
        } else if (isprint(c) && ls < lmax) {
            *s++ = c;
            ls++;
            Jgputch(c,col);
        } else if (! ok) {
            Jputch('\a');                           /*non Jgputch <3>*/
        }
    } while (! ok);

    *s = 0;                                         /*termina stringa*/
    Jgotoxy(px,py);                                 /*torna dov'era*/

    return ls;                                      /*ritorna lunghezza*/
}

/* Note su GinputStringa:
    <1> GinputStringa  simile a quella presentata nel programma INPCON
        della lezione E1, con la differenza che non usa l'inverse (in
        modo grafico non  facile ottenerlo in maniera portatile).
    <2> Il carattere viene letto simulando un cursore lampeggiante, dato
        che in modo grafico quello normale non  disponibile.
    <3> Jgputch non riconosce \a: stamperebbe il simbolo corrispondente.
*/


/********************************************************************
 * Ggetkey: aspetta un carattere in modo grafico con cursore simulato
 *          con simbolo cs, lampeggiante in colore col.
 ********************************************************************/

int Ggetkey(int cs)
{
    int tlamp = 4;        /*tempo di lampeggio cursore, in tick <1>*/
    int curs = 0;         /*cursore attualmente spento <2>*/
    clock_t t;
    int x,y,c;

    x = Jgposx(); y = Jgposy();                     /*legge pos. corrente*/
    t = clock();                                    /*tempo attuale*/

    while (! kbhit()) {                             /*aspetta un tasto <3>*/
        while (clock() < t && ! kbhit());           /*aspetta lampeggio <4>*/
        curs = ! curs;                              /*lampeggia <5>*/
        Jgputch(curs ? cs : ' ',col);               /*disegna cursore <6>*/
        Jgotoxy(x,y);
        t = clock()+tlamp;                          /*prepara lampeggio <7>*/
    }

    if (curs) {                                     /*toglie se visibile <8>*/
        Jgputch(' ',col);
        Jgotoxy(x,y);
    }

    return Jgetkey(0);                              /*legge carattere <9>*/
}

/* Note su Ggetkey:
    <1> Il cursore viene mostrato/nascosto ogni tlamp tick del timer.
    <2> La variabile curs ricorda se il cursore  visibile o no.
    <3> Il ciclo di lampeggio del cursore viene continuamente eseguito
        finch il buffer di tastiera rimane vuoto.
    <4> Aspetta il momento di invertire il cursore. Se viene premuto un
        tasto, smette subito di aspettare per non dare all'utente la
        sensazione di un ritardo nella risposta al tasto premuto. Al primo
        passaggio, il tempo  gi scaduto e non fa ritardi.
    <5> Inverte lo stato del cursore. Al primo passaggio lo accende.
    <6> Disegna fisicamente il cursore sul video, usando il carattere cs
        (se acceso) o uno spazio (se spento).
    <7> Calcola l'istante per l'inversione successiva del cursore.
    <8> Evita di dimenticare sul video un cursore visibile.
    <9> Dato che kbhit ha ritornato vero, c' un carattere in attesa:
        lo legge.
*/


/********************************************************************
 * GclearEOL: cancella fino a fine linea, resta dov'era.
 ********************************************************************/

void GclearEOL(void)
{
    int px,x;

    x = px = Jgposx();                              /*legge pos. corrente*/
    while (x++ < 80) {                              /*pulisce con spazi*/
        Jgputch(' ',col);
    }
    Jgotoxy(px,Jgposy());                           /*torna dov'era*/
}


/********************************************************************
 * GputsAt: stampa s alla posizione data, nel colore col.
 ********************************************************************/

void GputsAt(int x, int y, char *s)
{
    Jgotoxy(x,y);
    Jgputs(s,col);
}
