/*
   Questa libreria fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* JCONIO: 'libreria' di funzioni per Input/Ouput su console   */
/*         compilabili sia con Turbo C 2.0 che con Quick C 2.0 */
/*         Versione E6: aggiunte Jgettext, Jputtext.           */


#include "Jconio.h"                         /*prototipi funzioni e costanti*/

#include <dos.h>                            /*richiesto per chiamate a BIOS*/


/********************************************************************
 * Funzioni per il Turbo C 2.0 non definite come macro in Jconio.h
 ********************************************************************/

#ifdef __TURBOC__                           /*compila se Turbo C*/

  /* Jgettextcolor: ritorna colore corrente di foreground */

  int Jgettextcolor(void)
  {
      struct text_info ti;

      gettextinfo(&ti);                     /*legge dati correnti*/
      return (ti.attribute & 0x8f);         /*estrae foreground+blink*/
  }

  /* Jgettextback: ritorna colore corrente di background */

  int Jgettextback(void)
  {
      struct text_info ti;

      gettextinfo(&ti);                     /*legge dati correnti*/
      return ((ti.attribute & 0x70) >> 4);  /*estrae background*/
  }


/********************************************************************
 * Funzioni per il Quick C 2.0 non definite come macro in Jconio.h
 ********************************************************************/

#else                                       /*compila se Quick C*/

  /* Jputch: stampa il carattere c */

  void Jputch(int c)
  {
      char b[2];                            /*stringa di 2 char*/

      if (c == '\a') {                      /*se beep, lo suona*/
          putch(c);
      } else {
          b[0] = (char)c; b[1] = 0;         /*costruisce stringa*/
          _outtext(b);                      /*e la stampa*/
      }
  }

  /* Jposx: ritorna posizione orizzontale (colonna) del cursore */

  int Jposx(void)
  {
      struct rccoord rc;                    /*contiene posizione x,y*/

      rc = _gettextposition();              /*legge posizione*/
      return(rc.col);                       /*ritorna colonna*/
  }

  /* Jposx: ritorna posizione verticale (riga) del cursore */

  int Jposy(void)
  {
      struct rccoord rc;                    /*contiene posizione x,y*/

      rc = _gettextposition();              /*legge posizione*/
      return(rc.row);                       /*ritorna riga*/
  }

  /* Jgettext: legge area video da x1,y1 a x2,y2 in memoria puntata da p:
               per ogni carattere legge 2 byte: codice ed attributi.
  */

  void Jgettext(int x1, int y1, int x2, int y2, char *p)
  {
      union REGS ir,or;                     /*per interfaccia BIOS*/
      int ch,cv;                            /*posizione corrente cursore*/
      int h,v;

      ch = Jposx();                         /*registra posizione cursore*/
      cv = Jposy();
      ir.h.ah = 0x08;                       /*Read Character at Cursor*/
      ir.h.bh = 0;                          /*pagina video 0*/
      for (v = y1; v <= y2; v++) {          /*scandisce per righe*/
          for (h = x1; h <= x2; h++) {      /*e per colonne*/
              Jgotoxy(h,v);                 /*posiziona cursore*/
              int86(0x10,&ir,&or);          /*chiama BIOS Video*/
              *p++ = or.h.al;               /*copia codice carattere*/
              *p++ = or.h.ah;               /*ed attributi*/
          }
      }
      Jgotoxy(ch,cv);                       /*riposiziona cursore*/
  }

  /* Jputtext: scrive da memoria puntata da p in area video x1,y1..x2,y2:
               per ogni carattere vuole 2 byte: codice ed attributi.
  */

  void Jputtext(int x1, int y1, int x2, int y2, char *p)
  {
      union REGS ir,or;                     /*per interfaccia BIOS*/
      int ch,cv;                            /*posizione corrente cursore*/
      int h,v;

      ch = Jposx();                         /*registra posizione cursore*/
      cv = Jposy();
      ir.h.ah = 0x09;                       /*Write Character and Attribute*/
      ir.h.bh = 0;                          /*pagina video 0*/
      ir.x.cx = 1;                          /*un carattere alla volta*/
      for (v = y1; v <= y2; v++) {          /*scandisce per righe*/
          for (h = x1; h <= x2; h++) {      /*e per colonne*/
              Jgotoxy(h,v);                 /*posiziona cursore*/
              ir.h.al = *p++;               /*codice del carattere*/
              ir.h.bl = *p++;               /*ed attributi*/
              int86(0x10,&ir,&or);          /*chiama BIOS Video*/
          }
      }
      Jgotoxy(ch,cv);                       /*riposiziona cursore*/
  }


#endif                                      /*fine compilazione condizionale*/


/********************************************************************
 * Jcursor: spegne il cursore se vis  falsa (0), lo accende se vera.
 * Il controllo dell'accensione del cursore non  disponibile in
 * nessuna delle librerie fornite, per cui lo costruiamo usando
 * chiamate dirette al BIOS, compilabili da entrambi i compilatori.
 ********************************************************************/

void Jcursor(int vis)
{
    static int v = 1;                       /*di solito, cursore visibile*/
    static unsigned char start;             /*linea iniziale del cursore*/
    static unsigned char end;               /*linea finale del cursore*/

    union REGS ir;                          /*per interfaccia BIOS*/
    union REGS or;

    if (vis != v) {                         /*agisce solo se cambia*/
        if (vis) {                          /*se deve mostrarlo*/
            ir.h.ah = 0x01;                 /*Set Cursor Size*/
            ir.h.ch = start;                /*valori salvati prima*/
            ir.h.cl = end;
            int86(0x10,&ir,&or);            /*chiama BIOS Video*/
        } else {                            /*se deve nasconderlo*/
            ir.h.ah = 0x03;                 /*Read Cursor Pos, Size*/
            ir.h.bh = 0;                    /*pagina video 0*/
            int86(0x10,&ir,&or);            /*chiama BIOS Video*/
            start = or.h.ch;                /*registra dati cursore*/
            end = or.h.cl;
            ir.h.ah = 0x01;                 /*Set Cursor Size*/
            ir.h.ch = 0x20;                 /*nessun cursore*/
            int86(0x10,&ir,&or);            /*chiama BIOS Video*/
        }
        v = vis;                            /*registra nuovo stato*/
    }
}


/********************************************************************
 * Jgetkey: legge qualunque tasto, inclusi i tasti speciali con
 * doppio codice, per i quali ritorna 1000 pi il codice di scansione.
 * Se cur==vero, accende il cursore prima dell'input e lo spegne
 * dopo l'input, se cur==falso non tocca il cursore.
 ********************************************************************/

int Jgetkey(int cur)
{
    int c;

    if (cur)  Jcursor(1);                   /*mostra cursore se richiesto*/

    c = getch();                            /*legge un tasto*/
    if (c == 0) {                           /*se speciale*/
        c = getch()+1000;                   /*ne legge un altro, corregge*/
    }

    if (cur)  Jcursor(0);                   /*nasconde cursore se richiesto*/
    return c;                               /*ritorna tasto*/
}


/********************************************************************
 * Jclrkey: cancella il buffer di tastiera leggendo ed ignorando
 * tutti i caratteri contenuti.
 ********************************************************************/

void Jclrkey(void)
{
    while (kbhit()) {                       /*finch ci sono caratteri*/
        getch();                            /*li legge e li ignora*/
    }
}
