/*
   Questa libreria fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* JGRAPH: 'libreria' (in realt un normale modulo) di funzioni grafiche */
/*         compilabili con Turbo C 2.0 e con Quick C 2.0, versione E7.   */


#include <dos.h>

#include "Jgraph.h"                         /*prototipi funzioni e costanti*/


/********************************************************************
 * Funzioni per il Turbo C 2.0 non definite come macro in Jgraph.h
 ********************************************************************/

#ifdef __TURBOC__                                   /*compila se Turbo C*/

  #include <stddef.h>                               /*per definire NULL <1>*/

  /* Jinitgraph: sceglie il modo grafico pi definito, ritorna vero se ok */

  int Jinitgraph(void)                              /*<2>*/
  {
      int driver = DETECT;                          /*sceglie da s*/
      int modo;

      initgraph(&driver,&modo,NULL);                /*avvia grafica*/
      return (driver > 0);                          /*vero se ok*/
  }

  /* Note su Jinitgraph:
      <1> <stddef.h>  uno header che contiene le definizioni di alcune
          costanti fondamentali. Lo usiamo per definire NULL senza dover
          includere il ben pi corposo <stdio.h>, che potrebbe aumentare
          inutilmente le dimensioni del programma. Il valore di NULL non
           noto, ed  quindi sconsigliabile definirlo direttamente.
      <2> La funzione  commentata nel programma RIMBALZA della lezione E4.
  */


/********************************************************************
 * Funzioni per il Quick C 2.0 non definite come macro in Jconio.h
 ********************************************************************/

#else                                               /*compila se Quick C*/

  /* Jinitgraph: sceglie il modo grafico pi definito, ritorna vero se ok */

  int Jinitgraph(void)
  {
      int modi[] = {          /*lista dei modi in ordine di preferenza*/
          _VRES16COLOR,       /*VGA 640 x 480*/
          _ERESCOLOR,         /*EGA colore 640 x 350*/
          _ERESNOCOLOR,       /*EGA monocromatica 640 x 350*/
          _ORESCOLOR,         /*Olivetti 640 x 400*/
          _HERCMONO,          /*Hercules 720 x 348*/
          _HRESBW,            /*CGA 640 x 400*/
          _DEFAULTMODE        /*segna fine lista*/
      };
      int i;

      i = 0;                                        /*avvia grafica*/
      while (modi[i] != _DEFAULTMODE
        && _setvideomode(modi[i]) == 0) {
          i++;
      }

      return (modi[i] != _DEFAULTMODE);             /*1 se ok*/
  }

  /* Note su Jinitgraph:
      <1> La funzione  commentata nel programma RIMBALZA della lezione E4.
  */

  /* JgetQuickGraphInfo: dimens. x se n==1, y se n==2, n. colori se n==3. */

  int JgetQuickGraphInfo(int n)
  {
      static int prima = -1;           /*vera se prima chiamata*/
      static struct videoconfig dv;    /*dati del modo video corrente*/

      if (prima) {                                  /*prima chiamata*/
          _getvideoconfig(&dv);                     /*legge dati video*/
          prima = 0;                                /*non rifarlo pi*/
      }

      return (n == 1) ? dv.numxpixels :
             (n == 2) ? dv.numypixels : dv.numcolors;
  }


#endif                                      /*fine compilazione condizionale*/


/********************************************************************
 * Funzioni compilabili sia col Turbo C 2.0 che col Quick C 2.0.
 ********************************************************************/

/* Jgputch: stampa carattere c in colore col */

void Jgputch(int c,int col)
{
    union REGS r;     /*registri per chiamata BIOS*/

    r.h.ah = 0x0e;                                  /*Write Char as TTY*/
    r.h.al = c;                                     /*codice carattere*/
    r.h.bl = col;                                   /*e colore*/
    r.h.bh = 0;                                     /*pagina video 0*/
    int86(0x10,&r,&r);                              /*chiama BIOS video*/
}

/* Jgputs: stampa stringa s in colore col */

void Jgputs(char *s,int col)
{
    int c;

    while ((c = *s++) != 0) {
        Jgputch(c,col);
    }
}

/* Jgposx: ritorna la posizione orizzontale corrente */

int  Jgposx(void)
{
    union REGS r;     /*registri per chiamata BIOS*/

    r.h.ah = 0x03;                                  /*Read Cursor Position*/
    r.h.bh = 0;                                     /*pagina video 0*/
    int86(0x10,&r,&r);                              /*chiama BIOS video*/
    return r.h.dl+1;                                /*ritorna posizione h*/
}

/* Jgposx: ritorna la posizione verticale corrente */

int  Jgposy(void)
{
    union REGS r;     /*registri per chiamata BIOS*/

    r.h.ah = 0x03;                                  /*Read Cursor Position*/
    r.h.bh = 0;                                     /*pagina video 0*/
    int86(0x10,&r,&r);                              /*chiama BIOS video*/
    return r.h.dh+1;                                /*ritorna posizione v*/
}
