/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* MEMVIDEO: accesso diretto alla memoria video */


#include <stdio.h>
#include <ctype.h>
#include <conio.h>
#include <dos.h>


#define  HD  80     /*dimensione orizzontale video*/
#define  VD  25     /*dimensione verticale video*/


/* definisce la macro per costruire pointer, per il Quick C: <1> */

#ifndef MK_FP
  #define  MK_FP(seg,ofs)  ((void far *) \
                           (((unsigned long)(seg) << 16) | (unsigned)(ofs)))
#endif


/* definisce il tipo CarVideo: un carattere nella memoria video: <2>*/

typedef struct {
    unsigned int c    : 8;  /*un byte di codice del carattere*/
    unsigned int colf : 4;  /*e uno di attributi: colore di foreground*/
    unsigned int colb : 3;  /*colore di background*/
    unsigned int lamp : 1;  /*lampeggio*/
} CarVideo;


/* definisce il tipo 'far pointer a memoria video', vista come array: <3>*/

typedef CarVideo (far *PtrMemVid)[HD];      /*pointer a una linea video <4>*/


/* pointer globale alla memoria video: */

PtrMemVid vp;                               /*<5>*/


/* prototipi funzioni: */

PtrMemVid IndirizzoMemVideo(void);
void AvanzaLettere(void);
void CambiaColori(void);
void AspettaReturn(void);
void CancellaVideo(void);
void ScriveStringa(int h, int v, int col, int back, char *s);


/* Note sulle dichiarazioni:
    <1> La macro MK_FP, definita nel Turbo C ma assente nel Quick C,
        costruisce un pointer far a partire dal numero di paragrafo (seg)
        e dall'offset (ofs). E' strettamente non portatile: dipende da come
        un pointer far  rappresentato in memoria (in questo caso da un
        unsigned long, 32 bit).
    <2> Ogni carattere del video  controllato da due byte in memoria: uno
        per il codice ed uno per gli attributi. Questa struttura rappresenta,
        in modo non portatile, le sue varie componenti. Accedere alla memoria
        video in questo modo non  particolarmente efficiente, ma  facile
        e garantisce contro molti possibili errori.
    <3> Con questa definizione, la memoria video  usabile come un array
        bidimensionale (matrice rettangolare) di strutture CarVideo. Ancora
        una volta, l'efficienza  sacrificata alla pulizia di programmazione.
    <4> PtrMemVid  definito come pointer far ad un array di HD (cio 80)
        CarVideo. Punta cio ad una linea video, ed applicando un secondo
        indice si pu indirizzare un singolo carattere. Incrementando un
        PtrMemVid, questo punta alla successiva linea di HD CarVideo.
    <5> vp verr inizializzato con l'indirizzo del primo byte della memoria
        video, in modo da poterlo usare come array di VD x HD CarVideo,
        ciascuno dei quali rappresenta un carattere.
*/


/********************************************************************
 * main
 ********************************************************************/

main()
{
    vp = IndirizzoMemVideo();                       /*punta a mem.video <1>*/

    printf("\nLa memoria video di questa pagina"
           " si trova all'indirizzo %Fp.",vp);      /*mostra pointer <2>*/

    printf("\n\nPremi Return per cambiare ogni lettera"
           " visibile sul video\n"
           "nella successiva lettera dell'alfabeto: ");

    AspettaReturn();
    AvanzaLettere();

    printf("\n\nPremi nuovamente Return per cambiare "
           "i colori di background\n"
           "e di foreground di ogni carattere del video: ");

    AspettaReturn();
    CambiaColori();

    ScriveStringa(1,24,0,7," Premi ancora una volta Return per cancellare  ");
    ScriveStringa(1,25,0,7," il video e rimettere a posto i colori:        ");

    AspettaReturn();
    CancellaVideo();

    putchar('\n');
}

/* Note su main:
    <1> Vedi nota 5 delle dichiarazioni. La IndirizzoMemVideo determina la
        posizione della memoria video, e restituisce un pointer a questa.
    <2> La specifica %p stampa un pointer nel formato del computer usato.
        Nei PC IBM e compatibili, %Fp stampa un pointer far nella forma
        segmento:offset.
*/


/********************************************************************
 * IndirizzoMemVideo: ritorna un far pointer a inizio memoria video.
 ********************************************************************/

PtrMemVid IndirizzoMemVideo(void)
{
    union REGS r;       /*per chiamata BIOS*/
    unsigned   ob;      /*offset base memoria video <1>*/

    r.h.ah = 0x0f;                                  /*Get Current Video Mode*/
    int86(0x10,&r,&r);                              /*chiama BIOS video <2>*/
    ob = (r.h.al == 7) ? 0x0000 : 0x8000;           /*determina offset <3>*/
    ob += (r.h.bh*0x800);                           /*conta pagina <4>*/

    return (PtrMemVid) MK_FP(0xb000,ob);;           /*ritorna pointer <5>*/
}

/* Note su IndirizzoMemVideo:
    <1> Nei PC IBM e compatibili, la memoria video usata per i caratteri
        del modo testo pu iniziare a b000:0000 (modo 7: monocromatico),
        oppure a b800:0000 (gli altri modi). Notiamo che b800:0000 equivale
        a b000:8000, quindi possiamo fissare il segmento al paragrafo b000
        e variare solo l'offset in base al modo video corrente.
    <2> La funzione 0fh dell'interrupt 10h (funzioni video) ritorna il numero
        del modo video correntemente attivo.
    <3> Vedi nota 1.
    <4> Alcune schede hanno a disposizione varie 'pagine' video, che in
        memoria sono spaziate di 2K (800h). Per quanto siamo praticamente
        certi che sia selezionata la pagina zero, non ci costa nulla usare
        il numero della pagina corrente che il BIOS ritorna in bh, ed
        aggiustare l'offset per puntare con certezza alla pagina attiva.
    <5> La macro MK_FP (vedi nota 1 delle dichiarazioni) costruisce un
        pointer far a partire dal numero di segmento e dall'offset. Il
        primo  sempre b000, mentre il secondo  il valore calcolato in ob.
*/


/********************************************************************
 * AvanzaLettere: sostituisce ogni lettera visibile con la successiva
 *                in ordine alfabetico, non tocca gli altri caratteri.
 *                Accede alla memoria video con il pointer globale vp.
 ********************************************************************/

void AvanzaLettere(void)
{
    int  h,v;

    for (v = 0; v < VD; v++) {
        for (h = 0; h < HD; h++) {
            if (isalpha(vp[v][h].c)) {              /*<1>*/
                vp[v][h].c++;
            }
        }
    }
}

/* Note su AvanzaLettere:
    <1> vp[v][h].c  il codice del CarVideo che sta alla riga v ed alla
        colonna h. Per quanto questa sintassi sia molto semplice e leggibile,
         spaventosamente inefficiente (anche se pi veloce delle chiamate
        al BIOS). Tra l'altro, isalpha potrebbe valutare due volte il suo
        argomento, causando la ripetizione delle lente operazioni sui bit
        field. Si potrebbero migliorare le cose definendo un pointer ad
        un CarVideo, e puntandolo all'inizio della linea corrente ad ogni
        ciclo esterno. La massima velocit si otterrebbe con un semplice
        pointer a char, abbandonando tutta la costruzione che consente di
        usare la memoria video come un array di strutture. Una simile
        soluzione va comunque adottata solo se quella usata  insufficiente:
        non conviene ridurre la leggibilit, o immergersi in dettagli
        hardware, se non se ne ricava alcun guadagno: su molti computer
        la velocit di questa funzione  pi che accettabile. Su alcune
        schede CGA l'accesso diretto alla memoria video pu causare
        fastidiosi disturbi intermittenti (flicker).
*/


/********************************************************************
 * CambiaColori: cambia il colore di foreground e background di ogni
 *               carattere del video. Accede alla memoria video con
 *               il pointer globale vp.
 ********************************************************************/

void CambiaColori(void)
{
    int  h,v;

    for (v = 0; v < VD; v++) {
        for (h = 0; h < HD; h++) {
            vp[v][h].colb = v%8;                    /*<1>*/
            vp[v][h].colf = h%16;
        }
    }
}

/* Note su CambiaColori:
    <1> Vedi nota 1 di AvanzaLettere. Notiamo anche l'uso di due lente
        operazioni di modulo, facilmente sostituibili con incrementi e
        test se fosse richiesto un aumento di efficienza.
*/


/********************************************************************
 * CancellaVideo: riempie il video di spazi, con colore di foreground
 *                bianco (7) e background nero (0).
 ********************************************************************/

void CancellaVideo(void)
{
    CarVideo cv;
    int  h,v;

    cv.c = ' ';                                     /*prepara il carattere*/
    cv.colf = 7;                                    /*bianco*/
    cv.colb = 0;                                    /*su fondo nero*/
    cv.lamp = 0;                                    /*no lampeggio*/

    for (v = 0; v < VD; v++) {
        for (h = 0; h < HD; h++) {
            vp[v][h] = cv;                          /*riempie il video <1>*/
        }
    }
}

/* Note su CancellaVideo:
    <1> Vedi nota 1 di AvanzaLettere. Si poteva anche considerare il fatto
        che le righe successive del video sono contigue, e fare un solo
        ciclo che agisse su un array unidimensionale di VD x HD elementi.
        Ma, visto che la velocit  pi che sufficiente, perch complicarsi
        la vita inutilmente?
*/


/********************************************************************
 * ScriveStringa: scrive la stringa s[] direttamente in memoria video,
 *                alla posizione h,v con i colori dati (1 sola linea).
 ********************************************************************/

void ScriveStringa(int h, int v, int col, int back, char *s)
{
    CarVideo cv;      /*carattere di lavoro*/

    cv.colf = col;                                  /*questi non cambiano*/
    cv.colb = back;
    cv.lamp = 0;

    h--; v--;                                       /*parte da zero*/
    while (*s) {                                    /*scandisce stringa*/
        cv.c = *s++;                                /*prepara carattere*/
        vp[v][h++] = cv;                            /*e lo mostra*/
    }
}


/********************************************************************
 * AspettaReturn: aspetta che venga premuto il tasto Return.
 ********************************************************************/

void AspettaReturn(void)
{
    while (kbhit()) {                               /*pulisce buffer*/
        getch();
    }
    while (getch() != '\r') {                       /*aspetta tasto*/
        putch('\a');                                /*segnala errore*/
    }
}
