/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* PAROLA: gioco per due persone basato sulle stringhe */


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <ctype.h>


void Titoli(void);                                  /*prototipi funzioni*/
void CancellaSchermo(void);
int  PreparaPartita(void);
void InputNomeGioca(int g);
void InputParola(int g);
int  ProvaLettera(int g);
int  InputTentativo(int g);
void MostraSituazione(void);
void Vittoria(int g);
void PutNchar(int n, int c);
int  InputStringa(char *prompt, char *s, int lmin, int lmax);
void StrMinuscola(char *s);


#define MAXNOME   20     /*lunghezza massima nome giocatore*/
#define MINPAROLA 8      /*lunghezza minima parola da indovinare*/
#define MAXPAROLA 16     /*lunghezza massima parola da indovinare*/

enum stati { NO, SI, VINCE, BASTA };      /*stati di gioco <1>*/

char nomeGioca[2][MAXNOME+1];   /*nomi dei due giocatori <2>*/
char parola[2][MAXPAROLA+1];    /*parole da indovinare*/
char finora[2][MAXPAROLA+1];    /*lettere finora indovinate*/
char provate[2][26+1];          /*lettere finora provate <3>*/

/* Note sulle dichiarazioni:
    <1> Quando il valore delle costanti non ha significato, la enum consente
        di definirne una serie (tutte diverse) da usare poi esclusivamente
        per nome, in modo da poter distinguere tra varie situazioni. Si
        tratta comunque di comuni valori numerici di tipo int.
    <2> Questa linea dichiara un array bidimensionale di caratteri, che in
        pratica si usa come un array di stringhe: nomeGioca[0][]  una
        stringa di MAXNOME elementi (pi uno per lo zero), e nomeGioca[1][]
         un'altra stringa identica. Quindi nomeGioca[0][3]  il carattere
        numero 3 (cio il quarto) della stringa nomeGioca[0][].
    <3> Questa stringa pu contenere al massimo le 26 lettere dell'alfabeto,
        pi lo zero finale che  indicato separatamente per chiarezza. In
        realt il meccanismo di gioco  tale che la stringa non potr
        contenere pi di 25 elementi, ma  meglio sbagliare per eccesso che
        per difetto.
*/


/********************************************************************
 * main
 ********************************************************************/

main()
{
    int gioca;           /*giocatore di turno (0,1)*/
    int stato;           /*stato del gioco*/

    Titoli();
    gioca = PreparaPartita();                       /*inizializza*/

    do {
        gioca = 1-gioca;                            /*cambia giocatore <1>*/
        do {
            stato = ProvaLettera(gioca);            /*fa un tentativo <2>*/
        } while (stato == SI);                      /*ripete se indovina*/
    } while (stato != VINCE && stato != BASTA);

    if (stato == VINCE) {
        Vittoria(gioca);                            /*celebra vittoria*/
    } else {
        puts("- D'accordo, interrompo questa appassionante partita.\n");
    }
    puts("Ciao!");
}

/*  Note su main:
    <1> Questo  un metodo semplice per alternare tra i giocatori 0 e 1.
    <2> Il valore ritornato da ProvaLettera ed assegnato alla variabile
        stato sar una delle costanti definite con enum (vedi nota 1 delle
        dichiarazioni).
*/


/********************************************************************
 * Titoli: mostra titolo ed istruzioni del gioco.
 ********************************************************************/

void Titoli(void)
{
    CancellaSchermo();
    printf("%60s","- Parola: una sfida tra due giocatori -\n\n\n");   /*<1>*/
    puts("Regole:\n\n"
         "- Ogni giocatore imposta in segreto una parola da indovinare,\n"
         "  poi si gioca introducendo a turno una lettera.\n\n"
         "- Se la lettera c' nella parola, viene mostrata\n"
         "  e si ha diritto ad un altro tentativo.\n\n"
         "- Vince chi completa per primo l'intera parola.\n\n");
}

/*  Note su Titoli:
    <1> Indicando %60s, la stringa indicata viene stampata a destra in un
        campo di 60 caratteri: in pratica, data la sua lunghezza, appare
        centrata negli 80 caratteri del video.
*/


/********************************************************************
 * CancellaSchermo: pulisce il video mandando 25 newline.
 ********************************************************************/

void CancellaSchermo(void)
{
    int i;

    for (i = 0; i < 25; i++)  putchar('\n');
}

/*  Note su CancellaSchermo:
    <1> Il C standard ANSI non dispone di funzioni per cancellare il video
        (quelle dei computer MS-DOS sono trattate nella parte esperti), per
        cui usiamo il vecchio sistema di stampare 25 newline in modo da far
        scorrere fuori dal bordo superiore tutte le linee che fossero state
        precedentemente stampate.
*/


/********************************************************************
 * PreparaPartita: inizializza gioco, input nomi dei giocatori nelle
 *                 stringhe globali nomeGioca[][], parole da indovinare
 *                 nelle stringhe globali parola[][], azzera stringhe
 *                 globali finora[][] e provate[][], estrae e ritorna
 *                 primo giocatore.
 ********************************************************************/

int PreparaPartita(void)
{
    int i,lp;

    srand((unsigned int)time(NULL));                /*avvia random <1>*/

    InputNomeGioca(0);                              /*nomi giocatori <2>*/
    InputNomeGioca(1);
    InputParola(0);                                 /*parole segrete*/
    InputParola(1);

    for (i = 0; i < 2; i++) {                       /*finora, niente <3>*/
        lp = strlen(parola[i]);
        finora[i][lp] = 0;                          /*<4>*/
        while (lp--) {                              /*<5>*/
            finora[i][lp] = '.';
        }
        provate[i][0] = 0;                          /*stringa nulla <6>*/
    }

    return rand()%2;                                /*sceglie giocatore <7>*/
}

/*  Note su PreparaPartita:
    <1> Questo  il solito sistema per inizializzare il generatore di numeri
        pseudocasuali, da usarsi poi con la funzione rand.
    <2> Per due sole chiamate  inutile usare un ciclo. Queste funzioni
        agiscono direttamente sulle variabili globali, per evitare un
        continuo passaggio di argomenti sempre uguali.
    <3> Questo ciclo costruisce, per ciascun giocatore, una stringa finora[]
        lunga come parola[] ma contenente solo punti (.), che rappresentano
        le lettere finora indovinate.
    <4> Usa subito lp per mettere lo zero finale perch poi user la stessa
        lp per contare all'indietro, risparmiando una variabile. finora[i][]
         una delle due stringhe dell'array finora[][].
    <5> Per riempire una stringa di caratteri tutti uguali, si potrebbe
        anche usare la funzione memset. Visto che non ci sono problemi di
        velocit, abbiamo preferito una forma pi esplicita.
    <6> Mettendo uno zero nel primo carattere di una stringa, questa 
        una stringa nulla. Era equivalente scrivere: *provate[i] = 0.
    <7> Ritorna il resto della divisione per due del numero pseudocasuale
        estratto con rand: se  pari torna zero, se  dispari torna 1, con
        (circa) uguale probabilit.
*/


/********************************************************************
 * InputNomeGioca: chiede nome giocatore g, lo scrive nella stringa
 *                 globale nomeGioca[g][].
 ********************************************************************/

void InputNomeGioca(int g)
{
    char prompt[80];

    sprintf(prompt,"Nome del giocatore n. %d:",g+1); /*<1>*/
    InputStringa(prompt,nomeGioca[g],3,MAXNOME);     /*<2>*/

    StrMinuscola(nomeGioca[g]);                      /*tutte minuscole <3>*/
    nomeGioca[g][0] = toupper(nomeGioca[g][0]);      /*tranne l'iniziale*/
    putchar('\n');
}

/*  Note su InputNomeGioca:
    <1> La sprintf non stampa sul video, ma nella stringa prompt[], che
        ovviamente deve avere abbastanza spazio per contenere tutti i
        caratteri stampati.
    <2> nomeGioca[g]  l'indirizzo della stringa nomeGioca[g][]: un elemento
        di un array di stringhe si usa quindi come una normale stringa.
    <3> Convertire un nome proprio a tutte minuscole tranne l'iniziale
        rende pi simpatica la presentazione. Anche l'occhio vuole la sua
        parte, cosa che troppi programmatori tendono a sottovalutare.
*/


/********************************************************************
 * InputParola: chiede a g la parola da indovinare, la scrive nella
 *              stringa globale parola[1-g][], cio la parola che
 *              l'altro giocatore dovr indovinare.
 ********************************************************************/

void InputParola(int g)
{
    char prompt[160];                               /*<1>*/
    int  ag;
    int  i,ok,c;

    ag = 1-g;                                       /*altro giocatore*/
    do {
        sprintf(prompt,"%s, scrivi la parola che %s\n"
                       "dovr indovinare (%s non guardare) :",
                       nomeGioca[g],nomeGioca[ag],nomeGioca[ag]);
        InputStringa(prompt,parola[ag],MINPAROLA,MAXPAROLA);

        StrMinuscola(parola[ag]);                     /*tutte minuscole*/
        ok = 1;                                       /*supponiamo ok*/
        for(i = strlen(parola[ag])-1; i >= 0; i--) {  /*test lettere <2>*/
            c = parola[ag][i];
            if (c < 'a' || c > 'z')  ok = 0;          /*<3>*/
        }
        if (! ok) {
            puts("\n- Puoi usare solo lettere, non accentate.\a\n");
        }
    } while (! ok);

    CancellaSchermo();                                /*nasconde parola*/
}

/*  Note su InputParola:
    <1> Dato che il contenuto di prompt[] occuper pi di una linea video,
        meglio sbagliare per eccesso che per difetto, in modo da non
        rischiare scritture accidentali fuori della stringa. Tanto si tratta
        di un array locale, che sparisce quando la funzione ritorna.
    <2> Contando all'indietro si risparmia il confronto ad ogni ciclo con
        strlen(parola[ag]), che implica un'inutile chiamata di funzione. Il
        conteggio alla rovescia  spesso pi efficiente.
    <3> Non abbiamo usato la funzione isalpha perch questa non  definita
        per i caratteri non ASCII del PC IBM, e potrebbe quindi creare
        problemi se li accettasse (come avviene con alcuni compilatori).
*/


/********************************************************************
 * ProvaLettera: chiede a g di provare una lettera (o "basta" per
 *               smettere), mostra risultato lettera, torna SI se la
 *               lettera  valida (e aggiorna finora[g][]), NO se non
 *                valida, VINCE se parola completa, BASTA se smette.
 ********************************************************************/

int  ProvaLettera(int g)
{
    int  t;          /*tentativo*/
    char tz[2];      /*tentativo, come stringa <1>*/
    int  st;         /*stato*/
    int  i;

    t = InputTentativo(g);                          /*chiede lettera <2>*/
    tz[0] = t;                                      /*crea stringa <3>*/
    tz[1] = 0;

    if (t == EOF) {                                 /*guarda se finito*/
        st = BASTA;                                 /*<4>*/
    } else if (strchr(parola[g],t) == NULL) {       /*se non c' <5>*/
        st = NO;
        printf("- La lettera '%c' non  nella"
               " parola da indovinare!\a\n\n",t);
        strcat(provate[g],tz);                      /*provata <6>*/
    } else {
        st = SI;                                    /*c', aggiorna <7>*/
        for (i = strlen(parola[g])-1; i >= 0; i--) {
            if (parola[g][i] == t) {
                finora[g][i] = t;                   /*copia se ok*/
            }
        }
        strcat(provate[g],tz);                      /*provata*/
        if (strcmp(finora[g],parola[g]) == 0) {     /*test vittoria <8>*/
            st = VINCE;
        }
    }

    return st;                                      /*ritorna stato*/
}

/*  Note su ProvaLettera:
    <1> La strcat (usata pi avanti) concatena una stringa ad un'altra
        stringa, e non accetta (come in BASIC) un carattere. Per concatenare
        un carattere ad una stringa senza manipolare direttamente gli
        elementi della stringa stessa, si pu creare una stringa di due
        elementi: il carattere in questione pi lo zero finale.
    <2> La InputTentativo ritorna un carattere, o EOF se il giocatore ha
        chiesto di terminare la partita. EOF non pu per definizione essere
        confuso con alcun carattere stampabile.
    <3> La stringa tz[] contiene il carattere t seguito da uno zero.
    <4> Usando i nomi simbolici definiti con enum (nota 1 del main) non si
        devono scrivere numeri, e la leggibilit ne guadagna parecchio.
    <5> La strchr cerca un carattere in una stringa, e torna NULL se non lo
        trova. NULL  un pointer non valido, definito in <stdio.h>.
    <6> La strcat aggiunge la stringa tz[] in coda alla stringa provate[g][],
        spostando lo zero finale. tz[] contiene solo il carattere t, pi lo
        zero finale (vedi nota 1). La stringa provate[g][] contiene tutte
        le lettere provate dal giocatore g, indipendentemente dall'esito.
    <7> Per ogni occorrenza del carattere t nella stringa parola[g][], lo
        copia nella corrispondente posizione della stringa finora[g][].
    <8> Se tutte le lettere di parola[g][] sono state trasferite in
        finora[g][], il giocatore ha indovinato l'intera parola.
*/


/********************************************************************
 * InputTentativo: mostra situazione, ritorna lettera introdotta dal
 *                 giocatore g, o EOF se scrive "basta".
 ********************************************************************/

int InputTentativo(int g)
{
    char tenta[6];      /*tentativo (vale la prima lettera)*/
    char prompt[80];
    int  ok,t,i,basta;

    do {
        MostraSituazione();                         /*mostra parole*/
        sprintf(prompt,"%s, prova una lettera (\"basta\" per smettere):",
                nomeGioca[g]);
        InputStringa(prompt,tenta,1,5);             /*input tentativo <1>*/
        putchar('\n');
        StrMinuscola(tenta);                        /*converte in minuscole*/
        basta = (strcmp(tenta,"basta") == 0);       /*vero se "basta"*/

        if (! basta) {                              /*se non deve uscire*/
            t = tenta[0];                           /*tentativo <2>*/
            ok = 1;                                 /*supponiamo ok*/
            if (t < 'a' || t > 'z') {               /*test se valida <3>*/
                puts("- Devi scrivere una lettera,"
                     " non accentata.\a\n");
                ok = 0;
            }
            if (strchr(provate[g],t) != NULL) {     /*test se provata <4>*/
                printf("- Hai gi provato la lettera"
                       " %c.\a\n\n",t);
                ok = 0;
            }
        }
    } while (! ok && ! basta);

    return ( basta ? EOF : t);                      /*EOF se basta <5>*/
}

/*  Note su InputTentativo:
    <1> Accetta fino a 5 caratteri, per consentire di scrivere "basta".
    <2> Usa solo il primo carattere, se ne ha scritto pi d'uno.
    <3> Vedi nota 3 di InputParola. Si poteve anche scrivere una funzione
        di test (es. LetteraValida) per non ripetere questo confronto.
    <4> Cerca la lettera nella stringa contenente quelle gi provate.
    <5> Se il giocatore ha scritto "basta" ritorna EOF, altrimenti ritorna
        la lettera introdotta.
*/


/********************************************************************
 * MostraSituazione: mostra parole con lettere indovinate e provate,
 *                   usando le stringhe globali finora[][].
 ********************************************************************/

void MostraSituazione(void)
{
    int  n;

    n = printf("   %s: %s",nomeGioca[0],finora[0]);  /*n.1 a sinistra <1>*/
    PutNchar(40-n,' ');                              /*n.2 al centro <2>*/
    printf("%s: %s\n\n",nomeGioca[1],finora[1]);
    n = printf("   (provate: %s)",provate[0]);
    PutNchar(40-n,' ');                              /*n.2 al centro*/
    printf("(provate: %s)\n\n",provate[1]);
}

/*  Note su MostraSituazione:
    <1> La printf restituisce il numero di caratteri stampati, che permette
        cos di conoscere la posizione di stampa sul video.
    <2> Allinea per differenza tra la posizione voluta e quella nota.
*/


/********************************************************************
 * Vittoria: celebra vittoria del giocatore g.
 ********************************************************************/

void Vittoria(int g)
{
    int  ls;
    char buf[80];

    PutNchar(78,'-');
    puts("\n");                                     /*due a capo*/
    MostraSituazione();
    PutNchar(78,'-');
    puts("\n");

    sprintf(buf,"||   %s vince la partita!   ||",
            nomeGioca[g]);
    ls = strlen(buf)-4;                             /*lunghezza stringa*/
    printf("   ++");
    PutNchar(ls,'=');
    puts("++\a");
    printf("   ||%*s||\a\n",ls,"");                 /*<1>*/
    printf("   %s\a\n",buf);
    printf("   ||%*s||\a\n",ls,"");
    printf("   ++");                                /*<2>*/
    PutNchar(ls,'=');
    puts("++\a\n");

    printf("   (la parola di %s era \"%s\".\n\n",   /*mostra soluzione*/
           nomeGioca[1-g],parola[1-g]);
}

/*  Note su Vittoria:
    <1> Indicando %*s, la printf stampa la stringa s in un campo di larghezza
        non determinata al momento della compilazione, ma pari al valore di
        una variabile passata come parametro aggiuntivo, in questo caso ls
        ( un'altra delle possibilit della printf). Dato che la stringa da
        stampare  una stringa nulla (""), l'effetto  di stampare ls spazi.
    <2> Le quattro printf si potevano ovviamente riunire in una sola.
*/


/********************************************************************
 * PutNchar: stampa n caratteri c di seguito.
 ********************************************************************/

void PutNchar(int n, int c)
{
    while (n--)  putchar(c);
}


/********************************************************************
 * InputStringa: mostra prompt, input stringa nell'array stringa s[]
 *               non oltre lmax caratteri, mette zero finale, ritorna
 *               lunghezza della stringa introdotta.
 ********************************************************************/

int InputStringa(char *prompt, char *s, int lmin, int lmax)     /*<1>*/
{
    char buf[128];      /*buffer di linea*/
    int  ls;            /*lunghezza stringa*/

    do {
        printf("%s ",prompt);                       /*mostra prompt*/
        if (gets(buf) == NULL) {                    /*legge stringa*/
            ls = 0;                                 /*nulla se EOF*/
        } else {
            ls = strlen(buf);                       /*calcola lunghezza*/
        }
        if (ls < lmin || ls > lmax) {               /*guarda se valida*/
            printf("\n- da %d a %d caratteri.\a\n\n",
                   lmin,lmax);
            ls = 0;
        }
    } while (ls == 0);                              /*ripete se non ok*/

    strcpy(s,buf);                                  /*copia in s*/
    return ls;                                      /*ritorna lunghezza*/
}

/*  Note su InputStringa:
    <1> InputStringa  simile alla funzione dallo stesso nome commentata
        nel programma FIGURA, con l'aggiunta del controllo di lunghezza
        minima. Inoltre, una stringa che supera la lunghezza massima non
        viene troncata ma viene nuovamente richiesto l'input.
*/


/********************************************************************
 * StrMinuscola: converte la stringa s a sole lettere minuscole.
 ********************************************************************/

void StrMinuscola(char *s)     /*<1>*/
{
    while ( (*s = tolower(*s)) != 0)
        s++;
}

/*  Note su StrMinuscola:
    <1> StrMinuscola  commentata nel programma FIGURA.
*/
