/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* SEPARA: scompone una frase nelle parole che la compongono */


#include <stdio.h>
#include <ctype.h>


char *InizioParola(char *);
char *FineParola(char *);
void *StampaParola(char *p1, char *p2);
int  InputStringa(char *prompt, char *s, int lmax); /*prototipi funzioni*/


#define MAXLEN 60               /*lunghezza massima della frase*/


main()
{
    char frase[MAXLEN+1];               /*spazio per frase <1>*/
    char domanda[] = "Cosa mi dici?";   /*prompt <2>*/
    char *pi,*pf;                       /*pointer di lavoro <3>*/

    printf("\n Separazione delle parole di una frase:\n\n");

    InputStringa(domanda,frase,MAXLEN);             /*input frase <4>*/

    printf("\nUna parola alla volta:\n\n");         /*separa parole*/
    pf = frase;                                     /*punta all'inizio <5>*/
    while ( (pi = InizioParola(pf)) != NULL) {      /*e scandisce <6>*/
        pf = FineParola(pi);                        /*trova fine parola <7>*/
        StampaParola(pi,pf);                        /*<8>*/
    }
}

/*  Note su main:
    <1> L'array frase[] conterr una normale stringa C con non pi di MAXLEN
        caratteri, pi uno per lo zero finale usato come terminatore.
    <2> L'array domanda[] viene dimensionato in base al numero di caratteri
        della costante stringa "Cosa mi dici?", con la quale viene subito
        inizializzato. Il dimensionamento comprende lo zero finale, ed 
        quindi equivalente a domanda[14].
    <3> pi e pf sono pointer a char, e possono quindi essere utilizzati per
        puntare ai (contenere l'indirizzo dei) caratteri di una stringa.
    <4> La funzione InputStringa mette in frase[] una normale stringa C,
        terminata con uno zero dopo l'ultimo carattere.
    <5> Questo equivale a scrivere: pf = &frase[0], cio nel pointer pf viene
        messo l'indirizzo del primo elemento dell'array frase[]. In altri
        termini pf punta al primo elemento di frase, cio *pf  frase[0].
    <6> La funzione InizioParola riceve un pointer ad un elemento di una
        stringa (in questo caso frase[]), e ritorna un pointer al primo
        elemento successivo che sia alfanumerico (una lettera o una cifra),
        oppure torna NULL se ha trovato invece la fine della stringa.
    <7> La FineParola  simile alla InizioParola, ma ritorna un pointer al
        primo elemento non alfanumerico della stringa.
    <8> La StampaParola stampa i caratteri compresi tra i due indirizzi dati.
        Notare che nessuna di queste tre funzioni ha bisogno di conoscere
        l'inizio della stringa.
*/


/********************************************************************
 * InizioParola: avanza p fino al primo carattere alfanumerico,
 *               ritorna il nuovo pointer o NULL se fine stringa.
 ********************************************************************/

char *InizioParola(char *p)     /*<1>*/
{
    while (*p && !isalnum(*p)) {                    /*<2>*/
        p++;                                        /*<3>*/
    }
    return (*p) ? p : NULL;                         /*<4>*/
}

/*  Note su InizioParola:
    <1> Il pointer p deve contenere l'indirizzo di un carattere appartenente
        ad una stringa, cio di un elemento di un array di char (in questo
        caso frase[] del main) terminato con zero. Dal punto di vista di
        InizioParola, p pu essere trattata come una stringa, cio un pointer
        ad un array di char terminato con 0, di cui *p  il primo carattere.
    <2> *p  un'abbreviazione (comunemente usata) per: *p != 0. Il secondo
        test, che usa la funzione isalnum per chiedersi se *p  un carattere
        alfanumerico, viene eseguito solo se il primo  falso, cio se p
        non punta allo zero che termina la stringa.
        La funzione isalnum non  definita per i caratteri non appartenenti
        al set ASCII, ad esempio le lettere accentate del PC IBM: in questo
        caso, il suo risultato  indeterminato.
    <3> Ad ogni ciclo, p viene fatto avanzare in modo che punti al successivo
        elemento della stringa.
    <4> Se il ciclo  terminato perch p  giunto alla fine della stringa,
        *p  lo zero finale: in questo caso ritorna NULL, altrimenti (se *p
         diverso da zero) ritorna il contenuto del pointer p, cio
        l'indirizzo del primo carattere che soddisfa isalnum.
*/


/********************************************************************
 * FineParola: avanza p fino al primo carattere non alfanumerico,
 *             ritorna il nuovo pointer.
 ********************************************************************/

char *FineParola(char *p)       /*<1>*/
{
    while (isalnum(*p))  p++;                       /*<2>*/
    return p;
}

/*  Note su FineParola:
    <1> FineParola funziona come la precedente InizioParola, ma  molto
        pi semplice non dovendo trattare in modo speciale la fine della
        stringa (lo 0):  un carattere non alfanumerico come gli altri.
    <2> Un ciclo semplicissimo come questo pu anche stare su una sola linea.
*/


/********************************************************************
 * StampaParola: stampa stringa da *p1 (incluso) a *p2 (escluso), con
 *               spazio iniziale ed a capo finale.
 ********************************************************************/

void *StampaParola(char *p1, char *p2)       /*<1>*/
{
    putchar(' ');
    while (p1 < p2)  putchar(*p1++);         /*<2>*/
    putchar('\n');
}

/*  Note su StampaParola:
    <1> Perch StampaParola funzioni, p1 e p2 devono puntare a caratteri
        appartenenti alla medesima stringa (array di char).
    <2> Il confronto tra due pointer  valido, purch puntino ad elementi
        dello stesso array. Dato che i pointer sono passati per valore
        (come le normali variabili semplici), l'alterazione di p1 non ha
        alcun effetto sul corrispondente argomento indicato nella chiamata.
        Una variazione del carattere puntato da p1, cio *p1, avrebbe invece
        effetto sulla stringa originaria.
*/


/********************************************************************
 * InputStringa: mostra prompt, input stringa nell'array stringa s[]
 *               non oltre lmax caratteri, mette zero finale, ritorna
 *               lunghezza della stringa introdotta.
 ********************************************************************/

int InputStringa(char *prompt, char *s, int lmax)   /*<1>*/
{
    int nc;     /*numero di caratteri introdotti*/
    int c;

    do {
        printf("%s ",prompt);                       /*mostra prompt <2>*/
        nc = 0;
        while ( (c = getchar()) != '\n' && c != EOF && nc < lmax) {
            s[nc++] = c;                            /*legge stringa*/
        }
        if (nc == 0)  putchar('\a');                /*segnala errore*/
    } while (nc == 0);                              /*ripete se non ok*/
    s[nc] = 0;                                      /*mette terminatore <3>*/

    return nc;                                      /*ritorna lunghezza*/
}

/*  Note su InputStringa:
    <1> InputStringa  simile alla InputFrase del programma SPECCHIO, ma
        stampa la stringa che le viene passata come prompt, e mette lo
        zero di chiusura al termine della stringa introdotta.
    <2> La printf pu stampare una stringa come fa con le normali variabili,
        indicando %s nella stringa di formato.
    <3> Lo zero finale evita di dover tenere conto del numero di caratteri
        contenuti nella stringa, e consente di passare semplicemente dei
        pointer alle funzioni che devono operare sulla stringa stessa, come
        InizioParola, FineParola e StampaParola.
*/
