/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* SOMMASIN: grafico della somma di due sinusoidi */


#include <math.h>
#include <stdio.h>

#include "Jgraph.h"
#include "Jconio.h"
#include "grafutil.h"


#define  XMIN  0          /*inizio asse x*/
#define  XMAX  400        /*fine asse x (ultimo pixel usato)*/
#define  PDH   58         /*posizione orizzontale didascalie, in caratteri*/
#define  PDV1  1          /*posizione verticale primo gruppo didascalie*/
#define  PDV2  5          /*secondo gruppo*/
#define  PDV3  12         /*terzo gruppo*/
#define  PDV4  21         /*quarto gruppo (input)*/
#define  YF1   PDV2+5     /*posizione stampa prima frequenza*/
#define  YF2   PDV3+5     /*e seconda frequenza*/

#define  FMIN  50         /*frequenza minima ammessa*/
#define  FMAX  1800       /*e massima*/

#define  FREQ1 300        /*frequenza prima sinusoide*/


/* variabili globali: */

double grad;              /*costante di conversione gradi/radianti*/
double sin1[XMAX-XMIN+1]; /*Valori della prima sinusoide, per ogni pixel*/
double sin2[XMAX-XMIN+1]; /*Valori della seconda sinusoide, per ogni pixel*/

int asse1;                /*posizione verticale asse x prima sinusoide*/
int asse2;                /*seconda sinusoide*/
int asse3;                /*terza sinusoide*/
int amp;                  /*ampiezza massima prima e seconda sinusoide*/

int k;                    /*colore per disegno sinusoidi*/


/* prototipi funzioni: */

void SinusoidiBase(int f);
void AggiornaSinusoidi(int f);
int  InputFrequenza2(int mi,int ma);
void Didascalie(void);
void MostraFrequenze(int f1, int f2);


/********************************************************************
 * main
 ********************************************************************/

main()
{
    int  dimy;      /*numero pixel in verticale*/
    int  fr1;       /*frequenza prima sinusoide*/
    int  fr2;       /*e seconda sinusoide*/

    grad = atan(1.0)/45.0;                          /*prepara costante*/
    if (Jinitgraph() == 0) {                        /*inizializza grafica*/
        Jputs("\n- Non sono in grado di usare"
              " la grafica di questo computer.");
    } else {
        dimy = Jgetdimy();                          /*posizione assi x <1>*/
        asse1 = dimy/8;
        asse2 = (dimy*3)/8;
        asse3 = (dimy*6)/8;
        amp = asse1-2;                              /*e ampiezza massima*/
        k = Jgetcolors()-1;                         /*disegna in bianco <2>*/
        GsetUtilColor(k);                           /*scrive in bianco*/

        Jmoveto(XMIN,asse1); Jlineto(XMAX,asse1);   /*disegna assi*/
        Jmoveto(XMIN,asse2); Jlineto(XMAX,asse2);
        Jmoveto(XMIN,asse3); Jlineto(XMAX,asse3);

        fr1 = fr2 = FREQ1;                          /*frequenze iniziali*/
        Didascalie();                               /*mostra didascalie*/
        MostraFrequenze(fr1,fr2);                   /*e frequenze*/
        SinusoidiBase(fr1);                         /*sinusoidi iniziali <3>*/

        do {
            fr2 = InputFrequenza2(FMIN,FMAX);       /*chiede nuova freq.2*/
            if (fr2 != 0) {                         /*se valida*/
                MostraFrequenze(fr1,fr2);           /*aggiorna scritte*/
                AggiornaSinusoidi(fr2);             /*e disegno <4>*/
            }
        } while (fr2 != 0);

        Jclosegraph();                              /*torna in modo testo*/
    }
}

/* Note su main:
    <1> Le posizioni degli assi x per le tre sinusoidi sono calcolate in
        modo da spaziarsi correttamente indipendentemente dalla risoluzione
        verticale del modo grafico trovato. Lo stesso vale per l'ampiezza
        delle sinusoidi.
    <2> La variabile globale k contiene il colore usato per i disegni, mentre
        la GsetUtilColor sceglie il colore che verr usato da tutte le
        funzioni del modulo GRAFUTIL, cio scritte ed input.
    <3> Calcola le prime due sinusoidi, uguali (stessa frequenza), le disegna
        e le memorizza negli array sin1[] e sin2[]. Calcola e disegna la
        terza sinusoide come somma delle prime due.
    <4> Ricalcola la seconda e la terza sinusoide, le disegna cancellando
        quelle precedenti, memorizza la seconda in sin2[].
*/


/********************************************************************
 * SinusoidiBase: calcola due sinusoidi uguali di frequenza f Hertz
 *                in sin1[] e sin2[], le disegna. Disegna la terza
 *                sinusoide come somma delle prime due.
 ********************************************************************/

void SinusoidiBase(int f)
{
    double passo;     /*passo in radianti*/
    double ang;       /*angolo corrente, in radianti*/
    double val;       /*valore all'angolo corrente*/
    int    x,y;

    passo = grad*(360.*f/100.0)/(XMAX-XMIN);        /*calcola passo <1>*/
    ang = 0.0;                                      /*angolo iniziale*/

    for (x = XMIN; x <= XMAX; x++) {                /*per ogni pixel x*/
        val = sin(ang)*amp;                         /*calcola valore*/
        sin1[x-XMIN] = val;                         /*ricorda in sin1[]*/
        sin2[x-XMIN] = val;                         /*e in sin2[]*/
        y = (int)(floor(val+0.5));                  /*converte in pixel*/
        Jsetpixel(x,asse1-y,k);                     /*disegna prima sin*/
        Jsetpixel(x,asse2-y,k);                     /*seconda sinusoide*/

        val += val;                                 /*terza sin (somma) <2>*/
        y = (int)(floor(val+0.5));                  /*converte in pixel*/
        Jsetpixel(x,asse3-y,k);                     /*e disegna*/

        ang += passo;                               /*aggiorna l'angolo <3>*/
    }
}

/* Note su SinusoidiBase:
    <1> Il passo  calcolato in modo che una sinusoide di 100 Hz (cicli al
        secondo) occupi l'intero asse x. In altri termini, l'asse x
        rappresenta 10 millisecondi di tempo simulato. La variabile passo
        rappresenta l'incremento di angolo corrispondente ad un pixel video.
    <2> La terza sinusoide  la somma delle prime due, che in questo caso
        sono uguali. La somma va fatta sui valori effettivi (double) e non
        sulle posizioni di pixel, per evitare errori di approssimazione.
    <3> Prepara l'angolo corrispondente al pixel successivo.
*/


/********************************************************************
 * AggiornaSinusoidi: ricalcola sin2[] di frequenza f, la ridisegna
 *                    cancellando la precedente, calcola e disegna la
 *                    terza sinusoide come somma delle prime due.
 ********************************************************************/

void AggiornaSinusoidi(int f)
{
    double passo;     /*passo in radianti*/
    double ang;       /*angolo corrente, in radianti*/
    double val;       /*valore all'angolo corrente*/
    double val1;      /*valore di sin1 all'angolo corrente*/
    int    x,y;

    passo = grad*(360.*f/100.0)/(XMAX-XMIN);        /*calcola passo <1>*/
    ang = 0.0;                                      /*angolo iniziale*/

    for (x = XMIN; x <= XMAX; x++) {                /*per ogni pixel x*/
        val1 = sin1[x-XMIN];                        /*valore di sin1 <2>*/
        val = sin2[x-XMIN];                         /*valore prec. sin2 <3>*/
        y = (int)(floor(val+0.5));                  /*converte in pixel*/
        if (y != 0) {                               /*nero se non asse <4>*/
            Jsetpixel(x,asse2-y,0);
        }
        val += val1;                                /*valore prec. sin3 <5>*/
        y = (int)(floor(val+0.5));                  /*converte in pixel*/
        if (y != 0) {                               /*nero se non asse*/
            Jsetpixel(x,asse3-y,0);
        }

        val = sin(ang)*amp;                         /*nuovo valore sin2 <6>*/
        sin2[x-XMIN] = val;                         /*ricorda in sin2[]*/
        y = (int)(floor(val+0.5));                  /*converte in pixel*/
        Jsetpixel(x,asse2-y,k);                     /*disegna*/
        val += val1;                                /*nuovo valore sin3 <7>*/
        y = (int)(floor(val+0.5));                  /*converte in pixel*/
        Jsetpixel(x,asse3-y,k);                     /*disegna*/

        ang += passo;                               /*aggiorna l'angolo <8>*/
    }
}

/* Note su AggiornaSinusoidi:
    <1> Vedi nota 1 di SinusoidiBase.
    <2> Rilegge il valore della prima sinusoide in questo pixel, che sar
        usato due volte: per calcolare il valore precedente della terza
        sinusoide, e per calcolare il nuovo valore della stessa. In pratica,
        il valore della prima sinusoide in un dato punto  una costante.
    <3> Il valore precedente della seconda sinusoide serve per cancellare
        il pixel corrente prima di disegnare quello nuovo.
    <4> Evita di cancellare pixel appartenenti all'asse x.
    <5> Il valore precedente della terza sinusoide si ottiene sommando i
        valori precedenti delle prime due (vedi nota 2 di SinusoidiBase),
        e serve per cancellarla prima di diegnarne il nuovo valore.
    <6> Calcola, ricorda e disegna il nuovo valore della seconda sinusoide.
    <7> Calcola, ricorda e disegna il nuovo valore della terza sinusoide
        (vedi nota 2 di SinusoidiBase).
    <8> Vedi nota 3 di SinusoidiBase.
*/


/********************************************************************
 * InputFrequenza2: input frequenza seconda sinusoide, compresa
 *                  tra mi e ma. Ritorna frequenza o 0 se solo Return.
 ********************************************************************/

int InputFrequenza2(int mi, int ma)
{
    int  f;
    char buf[80];
    int  ok;

    GputsAt(PDH,PDV4,"Nuovo valore Freq2");
    sprintf(buf,"(%d-%d, CR esce):",mi,ma);         /*mostra limiti*/
    GputsAt(PDH,PDV4+1,buf);

    GputsAt(PDH,PDV4+3,"Freq2: ");                  /*mostra prompt*/
    GclearEOL();                                    /*ripulisce*/
    f = GinputInt(mi,ma);                           /*input frequenza*/

    Jgotoxy(PDH,PDV4); GclearEOL();                 /*ripulisce tutto*/
    Jgotoxy(PDH,PDV4+1); GclearEOL();
    Jgotoxy(PDH,PDV4+3); GclearEOL();

    return f;                                       /*ritorna frequenza*/
}


/********************************************************************
 * Didascalie: mostra didascalie fisse.
 ********************************************************************/

void Didascalie(void)
{
    static char sep[] = "------------------";

    GputsAt(PDH,PDV1,"Somma di sinusoidi");
    GputsAt(PDH,PDV1+1,"dal Corso di C");
    GputsAt(PDH,PDV1+2,"by E.Colombini");
    GputsAt(PDH,PDV2,sep);
    GputsAt(PDH,PDV2+2,"Frequenza della");
    GputsAt(PDH,PDV2+3,"sinusoide 1:");
    GputsAt(PDH,PDV3,sep);
    GputsAt(PDH,PDV3+2,"Frequenza della");
    GputsAt(PDH,PDV3+3,"sinusoide 2:");
    GputsAt(PDH,PDV3+7,sep);
}


/********************************************************************
 * MostraFrequenze: stampa frequenze correnti f1 e f2..
 ********************************************************************/

void MostraFrequenze(int f1, int f2)
{
    char buf[80];

    sprintf(buf,"Freq1 = %d Hz",f1);                /*prima frequenza*/
    GputsAt(PDH,YF1,buf);
    GclearEOL();                                    /*pulisce resto linea*/
    sprintf(buf,"Freq2 = %d Hz",f2);                /*seconda frequenza*/
    GputsAt(PDH,YF2,buf);
    GclearEOL();                                    /*pulisce resto linea*/
}
