/*
   Questo file fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


// C-CASELL: funzioni (metodi) della classe Casella


#include "c-casell.h"

#include <stddef.h>
#include <conio.h>


// variabili (componenti) statiche della classe Casella: <1>

int Casella::dx[5] = { 0, 0, 2, -2, 0 };    //offset passaggi N,S,E,W,centro
int Casella::dy[5] = { -1, 1, 0, 0, 0 };
int Casella::opposta[4] = { SUD, NORD, OVEST, EST };   //direzioni opposte

Timer  Casella::ti;                         //timer per sincronizzazione
Random Casella::r;                          //generatore di numeri casuali

/* Note sulle variabili statiche:
    <1> Di queste variabili esiste una sola copia, comune a tutti gli
        esemplari della classe Casella. Il timer, ad esempio, dev'essere uno
        solo, altrimenti non serve a nulla. Le variabili statiche possono
        essere molto utili, ad esempio per tenere il conto del numero di
        esemplari esistenti di una certa classe. Notare che non viene usata
        la parola static, ma il nome della variabile  preceduto da quello
        della classe.
*/


/********************************************************************
 * Funzioni della classe Casella
 ********************************************************************/

// costruttore: posiziona la casella, la disegna se visibile

Casella::Casella(int posx, int posy)
{
    x = posx;                                       //registra posizione
    y = posy;
    for (int d = 0; d < 4; d++) {
        adia[d] = NULL;                             //nessuna adiacente
        pass[d] = NULL;                             //nessun passaggio
    }
    st = CREATA;                                    //stato iniziale
    for (int v = y-1; v <= y+1; v++) {              //disegna
        gotoxy(x-2,v); cputs(MURO MURO MURO);
    }
}


// distruttore: cancella la casella, se visibile

Casella::~Casella()
{
    for (int v = y-1; v <= y+1; v++) {              //cancella
        gotoxy(x-2,v); cputs(NULLA NULLA NULLA);
    }
}


// Crea: crea labirinto per esplorazione ricorsiva

void Casella::Crea()
{
    st = COLLEG;                                    //casella collegata <1>
    MetteSimbolo(CENTRO,NULLA);                     //la segna

    Casella *ad;                                    //pointer a adiacente <2>
    int dir;                                        //direzione
    for (int n = 0; n < 4; n++) {                   //esplora direzioni
        dir = EstraeDirezione(n);                   //in ordine casuale <3>
        ad = adia[dir];                             //casella adiacente
        if (ad != NULL && ad->Stato() != COLLEG) {  //se c' e non collegata
            Collega(dir);                           //apre passaggio
            ad->Collega(opposta[dir]);              //da entrambe le parti
            MetteSimbolo(dir,NULLA);                //mostra passaggio
            ad->Crea();                             //ed esplora <4>
        }
    }
}

/* Note su Crea:
    <1> Ricorda che questa casella  gi stata collegata al labirinto, per
        non rifarlo.
    <2> In C++ si possono dichiarare variabili quando lo si desidera.
    <3> La EstraeDirezione ritorna le 4 direzioni (0,1,2,3) in ordine casuale.
    <4> Il procedimanto  ricorsivo: senza che nessun algoritmo 'veda' tutte
        le caselle, queste si chiamano a vicenda creando il labirinto. La
        ricorsione termina quando tutti i passaggi possibili di tutte le
        caselle del labirinto sono stati creati.
*/


// Cerca: cerca casella c (risolve labirinto) con ritardo di rit tick ad
//        ogni casella, ritorna vero se  la casella cercata.

int Casella::Cerca(Casella *c, int rit)
{
    ti.Sincro(rit);                                 //sincronizza <1>
    st = ESPLOR;                                    //casella esplorata <2>
    MetteSimbolo(CENTRO,ORMA);                      //la segna

    Casella *ad;                                    //pointer a adiacente
    int dir;                                        //direzione
    int trovata = (this == c);                      //guarda se ha finito <3>
    for (int n = 0; n < 4 && ! trovata; n++) {      //esplora direzioni
        dir = EstraeDirezione(n);                   //in ordine casuale
        ad = pass[dir];                             //casella collegata
        if (ad != NULL && ad->Stato() != ESPLOR) {  //se c' e non esplorata
            MetteSimbolo(dir,ORMA);                 //mostra passaggio
            trovata = ad->Cerca(c,rit);             //ed esplora <4>
            if (! trovata) {                        //se non ha finito
                MetteSimbolo(dir,NULLA);            //cancella orme
            }
        }
    }
    if (! trovata) {                                //se non ha finito
        ti.Sincro(rit);                             //sincronizza
        MetteSimbolo(CENTRO,NULLA);                 //cancella orma
    }
    return trovata;                                 //vero se ha finito
}

/* Note su Cerca:
    <1> La funzione Sincro della classe Timer aspetta il tempo stabilito
        rispetto all'ultima chiamata alla funzione stessa, come nel programma
        PISTONE della lezione E1. Viene usata, qui e pi sotto, per mostrare
        al rallentatore (se desiderato) il procedere della ricerca della
        soluzione. ti  un Timer comune a tutte le caselle (statico).
    <2> Ricorda che questa casella  gi stata esplorata, per non rifarlo.
    <3> this , per definizione, un pointer alla casella corrente. Se questa
         la casella c cercata, il lavoro  finito (il labirinto  risolto).
    <4> Il procedimento di ricerca della casella c (presumibilmente l'uscita)
         simile a quello di creazione. Vedi nota 4 della funzione Crea.
*/


// MetteSimbolo: disegna simbolo indicato nella direzione dir (4 = centro).

void Casella::MetteSimbolo(int dir, char *simb)
{
    gotoxy(x+dx[dir],y+dy[dir]);                    //si posiziona
    cputs(simb);                                    //e disegna simbolo
}


// EstraeDirezione: estrae una direzione a caso (0..3) tra le n+1 rimaste,
//                  va chiamata nell'ordine 0-1-2-3.

int Casella::EstraeDirezione(int n)
{
    int d;

    n = 3-n;                                        //inverte: 3-2-1-0
    if (n == 0) {                                   //0: ultima rimasta
        d = ord[0];
    } else {
        if (n == 3) {                               //3: inizializza
            for (int i = 0; i < 4; i++) {
                ord[i] = i;
            }
        }
        int x = r.RandTra(0,n);                     //estrae 0..n 
        d = ord[x];                                 //estrae dir. rimasta
        ord[x] = ord[n];                            //e la toglie
    }
    return d;                                       //direzione estratta
}
