 /*
   Questo file fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


// C-FRAZ: funzioni (metodi) della classe Frazione
//
// Questa  l'implementazione della classe Frazione, che definisce le
// funzioni che non sono gi definite inline nell'interfaccia (C-FRAZ.H).
// Il codice oggetto che si ottiene compilandola va linkato col resto
// del programma.


#include "c-fraz.h"


/********************************************************************
 * Funzioni della classe Frazione
 ********************************************************************/

// operator+ : somma se stessa alla frazione fr, ritorna risultato.

Frazione Frazione::operator+(Frazione fr)           //<1>
{
    Frazione r1 = Riduce(*this);                    //riduce operandi <2>
    Frazione r2 = Riduce(fr);
    long mcm = Mcm(r1.d,r2.d);                      //min. comune multiplo
    Frazione r(mcm/r1.d*r1.n+mcm/r2.d*r2.n,mcm);    //risultato <3>
    return Riduce(r);                               //riduce e ritorna
}

/* Note su operator+ :
    <1> Frazione::operator+  la funzione operator+ della classe Frazione.
        L'ulteriore "Frazione" all'inizio  il tipo del valore ritornato.
    <2> La Frazione r1 viene dichiarata ed usata nella stessa istruzione:
        riceve la Frazione restituita dalla funzione Riduce. this  per
        definizione un pointer all'oggetto corrente, quindi *this  l'oggetto
        corrente.
    <3> r  un altro oggetto di tipo Frazione, che viene inizializzato con
        i valori indicati. Questa non  la procedura pi semplice per
        calcolare la somma di due frazioni: abbiamo preferito minimizzare
        il rischio di overflow nei calcoli.
*/


// operator* : moltiplica se stessa per la frazione fr, ritorna risultato.

Frazione Frazione::operator*(Frazione fr)
{
    Frazione r1 = Riduce(*this);                    //riduce operandi
    Frazione r2 = Riduce(fr);
    return Riduce(Frazione(r1.n*r2.n,r1.d*r2.d));   //calcola e riduce
}


// Riduce: riduce frazione eliminando i fattori comuni.

Frazione Frazione::Riduce(Frazione fr)
{
    Frazione r;                                     //risultato

    if (fr.n == 0) {                                //se num 0, ritorna 0/1 
        r = Frazione(0,1);
    } else {
        long com = Mcd(fr.n,fr.d);                  //fattori comuni
        r = Frazione(fr.n/com,fr.d/com);
    }
    return r;
}


// Mcd: ritorna il massimo comune divisore del valore assoluto di due numeri.

long Frazione::Mcd(long n1, long n2)
{
    if (n1 < 0)  n1 = -n1;                          //elimina segno
    if (n2 < 0)  n2 = -n2;                         

    while (n1 != n2) {                              //finch diversi
        if (n1 > n2) {                              //sottrae min da max
            n1 -= n2;                               
        } else {
            n2 -= n1;
        }
    }
    return n1;                                      //ritorna mcd
}
