/*
   Questo file fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


// C-FRAZ.H: dichiara la classe Frazione
//
// Questa  l'interfaccia della classe Frazione, che va inclusa in tutti
// i programmi che vogliono dichiarare oggetti di tipo Frazione: dichiara
// la classe e le sue componenti (variabili e funzioni).
//
// La classe Frazione consente di trattare le frazioni (composte da
// numeratore e denominatore) come se fossero normali variabili.


#include <iostream.h>


#ifndef _C_FRAZ_H_        //non include due volte
#define _C_FRAZ_H_


/********************************************************************
 * Classe Frazione: frazioni come numeratore e denominatore.
 ********************************************************************/

class Frazione {
private:                                            //<1>
    long n;                                         //numeratore
    long d;                                         //denominatore

public:                                             //<2>
    Frazione(long num = 1, long den = 1)            //costruttore <3>
             { n = num; d = den; };

    long Num()  { return n; };                      //numeratore <4>
    long Den()  { return d; };                      //denominatore

    Frazione operator+(Frazione fr);                //somma <5>
    Frazione operator-(Frazione fr)                 //sottrazione <6>
        { return operator+(Frazione(-fr.n,fr.d)); };
    Frazione operator*(Frazione fr);                //moltiplicazione
    Frazione operator/(Frazione fr)                 //divisione
        { return operator*(Frazione(fr.d,fr.n)); };
    Frazione Riduce(Frazione fr);                   //riduce frazione <7>

    friend istream& operator>>(istream &in, Frazione &fr)  //input <8>
        { char c; return in >> fr.n >> c >> fr.d; };
    friend ostream& operator<<(ostream &out, Frazione fr)  //output <9>
        { return out << fr.n << '/' << fr.d; };

private:                                            //<10>
    long Mcd(long n1, long n2);                     //max comune divisore
    long Mcm(long n1, long n2)                      //minimo comune multiplo
        { return n1/Mcd(n1,n2)*n2; };
};

/* Note sulla classe Frazione:
    <1> Le componenti private non sono accessibili dall'esterno della classe:
        per accedervi occorre usare le apposite funzioni. Questo garantisce
        l'integrit dei dati: non possono essere alterati nemmeno per sbaglio.
        Se qualcosa non va, il problema  sicuramente nella classe.
    <2> Le componenti public sono visibili dall'esterno. Queste sono le
        funzioni con le quali si agisce su un oggetto della classe Frazione.
    <3> Questo  il costruttore, riconoscibile dal fatto che ha lo stesso
        nome della classe e non  indicato il tipo del valore di ritorno.
        Il costruttore viene chiamato automaticamente quando si dichiara, o
        comunque si crea, un oggetto della classe Frazione. Qui ci sono due
        argomenti di default: se non li si passa, la Frazione  inizializzata
        a 1/1. Il sorgente della funzione  talmente semplice che viene
        scritto inline: si tratta di una specie di macro che viene sostituita
        nel testo, evitando la perdita di tempo di una chiamata di funzione
        ed accelerando l'esecuzione del programma.
    <4> Dato che non si pu accedere direttamente alle componenti private,
        si devono usare le apposite funzioni (vedi nota 1). Anche queste
        funzioni sono inline (vedi nota 3).
    <5> La funzione operator+ definisce cosa avviene quando si esegue una
        somma (+) di due oggetti di tipo Frazione. Uno  quello corrente,
        l'altro  quello indicato come parametro. La funzione non  inline:
        questo  solo il prototipo, la definizione  nel file C-FRAZ.CPP.
    <6> La sottrazione  definita inline come un'addizione con una frazione
        col segno del numeratore cambiato. Scrivendo Frazione(-fr.n,fr.d)
        viene creato un oggetto provvisorio di tipo Frazione, che 
        inizializzato con i valori indicati. Al termine della funzione,
        l'oggetto provvisorio sparisce.
    <7> La funzione Riduce semplifica la frazione, eliminando i fattori
        comuni a numeratore e denominatore.
    <8> La sintassi per ridefinire l'operatore di input >>  un po' pi
        complessa che per gli altri operatori. Gli argomenti ed il valore
        di ritorno devono essere reference, ed il modo in cui  definita la
        classe istream richiede l'uso di friend (che non approfondiamo).
        Avendo ridefinito l'operatore >>, si pu eseguire l'input di un
        oggetto del tipo Frazione come fosse una normale variabile.
    <9> Si potrebbe migliorare l'operatore << (output) stampando solo il
        numeratore nel caso il denominatore sia 1. In questo caso sarebbe
        meglio non usare una funzione inline ( preferibile usare inline
        solo per funzioni molto brevi).
   <10> Queste funzioni sono definite private, e quondi non sono accessibili
        dall'esterno della classe. Sono per uso interno.
*/


#endif
