/*
   Questo file fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


// C-LABIR: funzioni (metodi) della classe Labirinto


#include "c-labir.h"


// costruttore: crea caselle labirinto (la prima a posx,posy)
//              delle dimensioni date, disegna rettangolo pieno.

Labirinto::Labirinto(int numh, int numv, int posh, int posv)
{
    nh = numh;                                      //dimensioni
    nv = numv;
    h0 = posh;                                      //e posizione
    v0 = posv;

    int pv = v0;
    for (int v = 0; v < nv; v++) {                  //crea caselle
        int ph = h0;
        for (int h = 0; h < nh; h++) {
            labi[v][h] = new Casella(ph,pv);        //crea e posiziona <1>
            ph += DISTH;
        }
        pv += DISTV;
    }

    SegnaAdiacenti();                               //collega adiacenti <2>

    entrata = labi[nv-1][r.RandTra(0,nh-1)];        //estrae entrata <3>
    uscita = labi[0][r.RandTra(0,nh-1)];            //e uscita
    entrata->MetteSimbolo(SUD,NULLA);               //porta di ingresso <4>
    uscita->MetteSimbolo(NORD,NULLA);               //porta di uscita
}

/* Note sul costruttore:
    <1> L'istruzione new crea un oggetto del tipo indicato, cio un esemplare
        della classe Casella, e ritorna un pointer a questo. Equivale alla
        vecchia malloc, ma l'oggetto viene anche inizializzato (viene cio
        chiamato il suo costruttore). labi[][]  un array di pointer ad
        oggetti di tipo Casella.
    <2> La SegnaAdiacenti comunica a ciascuna casella quali sono le sue
        vicine nelle quattro direzioni. Non si poteva farlo durante il
        ciclo, perch occorre che tutte le caselle siano state create.
    <3> entrata  un pointer alla casella di ingresso. L'oggetto r, che  un
        esemplare della classe Random,  dichiarato nello header C-LABI.H.
    <4> Dato che entrata  un pointer ad un oggetto, si usa la stessa
        sintassi -> dei pointer a strutture.
*/


// distruttore: elimina le caselle (che si auto-cancellano dal video).

Labirinto::~Labirinto()
{
    for (int v = 0; v < nv; v++) {                  //cancella caselle
        for (int h = 0; h < nh; h++) {
            delete labi[v][h];                      //<1>
        }
    }
}

/* Note sul distruttore:
    <1> L'istruzione delete distrugge l'oggetto puntato dal suo operando,
        cio la Casella puntata da labi[v][h], liberando la memoria occupata
        nello heap. Equivale alla vecchia free, ma viene anche chiamato il
        distruttore della classe a cui l'oggetto appartiene (Casella).
*/


// SegnaAdiacenti: registra adiacenze di ciascuna casella.

void Labirinto::SegnaAdiacenti()
{
    Casella *pc;                                    //pointer a casella <1>

    for (int v = 0; v < nv; v++) {                  //scandisce caselle
        for (int h = 0; h < nh; h++) {
            pc = labi[v][h];                        //punta a casella
            if (v > 0)                              //descrive adiacenze
                pc->Adiacente(NORD,labi[v-1][h]);   //<2>
            if (v < nv-1)
                pc->Adiacente(SUD,labi[v+1][h]);
            if (h > 0)
                pc->Adiacente(OVEST,labi[v][h-1]);
            if (h < nh-1)
                pc->Adiacente(EST,labi[v][h+1]);
        }
    }
}

/* Note su SegnaAdiacenti:
    <1> pc  un pointer ad un esemplare (oggetto) della classe Casella.
    <2> La funzione Adiacente della classe Casella registra un pointer alla
        casella adiacente nella direzione indicata.
*/


// Risolve: risolve labirinto con ritardo indicato tra le caselle

void Labirinto::Risolve(int rit)
{ 
    entrata->MetteSimbolo(SUD,ORMA);                //porta di ingresso <1>
    entrata->Cerca(uscita,rit);                     //cerca uscita <2>
    uscita->MetteSimbolo(NORD,ORMA);                //porta di uscita
}

/* Note su Risolve:
    <1> Disegna la porta di ingresso.
    <2> Il meccanismo  ricorsivo: chiamando la casella di ingresso
        (entrata), questa chiama la stessa funzione delle caselle adiacenti
        finch non viene raggiunta la casella indicata (uscita). Tutto il
        necessario perch questo funzioni  nella classe Casella. Da notare
        che non esiste alcun algoritmo esplicito sotto forma di funzione
        che esamini le varie caselle: ciascuna di esse  indipendente.
*/
