/*
   Questo file fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


// C-TIMER.H: dichiara la classe Timer
//
// Questa  l'interfaccia della classe Timer, che va inclusa in tutti
// i programmi che vogliono utilizzare oggetti di tipo Timer: dichiara
// la classe e le sue componenti (variabili e funzioni).
//
// La classe Timer consente di eseguire ritardi e sincronizzazioni,
// senza necessit di inizializzazioni o variabili aggiuntive.


#ifndef _C_TIMER_H_       //non include due volte
#define _C_TIMER_H_


#include <time.h>


/********************************************************************
 * Classe Timer: per ritardi e sincronizzazioni.
 ********************************************************************/

class Timer {
private:                                            //non visibili <1>
    clock_t t0;                                     //istante di creazione                              
public:
    Timer() { t0 = clock(); };                      //costruttore <2>
    void Aspetta(int tk);                           //aspetta tk tick <3>
    void Aspetta(double sec)                        //aspetta sec secondi
         { Aspetta(int(sec*CLK_TCK)+1); };          //<4>
    void Sincro(int tk);                            //sincronizza a tk tick
    void Sincro(double sec)                         //sincr. a sec secondi
         { Sincro(int(sec*CLK_TCK)+1); };             
};

/* Note sulla classe Timer:
    <1> Le componenti private non sono visibili dall'esterno della classe:
        possono quindi essere usate solo dalle funzioni appartenenti alla
        classe stessa.
    <2> Ogni volta che un oggetto (un esemplare) della classe Timer viene
        dichiarato (o creato con new), il suo costruttore legge l'istante di
        creazione nella variabile t0. Questo consente alla funzione Sincro
        di funzionare correttamente gi dalla prima chiamata.
    <3> Ci sono due funzioni Aspetta, che differiscono solo per il tipo
        dell'argomento: int o double. Se l'argomento  un int viene
        chiamata la prima (che aspetta il numero di tick indicato), se 
        un double viene chiamata la seconda (che aspetta invece il numero
        di secondi indicato). E' un sistema pratico, ma che forse potrebbe
        causare una certa confusione.
    <4> L'aggiunta di un tick riduce l'errore medio dovuto a due
        arrotondamenti per difetto: quello da long ad int, e quello della
        funzione Sincro, che mediamente perde mezzo tick (la lettura del
        clock avviene in un istante indefinito rispetto alla transizione).
        Sui tempi brevi rimane comunque una certa imprecisione.
*/

#endif
