/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* CARTE: distribuisce le carte del tressette, conta i punti */


#include <stdio.h>
#include <stdlib.h>
#include <time.h>


struct carta {              /*una carta da tressette <1>*/
    int nome;               /*nome carta ('1','2',...'F','C','R')*/
    int presa;              /*ordine di presa (se pi alto prende) <2>*/
    int valore;             /*valore nel tressette, in terzi di punto*/
    int seme;               /*simbolo del seme*/
};

struct carta serie[] = {    /*carte di un seme <3>*/
    '4',0,0,0,
    '5',1,0,0,
    '6',2,0,0,
    '7',3,0,0,
    'F',4,1,0,
    'C',5,1,0,
    'R',6,1,0,
    '1',7,3,0,
    '2',8,1,0,
    '3',9,1,0
};

char semi[4] = { '/', '*', 'Y', '|' };              /*spade..bastoni <4>*/

char *terzi[] = { "", " e un terzo", " e due terzi" };    /*per stampe <5>*/


#define NSERIE (sizeof(serie)/sizeof(struct carta)) /*carte per seme <6>*/
#define NSEMI  4                                    /*numero di semi*/
#define NCARTE (NSERIE*NSEMI)                       /*totale carte <7>*/
#define NGIOCA 4                                    /*numero giocatori*/
#define NMANO  (NCARTE/NGIOCA)                      /*carte per mano <8>*/


void NuovoMazzo(struct carta m[]);                  /*prototipi funzioni*/
void MischiaMazzo(struct carta m[]);
int  StampaCarte(int n, struct carta c[]);


/* Note sulle dichiarazioni:
    <1> La struttura carta comprende sotto un solo nome quattro variabili
        int. Dato che nessuna di queste variabili deve contenere numeri
        maggiori di 127, si potrebbero anche usare quattro char. A seconda
        del compilatore, questo potrebbe comportare un risparmio di memoria
        oppure solo un rallentamento (l'accesso a un singolo char  meno
        efficiente dell'accesso ad un int, i char convengono negli array).
    <2> L'ordine di presa non  usato in questo programma, ma lo sarebbe
        se si volesse scrivere un programma in grado di giocare una partita.
    <3> Un array di strutture pu essere inizializzato elencando il
        contenuto di ciascun elemento (volendo, tra graffe). Il programma
        pu poi modificarne il contenuto, se lo desidera: si tratta comunque
        di variabili.
    <4> Era lo stesso scrivere una stringa con i quattro simboli: "/*Y|". Il
        compilatore non avrebbe aggiunto lo zero finale, dato che 
        esplicitamente indicata la dimensione (4 elementi). Con la sintassi
        usata, si potrebbero anche usare int invece di char (non c' una
        differenza significativa). Usando il set di caratteri del PC IBM,
        si potrebbero trovare simboli migliori per spade, denari, coppe e
        bastoni, ad esempio i caratteri con codice 24, 1, 157 e 20.
    <5> terzi[]  un array di pointer a stringhe, non un array di stringhe.
        Ciascun elemento di terzi[] contiene solo un pointer a char: la
        relativa stringa dev'essere definita da qualche altra parte. In
        questo caso gli elementi di terzi[] sono inizializzati in modo che
        puntino alle tre costanti stringa indicate. Trattandosi di costanti,
        le stringhe puntate da terzi[] non possono essere modificate.
    <6> La dimensione dell'array di strutture serie[], che il compilatore ha
        calcolato in base al numero di inizializzatori forniti, pu essere
        calcolata dividendo l'occupazione di memoria complessiva dell'array
        (sizeof(serie)) per l'occupazione di memoria di un singolo elemento
        (sizeof(struct carta)): il risultato  il numero di elementi. Le
        parentesi attorno all'espressione sono vivamente consigliabili (vedi
        nota seguente).
    <7> Anche se il funzionamento di #define non  stato ancora descritto a
        fondo nel corso, anticipiamo che le espressioni indicate non sono
        precalcolate ma vengono sostituite nel testo prima della compilazione
        vera e propria. Di conseguenza  sempre meglio racchiuderle tra
        parentesi per evitare errate valutazioni delle espressioni, come in
        effetti avverrebbe in questo programma (vedi nota 3 di MischiaMazzo).
    <8> Queste dichiarazioni suppongono che il numero delle carte sia
        divisibile per il numero dei giocatori. Per generalizzare il
        programma, basterebbe definire un resto (es. NCARTE%NGIOCA), in base
        al quale decidere di eliminare una o pi carte (es. il 4 di bastoni).
*/


/********************************************************************
 * main
 ********************************************************************/

main()
{
    struct carta mazzo[NCARTE];                     /*mazzo di carte <1>*/
    struct carta mano[NGIOCA][NMANO];               /*mani dei giocatori <2>*/
    int          n,i,g;

    printf("\n Distribuzione delle carte del tressette:\n\n"
           "(%c = spade, %c = denari, %c = coppe, %c = bastoni)\n",
           semi[0],semi[1],semi[2],semi[3]);

    srand((unsigned int)time(NULL));                /*avvia random*/
    NuovoMazzo(mazzo);                              /*costruisce array <3>*/
    MischiaMazzo(mazzo);                            /*mischia*/

    n = 0;
    for (i = 0; i < NMANO; i++) {                   /*distribuisce <4>*/
        for (g = 0; g < NGIOCA; g++) {
            mano[g][i] = mazzo[n++];
        }
    }

    for (g = 0; g < NGIOCA; g++) {                  /*e stampa*/
        printf("\nGiocatore %d:  ",g+1);
        n = StampaCarte(NMANO,mano[g]);             /*ritorna punteggio <5>*/
        printf("  (%d punti%s)\n",n/3,terzi[n%3]);  /*<6>*/
    }
}

/*  Note su main:
    <1> mazzo[]  un normale array di strutture, che conterr tutte le carte
        da distribuire.
    <2> mano  un array bidimensionale di strutture, che contiene le carte
        distribuite, suddivise tra gli NGIOCA giocatori. Un'alternativa per
        risparmiare memoria e migliorare l'efficienza del programma sarebbe
        quella di usare non un array di strutture, ma un array di pointer a
        strutture, che puntino ai vari elementi di mazzo[], senza che questi
        vengano fisicamente spostati. Non l'abbiamo usata per facilitare la
        comprensione del programma, ma certo sarebbe la scelta preferita
        da un esperto programmatore C.
    <3> Come tutti gli array, un array di strutture si passa per riferimento,
        e pu quindi essere modificato dalla funzione chiamata.
    <4> Le carte sono distribuite una alla volta per migliorare la casualit
        della distribuzione. Dato che il mazzo era gi mischiato, sarebbe
        stato forse sufficiente dividerlo in quattro parti. In un programma
        completo, sarebbe anche il caso di riordinare le carte distribuite
        per facilitare la lettura ai giocatori.
    <5> Il punteggio ritornato  in terzi di punto, secondo i valori del
        tressette.
    <6> L'uso dell'array terzi[] semplifica notevolmente la routine,
        consentendo di stampare stringhe diverse a seconda del valore del
        resto, senza bisogno di ricorrere ad un if..elseif..else.
*/


/********************************************************************
 * NuovoMazzo: crea mazzo di carte nell'array di carte m[], con
 *             NSERIE carte per ciascuno degli NSEMI semi.
 ********************************************************************/

void NuovoMazzo(struct carta m[])       /*<1>*/
{
    int s,n;
    int nc;

    nc = 0;                                         /*nessuna carta*/
    for (s = 0; s < NSEMI; s++) {                   /*per ciascun seme*/
        for (n = 0; n < NSERIE; n++) {              /*copia la serie*/
            m[nc] = serie[n];                       /*<2>*/
            m[nc].seme = semi[s];                   /*aggiunge seme <3>*/
            nc++;
        }
    }
}

/*  Note su NuovoMazzo:
    <1> Il parametro ricevuto  un array di strutture, che  passato per
        riferimento: ogni modifica ha effetto sull'originale.
    <2> Una struttura pu essere assegnata come una normale variabile.
    <3> Per accedere ad una componente di una struttura si usa come sempre
        il punto.
*/


/********************************************************************
 * MischiaMazzo: mischia mazzo di NCARTE per scambi successivi.
 ********************************************************************/

void MischiaMazzo(struct carta m[])     /*<1>*/
{
    int          n;
    int          c1,c2;
    struct carta x;

    n = NCARTE*5;                                   /*numero di scambi <2>*/
    while (n--) {                                   /*effettua scambi:*/
        c1 = rand()%NCARTE;                         /*estrae due carte <3>*/
        c2 = rand()%NCARTE;                         /*<4>*/

        x = m[c1];                                  /*e le scambia <5>*/
        m[c1] = m[c2];
        m[c2] = x;
    }
}

/*  Note su MischiaMazzo:
    <1> Un sistema per mischiare velocemente un mazzo di carte consiste in
        una serie di scambi di due carte a caso (due elementi di m[]).
    <2> Con un numero di scambi pari a 5 volte il numero di carte, si
        ottiene un rimescolamento sufficiente per i nostri scopi.
    <3> c1 risulta compreso tra 0 e NCARTE-1. E' importante notare che questo
        meccanismo non funziona se nella definizione di NCARTE mancano le
        parentesi (vedi nota 7 delle dichiarazioni). Difatti NCARTE  in
        realt NSERIE*NSEMI: sostituendo si ottiene rand()%NSERIE*NSEMI, col
        risultato che rand()%NSERIE viene eseguito per primo, dando come
        risultato un numero tra 0 e NSERIE-1, che poi viene moltiplicato
        per NSEMI. In pratica, c1 risulta sempre multiplo di NSEMI e sono
        rimescolate sempre le stesse carte (coi valori dati, una ogni 4).
        Con le parentesi nella #define si ha invece rand()%(NSERIE/NSEMI),
        che funziona correttamente.
    <4> La seconda carta estratta (c2) pu essere uguale alla prima (c1), ma
        questo non comporta inconvenienti: la carta viene semplicemente
        scambiata con se stessa, cio non viene scambiata affatto.
    <5> Per scambiare due variabili occorre fare uso di una variabile
        intermedia dello stesso tipo, in questo caso x.
*/


/********************************************************************
 * StampaCarte: stampa n carte dall'array c[], ritorna valore totale.
 ********************************************************************/

int StampaCarte(int n, struct carta c[])    /*<1>*/
{
    struct carta *pc;                               /*pointer a carta <2>*/
    int          i,v;

    v = 0;                                          /*azzera valore*/
    for (pc = c; pc < c+n; pc++) {                  /*<3>*/
        printf("  %c%c",pc->nome,pc->seme);         /*stampa nome, seme <4>*/
        v += pc->valore;                            /*somma valore*/
    }

    return v;                                       /*valore totale*/
}

/*  Note su StampaCarte:
    <1> Anche questa funzione riceve un array di strutture per riferimento,
        cio riceve in realt l'indirizzo della prima struttura dell'array.
    <2> pc  un pointer ad una variabile di tipo struct carta, *pc  la
        variabile puntata da pc.
    <3> Il pointer pc scorre dal primo elemento dell'array (c equivale a
        &c[0]) fino all'ultimo. Un'alternativa di comprensione pi
        immediata, anche se meno efficiente, consisteva nell'usare la
        classica variabile intera di controllo del ciclo, ad esempio:
        for (i = 0; i < n; i++).
    <4> pc->nome  un'abbreviazione per (*pc).nome, cio la componente nome
        della struttura puntata da pc. Accedere attraverso pointer a
        strutture  molto pi efficiente che usare un indice (es. c[i].nome),
        perch internamente l'accesso a c[i] richiede una moltiplicazione
        mentre il pointer pc contiene gi l'indirizzo della struttura voluta.
*/
