/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* DATTILO: test di dattilografia: misura velocit ed errori di battitura */


#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>


void MostraDataOra(void);                           /*prototipi funzioni*/
int ValutaFrase(char *fr, char *fe);


char *frasi[] = {                                   /*frasi da copiare <1>*/
    "Nel mezzo del cammin di nostra vita mi ritrovai per una selva oscura",
    "Quel ramo del lago di Como che volge a mezzogiorno",
    "Le donne, i cavallier, l'arme, gli amori, le cortesie,"
                                " l'audaci imprese io canto",
    "La donzelletta vien dalla campagna in sul calar del sole"
};

#define NFRASI (sizeof(frasi)/sizeof(char *));      /*numero di frasi <2>*/

/* Note sulle dichiarazioni:
    <1> frasi[]  un array di pointer a char, ciascuno dei quali viene
        inizializzato in modo che punti ad una delle stringhe date. In
        pratica frasi[]  un array di costanti stringa.
    <2> Il numero di elementi  dato dalla dimensione complessiva dell'array
        frasi[] diviso per la dimensione di uno dei suoi elementi (che sono
        pointer a char (cio char *). In questo modo, per aggiungere una
        nuova stringa basta scriverla di seguito alle altre: il compilatore
        calcola da s il valore di NFRASI usato poi dal programma.
*/


/********************************************************************
 * main
 ********************************************************************/

main()
{
    time_t inizio;      /*tempo di inizio scrittura <1>*/
    time_t fine;        /*tempo di fine scrittura*/
    int    nf;          /*numero frase scelta*/
    char   fi[128];     /*frase introdotta*/
    int    ne;          /*numero errori*/
    int    ncar;        /*numero caratteri battuti*/
    double nsec;        /*secondi impiegati*/
    int    vel;         /*velocit in caratteri al minuto*/

    printf("\n Valutazione dell'abilit dattilografica:\n\n");

    srand((unsigned int)time(NULL));                /*avvia random <2>*/

    MostraDataOra();                                /*stampa data/ora*/

    nf = rand()%NFRASI;                             /*sceglie frase <3>*/

    printf("Riscrivi esattamente questa"            /*mostra frase <4>*/
           " frase:\n\n%s\n",frasi[nf]);

    inizio = time(NULL);                            /*tempo iniziale <5>*/
    gets(fi);                                       /*input test*/
    fine = time(NULL);                              /*tempo finale*/

    ne = ValutaFrase(fi,frasi[nf]);                 /*valuta <6>*/

    if (ne == 0) {
        puts("Complimenti, non hai fatto nemmeno un errore!\n");
    } else {
        printf("\nHai commesso %d errori.\n\n",ne);
    }

    ncar = strlen(fi);                              /*caratteri battuti*/
    nsec = difftime(fine,inizio);                   /*secondi impiegati <7>*/
    if (nsec <= 0.1)   nsec = 0.1;                  /*evita div. per 0 <8>*/
    vel = (int)( (ncar*60.0)/nsec );                /*velocit <9>*/

    printf("Hai scritto %d caratteri in %.1f secondi, per una \n"
           "velocit di battitura di %d caratteri al minuto.\n",
           ncar, nsec, vel);
}

/*  Note su main:
    <1> time_t  un tipo definito in <time.h> (dipende dal computer e dal
        compilatore), che va bene per contenere il valore ritornato da time.
    <2> Questa  la solita inizializzazione del generatore di numeri casuali.
    <3> Perch questa espressione ritorni correttamente un valore compreso
        tra 0 e NFRASI-1, NFRASI dev'essere stato definito con le parentesi
        intorno all'espressione (vedi nota 2 delle dichiarazioni, il problema
         discusso nel programma CARTE).
    <4> frasi[nf]  una stringa, il cui primo carattere  puntato
        dall'elemento nf dell'array frasi[].
    <5> Legge il tempo corrente dall'orologio interno del PC.
    <6> La ValutaFrase ritorna il numero di errori commessi.
    <7> Le due variabili fine e inizio sono di tipo time_t, ed il loro
        contenuto non  immediatamente interpretabile (dipende dal computer
        e dal compilatore). E' invece valido usare difftime per conoscere
        la differenza tra due tempi, che viene restituita come double.
    <8> Se l'utente battesse subito Return, si avrebbe una divisione per
        zero, con esito imprevedibile. Meglio evitarlo garantendo un tempo
        minimo di un decimo di secondo (la velocit indicata sar in questo
        caso irrealistica).
    <9> ncar  un intero, ma viene automaticamente convertito a double
        dato che  moltiplicato con una costante double. nsec  gi un
        double, e quindi non ci sono problemi di perdita di precisione.
        Alla fine, il risultato  convertito ad int con il cast (int).
        La conversione avverrebbe comunque, ma il cast la evidenzia per
        leggibilit. La parte frazionaria viene troncata, ma in questo
        caso un arrotondamento per difetto  proprio ci che vogliamo.
*/


/********************************************************************
 * MostraDataOra: stampa data e ora correnti.
 ********************************************************************/

void MostraDataOra(void)    /*<1>*/
{
    time_t    t;
    struct tm *tp;                                                /*<2>*/

    char *ng[] = { "Domenica", "Luned", "Marted", "Mercoled",  /*<3>*/
                   "Gioved", "Venerd", "Sabato" };

    char *nm[] = { "Gennaio", "Febbraio", "Marzo", "Aprile",
                   "Maggio", "Giugno", "Luglio", "Agosto",
                   "Settembre", "Ottobre", "Novembre", "Dicembre" };

    t = time(NULL);                                         /*legge tempo*/
    tp = localtime(&t);                                     /*<4>*/

    printf("Sono le %02d:%02d:%02d di %s %d %s %d.\n\n",    /*<5>*/
           tp->tm_hour, tp->tm_min, tp->tm_sec,
           ng[tp->tm_wday], tp->tm_mday, nm[tp->tm_mon],
           1900+tp->tm_year);
}

/*  Note su MostraDataOra:
    <1> La funzione mostra l'ora e la data esatte se l'orologio interno
        del computer funziona correttamente; in alcuni computer, privi di
        batteria interna, occorre indicare data e ora ad ogni avviamento.
    <2> tp  un pointer ad una struct tm, tipo definito in <time.h>.
    <3> ng[]  un array di pointer a stringhe, cio in pratica un array di
        costanti stringa (vedi nota 1 delle dichiarazioni).
    <4> La funzione localtime va chiamata passando l'inirizzo di una
        variabile di tipo time_t contenente il tempo da convertire (t), e
        ritorna un pointer ad una struttura di tipo tm. La struttura  una
        variabile statica che si trova nelle routine di libreria; non
        occorre quindi dichiararla. Dopo la chiamata a localtime, il suo
        contenuto si conserva fino alla successiva chiamata a funzioni che
        riguardano il tempo.
    <5> tp->tm_hour  l'elemento tm_hour della struttura puntata da tp, e
        cos per tutti gli altri. Il giorno della settimana parte da
        Domenica (0), e l'anno parte dal 1900.
*/


/********************************************************************
 * ValutaFrase: confronta frase fr con frase esatta fe, stampa un ^
 *              sotto ogni carattere scorretto (supponendo che sulla
 *              linea sovrastante sia stampata fr) e tenta di
 *              risincronizzarsi in caso di caratteri mancanti o in
 *              eccesso. Segna con * i caratteri mancanti o in eccesso
 *              a fine frase e ritorna il numero totale di errori.
 ********************************************************************/

int ValutaFrase(char *fr, char *fe)     /*<1>*/
{
    int n = 0;      /*numero errori*/
    int r;

    while (*fr && *fe) {                            /*scandisce frase <2>*/
        if (*fr == *fe) {                           /*controlla carattere*/
            putchar(' ');                           /*ok, avanza*/
        } else {
            n++;                                    /*errore, lo conta*/
            putchar('^');                           /*lo segna*/
            if (*fr == *(fe+1)) {                   /*manca carattere? <3>*/
                fe++;                               /*compensa*/
            } else if (*(fr+1) == *fe) {            /*in eccesso? <4>*/
                fr++;                               /*compensa*/
                putchar(' ');
            }
        }
        fr++;                                       /*al prossimo*/
        fe++;
    }

    r = strlen(fr)-strlen(fe);                      /*residui? <5>*/
    if (r < 0)  r = -r;
    while (r--) {
        putchar('*');                               /*segna residui*/
        n++;
    }

    putchar('\n');
    return n;                                       /*numero errori*/
}

/*  Note su ValutaFrase:
    <1> L'algoritmo di confronto non  dei migliori, ma nella sua semplicit
        riesce a recuperare gran parte degli errori di una singola lettera
        ed a riprendere il passo nei confronti successivi.
    <2> Dato che i pointer fr ed fe sono variabili locali (passaggio per
        valore), possono essere tranquillamente alterati: qui sono usati per
        scandire le stringhe. Da notare che (*fr && *fe)  un'abbreviazione
        per: (*fr != 0 && *fe != 0).
    <3> Se il carattere della frase introdotta non  uguale a quello in
        posizione corrispondente nella frase corretta, ma  uguale al
        successivo, suppone che non sia stato battuto il carattere intermedio
        e corregge di conseguenza per non segnare poi errori inesistenti.
    <4> Se il carattere della frase introdotta non  uguale a quello in
        posizione corrispondente nella frase corretta, ma il successivo lo
        , suppone che sia stato battuto un carattere di troppo e corregge
        di conseguenza.
    <5> Se una delle due frasi termina prima dell'altra, c' un errore per
        un numero di caratteri pari alla lunghezza residua.
*/
