/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* FORZE: somma vettoriale di n forze */


#include <stdio.h>
#include <math.h>
#include <string.h>


struct polare {         /*forza espressa in forma polare <1>*/
    double intens;      /*intensit*/
    double direz;       /*direzione (-180..+180)*/
};

struct cartes {         /*forza espressa in forma cartesiana <2>*/
    double x;           /*componente X*/
    double y;           /*componente Y*/
};


struct polare InputForza(void);                     /*prototipi funzioni*/
struct polare SommaForze(struct polare f1, struct polare f2);
void          MostraForza(struct polare f);
struct cartes PolareACartesiana(struct polare f);
struct polare CartesianaAPolare(struct cartes r);


#define PI    3.1415926536      /*pigreco*/
#define GRAD  PI/180.0          /*per conversione gradi/radianti*/
#define EPS   1e-7              /*minimo valore considerato != 0 <3>*/


/* Note sulle dichiarazioni:
    <1> La struttura 'polare' contiene due componenti: intensit e direzione,
        entrambe di tipo double. E' qui impiegata per conservare una forza
        espressa in coordinate polari.
    <2> La struttura 'cartes'  invece adatta a memorizzare una forza
        espressa in coordinate cartesiane, cio scomposta nelle sue due
        componenti ortogonali nel piano.
    <3> La costante EPS serve per decidere se un valore pu essere
        considerato zero, dato che le inevitabili imprecisioni delle funzioni
        trigonometriche non consentono di ottenere lo zero esatto.
*/


/********************************************************************
 * main
 ********************************************************************/

main()
{
    struct polare ftot;         /*totale delle forze <1>*/
    struct polare f;
    int           finito;

    printf("\n Composizione di forze nel piano:\n\n");

    ftot.intens = ftot.direz = 0.0;                 /*azzera totale*/
    do {
        f = InputForza();                           /*<2>*/
        finito = (f.intens == 0.0);
        if (! finito) {
            ftot = SommaForze(ftot,f);              /*<3>*/
            printf("\nForza risultante:  ");
            MostraForza(ftot);
            putchar('\n');
        }
    } while (! finito);
}

/*  Note su main:
    <1> La variabile ftot  una struct polare, quindi racchiude in un'unica
        entit due distinte variabili: ftot.intens e ftot.direz. Lo stesso
        vale per la variabile f dichiarata nella linea successiva.
    <2> La InputForza ritorna una variabile di tipo struct polare, che a sua
        volta  formata da due componenti. Questo consente di aggirare il
        limite delle funzioni C (possono ritornare un solo valore) senza
        ricorrere al passaggio per riferimento (pointer). Si paga per con
        una velocit di esecuzione leggermente inferiore.
    <3> La SommaForze riceve due variabili di tipo struct polare, e ritorna
        una variabile di tipo struct polare. Le variabili indicate come
        argomenti sono comunque passate per valore (cio ne viene passata
        una copia), anche se si tratta di strutture. Il passaggio per valore
        delle strutture  stato introdotto con lo standard ANSI.
*/


/********************************************************************
 * InputForza: ritorna forza in forma polare: intensit, direzione;
 *             se solo Return, ritorna 0.0,0.0.
 ********************************************************************/

struct polare InputForza(void)  /*<1>*/
{
    char          buf[128];     /*buffer di linea*/
    int           ok;           /*flag di stringa valida*/
    struct polare f;            /*forza introdotta <2>*/
    int           n;

    do {
        do {                                              /*legge stringa*/
            printf("Intensit, direzione forza (Return per smettere): ");
            ok = (gets(buf) != NULL);
            if (! ok)  putchar('\a');
        } while (! ok);                                   /*esce con ok vero*/

        n = sscanf(buf,"%lf,%lf",&f.intens,&f.direz);     /*input forza <3>*/
        if (*buf == 0) {                                  /*0,0 se Return*/
            f.intens = f.direz = 0.0;
        } else if (n < 2) {                               /*verifica*/
            puts("\n- Devi introdurre due valori separati da virgola.\n\a");
            ok = 0;
        } else if (f.direz < -180.0 || f.direz > 180.0) {
            puts("\n- La direzione dev'essere compresa tra -180 e +180.\n\a");
            ok = 0;
        }
    } while (! ok);                                       /*ripete se non ok*/

    return f;                                             /*forza <4>*/
}

/*  Note su InputForza:
    <1> La InputForza ritorna un valore di tipo struct polare, che comprende
        in realt due distinti valori.
    <2> Anche se f  una struttura,  una variabile locale come le altre.
    <3> La sscanf mette direttamente i valori convertiti dalla stringa buf[]
        nelle due componenti della struttura f.
    <4> Ritornando una struttura, la funzione ritorna pi di un valore (vedi
        note 1 e 2 del main).
*/


/********************************************************************
 * SommaForze: ritorna somma delle forze f1 e f2 in forma polare.
 ********************************************************************/

struct polare SommaForze(struct polare f1, struct polare f2)    /*<1>*/
{
    struct cartes r1;       /*componenti, in forma cartesiana*/
    struct cartes r2;

    r1 = PolareACartesiana(f1);                     /*conv. a cartesiane <2>*/
    r2 = PolareACartesiana(f2);

    r1.x += r2.x;                                   /*somma in r1 <3>*/
    r1.y += r2.y;

    f1 = CartesianaAPolare(r1);                     /*conv. a polare <4>*/
    return f1;                                      /*ritorna risultante*/
}

/*  Note su SommaForze:
    <1> La SommaForze riceve due variabili di tipo struct polare, e ne
        ritorna una terza contenente la loro somma vettoriale.
    <2> Per sommare due forze espresse in coordinate polari, conviene
        convertirle in forma cartesiana.
    <3> La somma di due forze espresse in coordinate cartesiane  banale. In
        questo caso si usa r1 (uno degli addendi) per contenere il risultato:
        il valore precedente di r1 va perso, ma tanto era solo una variabile
        locale che non serve pi. In questo modo si risparmia una variabile.
    <4> Il parametro f1, che non serve pi, viene riutilizzato per contenere
        il risultato. Questo non altera in alcun modo l'argomento utilizzato
        nella chiamata, perch il passaggio avviene per valore e quindi f1
         solamente una copia locale dell'argomento indicato nella funzione
        chiamante.
*/


/********************************************************************
 * MostraForza: stampa la forza f in forma polare.
 ********************************************************************/

void MostraForza(struct polare f)
{
    printf("intensit = %f",f.intens);
    if (fabs(f.intens) >= EPS) {                    /*se intens zero <1>*/
        printf(", direzione = %f",f.direz);
    }
    putchar('\n');
}

/*  Note su MostraForza:
    <1> Se l'intensit  nulla, non ha senso mostrare una direzione. Dato
        che le imprecisioni delle funzioni trigonometriche non consentono
        di avere lo zero esatto, consideriamo zero un valore quando il suo
        valore assoluto  minore della costante EPS definita all'inizio (vedi
        nota 3 del main).
*/


/********************************************************************
 * PolareACartesiana: ritorna l'equivalente in forma cartesiana (x,y)
 *                    della forza f ricevuta in forma polare.
 ********************************************************************/

struct cartes PolareACartesiana(struct polare f)    /*<1>*/
{
    struct cartes r;

    f.direz *= GRAD;                                /*passa in radianti <2>*/

    r.x = f.intens*cos(f.direz);                    /*ricava cartesiana <3>*/
    r.y = f.intens*sin(f.direz);

    return r;                                       /*ritorna cartesiana*/
}

/*  Note su PolareACartesiana:
    <1> Questa  una funzione di uso generale per convertire un valore
        dalla forma polare alla forma cartesiana.
    <2> Abbiamo scelto di conservare la direzione in gradi sessagesimali, ma
        le funzioni trigonometriche del C lavorano in radianti.
    <3> Come insegna la trigonometria applicata ad un triangolo rettangolo.
*/


/********************************************************************
 * CartesianaAPolare: ritorna l'equivalente in forma polare (intens,
 *                    direz) della forza f data in forma cartesiana.
 ********************************************************************/

struct polare CartesianaAPolare(struct cartes r)    /*<1>*/
{
    struct polare f;

    if (r.x == 0.0 && r.y == 0.0) {                 /*evita errori <2>*/
        f.intens = f.direz = 0.0;
    } else {
        f.intens = sqrt(r.x*r.x+r.y*r.y);           /*ricava polare <3>*/
        f.direz = atan2(r.y,r.x);                   /*<4>*/
        f.direz /= GRAD;                            /*converte in gradi <5>*/
    }

    return f;                                       /*ritorna polare*/
}

/*  Note su CartesianaAPolare:
    <1> Questa  una funzione di uso generale per convertire un valore
        dalla forma cartesiana alla forma polare.
    <2> Se entrambi gli argomenti sono nulli, la atan2 d errore. Lo evitiamo
        evitando di eseguirla e ritornando valori nulli per la forza in forma
        polare (se l'intensit  nulla, la direzione non ha significato).
    <3> Non  altro che l'ipotenusa di un triangolo rettangolo.
    <4> La funzione atan2 ritorna direttamente l'angolo che ci serve (in
        radianti). Senza questa sarebbe stato comunque sufficiente usare
        la atan (arcotangente), tenendo conto del segno di x per identificare
        il quadrante corretto.
    <5> Poich conserviamo la direzione in gradi sessagesimali, occorre
        riconvertire il valore (che la atan2 ha ritornato in radianti).
*/
