/*
   Questo programma fa parte del Corso di C
   Copyright (C) 1990 Gruppo Editoriale Jackson
   Copyright (C) 2002 Enrico Colombini
   Tutti i diritti riservati
*/


/* PIANO: disegna rettangoli sovrapposti su un piano */


#include <stdio.h>
#include <string.h>


#define XMIN    0       /*limiti delle coordinate*/
#define XMAX    78
#define YMIN    0
#define YMAX    22
#define MAXRETT 9       /*numero massimo di rettangoli*/


typedef struct {                /*definisce il tipo Punto <1>*/
    int x;                      /*coordinata orizzontale*/
    int y;                      /*coordinata verticale*/
} Punto;

typedef struct {                /*definisce il tipo Rettangolo <2>*/
    Punto as;                   /*vertice in alto a sinistra*/
    Punto bd;                   /*vertice in basso a destra*/
} Rettangolo;

typedef char  Linea[XMAX+1];    /*linea per matrice di disegno, pi 0 <3>*/
typedef Linea Piano[YMAX];      /*piano di disegno <4>*/


void AzzeraPiano(Piano m);                          /*prototipi funzioni*/
void DisegnaRettangolo(Rettangolo *ap, Piano m);
void MostraPiano(Piano m);
int  InputRettangolo(Rettangolo *rp);
int  InputPunto(Punto *pp, char *prompt, int retOk);

/*  Note sulle dichiarazioni:
    <1> In pratica il nome Punto diventa un'abbreviazione per la struttura
        descritta dopo typedef, cio Punto equivale a: struct {...}. Questo
        nuovo tipo (Punto) si usa esattamente come int, long o float.
    <2> Una variabile di tipo Rettangolo contiene due variabili di tipo Punto.
    <3> In quest'altro caso, il tipo Linea  un array di XMAX+1 char. Notare
        la corrispondenza della sintassi che segue typedef con la definizione
        di un normale array di char.
    <4> Piano  definito come un array di YMAX elementi di tipo Linea. Si
        possono quindi costruire nuovi tipi a partire da tipi definiti in
        precedenza, come gi visto alla nota 2.
*/


/********************************************************************
 * main
 ********************************************************************/

main()
{
    Rettangolo ar[MAXRETT];             /*array di rettangoli <1>*/
    int        nRett;                   /*numero rettangoli introdotti*/
    Piano      dis;                     /*piano di disegno <2>*/
    int        ok;
    int        i;

    printf("\n Composizione di forze nel piano:\n");

    nRett = 0;                                      /*numero rettangoli*/
    do {
        printf("\nCoordinate del rettangolo numero %d"
               " (Return per smettere):\n",nRett+1);
        ok = InputRettangolo(ar+nRett);             /*input rettangolo <3>*/
        if (ok)  nRett++;                           /*lo conta*/
    } while (ok && nRett < MAXRETT);                /*esce se max*/

    AzzeraPiano(dis);                               /*riempie di '0' <4>*/

    for (i = 0; i < nRett; i++) {                   /*disegna in memoria*/
        DisegnaRettangolo(ar+i,dis);                /*<5>*/
    }

    MostraPiano(dis);                               /*e stampa*/
}

/*  Note su main:
    <1> Non importa come sia definito il tipo Rettangolo: questa linea
        definisce un array di MAXRETT rettangoli. Tutto il main di questo
        esempio lavora ad alto livello, senza conoscere la struttura interna
        delle variabili che usa. Rettangolo si usa come se fosse un tipo
        base del C, ad esempio int, semplificando il lavoro e rendendo il
        tutto molto pi leggibile e maneggevole.
    <2> La variabile dis, che appare come una variabile semplice,  in realt
        un array, in cui ciascun elemento  di tipo Linea, e quindi composto
        a sua volta da un array di char. Questa dichiarazione equivale a:
        char[YMAX][XMAX+1] dis; che non solo sarebbe meno leggibile ma
        porrebbe il problema di come far conoscere alle altre funzioni la
        struttura della variabile. In questo modo non c' bisogno che sia
        globale la variabile dis, ma basta che il suo tipo (Piano) sia noto
        a tutte le funzioni che lo usano (grazie alla typedef iniziale).
    <3> Dato che ar[]  un array di rettangoli, ar+nRett equivale a:
        &ar[nRett], cio all'indirizzo dell'elemento nRett dell'array. Qui
        usiamo il passaggio per riferimento, in modo da evitare la copiatura
        del rettangolo nel passaggio alla funzione, e poi nel ritorno da
        questa (tra l'altro la funzione pu cos ritornare un altro valore,
        che in questo caso  0 se l'utente preme Return).
    <4> Dato che dis  un array (vedi la definizione del tipo Piano), il
        passaggio avviene per riferimento (senza copia), e la funzione
        riceve l'indirizzo dell'array dis, che pu quindi essere modificato.
    <5> Anche in questo caso viene passato l'indirizzo dell'elemento i
        dell'array ar[], cio l'indirizzo di un Rettangolo.
*/


/********************************************************************
 * AzzeraPiano: riempie di '0' tutte le caselle del piano m.
 ********************************************************************/

void AzzeraPiano(Piano m)       /*<1>*/
{
    int y;

    for (y = 0; y < YMAX; y++) {
        memset(m[y],'0',XMAX);                      /*riempie una linea <2>*/
        m[y][XMAX] = 0;                             /*e la termina <3>*/
    }
}

/*  Note su AzzeraPiano:
    <1> Dato che Piano  un array, il parametro m riceve l'indirizzo del
        primo elemento Linea che forma il piano da azzerare: m  quindi un
        pointer all'inizio della prima linea del piano.
    <2> Usiamo la funzione memset (che richiede <string.h>) per evitare un
        ciclo che scandisca la linea sostituendo ogni carattere con '0'.
        Notare che m[y]  una Linea, dunque un array di char (vedi typedef
        iniziale), ed  quindi l'indirizzo del primo elemento della linea.
    <3> Se m[y]  un array di char, m[y][x]  la componente x dell'array, che
         un singolo char. Leggendo ad alto livello, m[y][x]  il carattere x
        della Linea y del Piano m.
*/


/********************************************************************
 * DisegnaRettangolo: disegna il rettangolo puntato da *ap nel piano
 *                    m, incrementando il valore di ogni carattere
 *                    (pixel) che fa parte del rettangolo stesso.
 ********************************************************************/

void DisegnaRettangolo(Rettangolo *ap, Piano m)     /*<1>*/
{
    int x,y;

    for (y = ap->as.y; y <= ap->bd.y; y++) {        /*per righe <2>*/
        for (x = ap->as.x; x <= ap->bd.x; x++) {    /*e per colonne*/
            m[y][x]++;                              /*incrementa <3>*/
        }
    }
}

/*  Note su DisegnaRettangolo:
    <1> Poich il piano m  inizialmente riempito con caratteri '0', ed ogni
        rettangolo disegnato incrementa il valore dei caratteri che ne fanno
        parte, il numero stampato in un dato carattere indica quanti
        rettangoli includono quel punto. ap  un pointer ad una variabile
        di tipo Rettangolo (vedi nota 5 del main).
    <2> ap punta ad una variabile di tipo Rettangolo, che  una struttura,
        quindi ap->as punta alla componente as della struttura stessa. Poich
        la componente as di un Rettangolo  un Punto, cio a sua volta una
        struttura, ap->as.x punta alla componente x della componente as del
        rettangolo puntato da ap (ci vuole un po' per abituarsi a questa
        notazione). In pratica, ap->as.x  la coordinata x del vertice in
        alto a sinistra del rettangolo puntato da ap.
    <3> m[y][x]++ incrementa il carattere x della Linea m[y], cio il
        carattere x della Linea y del Piano m. Notare che m  un pointer,
        ma dato che punta ad un array pu essere usato anche con la
        notazione tipica degli array, cio con le parentesi quadre.
*/


/********************************************************************
 * MostraPiano: stampa il contenuto del piano m.
 ********************************************************************/

void MostraPiano(Piano m)
{
    int y;

    putchar('\n');
    for (y = 0; y < YMAX; y++) {
        puts(m[y]);                                 /*stampa una linea <1>*/
    }
}

/*  Note su MostraPiano:
    <1> Dato che un Piano  definito come un array di elementi di tipo Linea,
        ciascun elemento  un array di caratteri e pu essere trattato con le
        funzioni stringa (lo zero finale  stato aggiunto dalla AzzeraPiano).
*/


/********************************************************************
 * InputRettangolo: input coordinate rettangolo in *r, ritorna falso
 *                  (0) se Return sul primo input.
 ********************************************************************/
int InputRettangolo(Rettangolo *rp)     /*<1>*/
{
    int ok, valido;

    valido = InputPunto(&(rp->as),"Vertice superiore sinistro",1);  /*<2>*/
    if (valido) {
        do {
            InputPunto(&(rp->bd),"Vertice inferiore destro  ",0);
            ok = 1;
            if ((rp->bd.x < rp->as.x) || (rp->bd.y < rp->as.y)) {   /*<3>*/
                printf("\n- Le coordinate del secondo vertice"
                       " devono essere\n  maggiori o uguali a"
                       " quelle del primo (%d,%d).\n\n\a",
                       rp->as.x,rp->as.y);
                ok = 0;
            }
        } while (! ok);
    }

    return valido;
}

/*  Note su InputRettangolo:
    <1> rp contiene l'indirizzo di un Rettangolo, cio  un pointer ad un
        Rettangolo (vedi nota 3 del main).
    <2> rp->as  la componente as del Rettangolo puntato da rp, che  un
        Punto: il vertice superiore sinistro del Rettangolo (vedi anche nota
        2 di DisegnaRettangolo). &(rp->as)  l'indirizzo di questo Punto,
        che viene passato per riferimento alla InputPunto in modo che questa
        possa alterare le componenti del Punto in base ai valori introdotti
        dall'utente. Le parentesi non sono necessarie: si pu scrivere anche
        &rp->as, dato che -> ha la precedenza su &.
    <3> Vedi nota 2 di DisegnaRettangolo.
*/


/********************************************************************
 * InputPunto: input coordinate punto (XMIN..XMAX-1, YMIN..YMAX-1)
 *             in *pp, ritorna 1. Se retOk accetta anche Return come
 *             prima coordinata, e in questo caso ritorna zero.
 ********************************************************************/

int InputPunto(Punto *pp, char *prompt, int retOk)  /*<1>*/
{
    char buf[128];     /*buffer di linea*/
    int  ok;           /*flag di stringa valida*/
    int  nullo;        /* stato premuto solo Return, e retOk  vero*/

    do {
        do {                                              /*legge stringa*/
            printf("%s (x,y) : ",prompt);
            ok = (gets(buf) != NULL);
            nullo = 0;
            if (*buf == 0) {                              /*solo Return <2>*/
                nullo = retOk;                            /*accetta se retOk*/
                if (! nullo)  ok = 0;
            }
            if (! ok)  putchar('\a');
        } while (! ok);                                   /*esce con ok vero*/

        if (! nullo) {
            if (sscanf(buf,"%d,%d",&(pp->x),&(pp->y)) != 2) {  /*<3>*/
                puts("\n- Devi introdurre due valori"
                     " separati da virgola (es. 13,5).\n\a");
                ok = 0;
            } else if (pp->x < XMIN || pp->x > XMAX-1) {
                printf("\n- La coordinata orizzontale dev'essere"
                       " compresa tra %d e %d.\n\n\a",XMIN,XMAX-1);
                ok = 0;
            } else if (pp->y < YMIN || pp->y > YMAX-1) {
                printf("\n- La coordinata verticale dev'essere"
                       " compresa tra %d e %d.\n\n\a",YMIN,YMAX-1);
                ok = 0;
            }
        }

    } while (! ok);                                       /*ripete se non ok*/

    return (! nullo);                                     /*<4>*/
}

/*  Note su InputPunto:
    <1> pp contiene un pointer ad una struttura di tipo Punto.
    <2> E' inutile scrivere: if (strlen(buf) == 0). Se il primo carattere
        della stringa  uno zero, la stringa ha lunghezza zero.
    <3> pp->x  la componente x del Punto puntato da pp, e &pp->x (con o
        senza parentesi)  il suo indirizzo da passare alla sscanf perch
        questa possa alterarne il valore.
*/
